/****************************************************************************
** Meta object code from reading C++ file 'xresource.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/xsettings/xresource.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'xresource.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9XResourceE_t {};
} // unnamed namespace

template <> constexpr inline auto XResource::qt_create_metaobjectdata<qt_meta_tag_ZN9XResourceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "XResource",
        "XResourceKey",
        "Unknown",
        "Xft_DPI",
        "Xft_Antialias",
        "Xft_Hinting",
        "Xft_HintStyle",
        "Xft_RGBA",
        "Xft_LCDFilter",
        "Xcursor_Theme",
        "Xcursor_Size",
        "Xcursor_ThemeCore",
        "Gtk_FontName",
        "Gtk_ThemeName",
        "Gtk_IconThemeName",
        "Gtk_CursorThemeName",
        "Gtk_CursorThemeSize",
        "Gdk_WindowScalingFactor",
        "Gdk_UnscaledDPI",
        "Net_ThemeName",
        "Net_IconThemeName",
        "Net_SoundThemeName"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'XResourceKey'
        QtMocHelpers::EnumData<enum XResourceKey>(1, 1, QMC::EnumFlags{}).add({
            {    2, XResourceKey::Unknown },
            {    3, XResourceKey::Xft_DPI },
            {    4, XResourceKey::Xft_Antialias },
            {    5, XResourceKey::Xft_Hinting },
            {    6, XResourceKey::Xft_HintStyle },
            {    7, XResourceKey::Xft_RGBA },
            {    8, XResourceKey::Xft_LCDFilter },
            {    9, XResourceKey::Xcursor_Theme },
            {   10, XResourceKey::Xcursor_Size },
            {   11, XResourceKey::Xcursor_ThemeCore },
            {   12, XResourceKey::Gtk_FontName },
            {   13, XResourceKey::Gtk_ThemeName },
            {   14, XResourceKey::Gtk_IconThemeName },
            {   15, XResourceKey::Gtk_CursorThemeName },
            {   16, XResourceKey::Gtk_CursorThemeSize },
            {   17, XResourceKey::Gdk_WindowScalingFactor },
            {   18, XResourceKey::Gdk_UnscaledDPI },
            {   19, XResourceKey::Net_ThemeName },
            {   20, XResourceKey::Net_IconThemeName },
            {   21, XResourceKey::Net_SoundThemeName },
        }),
    };
    return QtMocHelpers::metaObjectData<XResource, qt_meta_tag_ZN9XResourceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject XResource::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractSettings::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9XResourceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9XResourceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9XResourceE_t>.metaTypes,
    nullptr
} };

void XResource::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<XResource *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *XResource::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *XResource::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9XResourceE_t>.strings))
        return static_cast<void*>(this);
    return AbstractSettings::qt_metacast(_clname);
}

int XResource::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractSettings::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
