/****************************************************************************
** Meta object code from reading C++ file 'woutputmanagerv1.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../waylib/src/server/protocols/woutputmanagerv1.h"
#include "wcursor.h"
#include "private/wserver_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'woutputmanagerv1.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Waylib6Server12WOutputStateE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WOutputState::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server12WOutputStateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WOutputState",
        "output",
        "WOutput*",
        "enabled",
        "mode",
        "wlr_output_mode*",
        "x",
        "int32_t",
        "y",
        "custom_mode_size",
        "QSize",
        "custom_mode_refresh",
        "transform",
        "WOutput::Transform",
        "scale",
        "adaptive_sync_enabled"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'output'
        QtMocHelpers::PropertyData<WOutput*>(1, 0x80000000 | 2, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'enabled'
        QtMocHelpers::PropertyData<bool>(3, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'mode'
        QtMocHelpers::PropertyData<wlr_output_mode*>(4, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'x'
        QtMocHelpers::PropertyData<int32_t>(6, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'y'
        QtMocHelpers::PropertyData<int32_t>(8, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'custom_mode_size'
        QtMocHelpers::PropertyData<QSize>(9, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'custom_mode_refresh'
        QtMocHelpers::PropertyData<int32_t>(11, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'transform'
        QtMocHelpers::PropertyData<WOutput::Transform>(12, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag),
        // property 'scale'
        QtMocHelpers::PropertyData<float>(14, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable),
        // property 'adaptive_sync_enabled'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WOutputState, qt_meta_tag_ZN6Waylib6Server12WOutputStateE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN6Waylib6Server12WOutputStateE[] = {
    QMetaObject::SuperData::link<Waylib::Server::WOutput::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject Waylib::Server::WOutputState::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server12WOutputStateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server12WOutputStateE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN6Waylib6Server12WOutputStateE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server12WOutputStateE_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WOutputState::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<WOutputState *>(_o);
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< WOutput* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<WOutput**>(_v) = _t->output; break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->enabled; break;
        case 2: *reinterpret_cast<wlr_output_mode**>(_v) = _t->mode; break;
        case 3: *reinterpret_cast<int32_t*>(_v) = _t->x; break;
        case 4: *reinterpret_cast<int32_t*>(_v) = _t->y; break;
        case 5: *reinterpret_cast<QSize*>(_v) = _t->customModeSize; break;
        case 6: *reinterpret_cast<int32_t*>(_v) = _t->customModeRefresh; break;
        case 7: *reinterpret_cast<WOutput::Transform*>(_v) = _t->transform; break;
        case 8: *reinterpret_cast<float*>(_v) = _t->scale; break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->adaptiveSyncEnabled; break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: QtMocHelpers::setProperty(_t->output, *reinterpret_cast<WOutput**>(_v)); break;
        case 1: QtMocHelpers::setProperty(_t->enabled, *reinterpret_cast<bool*>(_v)); break;
        case 2: QtMocHelpers::setProperty(_t->mode, *reinterpret_cast<wlr_output_mode**>(_v)); break;
        case 3: QtMocHelpers::setProperty(_t->x, *reinterpret_cast<int32_t*>(_v)); break;
        case 4: QtMocHelpers::setProperty(_t->y, *reinterpret_cast<int32_t*>(_v)); break;
        case 5: QtMocHelpers::setProperty(_t->customModeSize, *reinterpret_cast<QSize*>(_v)); break;
        case 6: QtMocHelpers::setProperty(_t->customModeRefresh, *reinterpret_cast<int32_t*>(_v)); break;
        case 7: QtMocHelpers::setProperty(_t->transform, *reinterpret_cast<WOutput::Transform*>(_v)); break;
        case 8: QtMocHelpers::setProperty(_t->scale, *reinterpret_cast<float*>(_v)); break;
        case 9: QtMocHelpers::setProperty(_t->adaptiveSyncEnabled, *reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN6Waylib6Server16WOutputManagerV1E_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WOutputManagerV1::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server16WOutputManagerV1E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WOutputManagerV1",
        "requestTestOrApply",
        "",
        "wlr_output_configuration_v1*",
        "config",
        "onlyTest"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'requestTestOrApply'
        QtMocHelpers::SignalData<void(wlr_output_configuration_v1 *, bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::Bool, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WOutputManagerV1, qt_meta_tag_ZN6Waylib6Server16WOutputManagerV1E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WOutputManagerV1::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server16WOutputManagerV1E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server16WOutputManagerV1E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server16WOutputManagerV1E_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WOutputManagerV1::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WOutputManagerV1 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->requestTestOrApply((*reinterpret_cast<std::add_pointer_t<wlr_output_configuration_v1*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WOutputManagerV1::*)(wlr_output_configuration_v1 * , bool )>(_a, &WOutputManagerV1::requestTestOrApply, 0))
            return;
    }
}

const QMetaObject *Waylib::Server::WOutputManagerV1::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WOutputManagerV1::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server16WOutputManagerV1E_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "WObject"))
        return static_cast< WObject*>(this);
    if (!strcmp(_clname, "WServerInterface"))
        return static_cast< WServerInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int Waylib::Server::WOutputManagerV1::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WOutputManagerV1::requestTestOrApply(wlr_output_configuration_v1 * _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
QT_WARNING_POP
