/****************************************************************************
** Meta object code from reading C++ file 'shortcutcontroller.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/modules/shortcut/shortcutcontroller.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'shortcutcontroller.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18ShortcutControllerE_t {};
} // unnamed namespace

template <> constexpr inline auto ShortcutController::qt_create_metaobjectdata<qt_meta_tag_ZN18ShortcutControllerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ShortcutController",
        "actionTriggered",
        "",
        "ShortcutAction",
        "action",
        "name",
        "isGesture",
        "KeyFlags",
        "keyFlags",
        "actionProgress",
        "progress",
        "actionFinished",
        "isTriggered",
        "Notify",
        "Workspace1",
        "Workspace2",
        "Workspace3",
        "Workspace4",
        "Workspace5",
        "Workspace6",
        "PrevWorkspace",
        "NextWorkspace",
        "ShowDesktop",
        "Maximize",
        "CancelMaximize",
        "MoveWindow",
        "CloseWindow",
        "ShowWindowMenu",
        "OpenMultiTaskView",
        "CloseMultiTaskView",
        "ToggleMultitaskView",
        "ToggleFpsDisplay",
        "Lockscreen",
        "ShutdownMenu",
        "Quit",
        "TaskSwitchNext",
        "TaskSwitchPrev",
        "TaskSwitchSameAppNext",
        "TaskSwitchSameAppPrev",
        "TileLeft",
        "TileRight"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'actionTriggered'
        QtMocHelpers::SignalData<void(enum ShortcutAction, const QString &, bool, KeyFlags)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 5 }, { QMetaType::Bool, 6 }, { 0x80000000 | 7, 8 },
        }}),
        // Signal 'actionTriggered'
        QtMocHelpers::SignalData<void(enum ShortcutAction, const QString &, bool)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 5 }, { QMetaType::Bool, 6 },
        }}),
        // Signal 'actionProgress'
        QtMocHelpers::SignalData<void(enum ShortcutAction, qreal, const QString &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QReal, 10 }, { QMetaType::QString, 5 },
        }}),
        // Signal 'actionFinished'
        QtMocHelpers::SignalData<void(enum ShortcutAction, const QString &, bool)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 5 }, { QMetaType::Bool, 12 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ShortcutAction'
        QtMocHelpers::EnumData<enum ShortcutAction>(3, 3, QMC::EnumIsScoped).add({
            {   13, ShortcutAction::Notify },
            {   14, ShortcutAction::Workspace1 },
            {   15, ShortcutAction::Workspace2 },
            {   16, ShortcutAction::Workspace3 },
            {   17, ShortcutAction::Workspace4 },
            {   18, ShortcutAction::Workspace5 },
            {   19, ShortcutAction::Workspace6 },
            {   20, ShortcutAction::PrevWorkspace },
            {   21, ShortcutAction::NextWorkspace },
            {   22, ShortcutAction::ShowDesktop },
            {   23, ShortcutAction::Maximize },
            {   24, ShortcutAction::CancelMaximize },
            {   25, ShortcutAction::MoveWindow },
            {   26, ShortcutAction::CloseWindow },
            {   27, ShortcutAction::ShowWindowMenu },
            {   28, ShortcutAction::OpenMultiTaskView },
            {   29, ShortcutAction::CloseMultiTaskView },
            {   30, ShortcutAction::ToggleMultitaskView },
            {   31, ShortcutAction::ToggleFpsDisplay },
            {   32, ShortcutAction::Lockscreen },
            {   33, ShortcutAction::ShutdownMenu },
            {   34, ShortcutAction::Quit },
            {   35, ShortcutAction::TaskSwitchNext },
            {   36, ShortcutAction::TaskSwitchPrev },
            {   37, ShortcutAction::TaskSwitchSameAppNext },
            {   38, ShortcutAction::TaskSwitchSameAppPrev },
            {   39, ShortcutAction::TileLeft },
            {   40, ShortcutAction::TileRight },
        }),
    };
    return QtMocHelpers::metaObjectData<ShortcutController, qt_meta_tag_ZN18ShortcutControllerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ShortcutController::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18ShortcutControllerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18ShortcutControllerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18ShortcutControllerE_t>.metaTypes,
    nullptr
} };

void ShortcutController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ShortcutController *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->actionTriggered((*reinterpret_cast<std::add_pointer_t<enum ShortcutAction>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<KeyFlags>>(_a[4]))); break;
        case 1: _t->actionTriggered((*reinterpret_cast<std::add_pointer_t<enum ShortcutAction>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 2: _t->actionProgress((*reinterpret_cast<std::add_pointer_t<enum ShortcutAction>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 3: _t->actionFinished((*reinterpret_cast<std::add_pointer_t<enum ShortcutAction>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ShortcutController::*)(ShortcutAction , const QString & , bool , KeyFlags )>(_a, &ShortcutController::actionTriggered, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShortcutController::*)(ShortcutAction , qreal , const QString & )>(_a, &ShortcutController::actionProgress, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShortcutController::*)(ShortcutAction , const QString & , bool )>(_a, &ShortcutController::actionFinished, 3))
            return;
    }
}

const QMetaObject *ShortcutController::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ShortcutController::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18ShortcutControllerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ShortcutController::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void ShortcutController::actionTriggered(ShortcutAction _t1, const QString & _t2, bool _t3, KeyFlags _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 2
void ShortcutController::actionProgress(ShortcutAction _t1, qreal _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3);
}

// SIGNAL 3
void ShortcutController::actionFinished(ShortcutAction _t1, const QString & _t2, bool _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
