/****************************************************************************
** Meta object code from reading C++ file 'mpvvideocontroller.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../wallpaper-factory/mpvvideocontroller.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mpvvideocontroller.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18MpvVideoControllerE_t {};
} // unnamed namespace

template <> constexpr inline auto MpvVideoController::qt_create_metaobjectdata<qt_meta_tag_ZN18MpvVideoControllerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MpvVideoController",
        "propertyChanged",
        "",
        "property",
        "QVariant",
        "value",
        "asyncReply",
        "data",
        "mpv_event",
        "event",
        "fileStarted",
        "fileLoaded",
        "endFile",
        "reason",
        "videoReconfig",
        "init",
        "observeProperty",
        "QByteArrayView",
        "mpv_format",
        "format",
        "uint64_t",
        "id",
        "unobserveProperty",
        "setProperty",
        "setPropertyAsync",
        "getProperty",
        "getPropertyAsync",
        "command",
        "params",
        "commandAsync"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'propertyChanged'
        QtMocHelpers::SignalData<void(const QByteArray &, const QVariant &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 3 }, { 0x80000000 | 4, 5 },
        }}),
        // Signal 'asyncReply'
        QtMocHelpers::SignalData<void(const QVariant &, mpv_event)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 7 }, { 0x80000000 | 8, 9 },
        }}),
        // Signal 'fileStarted'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fileLoaded'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'endFile'
        QtMocHelpers::SignalData<void(const QByteArray &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 13 },
        }}),
        // Signal 'videoReconfig'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'init'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'observeProperty'
        QtMocHelpers::SlotData<void(const QByteArrayView &, mpv_format, uint64_t)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 3 }, { 0x80000000 | 18, 19 }, { 0x80000000 | 20, 21 },
        }}),
        // Slot 'observeProperty'
        QtMocHelpers::SlotData<void(const QByteArrayView &, mpv_format)>(16, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 17, 3 }, { 0x80000000 | 18, 19 },
        }}),
        // Slot 'unobserveProperty'
        QtMocHelpers::SlotData<int(uint64_t)>(22, 2, QMC::AccessPublic, QMetaType::Int, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'setProperty'
        QtMocHelpers::SlotData<int(const QByteArrayView &, const QVariant &)>(23, 2, QMC::AccessPublic, QMetaType::Int, {{
            { 0x80000000 | 17, 3 }, { 0x80000000 | 4, 5 },
        }}),
        // Slot 'setPropertyAsync'
        QtMocHelpers::SlotData<int(const QByteArrayView &, const QVariant &, int)>(24, 2, QMC::AccessPublic, QMetaType::Int, {{
            { 0x80000000 | 17, 3 }, { 0x80000000 | 4, 5 }, { QMetaType::Int, 21 },
        }}),
        // Slot 'setPropertyAsync'
        QtMocHelpers::SlotData<int(const QByteArrayView &, const QVariant &)>(24, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { 0x80000000 | 17, 3 }, { 0x80000000 | 4, 5 },
        }}),
        // Slot 'getProperty'
        QtMocHelpers::SlotData<QVariant(const QByteArrayView &)>(25, 2, QMC::AccessPublic, 0x80000000 | 4, {{
            { 0x80000000 | 17, 3 },
        }}),
        // Slot 'getPropertyAsync'
        QtMocHelpers::SlotData<int(const QByteArrayView &, int)>(26, 2, QMC::AccessPublic, QMetaType::Int, {{
            { 0x80000000 | 17, 3 }, { QMetaType::Int, 21 },
        }}),
        // Slot 'getPropertyAsync'
        QtMocHelpers::SlotData<int(const QByteArrayView &)>(26, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { 0x80000000 | 17, 3 },
        }}),
        // Slot 'command'
        QtMocHelpers::SlotData<QVariant(const QVariant &)>(27, 2, QMC::AccessPublic, 0x80000000 | 4, {{
            { 0x80000000 | 4, 28 },
        }}),
        // Slot 'commandAsync'
        QtMocHelpers::SlotData<int(const QVariant &, int)>(29, 2, QMC::AccessPublic, QMetaType::Int, {{
            { 0x80000000 | 4, 28 }, { QMetaType::Int, 21 },
        }}),
        // Slot 'commandAsync'
        QtMocHelpers::SlotData<int(const QVariant &)>(29, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { 0x80000000 | 4, 28 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MpvVideoController, qt_meta_tag_ZN18MpvVideoControllerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MpvVideoController::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18MpvVideoControllerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18MpvVideoControllerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18MpvVideoControllerE_t>.metaTypes,
    nullptr
} };

void MpvVideoController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MpvVideoController *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->propertyChanged((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 1: _t->asyncReply((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<mpv_event>>(_a[2]))); break;
        case 2: _t->fileStarted(); break;
        case 3: _t->fileLoaded(); break;
        case 4: _t->endFile((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 5: _t->videoReconfig(); break;
        case 6: _t->init(); break;
        case 7: _t->observeProperty((*reinterpret_cast<std::add_pointer_t<QByteArrayView>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<mpv_format>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<uint64_t>>(_a[3]))); break;
        case 8: _t->observeProperty((*reinterpret_cast<std::add_pointer_t<QByteArrayView>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<mpv_format>>(_a[2]))); break;
        case 9: { int _r = _t->unobserveProperty((*reinterpret_cast<std::add_pointer_t<uint64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 10: { int _r = _t->setProperty((*reinterpret_cast<std::add_pointer_t<QByteArrayView>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 11: { int _r = _t->setPropertyAsync((*reinterpret_cast<std::add_pointer_t<QByteArrayView>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 12: { int _r = _t->setPropertyAsync((*reinterpret_cast<std::add_pointer_t<QByteArrayView>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 13: { QVariant _r = _t->getProperty((*reinterpret_cast<std::add_pointer_t<QByteArrayView>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariant*>(_a[0]) = std::move(_r); }  break;
        case 14: { int _r = _t->getPropertyAsync((*reinterpret_cast<std::add_pointer_t<QByteArrayView>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 15: { int _r = _t->getPropertyAsync((*reinterpret_cast<std::add_pointer_t<QByteArrayView>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 16: { QVariant _r = _t->command((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariant*>(_a[0]) = std::move(_r); }  break;
        case 17: { int _r = _t->commandAsync((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 18: { int _r = _t->commandAsync((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MpvVideoController::*)(const QByteArray & , const QVariant & )>(_a, &MpvVideoController::propertyChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MpvVideoController::*)(const QVariant & , mpv_event )>(_a, &MpvVideoController::asyncReply, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MpvVideoController::*)()>(_a, &MpvVideoController::fileStarted, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (MpvVideoController::*)()>(_a, &MpvVideoController::fileLoaded, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (MpvVideoController::*)(const QByteArray & )>(_a, &MpvVideoController::endFile, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (MpvVideoController::*)()>(_a, &MpvVideoController::videoReconfig, 5))
            return;
    }
}

const QMetaObject *MpvVideoController::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MpvVideoController::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18MpvVideoControllerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int MpvVideoController::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 19;
    }
    return _id;
}

// SIGNAL 0
void MpvVideoController::propertyChanged(const QByteArray & _t1, const QVariant & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void MpvVideoController::asyncReply(const QVariant & _t1, mpv_event _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void MpvVideoController::fileStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void MpvVideoController::fileLoaded()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void MpvVideoController::endFile(const QByteArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void MpvVideoController::videoReconfig()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
