/****************************************************************************
** Meta object code from reading C++ file 'wtextinput_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../waylib/src/server/protocols/private/wtextinput_p.h"
#include "wsurface.h"
#include "winputmethodv2_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wtextinput_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Waylib6Server10WTextInputE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WTextInput::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server10WTextInputE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WTextInput",
        "entityAboutToDestroy",
        "",
        "enabled",
        "disabled",
        "committed",
        "requestFocus",
        "requestLeave",
        "sendEnter",
        "WSurface*",
        "surface",
        "sendLeave",
        "sendDone",
        "handleIMCommitted",
        "WInputMethodV2*",
        "im"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'entityAboutToDestroy'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'enabled'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'disabled'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'committed'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestFocus'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestLeave'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'sendEnter'
        QtMocHelpers::SlotData<void(WSurface *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'sendLeave'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'sendDone'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleIMCommitted'
        QtMocHelpers::SlotData<void(WInputMethodV2 *)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WTextInput, qt_meta_tag_ZN6Waylib6Server10WTextInputE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WTextInput::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server10WTextInputE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server10WTextInputE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server10WTextInputE_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WTextInput::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WTextInput *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->entityAboutToDestroy(); break;
        case 1: _t->enabled(); break;
        case 2: _t->disabled(); break;
        case 3: _t->committed(); break;
        case 4: _t->requestFocus(); break;
        case 5: _t->requestLeave(); break;
        case 6: _t->sendEnter((*reinterpret_cast<std::add_pointer_t<WSurface*>>(_a[1]))); break;
        case 7: _t->sendLeave(); break;
        case 8: _t->sendDone(); break;
        case 9: _t->handleIMCommitted((*reinterpret_cast<std::add_pointer_t<WInputMethodV2*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WTextInput::*)()>(_a, &WTextInput::entityAboutToDestroy, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WTextInput::*)()>(_a, &WTextInput::enabled, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WTextInput::*)()>(_a, &WTextInput::disabled, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WTextInput::*)()>(_a, &WTextInput::committed, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (WTextInput::*)()>(_a, &WTextInput::requestFocus, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (WTextInput::*)()>(_a, &WTextInput::requestLeave, 5))
            return;
    }
}

const QMetaObject *Waylib::Server::WTextInput::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WTextInput::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server10WTextInputE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "WWaylandResource"))
        return static_cast< WWaylandResource*>(this);
    return QObject::qt_metacast(_clname);
}

int Waylib::Server::WTextInput::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WTextInput::entityAboutToDestroy()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Waylib::Server::WTextInput::enabled()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Waylib::Server::WTextInput::disabled()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Waylib::Server::WTextInput::committed()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Waylib::Server::WTextInput::requestFocus()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Waylib::Server::WTextInput::requestLeave()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
namespace {
struct qt_meta_tag_ZN6Waylib6Server3IMEE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::IME::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server3IMEE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::IME",
        "ChangeCause",
        "CC_InputMethod",
        "CC_Other",
        "ContentHint",
        "CH_None",
        "CH_Completion",
        "CH_Spellcheck",
        "CH_AutoCapitalization",
        "CH_Lowercase",
        "CH_Uppercase",
        "CH_Titlecase",
        "CH_HiddenText",
        "CH_SensitiveData",
        "CH_Latin",
        "CH_Multiline",
        "ContentPurpose",
        "CP_Normal",
        "CP_Alpha",
        "CP_Digits",
        "CP_Number",
        "CP_Phone",
        "CP_Url",
        "CP_Email",
        "CP_Name",
        "CP_Password",
        "CP_Pin",
        "CP_Date",
        "CP_Time",
        "CP_Datetime",
        "CP_Terminal",
        "Feature",
        "F_SurroundingText",
        "F_ContentType",
        "F_CursorRect",
        "KeyboardModifier",
        "KM_Shift",
        "KM_Caps",
        "KM_Ctrl",
        "KM_Alt",
        "KM_Mod2",
        "KM_Mod3",
        "KM_Logo",
        "KM_Mod5"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ChangeCause'
        QtMocHelpers::EnumData<ChangeCause>(1, 1, QMC::EnumFlags{}).add({
            {    2, ChangeCause::CC_InputMethod },
            {    3, ChangeCause::CC_Other },
        }),
        // enum 'ContentHint'
        QtMocHelpers::EnumData<ContentHint>(4, 4, QMC::EnumFlags{}).add({
            {    5, ContentHint::CH_None },
            {    6, ContentHint::CH_Completion },
            {    7, ContentHint::CH_Spellcheck },
            {    8, ContentHint::CH_AutoCapitalization },
            {    9, ContentHint::CH_Lowercase },
            {   10, ContentHint::CH_Uppercase },
            {   11, ContentHint::CH_Titlecase },
            {   12, ContentHint::CH_HiddenText },
            {   13, ContentHint::CH_SensitiveData },
            {   14, ContentHint::CH_Latin },
            {   15, ContentHint::CH_Multiline },
        }),
        // enum 'ContentPurpose'
        QtMocHelpers::EnumData<ContentPurpose>(16, 16, QMC::EnumFlags{}).add({
            {   17, ContentPurpose::CP_Normal },
            {   18, ContentPurpose::CP_Alpha },
            {   19, ContentPurpose::CP_Digits },
            {   20, ContentPurpose::CP_Number },
            {   21, ContentPurpose::CP_Phone },
            {   22, ContentPurpose::CP_Url },
            {   23, ContentPurpose::CP_Email },
            {   24, ContentPurpose::CP_Name },
            {   25, ContentPurpose::CP_Password },
            {   26, ContentPurpose::CP_Pin },
            {   27, ContentPurpose::CP_Date },
            {   28, ContentPurpose::CP_Time },
            {   29, ContentPurpose::CP_Datetime },
            {   30, ContentPurpose::CP_Terminal },
        }),
        // enum 'Feature'
        QtMocHelpers::EnumData<Feature>(31, 31, QMC::EnumFlags{}).add({
            {   32, Feature::F_SurroundingText },
            {   33, Feature::F_ContentType },
            {   34, Feature::F_CursorRect },
        }),
        // enum 'KeyboardModifier'
        QtMocHelpers::EnumData<KeyboardModifier>(35, 35, QMC::EnumFlags{}).add({
            {   36, KeyboardModifier::KM_Shift },
            {   37, KeyboardModifier::KM_Caps },
            {   38, KeyboardModifier::KM_Ctrl },
            {   39, KeyboardModifier::KM_Alt },
            {   40, KeyboardModifier::KM_Mod2 },
            {   41, KeyboardModifier::KM_Mod3 },
            {   42, KeyboardModifier::KM_Logo },
            {   43, KeyboardModifier::KM_Mod5 },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN6Waylib6Server3IMEE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN6Waylib6Server3IMEE =
    Waylib::Server::IME::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server3IMEE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN6Waylib6Server3IMEE =
    qt_staticMetaObjectContent_ZN6Waylib6Server3IMEE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN6Waylib6Server3IMEE =
    qt_staticMetaObjectContent_ZN6Waylib6Server3IMEE.relocatingData;

Q_CONSTINIT const QMetaObject Waylib::Server::IME::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN6Waylib6Server3IMEE.stringdata,
    qt_staticMetaObjectStaticContent_ZN6Waylib6Server3IMEE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN6Waylib6Server3IMEE.metaTypes,
    nullptr
} };

QT_WARNING_POP
