/****************************************************************************
** Meta object code from reading C++ file 'wsurface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../waylib/src/server/kernel/wsurface.h"
#include "woutput.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wsurface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Waylib6Server8WSurfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WSurface::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server8WSurfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WSurface",
        "QML.Element",
        "WaylandSurface",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Only create in C++",
        "mappedChanged",
        "",
        "bufferOffsetChanged",
        "hasSubsurfaceChanged",
        "newSubsurface",
        "WSurface*",
        "subsurface",
        "beforeDestroy",
        "preferredBufferScaleChanged",
        "outputEntered",
        "WOutput*",
        "output",
        "outputLeave",
        "commit",
        "committedState",
        "enterOutput",
        "leaveOutput",
        "outputs",
        "framePacingOutput",
        "inputRegionContains",
        "QPointF",
        "localPos",
        "map",
        "unmap",
        "mapped",
        "isSubsurface",
        "hasSubsurface",
        "needsFrame",
        "subsurfaces",
        "QList<WSurface*>",
        "preferredBufferScale",
        "uint32_t"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'mappedChanged'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'bufferOffsetChanged'
        QtMocHelpers::SignalData<void()>(9, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hasSubsurfaceChanged'
        QtMocHelpers::SignalData<void()>(10, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'newSubsurface'
        QtMocHelpers::SignalData<void(WSurface *)>(11, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'beforeDestroy'
        QtMocHelpers::SignalData<void()>(14, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'preferredBufferScaleChanged'
        QtMocHelpers::SignalData<void()>(15, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'outputEntered'
        QtMocHelpers::SignalData<void(WOutput *)>(16, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Signal 'outputLeave'
        QtMocHelpers::SignalData<void(WOutput *)>(19, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Signal 'commit'
        QtMocHelpers::SignalData<void(quint32)>(20, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 21 },
        }}),
        // Slot 'enterOutput'
        QtMocHelpers::SlotData<void(WOutput *)>(22, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'leaveOutput'
        QtMocHelpers::SlotData<void(WOutput *)>(23, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'outputs'
        QtMocHelpers::SlotData<void() const>(24, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'framePacingOutput'
        QtMocHelpers::SlotData<WOutput *() const>(25, 8, QMC::AccessPublic, 0x80000000 | 17),
        // Slot 'inputRegionContains'
        QtMocHelpers::SlotData<bool(const QPointF &) const>(26, 8, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Slot 'map'
        QtMocHelpers::SlotData<void()>(29, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'unmap'
        QtMocHelpers::SlotData<void()>(30, 8, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'mapped'
        QtMocHelpers::PropertyData<bool>(31, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'isSubsurface'
        QtMocHelpers::PropertyData<bool>(32, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'hasSubsurface'
        QtMocHelpers::PropertyData<bool>(33, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
        // property 'needsFrame'
        QtMocHelpers::PropertyData<bool>(34, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'subsurfaces'
        QtMocHelpers::PropertyData<QList<WSurface*>>(35, 0x80000000 | 36, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 3),
        // property 'preferredBufferScale'
        QtMocHelpers::PropertyData<uint32_t>(37, 0x80000000 | 38, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 5),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<WSurface, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WSurface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server8WSurfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server8WSurfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server8WSurfaceE_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WSurface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WSurface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->mappedChanged(); break;
        case 1: _t->bufferOffsetChanged(); break;
        case 2: _t->hasSubsurfaceChanged(); break;
        case 3: _t->newSubsurface((*reinterpret_cast<std::add_pointer_t<WSurface*>>(_a[1]))); break;
        case 4: _t->beforeDestroy(); break;
        case 5: _t->preferredBufferScaleChanged(); break;
        case 6: _t->outputEntered((*reinterpret_cast<std::add_pointer_t<WOutput*>>(_a[1]))); break;
        case 7: _t->outputLeave((*reinterpret_cast<std::add_pointer_t<WOutput*>>(_a[1]))); break;
        case 8: _t->commit((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1]))); break;
        case 9: _t->enterOutput((*reinterpret_cast<std::add_pointer_t<WOutput*>>(_a[1]))); break;
        case 10: _t->leaveOutput((*reinterpret_cast<std::add_pointer_t<WOutput*>>(_a[1]))); break;
        case 11: _t->outputs(); break;
        case 12: { WOutput* _r = _t->framePacingOutput();
            if (_a[0]) *reinterpret_cast<WOutput**>(_a[0]) = std::move(_r); }  break;
        case 13: { bool _r = _t->inputRegionContains((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 14: _t->map(); break;
        case 15: _t->unmap(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< WSurface* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WSurface::*)()>(_a, &WSurface::mappedChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSurface::*)()>(_a, &WSurface::bufferOffsetChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSurface::*)()>(_a, &WSurface::hasSubsurfaceChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSurface::*)(WSurface * )>(_a, &WSurface::newSubsurface, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSurface::*)()>(_a, &WSurface::beforeDestroy, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSurface::*)()>(_a, &WSurface::preferredBufferScaleChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSurface::*)(WOutput * )>(_a, &WSurface::outputEntered, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSurface::*)(WOutput * )>(_a, &WSurface::outputLeave, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSurface::*)(quint32 )>(_a, &WSurface::commit, 8))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<WSurface*> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->mapped(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isSubsurface(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->hasSubsurface(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->needsFrame(); break;
        case 4: *reinterpret_cast<QList<WSurface*>*>(_v) = _t->subsurfaces(); break;
        case 5: *reinterpret_cast<uint32_t*>(_v) = _t->preferredBufferScale(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 5: _t->setPreferredBufferScale(*reinterpret_cast<uint32_t*>(_v)); break;
        default: break;
        }
    }
    if (_c == QMetaObject::ResetProperty) {
        switch (_id) {
        case 5: _t->resetPreferredBufferScale(); break;
        default: break;
        }
    }
}

const QMetaObject *Waylib::Server::WSurface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WSurface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server8WSurfaceE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "WWaylandResource"))
        return static_cast< WWaylandResource*>(this);
    return QObject::qt_metacast(_clname);
}

int Waylib::Server::WSurface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WSurface::mappedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Waylib::Server::WSurface::bufferOffsetChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Waylib::Server::WSurface::hasSubsurfaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Waylib::Server::WSurface::newSubsurface(WSurface * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Waylib::Server::WSurface::beforeDestroy()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Waylib::Server::WSurface::preferredBufferScaleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Waylib::Server::WSurface::outputEntered(WOutput * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void Waylib::Server::WSurface::outputLeave(WOutput * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void Waylib::Server::WSurface::commit(quint32 _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}
QT_WARNING_POP
