/****************************************************************************
** Meta object code from reading C++ file 'wallpapermanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/wallpaper/wallpapermanager.h"
#include "woutput.h"
#include <woutput.h>
#include <output/output.h>
#include "core/rootsurfacecontainer.h"
#include "wcursor.h"
#include <woutputitem.h>
#include "private/wserver_p.h"
#include <wsurface.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wallpapermanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16WallpaperManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto WallpaperManager::qt_create_metaobjectdata<qt_meta_tag_ZN16WallpaperManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "WallpaperManager",
        "updateWallpaper",
        "",
        "updateWallpaperConfig",
        "onWallpaperAdded",
        "TreelandWallpaperInterfaceV1*",
        "interface",
        "onImageChanged",
        "workspaceIndex",
        "fileSource",
        "TreelandWallpaperInterfaceV1::WallpaperRoles",
        "roles",
        "onVideoChanged",
        "onWallpaperNotifierBound",
        "wl_resource*",
        "resource",
        "handleWallpaperSurfaceAdded",
        "TreelandWallpaperSurfaceInterfaceV1*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'updateWallpaper'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateWallpaperConfig'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onWallpaperAdded'
        QtMocHelpers::SlotData<void(TreelandWallpaperInterfaceV1 *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'onImageChanged'
        QtMocHelpers::SlotData<void(int, const QString &, TreelandWallpaperInterfaceV1::WallpaperRoles)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 }, { QMetaType::QString, 9 }, { 0x80000000 | 10, 11 },
        }}),
        // Slot 'onVideoChanged'
        QtMocHelpers::SlotData<void(int, const QString &, TreelandWallpaperInterfaceV1::WallpaperRoles)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 8 }, { QMetaType::QString, 9 }, { 0x80000000 | 10, 11 },
        }}),
        // Slot 'onWallpaperNotifierBound'
        QtMocHelpers::SlotData<void(wl_resource *)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'handleWallpaperSurfaceAdded'
        QtMocHelpers::SlotData<void(TreelandWallpaperSurfaceInterfaceV1 *)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WallpaperManager, qt_meta_tag_ZN16WallpaperManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject WallpaperManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16WallpaperManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16WallpaperManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16WallpaperManagerE_t>.metaTypes,
    nullptr
} };

void WallpaperManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WallpaperManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->updateWallpaper(); break;
        case 1: _t->updateWallpaperConfig(); break;
        case 2: _t->onWallpaperAdded((*reinterpret_cast<std::add_pointer_t<TreelandWallpaperInterfaceV1*>>(_a[1]))); break;
        case 3: _t->onImageChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<TreelandWallpaperInterfaceV1::WallpaperRoles>>(_a[3]))); break;
        case 4: _t->onVideoChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<TreelandWallpaperInterfaceV1::WallpaperRoles>>(_a[3]))); break;
        case 5: _t->onWallpaperNotifierBound((*reinterpret_cast<std::add_pointer_t<wl_resource*>>(_a[1]))); break;
        case 6: _t->handleWallpaperSurfaceAdded((*reinterpret_cast<std::add_pointer_t<TreelandWallpaperSurfaceInterfaceV1*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TreelandWallpaperInterfaceV1* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WallpaperManager::*)()>(_a, &WallpaperManager::updateWallpaper, 0))
            return;
    }
}

const QMetaObject *WallpaperManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *WallpaperManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16WallpaperManagerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int WallpaperManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void WallpaperManager::updateWallpaper()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
