/****************************************************************************
** Meta object code from reading C++ file 'rootsurfacecontainer.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/core/rootsurfacecontainer.h"
#include "woutput.h"
#include <woutput.h>
#include <output/output.h>
#include "core/rootsurfacecontainer.h"
#include "private/wserver_p.h"
#include <wsurface.h>
#include <wcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'rootsurfacecontainer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15OutputListModelE_t {};
} // unnamed namespace

template <> constexpr inline auto OutputListModel::qt_create_metaobjectdata<qt_meta_tag_ZN15OutputListModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OutputListModel",
        "QML.Element",
        "anonymous"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<OutputListModel, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject OutputListModel::staticMetaObject = { {
    QMetaObject::SuperData::link<ObjectListModel<Output>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15OutputListModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15OutputListModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15OutputListModelE_t>.metaTypes,
    nullptr
} };

void OutputListModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OutputListModel *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *OutputListModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OutputListModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15OutputListModelE_t>.strings))
        return static_cast<void*>(this);
    return ObjectListModel<Output>::qt_metacast(_clname);
}

int OutputListModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ObjectListModel<Output>::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN20RootSurfaceContainerE_t {};
} // unnamed namespace

template <> constexpr inline auto RootSurfaceContainer::qt_create_metaobjectdata<qt_meta_tag_ZN20RootSurfaceContainerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "RootSurfaceContainer",
        "primaryOutputChanged",
        "",
        "moveResizeFinised",
        "startMove",
        "SurfaceWrapper*",
        "surface",
        "startResize",
        "Qt::Edges",
        "edges",
        "cancelMoveResize",
        "outputLayout",
        "Waylib::Server::WOutputLayout*",
        "cursor",
        "Waylib::Server::WCursor*",
        "primaryOutput",
        "Output*",
        "outputModel",
        "OutputListModel*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'primaryOutputChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'moveResizeFinised'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'startMove'
        QtMocHelpers::SlotData<void(SurfaceWrapper *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'startResize'
        QtMocHelpers::SlotData<void(SurfaceWrapper *, Qt::Edges)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 8, 9 },
        }}),
        // Slot 'cancelMoveResize'
        QtMocHelpers::SlotData<void(SurfaceWrapper *)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'outputLayout'
        QtMocHelpers::PropertyData<Waylib::Server::WOutputLayout*>(11, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant | QMC::Final),
        // property 'cursor'
        QtMocHelpers::PropertyData<Waylib::Server::WCursor*>(13, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant | QMC::Final),
        // property 'primaryOutput'
        QtMocHelpers::PropertyData<Output*>(15, 0x80000000 | 16, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 0),
        // property 'outputModel'
        QtMocHelpers::PropertyData<OutputListModel*>(17, 0x80000000 | 18, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant | QMC::Final),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<RootSurfaceContainer, qt_meta_tag_ZN20RootSurfaceContainerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject RootSurfaceContainer::staticMetaObject = { {
    QMetaObject::SuperData::link<SurfaceContainer::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20RootSurfaceContainerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20RootSurfaceContainerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20RootSurfaceContainerE_t>.metaTypes,
    nullptr
} };

void RootSurfaceContainer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RootSurfaceContainer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->primaryOutputChanged(); break;
        case 1: _t->moveResizeFinised(); break;
        case 2: _t->startMove((*reinterpret_cast<std::add_pointer_t<SurfaceWrapper*>>(_a[1]))); break;
        case 3: _t->startResize((*reinterpret_cast<std::add_pointer_t<SurfaceWrapper*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::Edges>>(_a[2]))); break;
        case 4: _t->cancelMoveResize((*reinterpret_cast<std::add_pointer_t<SurfaceWrapper*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< SurfaceWrapper* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< SurfaceWrapper* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< SurfaceWrapper* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (RootSurfaceContainer::*)()>(_a, &RootSurfaceContainer::primaryOutputChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (RootSurfaceContainer::*)()>(_a, &RootSurfaceContainer::moveResizeFinised, 1))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< OutputListModel* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<Waylib::Server::WOutputLayout**>(_v) = _t->outputLayout(); break;
        case 1: *reinterpret_cast<Waylib::Server::WCursor**>(_v) = _t->cursor(); break;
        case 2: *reinterpret_cast<Output**>(_v) = _t->primaryOutput(); break;
        case 3: *reinterpret_cast<OutputListModel**>(_v) = _t->outputModel(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 2: _t->setPrimaryOutput(*reinterpret_cast<Output**>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *RootSurfaceContainer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *RootSurfaceContainer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20RootSurfaceContainerE_t>.strings))
        return static_cast<void*>(this);
    return SurfaceContainer::qt_metacast(_clname);
}

int RootSurfaceContainer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SurfaceContainer::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void RootSurfaceContainer::primaryOutputChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void RootSurfaceContainer::moveResizeFinised()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
