/****************************************************************************
** Meta object code from reading C++ file 'wtoplevelsurface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../waylib/src/server/kernel/wtoplevelsurface.h"
#include "woutput.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wtoplevelsurface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Waylib6Server16WToplevelSurfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WToplevelSurface::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server16WToplevelSurfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WToplevelSurface",
        "QML.Element",
        "ToplevelSurface",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Only create in C++",
        "activateChanged",
        "",
        "maximizeChanged",
        "minimizeChanged",
        "surfaceChanged",
        "parentSurfaceChanged",
        "fullscreenChanged",
        "titleChanged",
        "beforeDestroy",
        "appIdChanged",
        "minimumSizeChanged",
        "QSize",
        "size",
        "maximumSizeChanged",
        "requestMove",
        "WSeat*",
        "seat",
        "serial",
        "requestResize",
        "Qt::Edges",
        "edge",
        "requestMaximize",
        "requestCancelMaximize",
        "requestMinimize",
        "requestCancelMinimize",
        "requestFullscreen",
        "requestCancelFullscreen",
        "requestShowWindowMenu",
        "QPoint",
        "pos",
        "setMaximize",
        "on",
        "setMinimize",
        "setActivate",
        "setResizeing",
        "setFullScreen",
        "checkNewSize",
        "QSize*",
        "clipedSize",
        "resize",
        "close",
        "isActivated",
        "isMaximized",
        "isMinimized",
        "isFullScreen",
        "surface",
        "WSurface*",
        "parentSurface",
        "title",
        "appId"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'activateChanged'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'maximizeChanged'
        QtMocHelpers::SignalData<void()>(9, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'minimizeChanged'
        QtMocHelpers::SignalData<void()>(10, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'surfaceChanged'
        QtMocHelpers::SignalData<void()>(11, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'parentSurfaceChanged'
        QtMocHelpers::SignalData<void()>(12, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fullscreenChanged'
        QtMocHelpers::SignalData<void()>(13, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'titleChanged'
        QtMocHelpers::SignalData<void()>(14, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'beforeDestroy'
        QtMocHelpers::SignalData<void()>(15, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'appIdChanged'
        QtMocHelpers::SignalData<void()>(16, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'minimumSizeChanged'
        QtMocHelpers::SignalData<void(const QSize &)>(17, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Signal 'maximumSizeChanged'
        QtMocHelpers::SignalData<void(const QSize &)>(20, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Signal 'requestMove'
        QtMocHelpers::SignalData<void(WSeat *, quint32)>(21, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 }, { QMetaType::UInt, 24 },
        }}),
        // Signal 'requestResize'
        QtMocHelpers::SignalData<void(WSeat *, Qt::Edges, quint32)>(25, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 }, { 0x80000000 | 26, 27 }, { QMetaType::UInt, 24 },
        }}),
        // Signal 'requestMaximize'
        QtMocHelpers::SignalData<void()>(28, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestCancelMaximize'
        QtMocHelpers::SignalData<void()>(29, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestMinimize'
        QtMocHelpers::SignalData<void()>(30, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestCancelMinimize'
        QtMocHelpers::SignalData<void()>(31, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestFullscreen'
        QtMocHelpers::SignalData<void()>(32, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestCancelFullscreen'
        QtMocHelpers::SignalData<void()>(33, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestShowWindowMenu'
        QtMocHelpers::SignalData<void(WSeat *, QPoint, quint32)>(34, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 }, { 0x80000000 | 35, 36 }, { QMetaType::UInt, 24 },
        }}),
        // Slot 'setMaximize'
        QtMocHelpers::SlotData<void(bool)>(37, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'setMinimize'
        QtMocHelpers::SlotData<void(bool)>(39, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'setActivate'
        QtMocHelpers::SlotData<void(bool)>(40, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'setResizeing'
        QtMocHelpers::SlotData<void(bool)>(41, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'setFullScreen'
        QtMocHelpers::SlotData<void(bool)>(42, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Slot 'checkNewSize'
        QtMocHelpers::SlotData<bool(const QSize &, QSize *)>(43, 8, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 18, 19 }, { 0x80000000 | 44, 45 },
        }}),
        // Slot 'checkNewSize'
        QtMocHelpers::SlotData<bool(const QSize &)>(43, 8, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'resize'
        QtMocHelpers::SlotData<void(const QSize &)>(46, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'close'
        QtMocHelpers::SlotData<void()>(47, 8, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'isActivated'
        QtMocHelpers::PropertyData<bool>(48, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'isMaximized'
        QtMocHelpers::PropertyData<bool>(49, QMetaType::Bool, QMC::DefaultPropertyFlags, 1),
        // property 'isMinimized'
        QtMocHelpers::PropertyData<bool>(50, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
        // property 'isFullScreen'
        QtMocHelpers::PropertyData<bool>(51, QMetaType::Bool, QMC::DefaultPropertyFlags, 5),
        // property 'surface'
        QtMocHelpers::PropertyData<WSurface*>(52, 0x80000000 | 53, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 3),
        // property 'parentSurface'
        QtMocHelpers::PropertyData<WSurface*>(54, 0x80000000 | 53, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 4),
        // property 'title'
        QtMocHelpers::PropertyData<QString>(55, QMetaType::QString, QMC::DefaultPropertyFlags, 6),
        // property 'appId'
        QtMocHelpers::PropertyData<QString>(56, QMetaType::QString, QMC::DefaultPropertyFlags, 8),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<WToplevelSurface, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WToplevelSurface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server16WToplevelSurfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server16WToplevelSurfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server16WToplevelSurfaceE_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WToplevelSurface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WToplevelSurface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->activateChanged(); break;
        case 1: _t->maximizeChanged(); break;
        case 2: _t->minimizeChanged(); break;
        case 3: _t->surfaceChanged(); break;
        case 4: _t->parentSurfaceChanged(); break;
        case 5: _t->fullscreenChanged(); break;
        case 6: _t->titleChanged(); break;
        case 7: _t->beforeDestroy(); break;
        case 8: _t->appIdChanged(); break;
        case 9: _t->minimumSizeChanged((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1]))); break;
        case 10: _t->maximumSizeChanged((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1]))); break;
        case 11: _t->requestMove((*reinterpret_cast<std::add_pointer_t<WSeat*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<quint32>>(_a[2]))); break;
        case 12: _t->requestResize((*reinterpret_cast<std::add_pointer_t<WSeat*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::Edges>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<quint32>>(_a[3]))); break;
        case 13: _t->requestMaximize(); break;
        case 14: _t->requestCancelMaximize(); break;
        case 15: _t->requestMinimize(); break;
        case 16: _t->requestCancelMinimize(); break;
        case 17: _t->requestFullscreen(); break;
        case 18: _t->requestCancelFullscreen(); break;
        case 19: _t->requestShowWindowMenu((*reinterpret_cast<std::add_pointer_t<WSeat*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<quint32>>(_a[3]))); break;
        case 20: _t->setMaximize((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 21: _t->setMinimize((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 22: _t->setActivate((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 23: _t->setResizeing((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 24: _t->setFullScreen((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 25: { bool _r = _t->checkNewSize((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSize*>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 26: { bool _r = _t->checkNewSize((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 27: _t->resize((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1]))); break;
        case 28: _t->close(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WToplevelSurface::*)()>(_a, &WToplevelSurface::activateChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WToplevelSurface::*)()>(_a, &WToplevelSurface::maximizeChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WToplevelSurface::*)()>(_a, &WToplevelSurface::minimizeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WToplevelSurface::*)()>(_a, &WToplevelSurface::surfaceChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (WToplevelSurface::*)()>(_a, &WToplevelSurface::parentSurfaceChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (WToplevelSurface::*)()>(_a, &WToplevelSurface::fullscreenChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (WToplevelSurface::*)()>(_a, &WToplevelSurface::titleChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (WToplevelSurface::*)()>(_a, &WToplevelSurface::beforeDestroy, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (WToplevelSurface::*)()>(_a, &WToplevelSurface::appIdChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (WToplevelSurface::*)(const QSize & )>(_a, &WToplevelSurface::minimumSizeChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (WToplevelSurface::*)(const QSize & )>(_a, &WToplevelSurface::maximumSizeChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (WToplevelSurface::*)(WSeat * , quint32 )>(_a, &WToplevelSurface::requestMove, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (WToplevelSurface::*)(WSeat * , Qt::Edges , quint32 )>(_a, &WToplevelSurface::requestResize, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (WToplevelSurface::*)()>(_a, &WToplevelSurface::requestMaximize, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (WToplevelSurface::*)()>(_a, &WToplevelSurface::requestCancelMaximize, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (WToplevelSurface::*)()>(_a, &WToplevelSurface::requestMinimize, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (WToplevelSurface::*)()>(_a, &WToplevelSurface::requestCancelMinimize, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (WToplevelSurface::*)()>(_a, &WToplevelSurface::requestFullscreen, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (WToplevelSurface::*)()>(_a, &WToplevelSurface::requestCancelFullscreen, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (WToplevelSurface::*)(WSeat * , QPoint , quint32 )>(_a, &WToplevelSurface::requestShowWindowMenu, 19))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< WSurface* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isActivated(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isMaximized(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->isMinimized(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isFullScreen(); break;
        case 4: *reinterpret_cast<WSurface**>(_v) = _t->surface(); break;
        case 5: *reinterpret_cast<WSurface**>(_v) = _t->parentSurface(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->title(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->appId(); break;
        default: break;
        }
    }
}

const QMetaObject *Waylib::Server::WToplevelSurface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WToplevelSurface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server16WToplevelSurfaceE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "WWaylandResource"))
        return static_cast< WWaylandResource*>(this);
    return QObject::qt_metacast(_clname);
}

int Waylib::Server::WToplevelSurface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 29)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 29;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 29)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 29;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WToplevelSurface::activateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Waylib::Server::WToplevelSurface::maximizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Waylib::Server::WToplevelSurface::minimizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Waylib::Server::WToplevelSurface::surfaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Waylib::Server::WToplevelSurface::parentSurfaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Waylib::Server::WToplevelSurface::fullscreenChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Waylib::Server::WToplevelSurface::titleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void Waylib::Server::WToplevelSurface::beforeDestroy()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void Waylib::Server::WToplevelSurface::appIdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void Waylib::Server::WToplevelSurface::minimumSizeChanged(const QSize & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void Waylib::Server::WToplevelSurface::maximumSizeChanged(const QSize & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void Waylib::Server::WToplevelSurface::requestMove(WSeat * _t1, quint32 _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1, _t2);
}

// SIGNAL 12
void Waylib::Server::WToplevelSurface::requestResize(WSeat * _t1, Qt::Edges _t2, quint32 _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1, _t2, _t3);
}

// SIGNAL 13
void Waylib::Server::WToplevelSurface::requestMaximize()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void Waylib::Server::WToplevelSurface::requestCancelMaximize()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void Waylib::Server::WToplevelSurface::requestMinimize()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void Waylib::Server::WToplevelSurface::requestCancelMinimize()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void Waylib::Server::WToplevelSurface::requestFullscreen()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void Waylib::Server::WToplevelSurface::requestCancelFullscreen()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void Waylib::Server::WToplevelSurface::requestShowWindowMenu(WSeat * _t1, QPoint _t2, quint32 _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
