/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.freedesktop.DisplayManager.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef DISPLAYMANAGER_H
#define DISPLAYMANAGER_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.freedesktop.DisplayManager
 */
class DisplayManager: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.freedesktop.DisplayManager"; }

public:
    DisplayManager(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~DisplayManager();

    Q_PROPERTY(QDBusObjectPath LastSession READ lastSession)
    inline QDBusObjectPath lastSession() const
    { return qvariant_cast< QDBusObjectPath >(property("LastSession")); }

    Q_PROPERTY(QList<QDBusObjectPath> Seats READ seats)
    inline QList<QDBusObjectPath> seats() const
    { return qvariant_cast< QList<QDBusObjectPath> >(property("Seats")); }

    Q_PROPERTY(QList<QDBusObjectPath> Sessions READ sessions)
    inline QList<QDBusObjectPath> sessions() const
    { return qvariant_cast< QList<QDBusObjectPath> >(property("Sessions")); }

public Q_SLOTS: // METHODS
Q_SIGNALS: // SIGNALS
    void SeatAdded(const QDBusObjectPath &seat);
    void SeatRemoved(const QDBusObjectPath &seat);
    void SessionAdded(const QDBusObjectPath &session);
    void SessionRemoved(const QDBusObjectPath &session);
};

#endif
