/* Generated by wayland-scanner 1.26.0 */

/*
 * SPDX-FileCopyrightText: 2026 UnionTech Software Technology Co., Ltd.
 * SPDX-License-Identifier: MIT
 */

#include <stdbool.h>
#include <stdlib.h>
#include <stdint.h>
#include "wayland-util.h"

#ifndef __has_attribute
# define __has_attribute(x) 0  /* Compatibility with non-clang compilers. */
#endif

#if (__has_attribute(visibility) || defined(__GNUC__) && __GNUC__ >= 4)
#define WL_PRIVATE __attribute__ ((visibility("hidden")))
#else
#define WL_PRIVATE
#endif

extern const struct wl_interface treeland_keyboard_settings_v1_interface;
extern const struct wl_interface treeland_mouse_settings_v1_interface;
extern const struct wl_interface treeland_pointer_device_configuration_v1_interface;
extern const struct wl_interface treeland_touchpad_settings_v1_interface;
extern const struct wl_interface wl_seat_interface;

static const struct wl_interface *treeland_input_manager_unstable_v1_types[] = {
	NULL,
	NULL,
	&treeland_mouse_settings_v1_interface,
	&wl_seat_interface,
	&treeland_touchpad_settings_v1_interface,
	&wl_seat_interface,
	&treeland_keyboard_settings_v1_interface,
	&wl_seat_interface,
	NULL,
	&wl_seat_interface,
	NULL,
	&wl_seat_interface,
	&treeland_pointer_device_configuration_v1_interface,
	NULL,
	&treeland_pointer_device_configuration_v1_interface,
	NULL,
};

static const struct wl_message treeland_input_manager_v1_requests[] = {
	{ "destroy", "", treeland_input_manager_unstable_v1_types + 0 },
	{ "get_mouse_settings", "no", treeland_input_manager_unstable_v1_types + 2 },
	{ "get_touchpad_settings", "no", treeland_input_manager_unstable_v1_types + 4 },
	{ "get_keyboard_settings", "no", treeland_input_manager_unstable_v1_types + 6 },
};

static const struct wl_message treeland_input_manager_v1_events[] = {
	{ "capability_available", "uo", treeland_input_manager_unstable_v1_types + 8 },
	{ "capability_unavailable", "uo", treeland_input_manager_unstable_v1_types + 10 },
};

WL_PRIVATE const struct wl_interface treeland_input_manager_v1_interface = {
	"treeland_input_manager_v1", 1,
	4, treeland_input_manager_v1_requests,
	2, treeland_input_manager_v1_events,
};

static const struct wl_message treeland_pointer_device_configuration_v1_requests[] = {
	{ "destroy", "", treeland_input_manager_unstable_v1_types + 0 },
	{ "set_scroll_factor", "f", treeland_input_manager_unstable_v1_types + 0 },
	{ "set_handed_mode", "u", treeland_input_manager_unstable_v1_types + 0 },
	{ "set_accel_speed", "f", treeland_input_manager_unstable_v1_types + 0 },
	{ "set_acceleration_profile", "u", treeland_input_manager_unstable_v1_types + 0 },
	{ "set_send_events_mode", "u", treeland_input_manager_unstable_v1_types + 0 },
	{ "set_natural_scroll", "u", treeland_input_manager_unstable_v1_types + 0 },
	{ "set_disable_while_typing", "u", treeland_input_manager_unstable_v1_types + 0 },
	{ "set_tap_to_click", "u", treeland_input_manager_unstable_v1_types + 0 },
	{ "apply", "", treeland_input_manager_unstable_v1_types + 0 },
};

static const struct wl_message treeland_pointer_device_configuration_v1_events[] = {
	{ "feature", "u", treeland_input_manager_unstable_v1_types + 0 },
	{ "scroll_factor", "f", treeland_input_manager_unstable_v1_types + 0 },
	{ "handed_mode", "u", treeland_input_manager_unstable_v1_types + 0 },
	{ "accel_speed", "f", treeland_input_manager_unstable_v1_types + 0 },
	{ "acceleration_profile", "u", treeland_input_manager_unstable_v1_types + 0 },
	{ "send_events_mode", "u", treeland_input_manager_unstable_v1_types + 0 },
	{ "natural_scroll", "u", treeland_input_manager_unstable_v1_types + 0 },
	{ "disable_while_typing", "u", treeland_input_manager_unstable_v1_types + 0 },
	{ "tap_to_click", "u", treeland_input_manager_unstable_v1_types + 0 },
	{ "failed", "", treeland_input_manager_unstable_v1_types + 0 },
	{ "done", "u", treeland_input_manager_unstable_v1_types + 0 },
};

WL_PRIVATE const struct wl_interface treeland_pointer_device_configuration_v1_interface = {
	"treeland_pointer_device_configuration_v1", 1,
	10, treeland_pointer_device_configuration_v1_requests,
	11, treeland_pointer_device_configuration_v1_events,
};

static const struct wl_message treeland_mouse_settings_v1_requests[] = {
	{ "destroy", "", treeland_input_manager_unstable_v1_types + 0 },
	{ "get_pointer_configuration", "nu", treeland_input_manager_unstable_v1_types + 12 },
};

WL_PRIVATE const struct wl_interface treeland_mouse_settings_v1_interface = {
	"treeland_mouse_settings_v1", 1,
	2, treeland_mouse_settings_v1_requests,
	0, NULL,
};

static const struct wl_message treeland_touchpad_settings_v1_requests[] = {
	{ "destroy", "", treeland_input_manager_unstable_v1_types + 0 },
	{ "get_pointer_configuration", "nu", treeland_input_manager_unstable_v1_types + 14 },
};

WL_PRIVATE const struct wl_interface treeland_touchpad_settings_v1_interface = {
	"treeland_touchpad_settings_v1", 1,
	2, treeland_touchpad_settings_v1_requests,
	0, NULL,
};

static const struct wl_message treeland_keyboard_settings_v1_requests[] = {
	{ "destroy", "", treeland_input_manager_unstable_v1_types + 0 },
	{ "set_repeat", "ii", treeland_input_manager_unstable_v1_types + 0 },
	{ "set_num_lock", "u", treeland_input_manager_unstable_v1_types + 0 },
	{ "apply", "", treeland_input_manager_unstable_v1_types + 0 },
};

static const struct wl_message treeland_keyboard_settings_v1_events[] = {
	{ "feature", "u", treeland_input_manager_unstable_v1_types + 0 },
	{ "repeat", "ii", treeland_input_manager_unstable_v1_types + 0 },
	{ "num_lock", "u", treeland_input_manager_unstable_v1_types + 0 },
	{ "failed", "", treeland_input_manager_unstable_v1_types + 0 },
	{ "done", "", treeland_input_manager_unstable_v1_types + 0 },
};

WL_PRIVATE const struct wl_interface treeland_keyboard_settings_v1_interface = {
	"treeland_keyboard_settings_v1", 1,
	4, treeland_keyboard_settings_v1_requests,
	5, treeland_keyboard_settings_v1_events,
};

