/****************************************************************************
** Meta object code from reading C++ file 'shellhandler.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/core/shellhandler.h"
#include "private/wserver_p.h"
#include "woutput.h"
#include "workspace/workspace.h"
#include <wlayersurface.h>
#include <wxdgtoplevelsurface.h>
#include <wxwaylandsurface.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'shellhandler.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12ShellHandlerE_t {};
} // unnamed namespace

template <> constexpr inline auto ShellHandler::qt_create_metaobjectdata<qt_meta_tag_ZN12ShellHandlerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ShellHandler",
        "surfaceWrapperAdded",
        "",
        "SurfaceWrapper*",
        "wrapper",
        "surfaceWrapperAboutToRemove",
        "onXdgToplevelSurfaceAdded",
        "Waylib::Server::WXdgToplevelSurface*",
        "surface",
        "onXdgToplevelSurfaceRemoved",
        "onXdgPopupSurfaceAdded",
        "Waylib::Server::WXdgPopupSurface*",
        "onXdgPopupSurfaceRemoved",
        "onXWaylandSurfaceAdded",
        "Waylib::Server::WXWaylandSurface*",
        "onLayerSurfaceAdded",
        "Waylib::Server::WLayerSurface*",
        "onLayerSurfaceRemoved",
        "onInputPopupSurfaceV2Added",
        "Waylib::Server::WInputPopupSurface*",
        "onDockPreview",
        "std::vector<SurfaceWrapper*>",
        "surfaces",
        "Waylib::Server::WSurface*",
        "target",
        "QPoint",
        "pos",
        "ForeignToplevelManagerInterfaceV1::PreviewDirection",
        "direction",
        "onDockPreviewTooltip",
        "tooltip",
        "setupDockPreview",
        "onInputPopupSurfaceV2Removed",
        "onWindowMenuClosed"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'surfaceWrapperAdded'
        QtMocHelpers::SignalData<void(SurfaceWrapper *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'surfaceWrapperAboutToRemove'
        QtMocHelpers::SignalData<void(SurfaceWrapper *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'onXdgToplevelSurfaceAdded'
        QtMocHelpers::SlotData<void(Waylib::Server::WXdgToplevelSurface *)>(6, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'onXdgToplevelSurfaceRemoved'
        QtMocHelpers::SlotData<void(Waylib::Server::WXdgToplevelSurface *)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'onXdgPopupSurfaceAdded'
        QtMocHelpers::SlotData<void(Waylib::Server::WXdgPopupSurface *)>(10, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 11, 8 },
        }}),
        // Slot 'onXdgPopupSurfaceRemoved'
        QtMocHelpers::SlotData<void(Waylib::Server::WXdgPopupSurface *)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 11, 8 },
        }}),
        // Slot 'onXWaylandSurfaceAdded'
        QtMocHelpers::SlotData<void(Waylib::Server::WXWaylandSurface *)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 14, 8 },
        }}),
        // Slot 'onLayerSurfaceAdded'
        QtMocHelpers::SlotData<void(Waylib::Server::WLayerSurface *)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 16, 8 },
        }}),
        // Slot 'onLayerSurfaceRemoved'
        QtMocHelpers::SlotData<void(Waylib::Server::WLayerSurface *)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 16, 8 },
        }}),
        // Slot 'onInputPopupSurfaceV2Added'
        QtMocHelpers::SlotData<void(Waylib::Server::WInputPopupSurface *)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 8 },
        }}),
        // Slot 'onDockPreview'
        QtMocHelpers::SlotData<void(std::vector<SurfaceWrapper*>, Waylib::Server::WSurface *, QPoint, ForeignToplevelManagerInterfaceV1::PreviewDirection)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 21, 22 }, { 0x80000000 | 23, 24 }, { 0x80000000 | 25, 26 }, { 0x80000000 | 27, 28 },
        }}),
        // Slot 'onDockPreviewTooltip'
        QtMocHelpers::SlotData<void(QString, Waylib::Server::WSurface *, QPoint, ForeignToplevelManagerInterfaceV1::PreviewDirection)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 30 }, { 0x80000000 | 23, 24 }, { 0x80000000 | 25, 26 }, { 0x80000000 | 27, 28 },
        }}),
        // Slot 'setupDockPreview'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onInputPopupSurfaceV2Removed'
        QtMocHelpers::SlotData<void(Waylib::Server::WInputPopupSurface *)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 19, 8 },
        }}),
        // Slot 'onWindowMenuClosed'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ShellHandler, qt_meta_tag_ZN12ShellHandlerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ShellHandler::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12ShellHandlerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12ShellHandlerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12ShellHandlerE_t>.metaTypes,
    nullptr
} };

void ShellHandler::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ShellHandler *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->surfaceWrapperAdded((*reinterpret_cast<std::add_pointer_t<SurfaceWrapper*>>(_a[1]))); break;
        case 1: _t->surfaceWrapperAboutToRemove((*reinterpret_cast<std::add_pointer_t<SurfaceWrapper*>>(_a[1]))); break;
        case 2: _t->onXdgToplevelSurfaceAdded((*reinterpret_cast<std::add_pointer_t<Waylib::Server::WXdgToplevelSurface*>>(_a[1]))); break;
        case 3: _t->onXdgToplevelSurfaceRemoved((*reinterpret_cast<std::add_pointer_t<Waylib::Server::WXdgToplevelSurface*>>(_a[1]))); break;
        case 4: _t->onXdgPopupSurfaceAdded((*reinterpret_cast<std::add_pointer_t<Waylib::Server::WXdgPopupSurface*>>(_a[1]))); break;
        case 5: _t->onXdgPopupSurfaceRemoved((*reinterpret_cast<std::add_pointer_t<Waylib::Server::WXdgPopupSurface*>>(_a[1]))); break;
        case 6: _t->onXWaylandSurfaceAdded((*reinterpret_cast<std::add_pointer_t<Waylib::Server::WXWaylandSurface*>>(_a[1]))); break;
        case 7: _t->onLayerSurfaceAdded((*reinterpret_cast<std::add_pointer_t<Waylib::Server::WLayerSurface*>>(_a[1]))); break;
        case 8: _t->onLayerSurfaceRemoved((*reinterpret_cast<std::add_pointer_t<Waylib::Server::WLayerSurface*>>(_a[1]))); break;
        case 9: _t->onInputPopupSurfaceV2Added((*reinterpret_cast<std::add_pointer_t<Waylib::Server::WInputPopupSurface*>>(_a[1]))); break;
        case 10: _t->onDockPreview((*reinterpret_cast<std::add_pointer_t<std::vector<SurfaceWrapper*>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Waylib::Server::WSurface*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<ForeignToplevelManagerInterfaceV1::PreviewDirection>>(_a[4]))); break;
        case 11: _t->onDockPreviewTooltip((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Waylib::Server::WSurface*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<ForeignToplevelManagerInterfaceV1::PreviewDirection>>(_a[4]))); break;
        case 12: _t->setupDockPreview(); break;
        case 13: _t->onInputPopupSurfaceV2Removed((*reinterpret_cast<std::add_pointer_t<Waylib::Server::WInputPopupSurface*>>(_a[1]))); break;
        case 14: _t->onWindowMenuClosed(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Waylib::Server::WSurface* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Waylib::Server::WSurface* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ShellHandler::*)(SurfaceWrapper * )>(_a, &ShellHandler::surfaceWrapperAdded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ShellHandler::*)(SurfaceWrapper * )>(_a, &ShellHandler::surfaceWrapperAboutToRemove, 1))
            return;
    }
}

const QMetaObject *ShellHandler::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ShellHandler::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12ShellHandlerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ShellHandler::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void ShellHandler::surfaceWrapperAdded(SurfaceWrapper * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void ShellHandler::surfaceWrapperAboutToRemove(SurfaceWrapper * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
