/****************************************************************************
** Meta object code from reading C++ file 'personalizationmanagerinterfacev1.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/modules/personalization/personalizationmanagerinterfacev1.h"
#include "private/wserver_p.h"
#include "woutput.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'personalizationmanagerinterfacev1.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN30PersonalizationWindowContextV1E_t {};
} // unnamed namespace

template <> constexpr inline auto PersonalizationWindowContextV1::qt_create_metaobjectdata<qt_meta_tag_ZN30PersonalizationWindowContextV1E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PersonalizationWindowContextV1",
        "backgroundTypeChanged",
        "",
        "cornerRadiusChanged",
        "shadowChanged",
        "borderChanged",
        "windowStateChanged",
        "WindowState",
        "NoTitleBar"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'backgroundTypeChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cornerRadiusChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shadowChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'borderChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'windowStateChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'WindowState'
        QtMocHelpers::EnumData<enum WindowState>(7, 7, QMC::EnumFlags{}).add({
            {    8, WindowState::NoTitleBar },
        }),
    };
    return QtMocHelpers::metaObjectData<PersonalizationWindowContextV1, qt_meta_tag_ZN30PersonalizationWindowContextV1E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PersonalizationWindowContextV1::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN30PersonalizationWindowContextV1E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN30PersonalizationWindowContextV1E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN30PersonalizationWindowContextV1E_t>.metaTypes,
    nullptr
} };

void PersonalizationWindowContextV1::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PersonalizationWindowContextV1 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->backgroundTypeChanged(); break;
        case 1: _t->cornerRadiusChanged(); break;
        case 2: _t->shadowChanged(); break;
        case 3: _t->borderChanged(); break;
        case 4: _t->windowStateChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PersonalizationWindowContextV1::*)()>(_a, &PersonalizationWindowContextV1::backgroundTypeChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PersonalizationWindowContextV1::*)()>(_a, &PersonalizationWindowContextV1::cornerRadiusChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PersonalizationWindowContextV1::*)()>(_a, &PersonalizationWindowContextV1::shadowChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (PersonalizationWindowContextV1::*)()>(_a, &PersonalizationWindowContextV1::borderChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (PersonalizationWindowContextV1::*)()>(_a, &PersonalizationWindowContextV1::windowStateChanged, 4))
            return;
    }
}

const QMetaObject *PersonalizationWindowContextV1::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PersonalizationWindowContextV1::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN30PersonalizationWindowContextV1E_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int PersonalizationWindowContextV1::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void PersonalizationWindowContextV1::backgroundTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void PersonalizationWindowContextV1::cornerRadiusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void PersonalizationWindowContextV1::shadowChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void PersonalizationWindowContextV1::borderChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void PersonalizationWindowContextV1::windowStateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
namespace {
struct qt_meta_tag_ZN30PersonalizationCursorContextV1E_t {};
} // unnamed namespace

template <> constexpr inline auto PersonalizationCursorContextV1::qt_create_metaobjectdata<qt_meta_tag_ZN30PersonalizationCursorContextV1E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PersonalizationCursorContextV1",
        "commit",
        "",
        "PersonalizationCursorContextV1*",
        "context",
        "getSize",
        "getTheme"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'commit'
        QtMocHelpers::SignalData<void(PersonalizationCursorContextV1 *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'getSize'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'getTheme'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PersonalizationCursorContextV1, qt_meta_tag_ZN30PersonalizationCursorContextV1E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PersonalizationCursorContextV1::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN30PersonalizationCursorContextV1E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN30PersonalizationCursorContextV1E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN30PersonalizationCursorContextV1E_t>.metaTypes,
    nullptr
} };

void PersonalizationCursorContextV1::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PersonalizationCursorContextV1 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->commit((*reinterpret_cast<std::add_pointer_t<PersonalizationCursorContextV1*>>(_a[1]))); break;
        case 1: _t->getSize(); break;
        case 2: _t->getTheme(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PersonalizationCursorContextV1* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PersonalizationCursorContextV1::*)(PersonalizationCursorContextV1 * )>(_a, &PersonalizationCursorContextV1::commit, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PersonalizationCursorContextV1::*)()>(_a, &PersonalizationCursorContextV1::getSize, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PersonalizationCursorContextV1::*)()>(_a, &PersonalizationCursorContextV1::getTheme, 2))
            return;
    }
}

const QMetaObject *PersonalizationCursorContextV1::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PersonalizationCursorContextV1::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN30PersonalizationCursorContextV1E_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int PersonalizationCursorContextV1::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void PersonalizationCursorContextV1::commit(PersonalizationCursorContextV1 * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void PersonalizationCursorContextV1::getSize()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void PersonalizationCursorContextV1::getTheme()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
namespace {
struct qt_meta_tag_ZN34PersonalizationAppearanceContextV1E_t {};
} // unnamed namespace

template <> constexpr inline auto PersonalizationAppearanceContextV1::qt_create_metaobjectdata<qt_meta_tag_ZN34PersonalizationAppearanceContextV1E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PersonalizationAppearanceContextV1",
        "roundCornerRadiusChanged",
        "",
        "int32_t",
        "radius",
        "iconThemeChanged",
        "iconTheme",
        "activeColorChanged",
        "color",
        "windowOpacityChanged",
        "uint32_t",
        "opacity",
        "windowThemeTypeChanged",
        "type",
        "titlebarHeightChanged",
        "height",
        "requestRoundCornerRadius",
        "requestIconTheme",
        "requestActiveColor",
        "requestWindowOpacity",
        "requestWindowThemeType",
        "requestWindowTitlebarHeight"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'roundCornerRadiusChanged'
        QtMocHelpers::SignalData<void(int32_t)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'iconThemeChanged'
        QtMocHelpers::SignalData<void(const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'activeColorChanged'
        QtMocHelpers::SignalData<void(const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Signal 'windowOpacityChanged'
        QtMocHelpers::SignalData<void(uint32_t)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'windowThemeTypeChanged'
        QtMocHelpers::SignalData<void(uint32_t)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 13 },
        }}),
        // Signal 'titlebarHeightChanged'
        QtMocHelpers::SignalData<void(uint32_t)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 15 },
        }}),
        // Signal 'requestRoundCornerRadius'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestIconTheme'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestActiveColor'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestWindowOpacity'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestWindowThemeType'
        QtMocHelpers::SignalData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestWindowTitlebarHeight'
        QtMocHelpers::SignalData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PersonalizationAppearanceContextV1, qt_meta_tag_ZN34PersonalizationAppearanceContextV1E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PersonalizationAppearanceContextV1::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN34PersonalizationAppearanceContextV1E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN34PersonalizationAppearanceContextV1E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN34PersonalizationAppearanceContextV1E_t>.metaTypes,
    nullptr
} };

void PersonalizationAppearanceContextV1::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PersonalizationAppearanceContextV1 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->roundCornerRadiusChanged((*reinterpret_cast<std::add_pointer_t<int32_t>>(_a[1]))); break;
        case 1: _t->iconThemeChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->activeColorChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->windowOpacityChanged((*reinterpret_cast<std::add_pointer_t<uint32_t>>(_a[1]))); break;
        case 4: _t->windowThemeTypeChanged((*reinterpret_cast<std::add_pointer_t<uint32_t>>(_a[1]))); break;
        case 5: _t->titlebarHeightChanged((*reinterpret_cast<std::add_pointer_t<uint32_t>>(_a[1]))); break;
        case 6: _t->requestRoundCornerRadius(); break;
        case 7: _t->requestIconTheme(); break;
        case 8: _t->requestActiveColor(); break;
        case 9: _t->requestWindowOpacity(); break;
        case 10: _t->requestWindowThemeType(); break;
        case 11: _t->requestWindowTitlebarHeight(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PersonalizationAppearanceContextV1::*)(int32_t )>(_a, &PersonalizationAppearanceContextV1::roundCornerRadiusChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PersonalizationAppearanceContextV1::*)(const QString & )>(_a, &PersonalizationAppearanceContextV1::iconThemeChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PersonalizationAppearanceContextV1::*)(const QString & )>(_a, &PersonalizationAppearanceContextV1::activeColorChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (PersonalizationAppearanceContextV1::*)(uint32_t )>(_a, &PersonalizationAppearanceContextV1::windowOpacityChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (PersonalizationAppearanceContextV1::*)(uint32_t )>(_a, &PersonalizationAppearanceContextV1::windowThemeTypeChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (PersonalizationAppearanceContextV1::*)(uint32_t )>(_a, &PersonalizationAppearanceContextV1::titlebarHeightChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (PersonalizationAppearanceContextV1::*)()>(_a, &PersonalizationAppearanceContextV1::requestRoundCornerRadius, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (PersonalizationAppearanceContextV1::*)()>(_a, &PersonalizationAppearanceContextV1::requestIconTheme, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (PersonalizationAppearanceContextV1::*)()>(_a, &PersonalizationAppearanceContextV1::requestActiveColor, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (PersonalizationAppearanceContextV1::*)()>(_a, &PersonalizationAppearanceContextV1::requestWindowOpacity, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (PersonalizationAppearanceContextV1::*)()>(_a, &PersonalizationAppearanceContextV1::requestWindowThemeType, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (PersonalizationAppearanceContextV1::*)()>(_a, &PersonalizationAppearanceContextV1::requestWindowTitlebarHeight, 11))
            return;
    }
}

const QMetaObject *PersonalizationAppearanceContextV1::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PersonalizationAppearanceContextV1::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN34PersonalizationAppearanceContextV1E_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int PersonalizationAppearanceContextV1::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void PersonalizationAppearanceContextV1::roundCornerRadiusChanged(int32_t _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void PersonalizationAppearanceContextV1::iconThemeChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void PersonalizationAppearanceContextV1::activeColorChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void PersonalizationAppearanceContextV1::windowOpacityChanged(uint32_t _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void PersonalizationAppearanceContextV1::windowThemeTypeChanged(uint32_t _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void PersonalizationAppearanceContextV1::titlebarHeightChanged(uint32_t _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void PersonalizationAppearanceContextV1::requestRoundCornerRadius()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void PersonalizationAppearanceContextV1::requestIconTheme()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void PersonalizationAppearanceContextV1::requestActiveColor()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void PersonalizationAppearanceContextV1::requestWindowOpacity()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void PersonalizationAppearanceContextV1::requestWindowThemeType()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void PersonalizationAppearanceContextV1::requestWindowTitlebarHeight()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}
namespace {
struct qt_meta_tag_ZN28PersonalizationFontContextV1E_t {};
} // unnamed namespace

template <> constexpr inline auto PersonalizationFontContextV1::qt_create_metaobjectdata<qt_meta_tag_ZN28PersonalizationFontContextV1E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PersonalizationFontContextV1",
        "fontChanged",
        "",
        "font",
        "monoFontChanged",
        "fontSizeChanged",
        "uint32_t",
        "size",
        "requestFont",
        "requestMonoFont",
        "requestFontSize"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fontChanged'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'monoFontChanged'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'fontSizeChanged'
        QtMocHelpers::SignalData<void(uint32_t)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'requestFont'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestMonoFont'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestFontSize'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PersonalizationFontContextV1, qt_meta_tag_ZN28PersonalizationFontContextV1E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PersonalizationFontContextV1::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28PersonalizationFontContextV1E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28PersonalizationFontContextV1E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN28PersonalizationFontContextV1E_t>.metaTypes,
    nullptr
} };

void PersonalizationFontContextV1::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PersonalizationFontContextV1 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fontChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->monoFontChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->fontSizeChanged((*reinterpret_cast<std::add_pointer_t<uint32_t>>(_a[1]))); break;
        case 3: _t->requestFont(); break;
        case 4: _t->requestMonoFont(); break;
        case 5: _t->requestFontSize(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PersonalizationFontContextV1::*)(const QString & )>(_a, &PersonalizationFontContextV1::fontChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PersonalizationFontContextV1::*)(const QString & )>(_a, &PersonalizationFontContextV1::monoFontChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PersonalizationFontContextV1::*)(uint32_t )>(_a, &PersonalizationFontContextV1::fontSizeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (PersonalizationFontContextV1::*)()>(_a, &PersonalizationFontContextV1::requestFont, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (PersonalizationFontContextV1::*)()>(_a, &PersonalizationFontContextV1::requestMonoFont, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (PersonalizationFontContextV1::*)()>(_a, &PersonalizationFontContextV1::requestFontSize, 5))
            return;
    }
}

const QMetaObject *PersonalizationFontContextV1::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PersonalizationFontContextV1::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28PersonalizationFontContextV1E_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int PersonalizationFontContextV1::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void PersonalizationFontContextV1::fontChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void PersonalizationFontContextV1::monoFontChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void PersonalizationFontContextV1::fontSizeChanged(uint32_t _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void PersonalizationFontContextV1::requestFont()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void PersonalizationFontContextV1::requestMonoFont()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void PersonalizationFontContextV1::requestFontSize()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
namespace {
struct qt_meta_tag_ZN15PersonalizationE_t {};
} // unnamed namespace

template <> constexpr inline auto Personalization::qt_create_metaobjectdata<qt_meta_tag_ZN15PersonalizationE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Personalization",
        "QML.Element",
        "anonymous",
        "backgroundTypeChanged",
        "",
        "cornerRadiusChanged",
        "shadowChanged",
        "borderChanged",
        "windowStateChanged",
        "resetProperties",
        "backgroundType",
        "BackgroundType",
        "cornerRadius",
        "int32_t",
        "shadow",
        "Shadow",
        "border",
        "Border",
        "noTitlebar",
        "Normal",
        "Wallpaper",
        "Blur"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'backgroundTypeChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cornerRadiusChanged'
        QtMocHelpers::SignalData<void()>(5, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shadowChanged'
        QtMocHelpers::SignalData<void()>(6, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'borderChanged'
        QtMocHelpers::SignalData<void()>(7, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'windowStateChanged'
        QtMocHelpers::SignalData<void()>(8, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resetProperties'
        QtMocHelpers::SlotData<void()>(9, 4, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'backgroundType'
        QtMocHelpers::PropertyData<enum BackgroundType>(10, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 'cornerRadius'
        QtMocHelpers::PropertyData<int32_t>(12, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 1),
        // property 'shadow'
        QtMocHelpers::PropertyData<Shadow>(14, 0x80000000 | 15, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 2),
        // property 'border'
        QtMocHelpers::PropertyData<Border>(16, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 3),
        // property 'noTitlebar'
        QtMocHelpers::PropertyData<bool>(18, QMetaType::Bool, QMC::DefaultPropertyFlags, 4),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'BackgroundType'
        QtMocHelpers::EnumData<enum BackgroundType>(11, 11, QMC::EnumFlags{}).add({
            {   19, BackgroundType::Normal },
            {   20, BackgroundType::Wallpaper },
            {   21, BackgroundType::Blur },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<Personalization, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Personalization::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15PersonalizationE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15PersonalizationE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15PersonalizationE_t>.metaTypes,
    nullptr
} };

void Personalization::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Personalization *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->backgroundTypeChanged(); break;
        case 1: _t->cornerRadiusChanged(); break;
        case 2: _t->shadowChanged(); break;
        case 3: _t->borderChanged(); break;
        case 4: _t->windowStateChanged(); break;
        case 5: _t->resetProperties(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Personalization::*)()>(_a, &Personalization::backgroundTypeChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Personalization::*)()>(_a, &Personalization::cornerRadiusChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Personalization::*)()>(_a, &Personalization::shadowChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Personalization::*)()>(_a, &Personalization::borderChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Personalization::*)()>(_a, &Personalization::windowStateChanged, 4))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<enum BackgroundType*>(_v) = _t->backgroundType(); break;
        case 1: *reinterpret_cast<int32_t*>(_v) = _t->cornerRadius(); break;
        case 2: *reinterpret_cast<Shadow*>(_v) = _t->shadow(); break;
        case 3: *reinterpret_cast<Border*>(_v) = _t->border(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->noTitlebar(); break;
        default: break;
        }
    }
}

const QMetaObject *Personalization::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Personalization::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15PersonalizationE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Personalization::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void Personalization::backgroundTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Personalization::cornerRadiusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Personalization::shadowChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Personalization::borderChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Personalization::windowStateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
namespace {
struct qt_meta_tag_ZN33PersonalizationManagerInterfaceV1E_t {};
} // unnamed namespace

template <> constexpr inline auto PersonalizationManagerInterfaceV1::qt_create_metaobjectdata<qt_meta_tag_ZN33PersonalizationManagerInterfaceV1E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PersonalizationManagerInterfaceV1",
        "userIdChanged",
        "",
        "uid_t",
        "uid",
        "lockscreenChanged",
        "cursorThemeChanged",
        "name",
        "cursorSizeChanged",
        "QSize",
        "size",
        "windowContextCreated",
        "PersonalizationWindowContextV1*",
        "context",
        "userId",
        "cursorTheme",
        "cursorSize"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'userIdChanged'
        QtMocHelpers::SignalData<void(uid_t)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'lockscreenChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cursorThemeChanged'
        QtMocHelpers::SignalData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Signal 'cursorSizeChanged'
        QtMocHelpers::SignalData<void(const QSize &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'windowContextCreated'
        QtMocHelpers::SignalData<void(PersonalizationWindowContextV1 *)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'userId'
        QtMocHelpers::PropertyData<uid_t>(14, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 0),
        // property 'cursorTheme'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 2),
        // property 'cursorSize'
        QtMocHelpers::PropertyData<QSize>(16, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 3),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PersonalizationManagerInterfaceV1, qt_meta_tag_ZN33PersonalizationManagerInterfaceV1E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PersonalizationManagerInterfaceV1::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN33PersonalizationManagerInterfaceV1E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN33PersonalizationManagerInterfaceV1E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN33PersonalizationManagerInterfaceV1E_t>.metaTypes,
    nullptr
} };

void PersonalizationManagerInterfaceV1::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PersonalizationManagerInterfaceV1 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->userIdChanged((*reinterpret_cast<std::add_pointer_t<uid_t>>(_a[1]))); break;
        case 1: _t->lockscreenChanged(); break;
        case 2: _t->cursorThemeChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->cursorSizeChanged((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1]))); break;
        case 4: _t->windowContextCreated((*reinterpret_cast<std::add_pointer_t<PersonalizationWindowContextV1*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PersonalizationWindowContextV1* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PersonalizationManagerInterfaceV1::*)(uid_t )>(_a, &PersonalizationManagerInterfaceV1::userIdChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PersonalizationManagerInterfaceV1::*)()>(_a, &PersonalizationManagerInterfaceV1::lockscreenChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PersonalizationManagerInterfaceV1::*)(const QString & )>(_a, &PersonalizationManagerInterfaceV1::cursorThemeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (PersonalizationManagerInterfaceV1::*)(const QSize & )>(_a, &PersonalizationManagerInterfaceV1::cursorSizeChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (PersonalizationManagerInterfaceV1::*)(PersonalizationWindowContextV1 * )>(_a, &PersonalizationManagerInterfaceV1::windowContextCreated, 4))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<uid_t*>(_v) = _t->userId(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->cursorTheme(); break;
        case 2: *reinterpret_cast<QSize*>(_v) = _t->cursorSize(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setUserId(*reinterpret_cast<uid_t*>(_v)); break;
        case 1: _t->setCursorTheme(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setCursorSize(*reinterpret_cast<QSize*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *PersonalizationManagerInterfaceV1::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PersonalizationManagerInterfaceV1::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN33PersonalizationManagerInterfaceV1E_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "WServerInterface"))
        return static_cast< WServerInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int PersonalizationManagerInterfaceV1::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void PersonalizationManagerInterfaceV1::userIdChanged(uid_t _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void PersonalizationManagerInterfaceV1::lockscreenChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void PersonalizationManagerInterfaceV1::cursorThemeChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void PersonalizationManagerInterfaceV1::cursorSizeChanged(const QSize & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void PersonalizationManagerInterfaceV1::windowContextCreated(PersonalizationWindowContextV1 * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
