// This file was generated by qtwaylandscanner
// source file is treeland-wallpaper-shell-unstable-v1.xml

#ifndef QT_WAYLAND_TREELAND_WALLPAPER_SHELL_UNSTABLE_V1
#define QT_WAYLAND_TREELAND_WALLPAPER_SHELL_UNSTABLE_V1

#include "wayland-treeland-wallpaper-shell-unstable-v1-client-protocol.h"
#include <QByteArray>
#include <QString>

struct wl_registry;

QT_BEGIN_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_GCC("-Wmissing-field-initializers")
QT_WARNING_DISABLE_CLANG("-Wmissing-field-initializers")

namespace QtWayland {
    class  treeland_wallpaper_notifier_v1
    {
    public:
        treeland_wallpaper_notifier_v1(struct ::wl_registry *registry, uint32_t id, int version);
        treeland_wallpaper_notifier_v1(struct ::treeland_wallpaper_notifier_v1 *object);
        treeland_wallpaper_notifier_v1();

        virtual ~treeland_wallpaper_notifier_v1();

        void init(struct ::wl_registry *registry, uint32_t id, int version);
        void init(struct ::treeland_wallpaper_notifier_v1 *object);

        struct ::treeland_wallpaper_notifier_v1 *object() { return m_treeland_wallpaper_notifier_v1; }
        const struct ::treeland_wallpaper_notifier_v1 *object() const { return m_treeland_wallpaper_notifier_v1; }
        static treeland_wallpaper_notifier_v1 *fromObject(struct ::treeland_wallpaper_notifier_v1 *object);

        bool isInitialized() const;

        uint32_t version() const;
        static const struct ::wl_interface *interface();

        enum wallpaper_source_type {
            wallpaper_source_type_image = 0, // The wallpaper source is an image
            wallpaper_source_type_video = 1, // The wallpaper source is a video
        };

        void destroy();

    protected:
        virtual void treeland_wallpaper_notifier_v1_add(uint32_t source_type, const QString &file_source);
        virtual void treeland_wallpaper_notifier_v1_remove(const QString &file_source);

    private:
        void init_listener();
        static const struct treeland_wallpaper_notifier_v1_listener m_treeland_wallpaper_notifier_v1_listener;
        static void handle_add(
            void *data,
            struct ::treeland_wallpaper_notifier_v1 *object,
            uint32_t source_type,
            const char *file_source);
        static void handle_remove(
            void *data,
            struct ::treeland_wallpaper_notifier_v1 *object,
            const char *file_source);
        struct ::treeland_wallpaper_notifier_v1 *m_treeland_wallpaper_notifier_v1;
    };

    class  treeland_wallpaper_shell_v1
    {
    public:
        treeland_wallpaper_shell_v1(struct ::wl_registry *registry, uint32_t id, int version);
        treeland_wallpaper_shell_v1(struct ::treeland_wallpaper_shell_v1 *object);
        treeland_wallpaper_shell_v1();

        virtual ~treeland_wallpaper_shell_v1();

        void init(struct ::wl_registry *registry, uint32_t id, int version);
        void init(struct ::treeland_wallpaper_shell_v1 *object);

        struct ::treeland_wallpaper_shell_v1 *object() { return m_treeland_wallpaper_shell_v1; }
        const struct ::treeland_wallpaper_shell_v1 *object() const { return m_treeland_wallpaper_shell_v1; }
        static treeland_wallpaper_shell_v1 *fromObject(struct ::treeland_wallpaper_shell_v1 *object);

        bool isInitialized() const;

        uint32_t version() const;
        static const struct ::wl_interface *interface();

        void destroy();
        struct ::treeland_wallpaper_surface_v1 *get_treeland_wallpaper_surface(struct ::wl_surface *surface, const QString &file_source);

    private:
        struct ::treeland_wallpaper_shell_v1 *m_treeland_wallpaper_shell_v1;
    };

    class  treeland_wallpaper_surface_v1
    {
    public:
        treeland_wallpaper_surface_v1(struct ::wl_registry *registry, uint32_t id, int version);
        treeland_wallpaper_surface_v1(struct ::treeland_wallpaper_surface_v1 *object);
        treeland_wallpaper_surface_v1();

        virtual ~treeland_wallpaper_surface_v1();

        void init(struct ::wl_registry *registry, uint32_t id, int version);
        void init(struct ::treeland_wallpaper_surface_v1 *object);

        struct ::treeland_wallpaper_surface_v1 *object() { return m_treeland_wallpaper_surface_v1; }
        const struct ::treeland_wallpaper_surface_v1 *object() const { return m_treeland_wallpaper_surface_v1; }
        static treeland_wallpaper_surface_v1 *fromObject(struct ::treeland_wallpaper_surface_v1 *object);

        bool isInitialized() const;

        uint32_t version() const;
        static const struct ::wl_interface *interface();

        enum error {
            error_invalid_source = 1, // The specified wallpaper source is invalid, unsupported, or failed to load.
            error_permission_denied = 2, // Permission denied when opening the specified wallpaper source
        };

        void destroy();
        void source_failed(uint32_t error);
        void ready();

    protected:
        virtual void treeland_wallpaper_surface_v1_position(wl_fixed_t position);
        virtual void treeland_wallpaper_surface_v1_pause();
        virtual void treeland_wallpaper_surface_v1_set_playback_rate(wl_fixed_t rate);
        virtual void treeland_wallpaper_surface_v1_play();
        virtual void treeland_wallpaper_surface_v1_slow_down(uint32_t duration);

    private:
        void init_listener();
        static const struct treeland_wallpaper_surface_v1_listener m_treeland_wallpaper_surface_v1_listener;
        static void handle_position(
            void *data,
            struct ::treeland_wallpaper_surface_v1 *object,
            wl_fixed_t position);
        static void handle_pause(
            void *data,
            struct ::treeland_wallpaper_surface_v1 *object);
        static void handle_set_playback_rate(
            void *data,
            struct ::treeland_wallpaper_surface_v1 *object,
            wl_fixed_t rate);
        static void handle_play(
            void *data,
            struct ::treeland_wallpaper_surface_v1 *object);
        static void handle_slow_down(
            void *data,
            struct ::treeland_wallpaper_surface_v1 *object,
            uint32_t duration);
        struct ::treeland_wallpaper_surface_v1 *m_treeland_wallpaper_surface_v1;
    };
}

QT_WARNING_POP
QT_END_NAMESPACE

#endif
