/****************************************************************************
** Resource object code
**
** Created by: The Resource Compiler for Qt version 6.11.2
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#ifdef _MSC_VER
// disable informational message "function ... selected for automatic inline expansion"
#pragma warning (disable: 4711)
#endif

static const unsigned char qt_resource_data[] = {
  // qmldir
  0x0,0x0,0x2,0x2b,
  0x28,
  0xb5,0x2f,0xfd,0x60,0x4c,0x6,0xd,0x11,0x0,0x16,0x19,0x49,0x1e,0x40,0x73,0x1b,
  0x73,0x23,0x25,0xc9,0x94,0xd4,0x3b,0xb2,0xae,0x1c,0x95,0x8a,0x4a,0xa4,0x4f,0xa3,
  0x16,0xed,0xfa,0xfd,0x22,0x9e,0x32,0x8c,0x19,0x2,0x11,0x40,0x0,0x40,0x0,0x43,
  0x0,0x51,0xf4,0x99,0x88,0x0,0xb9,0x62,0xd4,0xdf,0xe3,0xaa,0x71,0x95,0x2,0x9c,
  0xac,0x40,0xef,0xfd,0xac,0x7d,0x20,0x5b,0x99,0x40,0x63,0xdc,0xea,0xe3,0x7c,0xc4,
  0x4a,0x7,0x61,0x32,0x83,0x38,0x5d,0xef,0xbf,0x7e,0xdc,0x4e,0x91,0xdf,0x5f,0x96,
  0xa1,0xd7,0xc9,0xf3,0x2b,0xb7,0xfc,0x34,0x97,0xac,0xea,0x13,0x48,0x39,0xb6,0xb6,
  0x2,0x14,0xfb,0xbd,0x24,0x65,0xf9,0x60,0xe5,0xad,0xcd,0xdb,0x70,0x75,0x4b,0x4a,
  0x9a,0x9a,0xe5,0x9d,0xbe,0xeb,0x83,0xb2,0xc4,0x78,0x94,0x2a,0x98,0xd1,0xa9,0xe3,
  0xea,0xe6,0xb6,0xe2,0x7a,0x3f,0xbd,0x99,0x22,0x3c,0xd,0xca,0xbf,0x93,0xd7,0x87,
  0x47,0xbd,0x19,0x24,0x8,0xc,0xa0,0x8f,0xef,0x23,0x84,0xe3,0x69,0xad,0x47,0x9e,
  0x28,0x94,0x4b,0x57,0xa3,0x1e,0x51,0x4d,0x1f,0x20,0x4c,0x5e,0xb8,0xac,0x38,0x6a,
  0x0,0x11,0x7c,0xa6,0x18,0x4c,0xc2,0x23,0x33,0x2b,0x25,0xb9,0x72,0x5a,0x3e,0x8a,
  0xb9,0x98,0x8d,0xd7,0x5f,0x41,0x45,0x5c,0x4d,0x69,0x67,0xad,0x75,0xeb,0xb8,0xf4,
  0x11,0x41,0x6b,0x5d,0x81,0xe3,0xaf,0x56,0xeb,0x22,0xf9,0xc1,0x45,0xc0,0xa7,0xe0,
  0x6c,0xda,0x23,0x1,0xcf,0x53,0xb1,0x8d,0x92,0x97,0x40,0xb0,0x39,0x7d,0xda,0x28,
  0x98,0xfc,0x24,0xae,0x58,0x5e,0x98,0x53,0xe1,0x54,0x9d,0x49,0xca,0x3b,0xd1,0xa9,
  0x25,0xb4,0x62,0xeb,0xfd,0x4c,0x59,0x25,0xf5,0xf8,0x2f,0xbd,0x57,0x93,0x94,0x1a,
  0xb4,0x65,0xc5,0xfa,0xa9,0x69,0x99,0x9a,0x96,0xa1,0x9f,0x93,0xb2,0x22,0x2e,0xdd,
  0x79,0xa8,0x51,0xe,0x99,0x19,0x91,0x9a,0xd1,0xc2,0x42,0x61,0x38,0x20,0x44,0xc8,
  0x3c,0x9b,0x36,0x31,0xc,0xc9,0x11,0x62,0x14,0x22,0xc4,0x22,0xa9,0x21,0x4d,0xca,
  0x23,0xc5,0x1,0xb4,0x0,0x7b,0x2f,0xf7,0x95,0xd0,0x60,0x80,0x2e,0x8e,0xf6,0xb7,
  0xdb,0xaf,0xc1,0x83,0x35,0x88,0xc,0xcb,0x85,0xb9,0xa8,0x9c,0x5b,0x48,0x49,0xb8,
  0xaf,0x5,0x59,0x54,0xa0,0xb0,0x63,0xaa,0xc0,0xfa,0xbb,0xb6,0xbc,0x56,0x9b,0x95,
  0x72,0x59,0xeb,0xb5,0x2c,0xf6,0x27,0xe,0x89,0x9e,0xf,0xe3,0x99,0xb2,0x40,0x5c,
  0x88,0x83,0xb3,0x51,0xf1,0x8b,0xcb,0xf5,0x74,0x82,0xed,0x3c,0xfc,0x4e,0xb6,0x4f,
  0xb5,0x21,0x10,0x64,0xa2,0xda,0xf,0x3c,0xe9,0xca,0x8c,0x89,0x1b,0x1b,0x2c,0xc3,
  0xc5,0x7e,0xe1,0x6b,0xa4,0xb,0xcb,0xdd,0xd1,0xf1,0x4f,0x2c,0xd8,0x99,0x86,0x6e,
  0x72,0xbf,0xad,0x82,0x97,0x2c,0x21,0x35,0xfd,0x32,0xcc,0x6b,0xb3,0xd0,0xc9,0x97,
  0x14,0xab,0xf9,0x96,0xd4,0x35,0x89,0xbf,0xf2,0x2a,0x30,0x4d,0xf4,0x20,0x4d,0xe9,
  0xda,0x4,0x57,0xdd,0xe9,0x67,0x6a,0x39,0x41,0xc5,0xe0,0x82,0xe1,0xde,0x90,0x83,
  0x1e,0xb5,0x5c,0xd6,0x8f,0xb9,0x2a,0xd2,0xe0,0x74,0x9f,0x61,0xe9,0x41,0x80,0xe8,
  0x95,0x7d,0xb9,0xd0,0x61,0x9c,0xa5,0x10,0x1a,0x55,0x65,0x35,0xde,0xda,0x91,0xb0,
  0xbc,0x64,0xd0,0x6,0x55,0x8d,0xb7,0x76,0x7d,0xc1,0x85,0xb6,0x35,0x77,0x37,0x1c,
  0x5c,0xf5,0x5c,0xc,0x6b,0xf6,0x52,0xab,0xa1,0xaa,
  
};

static const unsigned char qt_resource_name[] = {
  // qt
  0x0,0x2,
  0x0,0x0,0x7,0x84,
  0x0,0x71,
  0x0,0x74,
    // qml
  0x0,0x3,
  0x0,0x0,0x78,0x3c,
  0x0,0x71,
  0x0,0x6d,0x0,0x6c,
    // Treeland
  0x0,0x8,
  0x8,0xbc,0x2f,0x24,
  0x0,0x54,
  0x0,0x72,0x0,0x65,0x0,0x65,0x0,0x6c,0x0,0x61,0x0,0x6e,0x0,0x64,
    // qmldir
  0x0,0x6,
  0x7,0x84,0x2b,0x2,
  0x0,0x71,
  0x0,0x6d,0x0,0x6c,0x0,0x64,0x0,0x69,0x0,0x72,
  
};

static const unsigned char qt_resource_struct[] = {
  // :
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x1,
0x0,0x0,0x1,0xa0,0x14,0xec,0xd3,0x20,
  // :/qt
  0x0,0x0,0x0,0x0,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x2,
0x0,0x0,0x1,0xa0,0x14,0xec,0xd3,0x20,
  // :/qt/qml
  0x0,0x0,0x0,0xa,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x3,
0x0,0x0,0x1,0xa0,0x14,0xec,0xd3,0x20,
  // :/qt/qml/Treeland
  0x0,0x0,0x0,0x16,0x0,0x2,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x4,
0x0,0x0,0x1,0xa0,0x14,0xec,0xd3,0x20,
  // :/qt/qml/Treeland/qmldir
  0x0,0x0,0x0,0x2c,0x0,0x4,0x0,0x0,0x0,0x1,0x0,0x0,0x0,0x0,
0x0,0x0,0x1,0xa0,0x14,0xec,0xd3,0x20,

};

#ifdef QT_NAMESPACE
#  define QT_RCC_PREPEND_NAMESPACE(name) ::QT_NAMESPACE::name
#  define QT_RCC_MANGLE_NAMESPACE0(x) x
#  define QT_RCC_MANGLE_NAMESPACE1(a, b) a##_##b
#  define QT_RCC_MANGLE_NAMESPACE2(a, b) QT_RCC_MANGLE_NAMESPACE1(a,b)
#  define QT_RCC_MANGLE_NAMESPACE(name) QT_RCC_MANGLE_NAMESPACE2( \
        QT_RCC_MANGLE_NAMESPACE0(name), QT_RCC_MANGLE_NAMESPACE0(QT_NAMESPACE))
#else
#   define QT_RCC_PREPEND_NAMESPACE(name) name
#   define QT_RCC_MANGLE_NAMESPACE(name) name
#endif

#if defined(QT_INLINE_NAMESPACE)
inline namespace QT_NAMESPACE {
#elif defined(QT_NAMESPACE)
namespace QT_NAMESPACE {
#endif

bool qRegisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);
bool qUnregisterResourceData(int, const unsigned char *, const unsigned char *, const unsigned char *);

#if defined(__ELF__) || defined(__APPLE__)
static inline unsigned char qResourceFeatureZstd()
{
    extern const unsigned char qt_resourceFeatureZstd;
    return qt_resourceFeatureZstd;
}
#else
unsigned char qResourceFeatureZstd();
#endif

#ifdef QT_NAMESPACE
}
#endif

int QT_RCC_MANGLE_NAMESPACE(qInitResources_qmake_Treeland)();
int QT_RCC_MANGLE_NAMESPACE(qInitResources_qmake_Treeland)()
{
    int version = 3;
    QT_RCC_PREPEND_NAMESPACE(qRegisterResourceData)
        (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_qmake_Treeland)();
int QT_RCC_MANGLE_NAMESPACE(qCleanupResources_qmake_Treeland)()
{
    int version = 3;
    version += QT_RCC_PREPEND_NAMESPACE(qResourceFeatureZstd());
    QT_RCC_PREPEND_NAMESPACE(qUnregisterResourceData)
       (version, qt_resource_struct, qt_resource_name, qt_resource_data);
    return 1;
}

#ifdef __clang__
#   pragma clang diagnostic push
#   pragma clang diagnostic ignored "-Wexit-time-destructors"
#endif

namespace {
   struct initializer {
       initializer() { QT_RCC_MANGLE_NAMESPACE(qInitResources_qmake_Treeland)(); }
       ~initializer() { QT_RCC_MANGLE_NAMESPACE(qCleanupResources_qmake_Treeland)(); }
   } dummy;
}

#ifdef __clang__
#   pragma clang diagnostic pop
#endif
