/****************************************************************************
** Meta object code from reading C++ file 'lockscreen.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/core/lockscreen.h"
#include "woutput.h"
#include <woutput.h>
#include <output/output.h>
#include "core/rootsurfacecontainer.h"
#include <wsessionlock.h>
#include <wsessionlocksurface.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'lockscreen.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10LockScreenE_t {};
} // unnamed namespace

template <> constexpr inline auto LockScreen::qt_create_metaobjectdata<qt_meta_tag_ZN10LockScreenE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LockScreen",
        "QML.Element",
        "anonymous",
        "unlock",
        "",
        "onAnimationPlayed",
        "onAnimationPlayFinished",
        "onExternalLock",
        "WSessionLock*",
        "lock",
        "onLockSurfaceAdded",
        "WSessionLockSurface*",
        "surface",
        "onLockSurfaceRemoved",
        "onExternalLockAbandoned",
        "onExternalUnlock",
        "CurrentMode",
        "Lock",
        "Shutdown",
        "SwitchUser"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'unlock'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onAnimationPlayed'
        QtMocHelpers::SlotData<void()>(5, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onAnimationPlayFinished'
        QtMocHelpers::SlotData<void()>(6, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onExternalLock'
        QtMocHelpers::SlotData<void(WSessionLock *)>(7, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'onLockSurfaceAdded'
        QtMocHelpers::SlotData<void(WSessionLockSurface *)>(10, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'onLockSurfaceRemoved'
        QtMocHelpers::SlotData<void(WSessionLockSurface *)>(13, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'onExternalLockAbandoned'
        QtMocHelpers::SlotData<void()>(14, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onExternalUnlock'
        QtMocHelpers::SlotData<void()>(15, 4, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'CurrentMode'
        QtMocHelpers::EnumData<enum CurrentMode>(16, 16, QMC::EnumIsScoped).add({
            {   17, CurrentMode::Lock },
            {   18, CurrentMode::Shutdown },
            {   19, CurrentMode::SwitchUser },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<LockScreen, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject LockScreen::staticMetaObject = { {
    QMetaObject::SuperData::link<SurfaceContainer::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10LockScreenE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10LockScreenE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10LockScreenE_t>.metaTypes,
    nullptr
} };

void LockScreen::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LockScreen *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->unlock(); break;
        case 1: _t->onAnimationPlayed(); break;
        case 2: _t->onAnimationPlayFinished(); break;
        case 3: _t->onExternalLock((*reinterpret_cast<std::add_pointer_t<WSessionLock*>>(_a[1]))); break;
        case 4: _t->onLockSurfaceAdded((*reinterpret_cast<std::add_pointer_t<WSessionLockSurface*>>(_a[1]))); break;
        case 5: _t->onLockSurfaceRemoved((*reinterpret_cast<std::add_pointer_t<WSessionLockSurface*>>(_a[1]))); break;
        case 6: _t->onExternalLockAbandoned(); break;
        case 7: _t->onExternalUnlock(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (LockScreen::*)()>(_a, &LockScreen::unlock, 0))
            return;
    }
}

const QMetaObject *LockScreen::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LockScreen::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10LockScreenE_t>.strings))
        return static_cast<void*>(this);
    return SurfaceContainer::qt_metacast(_clname);
}

int LockScreen::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SurfaceContainer::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void LockScreen::unlock()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
