/****************************************************************************
** Meta object code from reading C++ file 'workspaceanimationcontroller.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/workspace/workspaceanimationcontroller.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'workspaceanimationcontroller.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN28WorkspaceAnimationControllerE_t {};
} // unnamed namespace

template <> constexpr inline auto WorkspaceAnimationController::qt_create_metaobjectdata<qt_meta_tag_ZN28WorkspaceAnimationControllerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "WorkspaceAnimationController",
        "QML.Element",
        "auto",
        "refWidthChanged",
        "",
        "refGapChanged",
        "refBounceChanged",
        "bounceFactorChanged",
        "refWrapChanged",
        "runningChanged",
        "viewportPosChanged",
        "pendingWorkspaceIndexChanged",
        "finished",
        "slideRunning",
        "toWorkspaceIndex",
        "slideNormal",
        "fromWorkspaceIndex",
        "slide",
        "bounce",
        "currentWorkspaceIndex",
        "Direction",
        "direction",
        "setRunning",
        "running",
        "startSlideAnimation",
        "startGestureSlide",
        "cb",
        "refWidth",
        "refGap",
        "refBounce",
        "bounceFactor",
        "refWrap",
        "viewportPos",
        "pendingWorkspaceIndex",
        "Left",
        "Right"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'refWidthChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'refGapChanged'
        QtMocHelpers::SignalData<void()>(5, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'refBounceChanged'
        QtMocHelpers::SignalData<void()>(6, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'bounceFactorChanged'
        QtMocHelpers::SignalData<void()>(7, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'refWrapChanged'
        QtMocHelpers::SignalData<void()>(8, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'runningChanged'
        QtMocHelpers::SignalData<void()>(9, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'viewportPosChanged'
        QtMocHelpers::SignalData<void()>(10, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pendingWorkspaceIndexChanged'
        QtMocHelpers::SignalData<void()>(11, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'finished'
        QtMocHelpers::SignalData<void()>(12, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slideRunning'
        QtMocHelpers::SlotData<void(uint)>(13, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 14 },
        }}),
        // Slot 'slideNormal'
        QtMocHelpers::SlotData<void(uint)>(15, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 16 },
        }}),
        // Slot 'slide'
        QtMocHelpers::SlotData<void(uint, uint)>(17, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 16 }, { QMetaType::UInt, 14 },
        }}),
        // Slot 'bounce'
        QtMocHelpers::SlotData<void(uint, enum Direction)>(18, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 19 }, { 0x80000000 | 20, 21 },
        }}),
        // Slot 'setRunning'
        QtMocHelpers::SlotData<void(bool)>(22, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 23 },
        }}),
        // Slot 'startSlideAnimation'
        QtMocHelpers::SlotData<void()>(24, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'startGestureSlide'
        QtMocHelpers::SlotData<void(qreal, bool)>(25, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 26 }, { QMetaType::Bool, 18 },
        }}),
        // Slot 'startGestureSlide'
        QtMocHelpers::SlotData<void(qreal)>(25, 4, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QReal, 26 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'refWidth'
        QtMocHelpers::PropertyData<qreal>(27, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 0),
        // property 'refGap'
        QtMocHelpers::PropertyData<qreal>(28, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 1),
        // property 'refBounce'
        QtMocHelpers::PropertyData<qreal>(29, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 2),
        // property 'bounceFactor'
        QtMocHelpers::PropertyData<qreal>(30, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 3),
        // property 'refWrap'
        QtMocHelpers::PropertyData<qreal>(31, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Final, 4),
        // property 'running'
        QtMocHelpers::PropertyData<bool>(23, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 5),
        // property 'viewportPos'
        QtMocHelpers::PropertyData<qreal>(32, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 6),
        // property 'pendingWorkspaceIndex'
        QtMocHelpers::PropertyData<uint>(33, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Final, 7),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Direction'
        QtMocHelpers::EnumData<enum Direction>(20, 20, QMC::EnumFlags{}).add({
            {   34, Direction::Left },
            {   35, Direction::Right },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<WorkspaceAnimationController, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject WorkspaceAnimationController::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28WorkspaceAnimationControllerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28WorkspaceAnimationControllerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN28WorkspaceAnimationControllerE_t>.metaTypes,
    nullptr
} };

void WorkspaceAnimationController::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WorkspaceAnimationController *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->refWidthChanged(); break;
        case 1: _t->refGapChanged(); break;
        case 2: _t->refBounceChanged(); break;
        case 3: _t->bounceFactorChanged(); break;
        case 4: _t->refWrapChanged(); break;
        case 5: _t->runningChanged(); break;
        case 6: _t->viewportPosChanged(); break;
        case 7: _t->pendingWorkspaceIndexChanged(); break;
        case 8: _t->finished(); break;
        case 9: _t->slideRunning((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 10: _t->slideNormal((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1]))); break;
        case 11: _t->slide((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[2]))); break;
        case 12: _t->bounce((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<enum Direction>>(_a[2]))); break;
        case 13: _t->setRunning((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->startSlideAnimation(); break;
        case 15: _t->startGestureSlide((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 16: _t->startGestureSlide((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WorkspaceAnimationController::*)()>(_a, &WorkspaceAnimationController::refWidthChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkspaceAnimationController::*)()>(_a, &WorkspaceAnimationController::refGapChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkspaceAnimationController::*)()>(_a, &WorkspaceAnimationController::refBounceChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkspaceAnimationController::*)()>(_a, &WorkspaceAnimationController::bounceFactorChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkspaceAnimationController::*)()>(_a, &WorkspaceAnimationController::refWrapChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkspaceAnimationController::*)()>(_a, &WorkspaceAnimationController::runningChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkspaceAnimationController::*)()>(_a, &WorkspaceAnimationController::viewportPosChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkspaceAnimationController::*)()>(_a, &WorkspaceAnimationController::pendingWorkspaceIndexChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (WorkspaceAnimationController::*)()>(_a, &WorkspaceAnimationController::finished, 8))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<qreal*>(_v) = _t->refWidth(); break;
        case 1: *reinterpret_cast<qreal*>(_v) = _t->refGap(); break;
        case 2: *reinterpret_cast<qreal*>(_v) = _t->refBounce(); break;
        case 3: *reinterpret_cast<qreal*>(_v) = _t->bounceFactor(); break;
        case 4: *reinterpret_cast<qreal*>(_v) = _t->refWrap(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->running(); break;
        case 6: *reinterpret_cast<qreal*>(_v) = _t->viewportPos(); break;
        case 7: *reinterpret_cast<uint*>(_v) = _t->pendingWorkspaceIndex(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setRefWidth(*reinterpret_cast<qreal*>(_v)); break;
        case 1: _t->setRefGap(*reinterpret_cast<qreal*>(_v)); break;
        case 2: _t->setRefBounce(*reinterpret_cast<qreal*>(_v)); break;
        case 3: _t->setBounceFactor(*reinterpret_cast<qreal*>(_v)); break;
        case 6: _t->setViewportPos(*reinterpret_cast<qreal*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *WorkspaceAnimationController::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *WorkspaceAnimationController::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28WorkspaceAnimationControllerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int WorkspaceAnimationController::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 17;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void WorkspaceAnimationController::refWidthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void WorkspaceAnimationController::refGapChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void WorkspaceAnimationController::refBounceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void WorkspaceAnimationController::bounceFactorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void WorkspaceAnimationController::refWrapChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void WorkspaceAnimationController::runningChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void WorkspaceAnimationController::viewportPosChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void WorkspaceAnimationController::pendingWorkspaceIndexChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void WorkspaceAnimationController::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}
QT_WARNING_POP
