/****************************************************************************
** Meta object code from reading C++ file 'xsettings.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/xsettings/xsettings.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'xsettings.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9XSettingsE_t {};
} // unnamed namespace

template <> constexpr inline auto XSettings::qt_create_metaobjectdata<qt_meta_tag_ZN9XSettingsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "XSettings",
        "XSettingsKey",
        "Unknown",
        "Xft_DPI",
        "Xft_Antialias",
        "Xft_Hinting",
        "Xft_HintStyle",
        "Xft_RGBA",
        "Xft_LCDFilter",
        "Xcursor_Theme",
        "Xcursor_Size",
        "Xcursor_ThemeCore",
        "Gdk_WindowScalingFactor",
        "Gdk_UnscaledDPI",
        "Gtk_FontName",
        "Gtk_ThemeName",
        "Gtk_IconThemeName",
        "Gtk_CursorThemeName",
        "Gtk_CursorThemeSize",
        "Gtk_RecentFilesEnabled",
        "Gtk_ShowStatusShapes",
        "Gtk_ShowInputMethodMenu",
        "Gtk_TimeoutInitial",
        "Gtk_TimeoutRepeat",
        "Gtk_DecorationLayout",
        "Gtk_IMModule",
        "Gtk_ShellShowsDesktop",
        "Gtk_MenuImages",
        "Gtk_EnablePrimaryPaste",
        "Gtk_KeynavUseCaret",
        "Gtk_ShellShowsAppMenu",
        "Gtk_CanChangeAccels",
        "Gtk_DialogsUseHeader",
        "Gtk_ToolbarStyle",
        "Gtk_KeyThemeName",
        "Gtk_IMPreeditStyle",
        "Gtk_EnableAnimations",
        "Gtk_ToolbarIconSize",
        "Gtk_IMStatusStyle",
        "Gtk_RecentFilesMaxAge",
        "Gtk_Modules",
        "Gtk_AutoMnemonics",
        "Gtk_ColorScheme",
        "Gtk_MenuBarAccel",
        "Gtk_ColorPalette",
        "Gtk_OverlayScrolling",
        "Gtk_SessionBusId",
        "Gtk_ShowUnicodeMenu",
        "Gtk_CursorBlinkTimeout",
        "Gtk_ButtonImages",
        "Gtk_TitlebarRightClick",
        "Gtk_TitlebarDoubleClick",
        "Gtk_TitlebarMiddleClick",
        "Gtk_MonospaceFontName",
        "Gtk_ApplicationPreferDarkTheme",
        "Gtk_PrimaryButtonWarpsSlider",
        "Net_DndDragThreshold",
        "Net_CursorBlinkTime",
        "Net_ThemeName",
        "Net_DoubleClickTime",
        "Net_CursorBlink",
        "Net_FallbackIconTheme",
        "Net_EnableEventSounds",
        "Net_IconThemeName",
        "Net_SoundThemeName",
        "Net_EnableInputFeedbackSounds",
        "Net_PreferDarkTheme",
        "XSettingsType",
        "XSettingsTypeInteger",
        "XSettingsTypeString",
        "XSettingsTypeColor"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'XSettingsKey'
        QtMocHelpers::EnumData<enum XSettingsKey>(1, 1, QMC::EnumFlags{}).add({
            {    2, XSettingsKey::Unknown },
            {    3, XSettingsKey::Xft_DPI },
            {    4, XSettingsKey::Xft_Antialias },
            {    5, XSettingsKey::Xft_Hinting },
            {    6, XSettingsKey::Xft_HintStyle },
            {    7, XSettingsKey::Xft_RGBA },
            {    8, XSettingsKey::Xft_LCDFilter },
            {    9, XSettingsKey::Xcursor_Theme },
            {   10, XSettingsKey::Xcursor_Size },
            {   11, XSettingsKey::Xcursor_ThemeCore },
            {   12, XSettingsKey::Gdk_WindowScalingFactor },
            {   13, XSettingsKey::Gdk_UnscaledDPI },
            {   14, XSettingsKey::Gtk_FontName },
            {   15, XSettingsKey::Gtk_ThemeName },
            {   16, XSettingsKey::Gtk_IconThemeName },
            {   17, XSettingsKey::Gtk_CursorThemeName },
            {   18, XSettingsKey::Gtk_CursorThemeSize },
            {   19, XSettingsKey::Gtk_RecentFilesEnabled },
            {   20, XSettingsKey::Gtk_ShowStatusShapes },
            {   21, XSettingsKey::Gtk_ShowInputMethodMenu },
            {   22, XSettingsKey::Gtk_TimeoutInitial },
            {   23, XSettingsKey::Gtk_TimeoutRepeat },
            {   24, XSettingsKey::Gtk_DecorationLayout },
            {   25, XSettingsKey::Gtk_IMModule },
            {   26, XSettingsKey::Gtk_ShellShowsDesktop },
            {   27, XSettingsKey::Gtk_MenuImages },
            {   28, XSettingsKey::Gtk_EnablePrimaryPaste },
            {   29, XSettingsKey::Gtk_KeynavUseCaret },
            {   30, XSettingsKey::Gtk_ShellShowsAppMenu },
            {   31, XSettingsKey::Gtk_CanChangeAccels },
            {   32, XSettingsKey::Gtk_DialogsUseHeader },
            {   33, XSettingsKey::Gtk_ToolbarStyle },
            {   34, XSettingsKey::Gtk_KeyThemeName },
            {   35, XSettingsKey::Gtk_IMPreeditStyle },
            {   36, XSettingsKey::Gtk_EnableAnimations },
            {   37, XSettingsKey::Gtk_ToolbarIconSize },
            {   38, XSettingsKey::Gtk_IMStatusStyle },
            {   39, XSettingsKey::Gtk_RecentFilesMaxAge },
            {   40, XSettingsKey::Gtk_Modules },
            {   41, XSettingsKey::Gtk_AutoMnemonics },
            {   42, XSettingsKey::Gtk_ColorScheme },
            {   43, XSettingsKey::Gtk_MenuBarAccel },
            {   44, XSettingsKey::Gtk_ColorPalette },
            {   45, XSettingsKey::Gtk_OverlayScrolling },
            {   46, XSettingsKey::Gtk_SessionBusId },
            {   47, XSettingsKey::Gtk_ShowUnicodeMenu },
            {   48, XSettingsKey::Gtk_CursorBlinkTimeout },
            {   49, XSettingsKey::Gtk_ButtonImages },
            {   50, XSettingsKey::Gtk_TitlebarRightClick },
            {   51, XSettingsKey::Gtk_TitlebarDoubleClick },
            {   52, XSettingsKey::Gtk_TitlebarMiddleClick },
            {   53, XSettingsKey::Gtk_MonospaceFontName },
            {   54, XSettingsKey::Gtk_ApplicationPreferDarkTheme },
            {   55, XSettingsKey::Gtk_PrimaryButtonWarpsSlider },
            {   56, XSettingsKey::Net_DndDragThreshold },
            {   57, XSettingsKey::Net_CursorBlinkTime },
            {   58, XSettingsKey::Net_ThemeName },
            {   59, XSettingsKey::Net_DoubleClickTime },
            {   60, XSettingsKey::Net_CursorBlink },
            {   61, XSettingsKey::Net_FallbackIconTheme },
            {   62, XSettingsKey::Net_EnableEventSounds },
            {   63, XSettingsKey::Net_IconThemeName },
            {   64, XSettingsKey::Net_SoundThemeName },
            {   65, XSettingsKey::Net_EnableInputFeedbackSounds },
            {   66, XSettingsKey::Net_PreferDarkTheme },
        }),
        // enum 'XSettingsType'
        QtMocHelpers::EnumData<enum XSettingsType>(67, 67, QMC::EnumFlags{}).add({
            {   68, XSettingsType::XSettingsTypeInteger },
            {   69, XSettingsType::XSettingsTypeString },
            {   70, XSettingsType::XSettingsTypeColor },
        }),
    };
    return QtMocHelpers::metaObjectData<XSettings, qt_meta_tag_ZN9XSettingsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject XSettings::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractSettings::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9XSettingsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9XSettingsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9XSettingsE_t>.metaTypes,
    nullptr
} };

void XSettings::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<XSettings *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *XSettings::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *XSettings::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9XSettingsE_t>.strings))
        return static_cast<void*>(this);
    return AbstractSettings::qt_metacast(_clname);
}

int XSettings::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractSettings::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
