/****************************************************************************
** Meta object code from reading C++ file 'wquickcoordmapper_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../waylib/src/server/qtquick/private/wquickcoordmapper_p.h"
#include "wquickobserver.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wquickcoordmapper_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Waylib6Server23WQuickCoordMapperHelperE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WQuickCoordMapperHelper::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server23WQuickCoordMapperHelperE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WQuickCoordMapperHelper",
        "QML.Element",
        "anonymous",
        "get",
        "WQuickCoordMapper*",
        "",
        "WQuickObserver*",
        "target"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'get'
        QtMocHelpers::SlotData<WQuickCoordMapper *(WQuickObserver *)>(3, 5, QMC::AccessPublic, 0x80000000 | 4, {{
            { 0x80000000 | 6, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<WQuickCoordMapperHelper, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WQuickCoordMapperHelper::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server23WQuickCoordMapperHelperE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server23WQuickCoordMapperHelperE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server23WQuickCoordMapperHelperE_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WQuickCoordMapperHelper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WQuickCoordMapperHelper *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { WQuickCoordMapper* _r = _t->get((*reinterpret_cast<std::add_pointer_t<WQuickObserver*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<WQuickCoordMapper**>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

const QMetaObject *Waylib::Server::WQuickCoordMapperHelper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WQuickCoordMapperHelper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server23WQuickCoordMapperHelperE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Waylib::Server::WQuickCoordMapperHelper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN6Waylib6Server25WQuickCoordMapperAttachedE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WQuickCoordMapperAttached::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server25WQuickCoordMapperAttachedE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WQuickCoordMapperAttached",
        "QML.Element",
        "anonymous",
        "helperChanged",
        "",
        "helper",
        "Waylib::Server::WQuickCoordMapperHelper*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'helperChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'helper'
        QtMocHelpers::PropertyData<Waylib::Server::WQuickCoordMapperHelper*>(5, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 0),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<WQuickCoordMapperAttached, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WQuickCoordMapperAttached::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server25WQuickCoordMapperAttachedE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server25WQuickCoordMapperAttachedE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server25WQuickCoordMapperAttachedE_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WQuickCoordMapperAttached::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WQuickCoordMapperAttached *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->helperChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WQuickCoordMapperAttached::*)()>(_a, &WQuickCoordMapperAttached::helperChanged, 0))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Waylib::Server::WQuickCoordMapperHelper* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<Waylib::Server::WQuickCoordMapperHelper**>(_v) = _t->helper(); break;
        default: break;
        }
    }
}

const QMetaObject *Waylib::Server::WQuickCoordMapperAttached::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WQuickCoordMapperAttached::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server25WQuickCoordMapperAttachedE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Waylib::Server::WQuickCoordMapperAttached::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WQuickCoordMapperAttached::helperChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
namespace {
struct qt_meta_tag_ZN6Waylib6Server17WQuickCoordMapperE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WQuickCoordMapper::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server17WQuickCoordMapperE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WQuickCoordMapper",
        "QML.Attached",
        "WQuickCoordMapperAttached",
        "QML.Element",
        "CoordMapper",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Only using for attached property"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
    });
    return QtMocHelpers::metaObjectData<WQuickCoordMapper, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WQuickCoordMapper::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server17WQuickCoordMapperE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server17WQuickCoordMapperE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server17WQuickCoordMapperE_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WQuickCoordMapper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WQuickCoordMapper *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *Waylib::Server::WQuickCoordMapper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WQuickCoordMapper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server17WQuickCoordMapperE_t>.strings))
        return static_cast<void*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int Waylib::Server::WQuickCoordMapper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
