/****************************************************************************
** Meta object code from reading C++ file 'mpvvideoitem.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../wallpaper-factory/mpvvideoitem.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mpvvideoitem.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12MpvVideoItemE_t {};
} // unnamed namespace

template <> constexpr inline auto MpvVideoItem::qt_create_metaobjectdata<qt_meta_tag_ZN12MpvVideoItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MpvVideoItem",
        "QML.Element",
        "auto",
        "mediaTitleChanged",
        "",
        "currentUrlChanged",
        "positionChanged",
        "durationChanged",
        "pauseChanged",
        "volumeChanged",
        "sourceChanged",
        "loopFileChanged",
        "speedChanged",
        "muteChanged",
        "scaleModeChanged",
        "panScanChanged",
        "refreshIntervalChanged",
        "fileStarted",
        "fileLoaded",
        "endFile",
        "reason",
        "videoReconfig",
        "ready",
        "observeProperty",
        "QByteArrayView",
        "property",
        "mpv_format",
        "format",
        "uint64_t",
        "id",
        "setProperty",
        "QVariant",
        "value",
        "command",
        "params",
        "onPropertyChanged",
        "onAsyncReply",
        "data",
        "mpv_event",
        "event",
        "updatePlaybackSpeed",
        "loadFile",
        "file",
        "setPropertyBlocking",
        "setPropertyAsync",
        "getProperty",
        "getPropertyAsync",
        "commandBlocking",
        "commandAsync",
        "expandText",
        "text",
        "unobserveProperty",
        "source",
        "mediaTitle",
        "position",
        "duration",
        "formattedPosition",
        "formattedDuration",
        "pause",
        "volume",
        "loopFile",
        "speed",
        "mute",
        "scaleMode",
        "VideoScaleMode",
        "panScan",
        "refreshInterval",
        "AsyncIds",
        "None",
        "SetVolume",
        "GetVolume",
        "ExpandText",
        "Property",
        "MediaTitle",
        "Position",
        "Duration",
        "Pause",
        "Volume",
        "Mute",
        "LoopFile",
        "Speed",
        "VideoUnscaled",
        "PanScan",
        "Scaled",
        "Unscaled"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'mediaTitleChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'currentUrlChanged'
        QtMocHelpers::SignalData<void()>(5, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'positionChanged'
        QtMocHelpers::SignalData<void()>(6, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'durationChanged'
        QtMocHelpers::SignalData<void()>(7, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pauseChanged'
        QtMocHelpers::SignalData<void()>(8, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'volumeChanged'
        QtMocHelpers::SignalData<void()>(9, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sourceChanged'
        QtMocHelpers::SignalData<void()>(10, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'loopFileChanged'
        QtMocHelpers::SignalData<void()>(11, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'speedChanged'
        QtMocHelpers::SignalData<void()>(12, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'muteChanged'
        QtMocHelpers::SignalData<void()>(13, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'scaleModeChanged'
        QtMocHelpers::SignalData<void()>(14, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'panScanChanged'
        QtMocHelpers::SignalData<void()>(15, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'refreshIntervalChanged'
        QtMocHelpers::SignalData<void()>(16, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fileStarted'
        QtMocHelpers::SignalData<void()>(17, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fileLoaded'
        QtMocHelpers::SignalData<void()>(18, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'endFile'
        QtMocHelpers::SignalData<void(const QByteArray &)>(19, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 20 },
        }}),
        // Signal 'videoReconfig'
        QtMocHelpers::SignalData<void()>(21, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ready'
        QtMocHelpers::SignalData<void()>(22, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'observeProperty'
        QtMocHelpers::SignalData<void(const QByteArrayView &, mpv_format, uint64_t)>(23, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 }, { 0x80000000 | 26, 27 }, { 0x80000000 | 28, 29 },
        }}),
        // Signal 'observeProperty'
        QtMocHelpers::SignalData<void(const QByteArrayView &, mpv_format)>(23, 4, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 24, 25 }, { 0x80000000 | 26, 27 },
        }}),
        // Signal 'setProperty'
        QtMocHelpers::SignalData<void(const QByteArrayView &, const QVariant &)>(30, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 }, { 0x80000000 | 31, 32 },
        }}),
        // Signal 'command'
        QtMocHelpers::SignalData<void(const QStringList &)>(33, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 34 },
        }}),
        // Slot 'onPropertyChanged'
        QtMocHelpers::SlotData<void(const QByteArrayView &, const QVariant &)>(35, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 24, 25 }, { 0x80000000 | 31, 32 },
        }}),
        // Slot 'onAsyncReply'
        QtMocHelpers::SlotData<void(const QVariant &, mpv_event)>(36, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 31, 37 }, { 0x80000000 | 38, 39 },
        }}),
        // Slot 'updatePlaybackSpeed'
        QtMocHelpers::SlotData<void()>(40, 4, QMC::AccessPrivate, QMetaType::Void),
        // Method 'loadFile'
        QtMocHelpers::MethodData<void(const QString &)>(41, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 42 },
        }}),
        // Method 'setPropertyBlocking'
        QtMocHelpers::MethodData<int(const QByteArrayView &, const QVariant &)>(43, 4, QMC::AccessPublic, QMetaType::Int, {{
            { 0x80000000 | 24, 25 }, { 0x80000000 | 31, 32 },
        }}),
        // Method 'setPropertyAsync'
        QtMocHelpers::MethodData<void(const QByteArrayView &, const QVariant &, int)>(44, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 }, { 0x80000000 | 31, 32 }, { QMetaType::Int, 29 },
        }}),
        // Method 'setPropertyAsync'
        QtMocHelpers::MethodData<void(const QByteArrayView &, const QVariant &)>(44, 4, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 24, 25 }, { 0x80000000 | 31, 32 },
        }}),
        // Method 'getProperty'
        QtMocHelpers::MethodData<QVariant(const QByteArrayView &)>(45, 4, QMC::AccessPublic, 0x80000000 | 31, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Method 'getPropertyAsync'
        QtMocHelpers::MethodData<void(const QByteArrayView &, int)>(46, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 }, { QMetaType::Int, 29 },
        }}),
        // Method 'getPropertyAsync'
        QtMocHelpers::MethodData<void(const QByteArrayView &)>(46, 4, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Method 'commandBlocking'
        QtMocHelpers::MethodData<QVariant(const QVariant &)>(47, 4, QMC::AccessPublic, 0x80000000 | 31, {{
            { 0x80000000 | 31, 34 },
        }}),
        // Method 'commandAsync'
        QtMocHelpers::MethodData<void(const QStringList &, int)>(48, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 34 }, { QMetaType::Int, 29 },
        }}),
        // Method 'commandAsync'
        QtMocHelpers::MethodData<void(const QStringList &)>(48, 4, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QStringList, 34 },
        }}),
        // Method 'expandText'
        QtMocHelpers::MethodData<QVariant(const QString &)>(49, 4, QMC::AccessPublic, 0x80000000 | 31, {{
            { QMetaType::QString, 50 },
        }}),
        // Method 'unobserveProperty'
        QtMocHelpers::MethodData<int(uint64_t)>(51, 4, QMC::AccessPublic, QMetaType::Int, {{
            { 0x80000000 | 28, 29 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'source'
        QtMocHelpers::PropertyData<QString>(52, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'mediaTitle'
        QtMocHelpers::PropertyData<QString>(53, QMetaType::QString, QMC::DefaultPropertyFlags, 0),
        // property 'position'
        QtMocHelpers::PropertyData<double>(54, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'duration'
        QtMocHelpers::PropertyData<double>(55, QMetaType::Double, QMC::DefaultPropertyFlags, 3),
        // property 'formattedPosition'
        QtMocHelpers::PropertyData<QString>(56, QMetaType::QString, QMC::DefaultPropertyFlags, 2),
        // property 'formattedDuration'
        QtMocHelpers::PropertyData<QString>(57, QMetaType::QString, QMC::DefaultPropertyFlags, 3),
        // property 'pause'
        QtMocHelpers::PropertyData<bool>(58, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'volume'
        QtMocHelpers::PropertyData<int>(59, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'loopFile'
        QtMocHelpers::PropertyData<bool>(60, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'speed'
        QtMocHelpers::PropertyData<double>(61, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'mute'
        QtMocHelpers::PropertyData<bool>(62, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 9),
        // property 'scaleMode'
        QtMocHelpers::PropertyData<enum VideoScaleMode>(63, 0x80000000 | 64, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 10),
        // property 'panScan'
        QtMocHelpers::PropertyData<double>(65, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 11),
        // property 'refreshInterval'
        QtMocHelpers::PropertyData<int>(66, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 12),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'AsyncIds'
        QtMocHelpers::EnumData<enum AsyncIds>(67, 67, QMC::EnumIsScoped).add({
            {   68, AsyncIds::None },
            {   69, AsyncIds::SetVolume },
            {   70, AsyncIds::GetVolume },
            {   71, AsyncIds::ExpandText },
        }),
        // enum 'Property'
        QtMocHelpers::EnumData<enum Property>(72, 72, QMC::EnumFlags{}).add({
            {   73, Property::MediaTitle },
            {   74, Property::Position },
            {   75, Property::Duration },
            {   76, Property::Pause },
            {   77, Property::Volume },
            {   78, Property::Mute },
            {   79, Property::LoopFile },
            {   80, Property::Speed },
            {   81, Property::VideoUnscaled },
            {   82, Property::PanScan },
        }),
        // enum 'VideoScaleMode'
        QtMocHelpers::EnumData<enum VideoScaleMode>(64, 64, QMC::EnumFlags{}).add({
            {   83, VideoScaleMode::Scaled },
            {   84, VideoScaleMode::Unscaled },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<MpvVideoItem, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject MpvVideoItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickFramebufferObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12MpvVideoItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12MpvVideoItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12MpvVideoItemE_t>.metaTypes,
    nullptr
} };

void MpvVideoItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MpvVideoItem *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->mediaTitleChanged(); break;
        case 1: _t->currentUrlChanged(); break;
        case 2: _t->positionChanged(); break;
        case 3: _t->durationChanged(); break;
        case 4: _t->pauseChanged(); break;
        case 5: _t->volumeChanged(); break;
        case 6: _t->sourceChanged(); break;
        case 7: _t->loopFileChanged(); break;
        case 8: _t->speedChanged(); break;
        case 9: _t->muteChanged(); break;
        case 10: _t->scaleModeChanged(); break;
        case 11: _t->panScanChanged(); break;
        case 12: _t->refreshIntervalChanged(); break;
        case 13: _t->fileStarted(); break;
        case 14: _t->fileLoaded(); break;
        case 15: _t->endFile((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 16: _t->videoReconfig(); break;
        case 17: _t->ready(); break;
        case 18: _t->observeProperty((*reinterpret_cast<std::add_pointer_t<QByteArrayView>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<mpv_format>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<uint64_t>>(_a[3]))); break;
        case 19: _t->observeProperty((*reinterpret_cast<std::add_pointer_t<QByteArrayView>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<mpv_format>>(_a[2]))); break;
        case 20: _t->setProperty((*reinterpret_cast<std::add_pointer_t<QByteArrayView>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 21: _t->command((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 22: _t->onPropertyChanged((*reinterpret_cast<std::add_pointer_t<QByteArrayView>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 23: _t->onAsyncReply((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<mpv_event>>(_a[2]))); break;
        case 24: _t->updatePlaybackSpeed(); break;
        case 25: _t->loadFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 26: { int _r = _t->setPropertyBlocking((*reinterpret_cast<std::add_pointer_t<QByteArrayView>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 27: _t->setPropertyAsync((*reinterpret_cast<std::add_pointer_t<QByteArrayView>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 28: _t->setPropertyAsync((*reinterpret_cast<std::add_pointer_t<QByteArrayView>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 29: { QVariant _r = _t->getProperty((*reinterpret_cast<std::add_pointer_t<QByteArrayView>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariant*>(_a[0]) = std::move(_r); }  break;
        case 30: _t->getPropertyAsync((*reinterpret_cast<std::add_pointer_t<QByteArrayView>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 31: _t->getPropertyAsync((*reinterpret_cast<std::add_pointer_t<QByteArrayView>>(_a[1]))); break;
        case 32: { QVariant _r = _t->commandBlocking((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariant*>(_a[0]) = std::move(_r); }  break;
        case 33: _t->commandAsync((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 34: _t->commandAsync((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 35: { QVariant _r = _t->expandText((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariant*>(_a[0]) = std::move(_r); }  break;
        case 36: { int _r = _t->unobserveProperty((*reinterpret_cast<std::add_pointer_t<uint64_t>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MpvVideoItem::*)()>(_a, &MpvVideoItem::mediaTitleChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MpvVideoItem::*)()>(_a, &MpvVideoItem::currentUrlChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MpvVideoItem::*)()>(_a, &MpvVideoItem::positionChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (MpvVideoItem::*)()>(_a, &MpvVideoItem::durationChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (MpvVideoItem::*)()>(_a, &MpvVideoItem::pauseChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (MpvVideoItem::*)()>(_a, &MpvVideoItem::volumeChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (MpvVideoItem::*)()>(_a, &MpvVideoItem::sourceChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (MpvVideoItem::*)()>(_a, &MpvVideoItem::loopFileChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (MpvVideoItem::*)()>(_a, &MpvVideoItem::speedChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (MpvVideoItem::*)()>(_a, &MpvVideoItem::muteChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (MpvVideoItem::*)()>(_a, &MpvVideoItem::scaleModeChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (MpvVideoItem::*)()>(_a, &MpvVideoItem::panScanChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (MpvVideoItem::*)()>(_a, &MpvVideoItem::refreshIntervalChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (MpvVideoItem::*)()>(_a, &MpvVideoItem::fileStarted, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (MpvVideoItem::*)()>(_a, &MpvVideoItem::fileLoaded, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (MpvVideoItem::*)(const QByteArray & )>(_a, &MpvVideoItem::endFile, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (MpvVideoItem::*)()>(_a, &MpvVideoItem::videoReconfig, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (MpvVideoItem::*)()>(_a, &MpvVideoItem::ready, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (MpvVideoItem::*)(const QByteArrayView & , mpv_format , uint64_t )>(_a, &MpvVideoItem::observeProperty, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (MpvVideoItem::*)(const QByteArrayView & , const QVariant & )>(_a, &MpvVideoItem::setProperty, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (MpvVideoItem::*)(const QStringList & )>(_a, &MpvVideoItem::command, 21))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->source(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->mediaTitle(); break;
        case 2: *reinterpret_cast<double*>(_v) = _t->position(); break;
        case 3: *reinterpret_cast<double*>(_v) = _t->duration(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->formattedPosition(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->formattedDuration(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->pause(); break;
        case 7: *reinterpret_cast<int*>(_v) = _t->volume(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->loopFile(); break;
        case 9: *reinterpret_cast<double*>(_v) = _t->speed(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->mute(); break;
        case 11: *reinterpret_cast<enum VideoScaleMode*>(_v) = _t->scaleMode(); break;
        case 12: *reinterpret_cast<double*>(_v) = _t->panScan(); break;
        case 13: *reinterpret_cast<int*>(_v) = _t->refreshInterval(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSource(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setPosition(*reinterpret_cast<double*>(_v)); break;
        case 6: _t->setPause(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setVolume(*reinterpret_cast<int*>(_v)); break;
        case 8: _t->setLoopFile(*reinterpret_cast<bool*>(_v)); break;
        case 9: _t->setSpeed(*reinterpret_cast<double*>(_v)); break;
        case 10: _t->setMute(*reinterpret_cast<bool*>(_v)); break;
        case 11: _t->setScaleMode(*reinterpret_cast<enum VideoScaleMode*>(_v)); break;
        case 12: _t->setPanScan(*reinterpret_cast<double*>(_v)); break;
        case 13: _t->setRefreshInterval(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *MpvVideoItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MpvVideoItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12MpvVideoItemE_t>.strings))
        return static_cast<void*>(this);
    return QQuickFramebufferObject::qt_metacast(_clname);
}

int MpvVideoItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickFramebufferObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 37)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 37;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 37)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 37;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void MpvVideoItem::mediaTitleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void MpvVideoItem::currentUrlChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void MpvVideoItem::positionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void MpvVideoItem::durationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void MpvVideoItem::pauseChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void MpvVideoItem::volumeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void MpvVideoItem::sourceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void MpvVideoItem::loopFileChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void MpvVideoItem::speedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void MpvVideoItem::muteChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void MpvVideoItem::scaleModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void MpvVideoItem::panScanChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void MpvVideoItem::refreshIntervalChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void MpvVideoItem::fileStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void MpvVideoItem::fileLoaded()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void MpvVideoItem::endFile(const QByteArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void MpvVideoItem::videoReconfig()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void MpvVideoItem::ready()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void MpvVideoItem::observeProperty(const QByteArrayView & _t1, mpv_format _t2, uint64_t _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1, _t2, _t3);
}

// SIGNAL 20
void MpvVideoItem::setProperty(const QByteArrayView & _t1, const QVariant & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1, _t2);
}

// SIGNAL 21
void MpvVideoItem::command(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1);
}
QT_WARNING_POP
