// This file was generated by qtwaylandscanner
// source file is /usr/share/treeland-protocols/treeland-personalization-manager-v1.xml

#ifndef QT_WAYLAND_SERVER_TREELAND_PERSONALIZATION_MANAGER_V1
#define QT_WAYLAND_SERVER_TREELAND_PERSONALIZATION_MANAGER_V1

#include "wayland-server-core.h"
#include "wayland-treeland-personalization-manager-v1-server-protocol.h"
#include <QByteArray>
#include <QMultiMap>
#include <QString>

#include <unistd.h>

#ifndef WAYLAND_VERSION_CHECK
#define WAYLAND_VERSION_CHECK(major, minor, micro) \
    ((WAYLAND_VERSION_MAJOR > (major)) || \
    (WAYLAND_VERSION_MAJOR == (major) && WAYLAND_VERSION_MINOR > (minor)) || \
    (WAYLAND_VERSION_MAJOR == (major) && WAYLAND_VERSION_MINOR == (minor) && WAYLAND_VERSION_MICRO >= (micro)))
#endif

QT_BEGIN_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_GCC("-Wmissing-field-initializers")
QT_WARNING_DISABLE_CLANG("-Wmissing-field-initializers")
QT_WARNING_DISABLE_GCC("-Wreorder")
QT_WARNING_DISABLE_CLANG("-Wreorder")

namespace QtWaylandServer {
    class  treeland_personalization_manager_v1
    {
    public:
        treeland_personalization_manager_v1(struct ::wl_client *client, int id, int version);
        treeland_personalization_manager_v1(struct ::wl_display *display, int version);
        treeland_personalization_manager_v1(struct ::wl_resource *resource);
        treeland_personalization_manager_v1();

        virtual ~treeland_personalization_manager_v1();

        class Resource
        {
        public:
            Resource() : treeland_personalization_manager_v1_object(nullptr), handle(nullptr) {}
            virtual ~Resource() {}

            treeland_personalization_manager_v1 *treeland_personalization_manager_v1_object;
            treeland_personalization_manager_v1 *object() { return treeland_personalization_manager_v1_object; } 
            struct ::wl_resource *handle;

            struct ::wl_client *client() const { return wl_resource_get_client(handle); }
            int version() const { return wl_resource_get_version(handle); }

            static Resource *fromResource(struct ::wl_resource *resource);
        };

        void init(struct ::wl_client *client, int id, int version);
        void init(struct ::wl_display *display, int version);
        void init(struct ::wl_resource *resource);

        Resource *add(struct ::wl_client *client, int version);
        Resource *add(struct ::wl_client *client, int id, int version);
        Resource *add(struct wl_list *resource_list, struct ::wl_client *client, int id, int version);

        Resource *resource() { return m_resource; }
        const Resource *resource() const { return m_resource; }

        const QMultiMap<struct ::wl_client*, Resource*> &resourceMap() const { return m_resource_map; }

        bool isGlobalRemoved() const { return m_globalRemovedEvent; }
        void globalRemove();

        bool isGlobal() const { return m_global != nullptr; }
        bool isResource() const { return m_resource != nullptr; }

        static const struct ::wl_interface *interface();
        static QByteArrayView interfaceName() { return interface()->name; }
        static int interfaceVersion() { return interface()->version; }

    protected:
        struct ::wl_global *m_global;

    protected:
        virtual Resource *allocate();

        virtual void destroy_global();

        virtual void bind_resource(Resource *resource);
        virtual void destroy_resource(Resource *resource);

        virtual void get_window_context(Resource *resource, uint32_t id, struct ::wl_resource *surface);
        virtual void get_cursor_context(Resource *resource, uint32_t id);
        virtual void get_font_context(Resource *resource, uint32_t id);
        virtual void get_appearance_context(Resource *resource, uint32_t id);
        virtual void destroy(Resource *resource);

    private:
        static void bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id);
        static void destroy_func(struct ::wl_resource *client_resource);
        static void display_destroy_func(struct ::wl_listener *listener, void *data);
        static int deferred_destroy_global_func(void *data);

        Resource *bind(struct ::wl_client *client, uint32_t id, int version);
        Resource *bind(struct ::wl_resource *handle);

        static const struct ::treeland_personalization_manager_v1_interface m_treeland_personalization_manager_v1_interface;

        static void handle_get_window_context(
            ::wl_client *client,
            struct wl_resource *resource,
            uint32_t id,
            struct ::wl_resource *surface);
        static void handle_get_cursor_context(
            ::wl_client *client,
            struct wl_resource *resource,
            uint32_t id);
        static void handle_get_font_context(
            ::wl_client *client,
            struct wl_resource *resource,
            uint32_t id);
        static void handle_get_appearance_context(
            ::wl_client *client,
            struct wl_resource *resource,
            uint32_t id);
        static void handle_destroy(
            ::wl_client *client,
            struct wl_resource *resource);

        QMultiMap<struct ::wl_client*, Resource*> m_resource_map;
        Resource *m_resource;
        struct ::wl_display *m_display;
        struct wl_event_source *m_globalRemovedEvent;
        struct DisplayDestroyedListener : ::wl_listener {
            treeland_personalization_manager_v1 *parent;
        };
        DisplayDestroyedListener m_displayDestroyedListener;
    };

    class  treeland_personalization_cursor_context_v1
    {
    public:
        treeland_personalization_cursor_context_v1(struct ::wl_client *client, int id, int version);
        treeland_personalization_cursor_context_v1(struct ::wl_display *display, int version);
        treeland_personalization_cursor_context_v1(struct ::wl_resource *resource);
        treeland_personalization_cursor_context_v1();

        virtual ~treeland_personalization_cursor_context_v1();

        class Resource
        {
        public:
            Resource() : treeland_personalization_cursor_context_v1_object(nullptr), handle(nullptr) {}
            virtual ~Resource() {}

            treeland_personalization_cursor_context_v1 *treeland_personalization_cursor_context_v1_object;
            treeland_personalization_cursor_context_v1 *object() { return treeland_personalization_cursor_context_v1_object; } 
            struct ::wl_resource *handle;

            struct ::wl_client *client() const { return wl_resource_get_client(handle); }
            int version() const { return wl_resource_get_version(handle); }

            static Resource *fromResource(struct ::wl_resource *resource);
        };

        void init(struct ::wl_client *client, int id, int version);
        void init(struct ::wl_display *display, int version);
        void init(struct ::wl_resource *resource);

        Resource *add(struct ::wl_client *client, int version);
        Resource *add(struct ::wl_client *client, int id, int version);
        Resource *add(struct wl_list *resource_list, struct ::wl_client *client, int id, int version);

        Resource *resource() { return m_resource; }
        const Resource *resource() const { return m_resource; }

        const QMultiMap<struct ::wl_client*, Resource*> &resourceMap() const { return m_resource_map; }

        bool isGlobalRemoved() const { return m_globalRemovedEvent; }
        void globalRemove();

        bool isGlobal() const { return m_global != nullptr; }
        bool isResource() const { return m_resource != nullptr; }

        static const struct ::wl_interface *interface();
        static QByteArrayView interfaceName() { return interface()->name; }
        static int interfaceVersion() { return interface()->version; }


        void send_verfity(int32_t success);
        void send_verfity(struct ::wl_resource *resource, int32_t success);
        void send_theme(const QString &name);
        void send_theme(struct ::wl_resource *resource, const QString &name);
        void send_size(uint32_t size);
        void send_size(struct ::wl_resource *resource, uint32_t size);
    protected:
        struct ::wl_global *m_global;

    protected:
        virtual Resource *allocate();

        virtual void destroy_global();

        virtual void bind_resource(Resource *resource);
        virtual void destroy_resource(Resource *resource);

        virtual void set_theme(Resource *resource, const QString &name);
        virtual void get_theme(Resource *resource);
        virtual void set_size(Resource *resource, uint32_t size);
        virtual void get_size(Resource *resource);
        virtual void commit(Resource *resource);
        virtual void destroy(Resource *resource);

    private:
        static void bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id);
        static void destroy_func(struct ::wl_resource *client_resource);
        static void display_destroy_func(struct ::wl_listener *listener, void *data);
        static int deferred_destroy_global_func(void *data);

        Resource *bind(struct ::wl_client *client, uint32_t id, int version);
        Resource *bind(struct ::wl_resource *handle);

        static const struct ::treeland_personalization_cursor_context_v1_interface m_treeland_personalization_cursor_context_v1_interface;

        static void handle_set_theme(
            ::wl_client *client,
            struct wl_resource *resource,
            const char *name);
        static void handle_get_theme(
            ::wl_client *client,
            struct wl_resource *resource);
        static void handle_set_size(
            ::wl_client *client,
            struct wl_resource *resource,
            uint32_t size);
        static void handle_get_size(
            ::wl_client *client,
            struct wl_resource *resource);
        static void handle_commit(
            ::wl_client *client,
            struct wl_resource *resource);
        static void handle_destroy(
            ::wl_client *client,
            struct wl_resource *resource);

        QMultiMap<struct ::wl_client*, Resource*> m_resource_map;
        Resource *m_resource;
        struct ::wl_display *m_display;
        struct wl_event_source *m_globalRemovedEvent;
        struct DisplayDestroyedListener : ::wl_listener {
            treeland_personalization_cursor_context_v1 *parent;
        };
        DisplayDestroyedListener m_displayDestroyedListener;
    };

    class  treeland_personalization_window_context_v1
    {
    public:
        treeland_personalization_window_context_v1(struct ::wl_client *client, int id, int version);
        treeland_personalization_window_context_v1(struct ::wl_display *display, int version);
        treeland_personalization_window_context_v1(struct ::wl_resource *resource);
        treeland_personalization_window_context_v1();

        virtual ~treeland_personalization_window_context_v1();

        class Resource
        {
        public:
            Resource() : treeland_personalization_window_context_v1_object(nullptr), handle(nullptr) {}
            virtual ~Resource() {}

            treeland_personalization_window_context_v1 *treeland_personalization_window_context_v1_object;
            treeland_personalization_window_context_v1 *object() { return treeland_personalization_window_context_v1_object; } 
            struct ::wl_resource *handle;

            struct ::wl_client *client() const { return wl_resource_get_client(handle); }
            int version() const { return wl_resource_get_version(handle); }

            static Resource *fromResource(struct ::wl_resource *resource);
        };

        void init(struct ::wl_client *client, int id, int version);
        void init(struct ::wl_display *display, int version);
        void init(struct ::wl_resource *resource);

        Resource *add(struct ::wl_client *client, int version);
        Resource *add(struct ::wl_client *client, int id, int version);
        Resource *add(struct wl_list *resource_list, struct ::wl_client *client, int id, int version);

        Resource *resource() { return m_resource; }
        const Resource *resource() const { return m_resource; }

        const QMultiMap<struct ::wl_client*, Resource*> &resourceMap() const { return m_resource_map; }

        bool isGlobalRemoved() const { return m_globalRemovedEvent; }
        void globalRemove();

        bool isGlobal() const { return m_global != nullptr; }
        bool isResource() const { return m_resource != nullptr; }

        static const struct ::wl_interface *interface();
        static QByteArrayView interfaceName() { return interface()->name; }
        static int interfaceVersion() { return interface()->version; }


        enum blend_mode {
            blend_mode_transparent = 0, // Normal blend mode, just composite over background with alpha channel
            blend_mode_wallpaper = 1, // Composite window over wallpaper
            blend_mode_blur = 2, // Blur the content of the window background
        };

        enum enable_mode {
            enable_mode_enable = 0, // force enable titlebar
            enable_mode_disable = 1, // force disable titlebar
        };
    protected:
        struct ::wl_global *m_global;

    protected:
        virtual Resource *allocate();

        virtual void destroy_global();

        virtual void bind_resource(Resource *resource);
        virtual void destroy_resource(Resource *resource);

        virtual void set_blend_mode(Resource *resource, int32_t mode);
        virtual void set_round_corner_radius(Resource *resource, int32_t radius);
        virtual void set_shadow(Resource *resource, int32_t radius, int32_t offset_x, int32_t offset_y, int32_t r, int32_t g, int32_t b, int32_t a);
        virtual void set_border(Resource *resource, int32_t width, int32_t r, int32_t g, int32_t b, int32_t a);
        virtual void set_titlebar(Resource *resource, int32_t mode);
        virtual void destroy(Resource *resource);

    private:
        static void bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id);
        static void destroy_func(struct ::wl_resource *client_resource);
        static void display_destroy_func(struct ::wl_listener *listener, void *data);
        static int deferred_destroy_global_func(void *data);

        Resource *bind(struct ::wl_client *client, uint32_t id, int version);
        Resource *bind(struct ::wl_resource *handle);

        static const struct ::treeland_personalization_window_context_v1_interface m_treeland_personalization_window_context_v1_interface;

        static void handle_set_blend_mode(
            ::wl_client *client,
            struct wl_resource *resource,
            int32_t mode);
        static void handle_set_round_corner_radius(
            ::wl_client *client,
            struct wl_resource *resource,
            int32_t radius);
        static void handle_set_shadow(
            ::wl_client *client,
            struct wl_resource *resource,
            int32_t radius,
            int32_t offset_x,
            int32_t offset_y,
            int32_t r,
            int32_t g,
            int32_t b,
            int32_t a);
        static void handle_set_border(
            ::wl_client *client,
            struct wl_resource *resource,
            int32_t width,
            int32_t r,
            int32_t g,
            int32_t b,
            int32_t a);
        static void handle_set_titlebar(
            ::wl_client *client,
            struct wl_resource *resource,
            int32_t mode);
        static void handle_destroy(
            ::wl_client *client,
            struct wl_resource *resource);

        QMultiMap<struct ::wl_client*, Resource*> m_resource_map;
        Resource *m_resource;
        struct ::wl_display *m_display;
        struct wl_event_source *m_globalRemovedEvent;
        struct DisplayDestroyedListener : ::wl_listener {
            treeland_personalization_window_context_v1 *parent;
        };
        DisplayDestroyedListener m_displayDestroyedListener;
    };

    class  treeland_personalization_font_context_v1
    {
    public:
        treeland_personalization_font_context_v1(struct ::wl_client *client, int id, int version);
        treeland_personalization_font_context_v1(struct ::wl_display *display, int version);
        treeland_personalization_font_context_v1(struct ::wl_resource *resource);
        treeland_personalization_font_context_v1();

        virtual ~treeland_personalization_font_context_v1();

        class Resource
        {
        public:
            Resource() : treeland_personalization_font_context_v1_object(nullptr), handle(nullptr) {}
            virtual ~Resource() {}

            treeland_personalization_font_context_v1 *treeland_personalization_font_context_v1_object;
            treeland_personalization_font_context_v1 *object() { return treeland_personalization_font_context_v1_object; } 
            struct ::wl_resource *handle;

            struct ::wl_client *client() const { return wl_resource_get_client(handle); }
            int version() const { return wl_resource_get_version(handle); }

            static Resource *fromResource(struct ::wl_resource *resource);
        };

        void init(struct ::wl_client *client, int id, int version);
        void init(struct ::wl_display *display, int version);
        void init(struct ::wl_resource *resource);

        Resource *add(struct ::wl_client *client, int version);
        Resource *add(struct ::wl_client *client, int id, int version);
        Resource *add(struct wl_list *resource_list, struct ::wl_client *client, int id, int version);

        Resource *resource() { return m_resource; }
        const Resource *resource() const { return m_resource; }

        const QMultiMap<struct ::wl_client*, Resource*> &resourceMap() const { return m_resource_map; }

        bool isGlobalRemoved() const { return m_globalRemovedEvent; }
        void globalRemove();

        bool isGlobal() const { return m_global != nullptr; }
        bool isResource() const { return m_resource != nullptr; }

        static const struct ::wl_interface *interface();
        static QByteArrayView interfaceName() { return interface()->name; }
        static int interfaceVersion() { return interface()->version; }


        void send_font(const QString &font_name);
        void send_font(struct ::wl_resource *resource, const QString &font_name);
        void send_monospace_font(const QString &font_name);
        void send_monospace_font(struct ::wl_resource *resource, const QString &font_name);
        void send_font_size(uint32_t font_size);
        void send_font_size(struct ::wl_resource *resource, uint32_t font_size);
    protected:
        struct ::wl_global *m_global;

    protected:
        virtual Resource *allocate();

        virtual void destroy_global();

        virtual void bind_resource(Resource *resource);
        virtual void destroy_resource(Resource *resource);

        virtual void set_font_size(Resource *resource, uint32_t size);
        virtual void get_font_size(Resource *resource);
        virtual void set_font(Resource *resource, const QString &font_name);
        virtual void get_font(Resource *resource);
        virtual void set_monospace_font(Resource *resource, const QString &font_name);
        virtual void get_monospace_font(Resource *resource);
        virtual void destroy(Resource *resource);

    private:
        static void bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id);
        static void destroy_func(struct ::wl_resource *client_resource);
        static void display_destroy_func(struct ::wl_listener *listener, void *data);
        static int deferred_destroy_global_func(void *data);

        Resource *bind(struct ::wl_client *client, uint32_t id, int version);
        Resource *bind(struct ::wl_resource *handle);

        static const struct ::treeland_personalization_font_context_v1_interface m_treeland_personalization_font_context_v1_interface;

        static void handle_set_font_size(
            ::wl_client *client,
            struct wl_resource *resource,
            uint32_t size);
        static void handle_get_font_size(
            ::wl_client *client,
            struct wl_resource *resource);
        static void handle_set_font(
            ::wl_client *client,
            struct wl_resource *resource,
            const char *font_name);
        static void handle_get_font(
            ::wl_client *client,
            struct wl_resource *resource);
        static void handle_set_monospace_font(
            ::wl_client *client,
            struct wl_resource *resource,
            const char *font_name);
        static void handle_get_monospace_font(
            ::wl_client *client,
            struct wl_resource *resource);
        static void handle_destroy(
            ::wl_client *client,
            struct wl_resource *resource);

        QMultiMap<struct ::wl_client*, Resource*> m_resource_map;
        Resource *m_resource;
        struct ::wl_display *m_display;
        struct wl_event_source *m_globalRemovedEvent;
        struct DisplayDestroyedListener : ::wl_listener {
            treeland_personalization_font_context_v1 *parent;
        };
        DisplayDestroyedListener m_displayDestroyedListener;
    };

    class  treeland_personalization_appearance_context_v1
    {
    public:
        treeland_personalization_appearance_context_v1(struct ::wl_client *client, int id, int version);
        treeland_personalization_appearance_context_v1(struct ::wl_display *display, int version);
        treeland_personalization_appearance_context_v1(struct ::wl_resource *resource);
        treeland_personalization_appearance_context_v1();

        virtual ~treeland_personalization_appearance_context_v1();

        class Resource
        {
        public:
            Resource() : treeland_personalization_appearance_context_v1_object(nullptr), handle(nullptr) {}
            virtual ~Resource() {}

            treeland_personalization_appearance_context_v1 *treeland_personalization_appearance_context_v1_object;
            treeland_personalization_appearance_context_v1 *object() { return treeland_personalization_appearance_context_v1_object; } 
            struct ::wl_resource *handle;

            struct ::wl_client *client() const { return wl_resource_get_client(handle); }
            int version() const { return wl_resource_get_version(handle); }

            static Resource *fromResource(struct ::wl_resource *resource);
        };

        void init(struct ::wl_client *client, int id, int version);
        void init(struct ::wl_display *display, int version);
        void init(struct ::wl_resource *resource);

        Resource *add(struct ::wl_client *client, int version);
        Resource *add(struct ::wl_client *client, int id, int version);
        Resource *add(struct wl_list *resource_list, struct ::wl_client *client, int id, int version);

        Resource *resource() { return m_resource; }
        const Resource *resource() const { return m_resource; }

        const QMultiMap<struct ::wl_client*, Resource*> &resourceMap() const { return m_resource_map; }

        bool isGlobalRemoved() const { return m_globalRemovedEvent; }
        void globalRemove();

        bool isGlobal() const { return m_global != nullptr; }
        bool isResource() const { return m_resource != nullptr; }

        static const struct ::wl_interface *interface();
        static QByteArrayView interfaceName() { return interface()->name; }
        static int interfaceVersion() { return interface()->version; }


        enum theme_type {
            theme_type_auto = 1, // window auto theme
            theme_type_light = 2, // window light theme
            theme_type_dark = 4, // window dark theme
        };

        enum error {
            error_invalid_round_corner_radius = 0, // Wrong round corner radius
            error_invalid_icon_theme = 1, // Wrong icon theme
            error_invalid_active_color = 2, // Wrong active color
            error_invalid_window_opacity = 4, // Wrong window opacity
            error_invalid_window_theme_type = 8, // Wrong theme type
            error_invalid_window_titlebar_height = 16, // Wrong window titlebar height
        };

        void send_round_corner_radius(int32_t radius);
        void send_round_corner_radius(struct ::wl_resource *resource, int32_t radius);
        void send_icon_theme(const QString &theme_name);
        void send_icon_theme(struct ::wl_resource *resource, const QString &theme_name);
        void send_active_color(const QString &active_color);
        void send_active_color(struct ::wl_resource *resource, const QString &active_color);
        void send_window_opacity(uint32_t opacity);
        void send_window_opacity(struct ::wl_resource *resource, uint32_t opacity);
        void send_window_theme_type(uint32_t type);
        void send_window_theme_type(struct ::wl_resource *resource, uint32_t type);
        void send_window_titlebar_height(uint32_t height);
        void send_window_titlebar_height(struct ::wl_resource *resource, uint32_t height);
    protected:
        struct ::wl_global *m_global;

    protected:
        virtual Resource *allocate();

        virtual void destroy_global();

        virtual void bind_resource(Resource *resource);
        virtual void destroy_resource(Resource *resource);

        virtual void set_round_corner_radius(Resource *resource, int32_t radius);
        virtual void get_round_corner_radius(Resource *resource);
        virtual void set_icon_theme(Resource *resource, const QString &theme);
        virtual void get_icon_theme(Resource *resource);
        virtual void set_active_color(Resource *resource, const QString &color);
        virtual void get_active_color(Resource *resource);
        virtual void set_window_opacity(Resource *resource, uint32_t opacity);
        virtual void get_window_opacity(Resource *resource);
        virtual void set_window_theme_type(Resource *resource, uint32_t type);
        virtual void get_window_theme_type(Resource *resource);
        virtual void set_window_titlebar_height(Resource *resource, uint32_t height);
        virtual void get_window_titlebar_height(Resource *resource);
        virtual void destroy(Resource *resource);

    private:
        static void bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id);
        static void destroy_func(struct ::wl_resource *client_resource);
        static void display_destroy_func(struct ::wl_listener *listener, void *data);
        static int deferred_destroy_global_func(void *data);

        Resource *bind(struct ::wl_client *client, uint32_t id, int version);
        Resource *bind(struct ::wl_resource *handle);

        static const struct ::treeland_personalization_appearance_context_v1_interface m_treeland_personalization_appearance_context_v1_interface;

        static void handle_set_round_corner_radius(
            ::wl_client *client,
            struct wl_resource *resource,
            int32_t radius);
        static void handle_get_round_corner_radius(
            ::wl_client *client,
            struct wl_resource *resource);
        static void handle_set_icon_theme(
            ::wl_client *client,
            struct wl_resource *resource,
            const char *theme);
        static void handle_get_icon_theme(
            ::wl_client *client,
            struct wl_resource *resource);
        static void handle_set_active_color(
            ::wl_client *client,
            struct wl_resource *resource,
            const char *color);
        static void handle_get_active_color(
            ::wl_client *client,
            struct wl_resource *resource);
        static void handle_set_window_opacity(
            ::wl_client *client,
            struct wl_resource *resource,
            uint32_t opacity);
        static void handle_get_window_opacity(
            ::wl_client *client,
            struct wl_resource *resource);
        static void handle_set_window_theme_type(
            ::wl_client *client,
            struct wl_resource *resource,
            uint32_t type);
        static void handle_get_window_theme_type(
            ::wl_client *client,
            struct wl_resource *resource);
        static void handle_set_window_titlebar_height(
            ::wl_client *client,
            struct wl_resource *resource,
            uint32_t height);
        static void handle_get_window_titlebar_height(
            ::wl_client *client,
            struct wl_resource *resource);
        static void handle_destroy(
            ::wl_client *client,
            struct wl_resource *resource);

        QMultiMap<struct ::wl_client*, Resource*> m_resource_map;
        Resource *m_resource;
        struct ::wl_display *m_display;
        struct wl_event_source *m_globalRemovedEvent;
        struct DisplayDestroyedListener : ::wl_listener {
            treeland_personalization_appearance_context_v1 *parent;
        };
        DisplayDestroyedListener m_displayDestroyedListener;
    };
}

QT_WARNING_POP
QT_END_NAMESPACE

#endif
