/****************************************************************************
** Meta object code from reading C++ file 'woutput.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../waylib/src/server/kernel/woutput.h"
#include "wcursor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'woutput.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Waylib6Server7WOutputE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WOutput::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server7WOutputE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WOutput",
        "QML.Element",
        "WaylandOutput",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Can't create in qml",
        "enabledChanged",
        "",
        "positionChanged",
        "QPoint",
        "pos",
        "modeChanged",
        "transformedSizeChanged",
        "beforeDestroy",
        "effectiveSizeChanged",
        "orientationChanged",
        "scaleChanged",
        "forceSoftwareCursorChanged",
        "bufferCommitted",
        "cursorAdded",
        "Waylib::Server::WCursor*",
        "cursor",
        "cursorRemoved",
        "cursorListChanged",
        "enabled",
        "size",
        "QSize",
        "orientation",
        "Transform",
        "scale",
        "forceSoftwareCursor",
        "name",
        "Normal",
        "R90",
        "R180",
        "R270",
        "Flipped",
        "Flipped90",
        "Flipped180",
        "Flipped270"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'enabledChanged'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'positionChanged'
        QtMocHelpers::SignalData<void(const QPoint &)>(9, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'modeChanged'
        QtMocHelpers::SignalData<void()>(12, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'transformedSizeChanged'
        QtMocHelpers::SignalData<void()>(13, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'beforeDestroy'
        QtMocHelpers::SignalData<void()>(14, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'effectiveSizeChanged'
        QtMocHelpers::SignalData<void()>(15, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'orientationChanged'
        QtMocHelpers::SignalData<void()>(16, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'scaleChanged'
        QtMocHelpers::SignalData<void()>(17, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'forceSoftwareCursorChanged'
        QtMocHelpers::SignalData<void()>(18, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'bufferCommitted'
        QtMocHelpers::SignalData<void()>(19, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cursorAdded'
        QtMocHelpers::SignalData<void(Waylib::Server::WCursor *)>(20, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Signal 'cursorRemoved'
        QtMocHelpers::SignalData<void(Waylib::Server::WCursor *)>(23, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Signal 'cursorListChanged'
        QtMocHelpers::SignalData<void()>(24, 8, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'enabled'
        QtMocHelpers::PropertyData<bool>(25, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'size'
        QtMocHelpers::PropertyData<QSize>(26, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 5),
        // property 'orientation'
        QtMocHelpers::PropertyData<enum Transform>(28, 0x80000000 | 29, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 6),
        // property 'scale'
        QtMocHelpers::PropertyData<float>(30, QMetaType::Float, QMC::DefaultPropertyFlags, 7),
        // property 'forceSoftwareCursor'
        QtMocHelpers::PropertyData<bool>(31, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'name'
        QtMocHelpers::PropertyData<QString>(32, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Transform'
        QtMocHelpers::EnumData<enum Transform>(29, 29, QMC::EnumFlags{}).add({
            {   33, Transform::Normal },
            {   34, Transform::R90 },
            {   35, Transform::R180 },
            {   36, Transform::R270 },
            {   37, Transform::Flipped },
            {   38, Transform::Flipped90 },
            {   39, Transform::Flipped180 },
            {   40, Transform::Flipped270 },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<WOutput, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WOutput::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server7WOutputE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server7WOutputE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server7WOutputE_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WOutput::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WOutput *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->enabledChanged(); break;
        case 1: _t->positionChanged((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 2: _t->modeChanged(); break;
        case 3: _t->transformedSizeChanged(); break;
        case 4: _t->beforeDestroy(); break;
        case 5: _t->effectiveSizeChanged(); break;
        case 6: _t->orientationChanged(); break;
        case 7: _t->scaleChanged(); break;
        case 8: _t->forceSoftwareCursorChanged(); break;
        case 9: _t->bufferCommitted(); break;
        case 10: _t->cursorAdded((*reinterpret_cast<std::add_pointer_t<Waylib::Server::WCursor*>>(_a[1]))); break;
        case 11: _t->cursorRemoved((*reinterpret_cast<std::add_pointer_t<Waylib::Server::WCursor*>>(_a[1]))); break;
        case 12: _t->cursorListChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WOutput::*)()>(_a, &WOutput::enabledChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutput::*)(const QPoint & )>(_a, &WOutput::positionChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutput::*)()>(_a, &WOutput::modeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutput::*)()>(_a, &WOutput::transformedSizeChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutput::*)()>(_a, &WOutput::beforeDestroy, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutput::*)()>(_a, &WOutput::effectiveSizeChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutput::*)()>(_a, &WOutput::orientationChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutput::*)()>(_a, &WOutput::scaleChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutput::*)()>(_a, &WOutput::forceSoftwareCursorChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutput::*)()>(_a, &WOutput::bufferCommitted, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutput::*)(Waylib::Server::WCursor * )>(_a, &WOutput::cursorAdded, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutput::*)(Waylib::Server::WCursor * )>(_a, &WOutput::cursorRemoved, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutput::*)()>(_a, &WOutput::cursorListChanged, 12))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isEnabled(); break;
        case 1: *reinterpret_cast<QSize*>(_v) = _t->effectiveSize(); break;
        case 2: *reinterpret_cast<enum Transform*>(_v) = _t->orientation(); break;
        case 3: *reinterpret_cast<float*>(_v) = _t->scale(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->forceSoftwareCursor(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 4: _t->setForceSoftwareCursor(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Waylib::Server::WOutput::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WOutput::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server7WOutputE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "WObject"))
        return static_cast< WObject*>(this);
    return QObject::qt_metacast(_clname);
}

int Waylib::Server::WOutput::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WOutput::enabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Waylib::Server::WOutput::positionChanged(const QPoint & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Waylib::Server::WOutput::modeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Waylib::Server::WOutput::transformedSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Waylib::Server::WOutput::beforeDestroy()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Waylib::Server::WOutput::effectiveSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Waylib::Server::WOutput::orientationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void Waylib::Server::WOutput::scaleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void Waylib::Server::WOutput::forceSoftwareCursorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void Waylib::Server::WOutput::bufferCommitted()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void Waylib::Server::WOutput::cursorAdded(Waylib::Server::WCursor * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void Waylib::Server::WOutput::cursorRemoved(Waylib::Server::WCursor * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void Waylib::Server::WOutput::cursorListChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}
QT_WARNING_POP
