#include <QtQml/qqmlprivate.h>
#include <QtCore/qdir.h>
#include <QtCore/qurl.h>
#include <QtCore/qhash.h>
#include <QtCore/qstring.h>

namespace QmlCacheGeneratedCode {
namespace _qt_qml_Treeland_core_qml_PrimaryOutput_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_Treeland_core_qml_CopyOutput_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_Treeland_core_qml_TitleBar_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_Treeland_core_qml_Decoration_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_Treeland_core_qml_WindowMenu_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_Treeland_core_qml_TaskBar_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_Treeland_core_qml_SurfaceContent_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_Treeland_core_qml_XdgShadow_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_Treeland_core_qml_Border_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_Treeland_core_qml_OutputMenuBar_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_Treeland_core_qml_WorkspaceSwitcher_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_Treeland_core_qml_WorkspaceProxy_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_Treeland_core_qml_FpsDisplay_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_Treeland_core_qml_Animations_GeometryAnimation_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_Treeland_core_qml_Animations_NewAnimation_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_Treeland_core_qml_Animations_MinimizeAnimation_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_Treeland_core_qml_Animations_ShowDesktopAnimation_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_Treeland_core_qml_Animations_LaunchpadAnimation_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_Treeland_core_qml_Animations_LayerShellAnimation_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_Treeland_core_qml_Effects_Blur_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_Treeland_core_qml_Effects__0x2b_vulkan_Blur_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_Treeland_core_qml_Effects_GlassEffect_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_Treeland_core_qml_Effects_LaunchpadCover_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_Treeland_core_qml_TaskSwitcher_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_Treeland_core_qml_TaskWindowPreview_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_Treeland_core_qml_SwitchViewDelegate_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_Treeland_core_qml_SwitchViewHighlightDelegate_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_Treeland_core_qml_DockPreview_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_Treeland_core_qml_FadeBehavior_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_Treeland_core_qml_CaptureSelectorLayer_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_Treeland_core_qml_WindowPickerLayer_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_Treeland_core_qml_EdgeTilePreview_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_Treeland_core_qml_LockScreenFallback_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}
namespace _qt_qml_Treeland_core_qml_PrelaunchSplash_qml { 
    extern const unsigned char qmlData[];
    extern const QQmlPrivate::AOTCompiledFunction aotBuiltFunctions[];
    const QQmlPrivate::CachedQmlUnit unit = {
        reinterpret_cast<const QV4::CompiledData::Unit*>(&qmlData), &aotBuiltFunctions[0], nullptr
    };
}

}
namespace {
struct Registry {
    Registry();
    ~Registry();
    QHash<QString, const QQmlPrivate::CachedQmlUnit*> resourcePathToCachedUnit;
    static const QQmlPrivate::CachedQmlUnit *lookupCachedUnit(const QUrl &url);
};

Q_GLOBAL_STATIC(Registry, unitRegistry)


Registry::Registry() {
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/PrimaryOutput.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_PrimaryOutput_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/CopyOutput.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_CopyOutput_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/TitleBar.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_TitleBar_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/Decoration.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_Decoration_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/WindowMenu.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_WindowMenu_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/TaskBar.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_TaskBar_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/SurfaceContent.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_SurfaceContent_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/XdgShadow.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_XdgShadow_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/Border.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_Border_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/OutputMenuBar.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_OutputMenuBar_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/WorkspaceSwitcher.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_WorkspaceSwitcher_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/WorkspaceProxy.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_WorkspaceProxy_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/FpsDisplay.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_FpsDisplay_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/Animations/GeometryAnimation.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_Animations_GeometryAnimation_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/Animations/NewAnimation.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_Animations_NewAnimation_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/Animations/MinimizeAnimation.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_Animations_MinimizeAnimation_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/Animations/ShowDesktopAnimation.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_Animations_ShowDesktopAnimation_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/Animations/LaunchpadAnimation.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_Animations_LaunchpadAnimation_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/Animations/LayerShellAnimation.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_Animations_LayerShellAnimation_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/Effects/Blur.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_Effects_Blur_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/Effects/+vulkan/Blur.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_Effects__0x2b_vulkan_Blur_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/Effects/GlassEffect.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_Effects_GlassEffect_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/Effects/LaunchpadCover.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_Effects_LaunchpadCover_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/TaskSwitcher.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_TaskSwitcher_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/TaskWindowPreview.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_TaskWindowPreview_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/SwitchViewDelegate.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_SwitchViewDelegate_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/SwitchViewHighlightDelegate.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_SwitchViewHighlightDelegate_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/DockPreview.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_DockPreview_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/FadeBehavior.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_FadeBehavior_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/CaptureSelectorLayer.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_CaptureSelectorLayer_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/WindowPickerLayer.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_WindowPickerLayer_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/EdgeTilePreview.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_EdgeTilePreview_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/LockScreenFallback.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_LockScreenFallback_qml::unit);
    resourcePathToCachedUnit.insert(QStringLiteral("/qt/qml/Treeland/core/qml/PrelaunchSplash.qml"), &QmlCacheGeneratedCode::_qt_qml_Treeland_core_qml_PrelaunchSplash_qml::unit);
    QQmlPrivate::RegisterQmlUnitCacheHook registration;
    registration.structVersion = 0;
    registration.lookupCachedQmlUnit = &lookupCachedUnit;
    QQmlPrivate::qmlregister(QQmlPrivate::QmlUnitCacheHookRegistration, &registration);
}

Registry::~Registry() {
    QQmlPrivate::qmlunregister(QQmlPrivate::QmlUnitCacheHookRegistration, quintptr(&lookupCachedUnit));
}

const QQmlPrivate::CachedQmlUnit *Registry::lookupCachedUnit(const QUrl &url) {
    if (url.scheme() != QLatin1String("qrc"))
        return nullptr;
    QString resourcePath = QDir::cleanPath(url.path());
    if (resourcePath.isEmpty())
        return nullptr;
    if (!resourcePath.startsWith(QLatin1Char('/')))
        resourcePath.prepend(QLatin1Char('/'));
    return unitRegistry()->resourcePathToCachedUnit.value(resourcePath, nullptr);
}
}
int QT_MANGLE_NAMESPACE(qInitResources_qmlcache_libtreeland)() {
    ::unitRegistry();
    return 1;
}
Q_CONSTRUCTOR_FUNCTION(QT_MANGLE_NAMESPACE(qInitResources_qmlcache_libtreeland))
int QT_MANGLE_NAMESPACE(qCleanupResources_qmlcache_libtreeland)() {
    return 1;
}
