/****************************************************************************
** Meta object code from reading C++ file 'wseat.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../waylib/src/server/kernel/wseat.h"
#include "private/wserver_p.h"
#include <wsurface.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wseat.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Waylib6Server16WSeatEventFilterE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WSeatEventFilter::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server16WSeatEventFilterE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WSeatEventFilter",
        "QML.Element",
        "anonymous"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<WSeatEventFilter, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WSeatEventFilter::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server16WSeatEventFilterE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server16WSeatEventFilterE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server16WSeatEventFilterE_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WSeatEventFilter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WSeatEventFilter *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *Waylib::Server::WSeatEventFilter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WSeatEventFilter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server16WSeatEventFilterE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Waylib::Server::WSeatEventFilter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN6Waylib6Server5WSeatE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WSeat::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server5WSeatE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WSeat",
        "keyboardChanged",
        "",
        "keyboardFocusSurfaceChanged",
        "requestCursorShape",
        "Waylib::Server::WGlobal::CursorShape",
        "shape",
        "requestCursorSurface",
        "Waylib::Server::WSurface*",
        "surface",
        "QPoint",
        "hotspot",
        "requestDrag",
        "alwaysUpdateHoverTargetChanged",
        "keyboard",
        "WInputDevice*",
        "keyboardFocus",
        "WSurface*",
        "alwaysUpdateHoverTarget"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'keyboardChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'keyboardFocusSurfaceChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestCursorShape'
        QtMocHelpers::SignalData<void(Waylib::Server::WGlobal::CursorShape)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'requestCursorSurface'
        QtMocHelpers::SignalData<void(Waylib::Server::WSurface *, const QPoint &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 }, { 0x80000000 | 10, 11 },
        }}),
        // Signal 'requestDrag'
        QtMocHelpers::SignalData<void(Waylib::Server::WSurface *)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'alwaysUpdateHoverTargetChanged'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'keyboard'
        QtMocHelpers::PropertyData<WInputDevice*>(14, 0x80000000 | 15, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 0),
        // property 'keyboardFocus'
        QtMocHelpers::PropertyData<WSurface*>(16, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::Final, 1),
        // property 'alwaysUpdateHoverTarget'
        QtMocHelpers::PropertyData<bool>(18, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 5),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WSeat, qt_meta_tag_ZN6Waylib6Server5WSeatE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WSeat::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server5WSeatE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server5WSeatE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server5WSeatE_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WSeat::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WSeat *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->keyboardChanged(); break;
        case 1: _t->keyboardFocusSurfaceChanged(); break;
        case 2: _t->requestCursorShape((*reinterpret_cast<std::add_pointer_t<Waylib::Server::WGlobal::CursorShape>>(_a[1]))); break;
        case 3: _t->requestCursorSurface((*reinterpret_cast<std::add_pointer_t<Waylib::Server::WSurface*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2]))); break;
        case 4: _t->requestDrag((*reinterpret_cast<std::add_pointer_t<Waylib::Server::WSurface*>>(_a[1]))); break;
        case 5: _t->alwaysUpdateHoverTargetChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WSeat::*)()>(_a, &WSeat::keyboardChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSeat::*)()>(_a, &WSeat::keyboardFocusSurfaceChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSeat::*)(Waylib::Server::WGlobal::CursorShape )>(_a, &WSeat::requestCursorShape, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSeat::*)(Waylib::Server::WSurface * , const QPoint & )>(_a, &WSeat::requestCursorSurface, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSeat::*)(Waylib::Server::WSurface * )>(_a, &WSeat::requestDrag, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSeat::*)()>(_a, &WSeat::alwaysUpdateHoverTargetChanged, 5))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<WInputDevice**>(_v) = _t->keyboard(); break;
        case 1: *reinterpret_cast<WSurface**>(_v) = _t->keyboardFocusSurface(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->alwaysUpdateHoverTarget(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setKeyboard(*reinterpret_cast<WInputDevice**>(_v)); break;
        case 1: _t->setKeyboardFocusSurface(*reinterpret_cast<WSurface**>(_v)); break;
        case 2: _t->setAlwaysUpdateHoverTarget(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Waylib::Server::WSeat::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WSeat::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server5WSeatE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "WObject"))
        return static_cast< WObject*>(this);
    if (!strcmp(_clname, "WServerInterface"))
        return static_cast< WServerInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int Waylib::Server::WSeat::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WSeat::keyboardChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Waylib::Server::WSeat::keyboardFocusSurfaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Waylib::Server::WSeat::requestCursorShape(Waylib::Server::WGlobal::CursorShape _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Waylib::Server::WSeat::requestCursorSurface(Waylib::Server::WSurface * _t1, const QPoint & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void Waylib::Server::WSeat::requestDrag(Waylib::Server::WSurface * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void Waylib::Server::WSeat::alwaysUpdateHoverTargetChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
