/****************************************************************************
** Meta object code from reading C++ file 'wxwaylandsurface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../waylib/src/server/protocols/wxwaylandsurface.h"
#include "woutput.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wxwaylandsurface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Waylib6Server16WXWaylandSurfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WXWaylandSurface::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server16WXWaylandSurfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WXWaylandSurface",
        "QML.Element",
        "XWaylandSurface",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Only create in C++",
        "associated",
        "",
        "aboutToDissociate",
        "parentXWaylandSurfaceChanged",
        "childrenChanged",
        "isToplevelChanged",
        "hasChildChanged",
        "bypassManagerChanged",
        "geometryChanged",
        "windowTypesChanged",
        "decorationsFlagsChanged",
        "requestConfigure",
        "QRect",
        "geometry",
        "ConfigureFlags",
        "flags",
        "requestActivate",
        "checkNewSize",
        "QSize",
        "size",
        "QSize*",
        "clipedSize",
        "resize",
        "configure",
        "setMaximize",
        "on",
        "setMinimize",
        "setFullScreen",
        "setActivate",
        "close",
        "restack",
        "WXWaylandSurface*",
        "sibling",
        "StackMode",
        "mode",
        "parentXWaylandSurface",
        "children",
        "QList<WXWaylandSurface*>",
        "isToplevel",
        "hasChild",
        "bypassManager",
        "windowTypes",
        "WindowTypes",
        "decorationsFlags",
        "DecorationsFlags",
        "XCB_STACK_MODE_ABOVE",
        "XCB_STACK_MODE_BELOW",
        "XCB_STACK_MODE_TOP_IF",
        "XCB_STACK_MODE_BOTTOM_IF",
        "XCB_STACK_MODE_OPPOSITE",
        "ConfigureFlag",
        "XCB_CONFIG_WINDOW_X",
        "XCB_CONFIG_WINDOW_Y",
        "XCB_CONFIG_WINDOW_POSITION",
        "XCB_CONFIG_WINDOW_WIDTH",
        "XCB_CONFIG_WINDOW_HEIGHT",
        "XCB_CONFIG_WINDOW_SIZE",
        "WindowType",
        "NET_WM_WINDOW_TYPE_NORMAL",
        "NET_WM_WINDOW_TYPE_UTILITY",
        "NET_WM_WINDOW_TYPE_TOOLTIP",
        "NET_WM_WINDOW_TYPE_DND",
        "NET_WM_WINDOW_TYPE_DROPDOWN_MENU",
        "NET_WM_WINDOW_TYPE_POPUP_MENU",
        "NET_WM_WINDOW_TYPE_COMBO",
        "NET_WM_WINDOW_TYPE_MENU",
        "NET_WM_WINDOW_TYPE_NOTIFICATION",
        "NET_WM_WINDOW_TYPE_SPLASH",
        "NET_WM_WINDOW_TYPE_DIALOG",
        "DecorationsFlag",
        "DecorationsAll",
        "DecorationsNoBorder",
        "DecorationsNoTitle"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'associated'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'aboutToDissociate'
        QtMocHelpers::SignalData<void()>(9, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'parentXWaylandSurfaceChanged'
        QtMocHelpers::SignalData<void()>(10, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'childrenChanged'
        QtMocHelpers::SignalData<void()>(11, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'isToplevelChanged'
        QtMocHelpers::SignalData<void()>(12, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hasChildChanged'
        QtMocHelpers::SignalData<void()>(13, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'bypassManagerChanged'
        QtMocHelpers::SignalData<void()>(14, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'geometryChanged'
        QtMocHelpers::SignalData<void()>(15, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'windowTypesChanged'
        QtMocHelpers::SignalData<void()>(16, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'decorationsFlagsChanged'
        QtMocHelpers::SignalData<void()>(17, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestConfigure'
        QtMocHelpers::SignalData<void(QRect, ConfigureFlags)>(18, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 20 }, { 0x80000000 | 21, 22 },
        }}),
        // Signal 'requestActivate'
        QtMocHelpers::SignalData<void()>(23, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'checkNewSize'
        QtMocHelpers::SlotData<bool(const QSize &, QSize *)>(24, 8, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 25, 26 }, { 0x80000000 | 27, 28 },
        }}),
        // Slot 'checkNewSize'
        QtMocHelpers::SlotData<bool(const QSize &)>(24, 8, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Slot 'resize'
        QtMocHelpers::SlotData<void(const QSize &)>(29, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Slot 'configure'
        QtMocHelpers::SlotData<void(const QRect &)>(30, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Slot 'setMaximize'
        QtMocHelpers::SlotData<void(bool)>(31, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 32 },
        }}),
        // Slot 'setMinimize'
        QtMocHelpers::SlotData<void(bool)>(33, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 32 },
        }}),
        // Slot 'setFullScreen'
        QtMocHelpers::SlotData<void(bool)>(34, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 32 },
        }}),
        // Slot 'setActivate'
        QtMocHelpers::SlotData<void(bool)>(35, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 32 },
        }}),
        // Slot 'close'
        QtMocHelpers::SlotData<void()>(36, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'restack'
        QtMocHelpers::SlotData<void(WXWaylandSurface *, enum StackMode)>(37, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 38, 39 }, { 0x80000000 | 40, 41 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'parentXWaylandSurface'
        QtMocHelpers::PropertyData<WXWaylandSurface*>(42, 0x80000000 | 38, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 2),
        // property 'children'
        QtMocHelpers::PropertyData<QList<WXWaylandSurface*>>(43, 0x80000000 | 44, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 3),
        // property 'isToplevel'
        QtMocHelpers::PropertyData<bool>(45, QMetaType::Bool, QMC::DefaultPropertyFlags, 4),
        // property 'hasChild'
        QtMocHelpers::PropertyData<bool>(46, QMetaType::Bool, QMC::DefaultPropertyFlags, 5),
        // property 'bypassManager'
        QtMocHelpers::PropertyData<bool>(47, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 6),
        // property 'geometry'
        QtMocHelpers::PropertyData<QRect>(20, 0x80000000 | 19, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 7),
        // property 'windowTypes'
        QtMocHelpers::PropertyData<WindowTypes>(48, 0x80000000 | 49, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 8),
        // property 'decorationsFlags'
        QtMocHelpers::PropertyData<DecorationsFlags>(50, 0x80000000 | 51, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 9),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'StackMode'
        QtMocHelpers::EnumData<enum StackMode>(40, 40, QMC::EnumFlags{}).add({
            {   52, StackMode::XCB_STACK_MODE_ABOVE },
            {   53, StackMode::XCB_STACK_MODE_BELOW },
            {   54, StackMode::XCB_STACK_MODE_TOP_IF },
            {   55, StackMode::XCB_STACK_MODE_BOTTOM_IF },
            {   56, StackMode::XCB_STACK_MODE_OPPOSITE },
        }),
        // enum 'ConfigureFlag'
        QtMocHelpers::EnumData<enum ConfigureFlag>(57, 57, QMC::EnumFlags{}).add({
            {   58, ConfigureFlag::XCB_CONFIG_WINDOW_X },
            {   59, ConfigureFlag::XCB_CONFIG_WINDOW_Y },
            {   60, ConfigureFlag::XCB_CONFIG_WINDOW_POSITION },
            {   61, ConfigureFlag::XCB_CONFIG_WINDOW_WIDTH },
            {   62, ConfigureFlag::XCB_CONFIG_WINDOW_HEIGHT },
            {   63, ConfigureFlag::XCB_CONFIG_WINDOW_SIZE },
        }),
        // enum 'WindowType'
        QtMocHelpers::EnumData<enum WindowType>(64, 64, QMC::EnumFlags{}).add({
            {   65, WindowType::NET_WM_WINDOW_TYPE_NORMAL },
            {   66, WindowType::NET_WM_WINDOW_TYPE_UTILITY },
            {   67, WindowType::NET_WM_WINDOW_TYPE_TOOLTIP },
            {   68, WindowType::NET_WM_WINDOW_TYPE_DND },
            {   69, WindowType::NET_WM_WINDOW_TYPE_DROPDOWN_MENU },
            {   70, WindowType::NET_WM_WINDOW_TYPE_POPUP_MENU },
            {   71, WindowType::NET_WM_WINDOW_TYPE_COMBO },
            {   72, WindowType::NET_WM_WINDOW_TYPE_MENU },
            {   73, WindowType::NET_WM_WINDOW_TYPE_NOTIFICATION },
            {   74, WindowType::NET_WM_WINDOW_TYPE_SPLASH },
            {   75, WindowType::NET_WM_WINDOW_TYPE_DIALOG },
        }),
        // enum 'DecorationsFlag'
        QtMocHelpers::EnumData<enum DecorationsFlag>(76, 76, QMC::EnumFlags{}).add({
            {   77, DecorationsFlag::DecorationsAll },
            {   78, DecorationsFlag::DecorationsNoBorder },
            {   79, DecorationsFlag::DecorationsNoTitle },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<WXWaylandSurface, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WXWaylandSurface::staticMetaObject = { {
    QMetaObject::SuperData::link<WToplevelSurface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server16WXWaylandSurfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server16WXWaylandSurfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server16WXWaylandSurfaceE_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WXWaylandSurface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WXWaylandSurface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->associated(); break;
        case 1: _t->aboutToDissociate(); break;
        case 2: _t->parentXWaylandSurfaceChanged(); break;
        case 3: _t->childrenChanged(); break;
        case 4: _t->isToplevelChanged(); break;
        case 5: _t->hasChildChanged(); break;
        case 6: _t->bypassManagerChanged(); break;
        case 7: _t->geometryChanged(); break;
        case 8: _t->windowTypesChanged(); break;
        case 9: _t->decorationsFlagsChanged(); break;
        case 10: _t->requestConfigure((*reinterpret_cast<std::add_pointer_t<QRect>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ConfigureFlags>>(_a[2]))); break;
        case 11: _t->requestActivate(); break;
        case 12: { bool _r = _t->checkNewSize((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSize*>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 13: { bool _r = _t->checkNewSize((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 14: _t->resize((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1]))); break;
        case 15: _t->configure((*reinterpret_cast<std::add_pointer_t<QRect>>(_a[1]))); break;
        case 16: _t->setMaximize((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: _t->setMinimize((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: _t->setFullScreen((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 19: _t->setActivate((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 20: _t->close(); break;
        case 21: _t->restack((*reinterpret_cast<std::add_pointer_t<WXWaylandSurface*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<enum StackMode>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< WXWaylandSurface* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WXWaylandSurface::*)()>(_a, &WXWaylandSurface::associated, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WXWaylandSurface::*)()>(_a, &WXWaylandSurface::aboutToDissociate, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WXWaylandSurface::*)()>(_a, &WXWaylandSurface::parentXWaylandSurfaceChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WXWaylandSurface::*)()>(_a, &WXWaylandSurface::childrenChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (WXWaylandSurface::*)()>(_a, &WXWaylandSurface::isToplevelChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (WXWaylandSurface::*)()>(_a, &WXWaylandSurface::hasChildChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (WXWaylandSurface::*)()>(_a, &WXWaylandSurface::bypassManagerChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (WXWaylandSurface::*)()>(_a, &WXWaylandSurface::geometryChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (WXWaylandSurface::*)()>(_a, &WXWaylandSurface::windowTypesChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (WXWaylandSurface::*)()>(_a, &WXWaylandSurface::decorationsFlagsChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (WXWaylandSurface::*)(QRect , ConfigureFlags )>(_a, &WXWaylandSurface::requestConfigure, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (WXWaylandSurface::*)()>(_a, &WXWaylandSurface::requestActivate, 11))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<WXWaylandSurface*> >(); break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< WXWaylandSurface* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<WXWaylandSurface**>(_v) = _t->parentXWaylandSurface(); break;
        case 1: *reinterpret_cast<QList<WXWaylandSurface*>*>(_v) = _t->children(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->isToplevel(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->hasChild(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->isBypassManager(); break;
        case 5: *reinterpret_cast<QRect*>(_v) = _t->geometry(); break;
        case 6: *reinterpret_cast<WindowTypes*>(_v) = _t->windowTypes(); break;
        case 7: *reinterpret_cast<DecorationsFlags*>(_v) = _t->decorationsFlags(); break;
        default: break;
        }
    }
}

const QMetaObject *Waylib::Server::WXWaylandSurface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WXWaylandSurface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server16WXWaylandSurfaceE_t>.strings))
        return static_cast<void*>(this);
    return WToplevelSurface::qt_metacast(_clname);
}

int Waylib::Server::WXWaylandSurface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = WToplevelSurface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WXWaylandSurface::associated()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Waylib::Server::WXWaylandSurface::aboutToDissociate()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Waylib::Server::WXWaylandSurface::parentXWaylandSurfaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Waylib::Server::WXWaylandSurface::childrenChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Waylib::Server::WXWaylandSurface::isToplevelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Waylib::Server::WXWaylandSurface::hasChildChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Waylib::Server::WXWaylandSurface::bypassManagerChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void Waylib::Server::WXWaylandSurface::geometryChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void Waylib::Server::WXWaylandSurface::windowTypesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void Waylib::Server::WXWaylandSurface::decorationsFlagsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void Waylib::Server::WXWaylandSurface::requestConfigure(QRect _t1, ConfigureFlags _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1, _t2);
}

// SIGNAL 11
void Waylib::Server::WXWaylandSurface::requestActivate()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}
QT_WARNING_POP
