/****************************************************************************
** Meta object code from reading C++ file 'helper.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/seat/helper.h"
#include "private/wserver_p.h"
#include <wsurface.h>
#include "woutput.h"
#include "wcursor.h"
#include "wsurface.h"
#include "wxdgtoplevelsurface.h"
#include <QDBusObjectPath>
#include <woutput.h>
#include <wlayersurface.h>
#include <wtoplevelsurface.h>
#include <wxdgsurface.h>
#include "core/rootsurfacecontainer.h"
#include "modules/capture/capture.h"
#include "surface/surfacewrapper.h"
#include "workspace/workspace.h"
#include "treelandconfig.hpp"
#include "treelanduserconfig.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'helper.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6HelperE_t {};
} // unnamed namespace

template <> constexpr inline auto Helper::qt_create_metaobjectdata<qt_meta_tag_ZN6HelperE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Helper",
        "QML.Element",
        "auto",
        "QML.Singleton",
        "true",
        "configChanged",
        "",
        "primaryOutputChanged",
        "activatedSurfaceChanged",
        "animationSpeedChanged",
        "outputModeChanged",
        "currentModeChanged",
        "noAnimationChanged",
        "blockActivateSurfaceChanged",
        "requestQuit",
        "launchpadMappedChanged",
        "WOutput*",
        "output",
        "mapped",
        "showDesktopRequested",
        "showDesktopStateChanged",
        "startLockscreened",
        "showAnimation",
        "modifierKeyReleased",
        "QKeyEvent*",
        "event",
        "activateSurface",
        "SurfaceWrapper*",
        "wrapper",
        "Qt::FocusReason",
        "reason",
        "WSeat*",
        "seat",
        "forceActivateSurface",
        "requestKeyboardFocus",
        "fakePressSurfaceBottomRightToReszie",
        "surface",
        "surfaceBelongsToCurrentSession",
        "onShowDesktop",
        "deleteTaskSwitch",
        "onSessionNew",
        "sessionId",
        "QDBusObjectPath",
        "objectPath",
        "onSessionLock",
        "onSessionUnlock",
        "addOutput",
        "isLaunchpad",
        "WLayerSurface*",
        "setLaunchpadMapped",
        "showDesktop",
        "startLockscreen",
        "currentWorkspaceWallpaper",
        "currentLockScreenWallpaper",
        "rootSurfaceContainer",
        "RootSurfaceContainer*",
        "animationSpeed",
        "outputMode",
        "OutputMode",
        "activatedSurface",
        "workspace",
        "Workspace*",
        "config",
        "TreelandUserConfig*",
        "globalConfig",
        "TreelandConfig*",
        "blockActivateSurface",
        "noAnimation",
        "rootContainer",
        "Copy",
        "Extension",
        "CurrentMode",
        "Normal",
        "LockScreen",
        "WindowSwitch",
        "Multitaskview"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'configChanged'
        QtMocHelpers::SignalData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'primaryOutputChanged'
        QtMocHelpers::SignalData<void()>(7, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'activatedSurfaceChanged'
        QtMocHelpers::SignalData<void()>(8, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'animationSpeedChanged'
        QtMocHelpers::SignalData<void()>(9, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'outputModeChanged'
        QtMocHelpers::SignalData<void()>(10, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'currentModeChanged'
        QtMocHelpers::SignalData<void()>(11, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'noAnimationChanged'
        QtMocHelpers::SignalData<void()>(12, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'blockActivateSurfaceChanged'
        QtMocHelpers::SignalData<void()>(13, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestQuit'
        QtMocHelpers::SignalData<void()>(14, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'launchpadMappedChanged'
        QtMocHelpers::SignalData<void(WOutput *, bool)>(15, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 }, { QMetaType::Bool, 18 },
        }}),
        // Signal 'showDesktopRequested'
        QtMocHelpers::SignalData<void(WOutput *)>(19, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Signal 'showDesktopStateChanged'
        QtMocHelpers::SignalData<void()>(20, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'startLockscreened'
        QtMocHelpers::SignalData<void(WOutput *, bool)>(21, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 }, { QMetaType::Bool, 22 },
        }}),
        // Signal 'modifierKeyReleased'
        QtMocHelpers::SignalData<void(QKeyEvent *)>(23, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Slot 'activateSurface'
        QtMocHelpers::SlotData<void(SurfaceWrapper *, Qt::FocusReason, WSeat *)>(26, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 }, { 0x80000000 | 29, 30 }, { 0x80000000 | 31, 32 },
        }}),
        // Slot 'activateSurface'
        QtMocHelpers::SlotData<void(SurfaceWrapper *, Qt::FocusReason)>(26, 6, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 27, 28 }, { 0x80000000 | 29, 30 },
        }}),
        // Slot 'activateSurface'
        QtMocHelpers::SlotData<void(SurfaceWrapper *)>(26, 6, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Slot 'forceActivateSurface'
        QtMocHelpers::SlotData<void(SurfaceWrapper *, Qt::FocusReason, WSeat *)>(33, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 }, { 0x80000000 | 29, 30 }, { 0x80000000 | 31, 32 },
        }}),
        // Slot 'forceActivateSurface'
        QtMocHelpers::SlotData<void(SurfaceWrapper *, Qt::FocusReason)>(33, 6, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 27, 28 }, { 0x80000000 | 29, 30 },
        }}),
        // Slot 'forceActivateSurface'
        QtMocHelpers::SlotData<void(SurfaceWrapper *)>(33, 6, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Slot 'requestKeyboardFocus'
        QtMocHelpers::SlotData<void(SurfaceWrapper *, Qt::FocusReason, WSeat *)>(34, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 }, { 0x80000000 | 29, 30 }, { 0x80000000 | 31, 32 },
        }}),
        // Slot 'requestKeyboardFocus'
        QtMocHelpers::SlotData<void(SurfaceWrapper *, Qt::FocusReason)>(34, 6, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 27, 28 }, { 0x80000000 | 29, 30 },
        }}),
        // Slot 'requestKeyboardFocus'
        QtMocHelpers::SlotData<void(SurfaceWrapper *)>(34, 6, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Slot 'fakePressSurfaceBottomRightToReszie'
        QtMocHelpers::SlotData<void(SurfaceWrapper *)>(35, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 36 },
        }}),
        // Slot 'surfaceBelongsToCurrentSession'
        QtMocHelpers::SlotData<bool(SurfaceWrapper *)>(37, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Slot 'onShowDesktop'
        QtMocHelpers::SlotData<void()>(38, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'deleteTaskSwitch'
        QtMocHelpers::SlotData<void()>(39, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onSessionNew'
        QtMocHelpers::SlotData<void(const QString &, const QDBusObjectPath &)>(40, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 41 }, { 0x80000000 | 42, 43 },
        }}),
        // Slot 'onSessionLock'
        QtMocHelpers::SlotData<void()>(44, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onSessionUnlock'
        QtMocHelpers::SlotData<void()>(45, 6, QMC::AccessPrivate, QMetaType::Void),
        // Method 'addOutput'
        QtMocHelpers::MethodData<void()>(46, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'isLaunchpad'
        QtMocHelpers::MethodData<bool(WLayerSurface *) const>(47, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 48, 36 },
        }}),
        // Method 'setLaunchpadMapped'
        QtMocHelpers::MethodData<void(WOutput *, bool)>(49, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 }, { QMetaType::Bool, 18 },
        }}),
        // Method 'showDesktop'
        QtMocHelpers::MethodData<void(WOutput *)>(50, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Method 'startLockscreen'
        QtMocHelpers::MethodData<void(WOutput *, bool)>(51, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 }, { QMetaType::Bool, 22 },
        }}),
        // Method 'currentWorkspaceWallpaper'
        QtMocHelpers::MethodData<QString(WOutput *)>(52, 6, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Method 'currentLockScreenWallpaper'
        QtMocHelpers::MethodData<QString(WOutput *)>(53, 6, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 16, 17 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'rootSurfaceContainer'
        QtMocHelpers::PropertyData<RootSurfaceContainer*>(54, 0x80000000 | 55, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant | QMC::Final),
        // property 'animationSpeed'
        QtMocHelpers::PropertyData<float>(56, QMetaType::Float, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 3),
        // property 'outputMode'
        QtMocHelpers::PropertyData<enum OutputMode>(57, 0x80000000 | 58, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 4),
        // property 'activatedSurface'
        QtMocHelpers::PropertyData<SurfaceWrapper*>(59, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 2),
        // property 'workspace'
        QtMocHelpers::PropertyData<Workspace*>(60, 0x80000000 | 61, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant | QMC::Final),
        // property 'config'
        QtMocHelpers::PropertyData<TreelandUserConfig*>(62, 0x80000000 | 63, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 0),
        // property 'globalConfig'
        QtMocHelpers::PropertyData<TreelandConfig*>(64, 0x80000000 | 65, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant | QMC::Final),
        // property 'blockActivateSurface'
        QtMocHelpers::PropertyData<bool>(66, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 7),
        // property 'noAnimation'
        QtMocHelpers::PropertyData<bool>(67, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 6),
        // property 'rootContainer'
        QtMocHelpers::PropertyData<RootSurfaceContainer*>(68, 0x80000000 | 55, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant | QMC::Final),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'OutputMode'
        QtMocHelpers::EnumData<enum OutputMode>(58, 58, QMC::EnumIsScoped).add({
            {   69, OutputMode::Copy },
            {   70, OutputMode::Extension },
        }),
        // enum 'CurrentMode'
        QtMocHelpers::EnumData<enum CurrentMode>(71, 71, QMC::EnumIsScoped).add({
            {   72, CurrentMode::Normal },
            {   73, CurrentMode::LockScreen },
            {   74, CurrentMode::WindowSwitch },
            {   75, CurrentMode::Multitaskview },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<Helper, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Helper::staticMetaObject = { {
    QMetaObject::SuperData::link<WSeatEventFilter::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6HelperE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6HelperE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6HelperE_t>.metaTypes,
    nullptr
} };

void Helper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Helper *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->configChanged(); break;
        case 1: _t->primaryOutputChanged(); break;
        case 2: _t->activatedSurfaceChanged(); break;
        case 3: _t->animationSpeedChanged(); break;
        case 4: _t->outputModeChanged(); break;
        case 5: _t->currentModeChanged(); break;
        case 6: _t->noAnimationChanged(); break;
        case 7: _t->blockActivateSurfaceChanged(); break;
        case 8: _t->requestQuit(); break;
        case 9: _t->launchpadMappedChanged((*reinterpret_cast<std::add_pointer_t<WOutput*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 10: _t->showDesktopRequested((*reinterpret_cast<std::add_pointer_t<WOutput*>>(_a[1]))); break;
        case 11: _t->showDesktopStateChanged(); break;
        case 12: _t->startLockscreened((*reinterpret_cast<std::add_pointer_t<WOutput*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 13: _t->modifierKeyReleased((*reinterpret_cast<std::add_pointer_t<QKeyEvent*>>(_a[1]))); break;
        case 14: _t->activateSurface((*reinterpret_cast<std::add_pointer_t<SurfaceWrapper*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::FocusReason>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<WSeat*>>(_a[3]))); break;
        case 15: _t->activateSurface((*reinterpret_cast<std::add_pointer_t<SurfaceWrapper*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::FocusReason>>(_a[2]))); break;
        case 16: _t->activateSurface((*reinterpret_cast<std::add_pointer_t<SurfaceWrapper*>>(_a[1]))); break;
        case 17: _t->forceActivateSurface((*reinterpret_cast<std::add_pointer_t<SurfaceWrapper*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::FocusReason>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<WSeat*>>(_a[3]))); break;
        case 18: _t->forceActivateSurface((*reinterpret_cast<std::add_pointer_t<SurfaceWrapper*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::FocusReason>>(_a[2]))); break;
        case 19: _t->forceActivateSurface((*reinterpret_cast<std::add_pointer_t<SurfaceWrapper*>>(_a[1]))); break;
        case 20: _t->requestKeyboardFocus((*reinterpret_cast<std::add_pointer_t<SurfaceWrapper*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::FocusReason>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<WSeat*>>(_a[3]))); break;
        case 21: _t->requestKeyboardFocus((*reinterpret_cast<std::add_pointer_t<SurfaceWrapper*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Qt::FocusReason>>(_a[2]))); break;
        case 22: _t->requestKeyboardFocus((*reinterpret_cast<std::add_pointer_t<SurfaceWrapper*>>(_a[1]))); break;
        case 23: _t->fakePressSurfaceBottomRightToReszie((*reinterpret_cast<std::add_pointer_t<SurfaceWrapper*>>(_a[1]))); break;
        case 24: { bool _r = _t->surfaceBelongsToCurrentSession((*reinterpret_cast<std::add_pointer_t<SurfaceWrapper*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 25: _t->onShowDesktop(); break;
        case 26: _t->deleteTaskSwitch(); break;
        case 27: _t->onSessionNew((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[2]))); break;
        case 28: _t->onSessionLock(); break;
        case 29: _t->onSessionUnlock(); break;
        case 30: _t->addOutput(); break;
        case 31: { bool _r = _t->isLaunchpad((*reinterpret_cast<std::add_pointer_t<WLayerSurface*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 32: _t->setLaunchpadMapped((*reinterpret_cast<std::add_pointer_t<WOutput*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 33: _t->showDesktop((*reinterpret_cast<std::add_pointer_t<WOutput*>>(_a[1]))); break;
        case 34: _t->startLockscreen((*reinterpret_cast<std::add_pointer_t<WOutput*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 35: { QString _r = _t->currentWorkspaceWallpaper((*reinterpret_cast<std::add_pointer_t<WOutput*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 36: { QString _r = _t->currentLockScreenWallpaper((*reinterpret_cast<std::add_pointer_t<WOutput*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< WOutput* >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< WOutput* >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< WOutput* >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< WSeat* >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< WSeat* >(); break;
            }
            break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< WSeat* >(); break;
            }
            break;
        case 32:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< WOutput* >(); break;
            }
            break;
        case 33:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< WOutput* >(); break;
            }
            break;
        case 34:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< WOutput* >(); break;
            }
            break;
        case 35:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< WOutput* >(); break;
            }
            break;
        case 36:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< WOutput* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Helper::*)()>(_a, &Helper::configChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Helper::*)()>(_a, &Helper::primaryOutputChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Helper::*)()>(_a, &Helper::activatedSurfaceChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Helper::*)()>(_a, &Helper::animationSpeedChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Helper::*)()>(_a, &Helper::outputModeChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (Helper::*)()>(_a, &Helper::currentModeChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (Helper::*)()>(_a, &Helper::noAnimationChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (Helper::*)()>(_a, &Helper::blockActivateSurfaceChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (Helper::*)()>(_a, &Helper::requestQuit, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (Helper::*)(WOutput * , bool )>(_a, &Helper::launchpadMappedChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (Helper::*)(WOutput * )>(_a, &Helper::showDesktopRequested, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (Helper::*)()>(_a, &Helper::showDesktopStateChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (Helper::*)(WOutput * , bool )>(_a, &Helper::startLockscreened, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (Helper::*)(QKeyEvent * )>(_a, &Helper::modifierKeyReleased, 13))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<RootSurfaceContainer**>(_v) = _t->rootSurfaceContainer(); break;
        case 1: *reinterpret_cast<float*>(_v) = _t->animationSpeed(); break;
        case 2: *reinterpret_cast<enum OutputMode*>(_v) = _t->outputMode(); break;
        case 3: *reinterpret_cast<SurfaceWrapper**>(_v) = _t->activatedSurface(); break;
        case 4: *reinterpret_cast<Workspace**>(_v) = _t->workspace(); break;
        case 5: *reinterpret_cast<TreelandUserConfig**>(_v) = _t->config(); break;
        case 6: *reinterpret_cast<TreelandConfig**>(_v) = _t->globalConfig(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->blockActivateSurface(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->noAnimation(); break;
        case 9: *reinterpret_cast<RootSurfaceContainer**>(_v) = _t->rootContainer(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setAnimationSpeed(*reinterpret_cast<float*>(_v)); break;
        case 2: _t->setOutputMode(*reinterpret_cast<enum OutputMode*>(_v)); break;
        case 7: _t->setBlockActivateSurface(*reinterpret_cast<bool*>(_v)); break;
        case 8: _t->setNoAnimation(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Helper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Helper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6HelperE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Waylib::Server::WObject"))
        return static_cast< Waylib::Server::WObject*>(this);
    return WSeatEventFilter::qt_metacast(_clname);
}

int Helper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = WSeatEventFilter::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 37)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 37;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 37)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 37;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void Helper::configChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Helper::primaryOutputChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Helper::activatedSurfaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Helper::animationSpeedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Helper::outputModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Helper::currentModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Helper::noAnimationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void Helper::blockActivateSurfaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void Helper::requestQuit()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void Helper::launchpadMappedChanged(WOutput * _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2);
}

// SIGNAL 10
void Helper::showDesktopRequested(WOutput * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void Helper::showDesktopStateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void Helper::startLockscreened(WOutput * _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1, _t2);
}

// SIGNAL 13
void Helper::modifierKeyReleased(QKeyEvent * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}
QT_WARNING_POP
