/****************************************************************************
** Meta object code from reading C++ file 'appconfig.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../appconfig.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'appconfig.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9AppConfigE_t {};
} // unnamed namespace

template <> constexpr inline auto AppConfig::qt_create_metaobjectdata<qt_meta_tag_ZN9AppConfigE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AppConfig",
        "DConfigKeyList",
        "enablePrelaunchSplash;lastWindowHeight;lastWindowWidth;splashDarkP"
        "alette;splashLightPalette;splashThemeType",
        "DConfigFileName",
        "org.deepin.dde.treeland.app",
        "DConfigFileVersion",
        "1.0",
        "configInitializeFailed",
        "",
        "configInitializeSucceed",
        "Dtk::Core::DConfig*",
        "config",
        "valueChanged",
        "key",
        "QVariant",
        "value",
        "enablePrelaunchSplashChanged",
        "lastWindowHeightChanged",
        "lastWindowWidthChanged",
        "splashDarkPaletteChanged",
        "splashLightPaletteChanged",
        "splashThemeTypeChanged",
        "isInitializeSucceed",
        "isInitializeSucceeded",
        "isInitializeFailed",
        "isInitializing",
        "keyList",
        "isDefaultValue",
        "enablePrelaunchSplashIsDefaultValue",
        "lastWindowHeightIsDefaultValue",
        "lastWindowWidthIsDefaultValue",
        "splashDarkPaletteIsDefaultValue",
        "splashLightPaletteIsDefaultValue",
        "splashThemeTypeIsDefaultValue",
        "enablePrelaunchSplash",
        "lastWindowHeight",
        "lastWindowWidth",
        "splashDarkPalette",
        "splashLightPalette",
        "splashThemeType"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'configInitializeFailed'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'configInitializeSucceed'
        QtMocHelpers::SignalData<void(Dtk::Core::DConfig *)>(9, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'valueChanged'
        QtMocHelpers::SignalData<void(const QString &, const QVariant &)>(12, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 }, { 0x80000000 | 14, 15 },
        }}),
        // Signal 'enablePrelaunchSplashChanged'
        QtMocHelpers::SignalData<void()>(16, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'lastWindowHeightChanged'
        QtMocHelpers::SignalData<void()>(17, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'lastWindowWidthChanged'
        QtMocHelpers::SignalData<void()>(18, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'splashDarkPaletteChanged'
        QtMocHelpers::SignalData<void()>(19, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'splashLightPaletteChanged'
        QtMocHelpers::SignalData<void()>(20, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'splashThemeTypeChanged'
        QtMocHelpers::SignalData<void()>(21, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'config'
        QtMocHelpers::MethodData<Dtk::Core::DConfig *() const>(11, 8, QMC::AccessPublic, 0x80000000 | 10),
        // Method 'isInitializeSucceed'
        QtMocHelpers::MethodData<bool() const>(22, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'isInitializeSucceeded'
        QtMocHelpers::MethodData<bool() const>(23, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'isInitializeFailed'
        QtMocHelpers::MethodData<bool() const>(24, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'isInitializing'
        QtMocHelpers::MethodData<bool() const>(25, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'keyList'
        QtMocHelpers::MethodData<QStringList() const>(26, 8, QMC::AccessPublic, QMetaType::QStringList),
        // Method 'isDefaultValue'
        QtMocHelpers::MethodData<bool(const QString &) const>(27, 8, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 13 },
        }}),
        // Method 'enablePrelaunchSplashIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(28, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'lastWindowHeightIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(29, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'lastWindowWidthIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(30, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'splashDarkPaletteIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(31, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'splashLightPaletteIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(32, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'splashThemeTypeIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(33, 8, QMC::AccessPublic, QMetaType::Bool),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'enablePrelaunchSplash'
        QtMocHelpers::PropertyData<bool>(34, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 3),
        // property 'lastWindowHeight'
        QtMocHelpers::PropertyData<qlonglong>(35, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 4),
        // property 'lastWindowWidth'
        QtMocHelpers::PropertyData<qlonglong>(36, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 5),
        // property 'splashDarkPalette'
        QtMocHelpers::PropertyData<QString>(37, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 6),
        // property 'splashLightPalette'
        QtMocHelpers::PropertyData<QString>(38, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 7),
        // property 'splashThemeType'
        QtMocHelpers::PropertyData<qlonglong>(39, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 8),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<AppConfig, qt_meta_tag_ZN9AppConfigE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject AppConfig::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9AppConfigE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9AppConfigE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9AppConfigE_t>.metaTypes,
    nullptr
} };

void AppConfig::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AppConfig *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->configInitializeFailed(); break;
        case 1: _t->configInitializeSucceed((*reinterpret_cast<std::add_pointer_t<Dtk::Core::DConfig*>>(_a[1]))); break;
        case 2: _t->valueChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 3: _t->enablePrelaunchSplashChanged(); break;
        case 4: _t->lastWindowHeightChanged(); break;
        case 5: _t->lastWindowWidthChanged(); break;
        case 6: _t->splashDarkPaletteChanged(); break;
        case 7: _t->splashLightPaletteChanged(); break;
        case 8: _t->splashThemeTypeChanged(); break;
        case 9: { Dtk::Core::DConfig* _r = _t->config();
            if (_a[0]) *reinterpret_cast<Dtk::Core::DConfig**>(_a[0]) = std::move(_r); }  break;
        case 10: { bool _r = _t->isInitializeSucceed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 11: { bool _r = _t->isInitializeSucceeded();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 12: { bool _r = _t->isInitializeFailed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 13: { bool _r = _t->isInitializing();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 14: { QStringList _r = _t->keyList();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 15: { bool _r = _t->isDefaultValue((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 16: { bool _r = _t->enablePrelaunchSplashIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 17: { bool _r = _t->lastWindowHeightIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 18: { bool _r = _t->lastWindowWidthIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 19: { bool _r = _t->splashDarkPaletteIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 20: { bool _r = _t->splashLightPaletteIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 21: { bool _r = _t->splashThemeTypeIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Dtk::Core::DConfig* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (AppConfig::*)()>(_a, &AppConfig::configInitializeFailed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppConfig::*)(Dtk::Core::DConfig * )>(_a, &AppConfig::configInitializeSucceed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppConfig::*)(const QString & , const QVariant & )>(_a, &AppConfig::valueChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppConfig::*)()>(_a, &AppConfig::enablePrelaunchSplashChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppConfig::*)()>(_a, &AppConfig::lastWindowHeightChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppConfig::*)()>(_a, &AppConfig::lastWindowWidthChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppConfig::*)()>(_a, &AppConfig::splashDarkPaletteChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppConfig::*)()>(_a, &AppConfig::splashLightPaletteChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (AppConfig::*)()>(_a, &AppConfig::splashThemeTypeChanged, 8))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->enablePrelaunchSplash(); break;
        case 1: *reinterpret_cast<qlonglong*>(_v) = _t->lastWindowHeight(); break;
        case 2: *reinterpret_cast<qlonglong*>(_v) = _t->lastWindowWidth(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->splashDarkPalette(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->splashLightPalette(); break;
        case 5: *reinterpret_cast<qlonglong*>(_v) = _t->splashThemeType(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setEnablePrelaunchSplash(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setLastWindowHeight(*reinterpret_cast<qlonglong*>(_v)); break;
        case 2: _t->setLastWindowWidth(*reinterpret_cast<qlonglong*>(_v)); break;
        case 3: _t->setSplashDarkPalette(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setSplashLightPalette(*reinterpret_cast<QString*>(_v)); break;
        case 5: _t->setSplashThemeType(*reinterpret_cast<qlonglong*>(_v)); break;
        default: break;
        }
    }
    if (_c == QMetaObject::ResetProperty) {
        switch (_id) {
        case 0: _t->resetEnablePrelaunchSplash(); break;
        case 1: _t->resetLastWindowHeight(); break;
        case 2: _t->resetLastWindowWidth(); break;
        case 3: _t->resetSplashDarkPalette(); break;
        case 4: _t->resetSplashLightPalette(); break;
        case 5: _t->resetSplashThemeType(); break;
        default: break;
        }
    }
}

const QMetaObject *AppConfig::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AppConfig::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9AppConfigE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int AppConfig::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void AppConfig::configInitializeFailed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void AppConfig::configInitializeSucceed(Dtk::Core::DConfig * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void AppConfig::valueChanged(const QString & _t1, const QVariant & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void AppConfig::enablePrelaunchSplashChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void AppConfig::lastWindowHeightChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void AppConfig::lastWindowWidthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void AppConfig::splashDarkPaletteChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void AppConfig::splashLightPaletteChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void AppConfig::splashThemeTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}
QT_WARNING_POP
