/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<capture.h>)
#  include <capture.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
Q_QMLTYPE_EXPORT void qml_register_types_Treeland_Capture()
{
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    qmlRegisterTypesAndRevisions<CaptureSourceSelector>("Treeland.Capture", 1);
    qmlRegisterEnum<CaptureSourceSelector::SelectionMode>("CaptureSourceSelector::SelectionMode");
    QT_WARNING_POP
    qmlRegisterModule("Treeland.Capture", 1, 0);
}

static const QQmlModuleRegistration treelandCaptureRegistration("Treeland.Capture", qml_register_types_Treeland_Capture);
