/****************************************************************************
** Meta object code from reading C++ file 'wquickcursor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../waylib/src/server/qtquick/wquickcursor.h"
#include "private/wserver_p.h"
#include "woutput.h"
#include "woutput.h"
#include "wquickoutputlayout.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wquickcursor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Waylib6Server20WQuickCursorAttachedE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WQuickCursorAttached::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server20WQuickCursorAttachedE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WQuickCursorAttached",
        "QML.Element",
        "anonymous",
        "shapeChanged",
        "",
        "shape",
        "WGlobal::CursorShape"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'shapeChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'shape'
        QtMocHelpers::PropertyData<WGlobal::CursorShape>(5, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 0),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<WQuickCursorAttached, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN6Waylib6Server20WQuickCursorAttachedE[] = {
    QMetaObject::SuperData::link<Waylib::Server::WGlobal::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject Waylib::Server::WQuickCursorAttached::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server20WQuickCursorAttachedE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server20WQuickCursorAttachedE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN6Waylib6Server20WQuickCursorAttachedE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server20WQuickCursorAttachedE_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WQuickCursorAttached::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WQuickCursorAttached *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->shapeChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WQuickCursorAttached::*)()>(_a, &WQuickCursorAttached::shapeChanged, 0))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<WGlobal::CursorShape*>(_v) = _t->shape(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setShape(*reinterpret_cast<WGlobal::CursorShape*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Waylib::Server::WQuickCursorAttached::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WQuickCursorAttached::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server20WQuickCursorAttachedE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Waylib::Server::WQuickCursorAttached::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WQuickCursorAttached::shapeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
namespace {
struct qt_meta_tag_ZN6Waylib6Server12WQuickCursorE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WQuickCursor::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server12WQuickCursorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WQuickCursor",
        "QML.Element",
        "Cursor",
        "QML.Attached",
        "WQuickCursorAttached",
        "validChanged",
        "",
        "cursorChanged",
        "themeNameChanged",
        "sourceSizeChanged",
        "currentRenderWindowChanged",
        "hotSpotChanged",
        "outputChanged",
        "invalidateSceneGraph",
        "onImageChanged",
        "updateCursor",
        "updateImplicitSize",
        "updateXCursorManager",
        "valid",
        "cursor",
        "Waylib::Server::WCursor*",
        "output",
        "Waylib::Server::WOutput*",
        "themeName",
        "sourceSize",
        "QSize",
        "hotSpot",
        "QPointF"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'validChanged'
        QtMocHelpers::SignalData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cursorChanged'
        QtMocHelpers::SignalData<void()>(7, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'themeNameChanged'
        QtMocHelpers::SignalData<void()>(8, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sourceSizeChanged'
        QtMocHelpers::SignalData<void()>(9, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'currentRenderWindowChanged'
        QtMocHelpers::SignalData<void()>(10, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hotSpotChanged'
        QtMocHelpers::SignalData<void()>(11, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'outputChanged'
        QtMocHelpers::SignalData<void()>(12, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'invalidateSceneGraph'
        QtMocHelpers::SlotData<void()>(13, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onImageChanged'
        QtMocHelpers::SlotData<void()>(14, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateCursor'
        QtMocHelpers::SlotData<void()>(15, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateImplicitSize'
        QtMocHelpers::SlotData<void()>(16, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateXCursorManager'
        QtMocHelpers::SlotData<void()>(17, 6, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'valid'
        QtMocHelpers::PropertyData<bool>(18, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 0),
        // property 'cursor'
        QtMocHelpers::PropertyData<Waylib::Server::WCursor*>(19, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'output'
        QtMocHelpers::PropertyData<Waylib::Server::WOutput*>(21, 0x80000000 | 22, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 6),
        // property 'themeName'
        QtMocHelpers::PropertyData<QString>(23, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'sourceSize'
        QtMocHelpers::PropertyData<QSize>(24, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
        // property 'hotSpot'
        QtMocHelpers::PropertyData<QPointF>(26, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 5),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<WQuickCursor, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WQuickCursor::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server12WQuickCursorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server12WQuickCursorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server12WQuickCursorE_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WQuickCursor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WQuickCursor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->validChanged(); break;
        case 1: _t->cursorChanged(); break;
        case 2: _t->themeNameChanged(); break;
        case 3: _t->sourceSizeChanged(); break;
        case 4: _t->currentRenderWindowChanged(); break;
        case 5: _t->hotSpotChanged(); break;
        case 6: _t->outputChanged(); break;
        case 7: _t->invalidateSceneGraph(); break;
        case 8: _t->d_func()->onImageChanged(); break;
        case 9: _t->d_func()->updateCursor(); break;
        case 10: _t->d_func()->updateImplicitSize(); break;
        case 11: _t->d_func()->updateXCursorManager(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WQuickCursor::*)()>(_a, &WQuickCursor::validChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WQuickCursor::*)()>(_a, &WQuickCursor::cursorChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WQuickCursor::*)()>(_a, &WQuickCursor::themeNameChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WQuickCursor::*)()>(_a, &WQuickCursor::sourceSizeChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (WQuickCursor::*)()>(_a, &WQuickCursor::currentRenderWindowChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (WQuickCursor::*)()>(_a, &WQuickCursor::hotSpotChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (WQuickCursor::*)()>(_a, &WQuickCursor::outputChanged, 6))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Waylib::Server::WCursor* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->valid(); break;
        case 1: *reinterpret_cast<Waylib::Server::WCursor**>(_v) = _t->cursor(); break;
        case 2: *reinterpret_cast<Waylib::Server::WOutput**>(_v) = _t->output(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->themeName(); break;
        case 4: *reinterpret_cast<QSize*>(_v) = _t->sourceSize(); break;
        case 5: *reinterpret_cast<QPointF*>(_v) = _t->hotSpot(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setCursor(*reinterpret_cast<Waylib::Server::WCursor**>(_v)); break;
        case 2: _t->setOutput(*reinterpret_cast<Waylib::Server::WOutput**>(_v)); break;
        case 3: _t->setThemeName(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setSourceSize(*reinterpret_cast<QSize*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Waylib::Server::WQuickCursor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WQuickCursor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server12WQuickCursorE_t>.strings))
        return static_cast<void*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int Waylib::Server::WQuickCursor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WQuickCursor::validChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Waylib::Server::WQuickCursor::cursorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Waylib::Server::WQuickCursor::themeNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Waylib::Server::WQuickCursor::sourceSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Waylib::Server::WQuickCursor::currentRenderWindowChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Waylib::Server::WQuickCursor::hotSpotChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Waylib::Server::WQuickCursor::outputChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
