/****************************************************************************
** Meta object code from reading C++ file 'wsessionlocksurface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../waylib/src/server/protocols/wsessionlocksurface.h"
#include "wcursor.h"
#include "woutput.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wsessionlocksurface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Waylib6Server19WSessionLockSurfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WSessionLockSurface::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server19WSessionLockSurfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WSessionLockSurface",
        "QML.Element",
        "WSessionLockSurface",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Only create in C++",
        "surfaceChanged",
        "",
        "checkNewSize",
        "QSize",
        "size",
        "QSize*",
        "clippedSize",
        "configureSize",
        "uint32_t",
        "newSize",
        "surface",
        "WSurface*",
        "output",
        "WOutput*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'surfaceChanged'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'checkNewSize'
        QtMocHelpers::SlotData<bool(const QSize &, QSize *)>(9, 8, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 10, 11 }, { 0x80000000 | 12, 13 },
        }}),
        // Method 'configureSize'
        QtMocHelpers::MethodData<uint32_t(const QSize &)>(14, 8, QMC::AccessPublic, 0x80000000 | 15, {{
            { 0x80000000 | 10, 16 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'surface'
        QtMocHelpers::PropertyData<WSurface*>(17, 0x80000000 | 18, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 'output'
        QtMocHelpers::PropertyData<WOutput*>(19, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<WSessionLockSurface, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WSessionLockSurface::staticMetaObject = { {
    QMetaObject::SuperData::link<WToplevelSurface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server19WSessionLockSurfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server19WSessionLockSurfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server19WSessionLockSurfaceE_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WSessionLockSurface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WSessionLockSurface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->surfaceChanged(); break;
        case 1: { bool _r = _t->checkNewSize((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSize*>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 2: { uint32_t _r = _t->configureSize((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1])));
            if (_a[0]) *reinterpret_cast<uint32_t*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WSessionLockSurface::*)()>(_a, &WSessionLockSurface::surfaceChanged, 0))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< WOutput* >(); break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< WSurface* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<WSurface**>(_v) = _t->surface(); break;
        case 1: *reinterpret_cast<WOutput**>(_v) = _t->output(); break;
        default: break;
        }
    }
}

const QMetaObject *Waylib::Server::WSessionLockSurface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WSessionLockSurface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server19WSessionLockSurfaceE_t>.strings))
        return static_cast<void*>(this);
    return WToplevelSurface::qt_metacast(_clname);
}

int Waylib::Server::WSessionLockSurface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = WToplevelSurface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WSessionLockSurface::surfaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
