/****************************************************************************
** Meta object code from reading C++ file 'wxwayland.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../waylib/src/server/protocols/wxwayland.h"
#include "private/wserver_p.h"
#include "wxwaylandsurface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wxwayland.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Waylib6Server9WXWaylandE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WXWayland::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server9WXWaylandE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WXWayland",
        "ready",
        "",
        "surfaceAdded",
        "WXWaylandSurface*",
        "surface",
        "surfaceRemoved",
        "toplevelAdded",
        "toplevelRemoved",
        "windowPropertyChanged",
        "xcb_atom_t",
        "atom",
        "XcbAtom",
        "AtomNone",
        "_NET_WM_WINDOW_TYPE_NORMAL",
        "_NET_WM_WINDOW_TYPE_UTILITY",
        "_NET_WM_WINDOW_TYPE_TOOLTIP",
        "_NET_WM_WINDOW_TYPE_DND",
        "_NET_WM_WINDOW_TYPE_DROPDOWN_MENU",
        "_NET_WM_WINDOW_TYPE_POPUP_MENU",
        "_NET_WM_WINDOW_TYPE_COMBO",
        "_NET_WM_WINDOW_TYPE_MENU",
        "_NET_WM_WINDOW_TYPE_NOTIFICATION",
        "_NET_WM_WINDOW_TYPE_SPLASH",
        "_NET_WM_WINDOW_TYPE_DIALOG",
        "_NET_SUPPORTED",
        "AtomCount"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'ready'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'surfaceAdded'
        QtMocHelpers::SignalData<void(WXWaylandSurface *)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Signal 'surfaceRemoved'
        QtMocHelpers::SignalData<void(WXWaylandSurface *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Signal 'toplevelAdded'
        QtMocHelpers::SignalData<void(WXWaylandSurface *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Signal 'toplevelRemoved'
        QtMocHelpers::SignalData<void(WXWaylandSurface *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Signal 'windowPropertyChanged'
        QtMocHelpers::SignalData<void(WXWaylandSurface *, xcb_atom_t)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { 0x80000000 | 10, 11 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'XcbAtom'
        QtMocHelpers::EnumData<enum XcbAtom>(12, 12, QMC::EnumFlags{}).add({
            {   13, XcbAtom::AtomNone },
            {   14, XcbAtom::_NET_WM_WINDOW_TYPE_NORMAL },
            {   15, XcbAtom::_NET_WM_WINDOW_TYPE_UTILITY },
            {   16, XcbAtom::_NET_WM_WINDOW_TYPE_TOOLTIP },
            {   17, XcbAtom::_NET_WM_WINDOW_TYPE_DND },
            {   18, XcbAtom::_NET_WM_WINDOW_TYPE_DROPDOWN_MENU },
            {   19, XcbAtom::_NET_WM_WINDOW_TYPE_POPUP_MENU },
            {   20, XcbAtom::_NET_WM_WINDOW_TYPE_COMBO },
            {   21, XcbAtom::_NET_WM_WINDOW_TYPE_MENU },
            {   22, XcbAtom::_NET_WM_WINDOW_TYPE_NOTIFICATION },
            {   23, XcbAtom::_NET_WM_WINDOW_TYPE_SPLASH },
            {   24, XcbAtom::_NET_WM_WINDOW_TYPE_DIALOG },
            {   25, XcbAtom::_NET_SUPPORTED },
            {   26, XcbAtom::AtomCount },
        }),
    };
    return QtMocHelpers::metaObjectData<WXWayland, qt_meta_tag_ZN6Waylib6Server9WXWaylandE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WXWayland::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server9WXWaylandE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server9WXWaylandE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server9WXWaylandE_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WXWayland::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WXWayland *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->ready(); break;
        case 1: _t->surfaceAdded((*reinterpret_cast<std::add_pointer_t<WXWaylandSurface*>>(_a[1]))); break;
        case 2: _t->surfaceRemoved((*reinterpret_cast<std::add_pointer_t<WXWaylandSurface*>>(_a[1]))); break;
        case 3: _t->toplevelAdded((*reinterpret_cast<std::add_pointer_t<WXWaylandSurface*>>(_a[1]))); break;
        case 4: _t->toplevelRemoved((*reinterpret_cast<std::add_pointer_t<WXWaylandSurface*>>(_a[1]))); break;
        case 5: _t->windowPropertyChanged((*reinterpret_cast<std::add_pointer_t<WXWaylandSurface*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<xcb_atom_t>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WXWayland::*)()>(_a, &WXWayland::ready, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WXWayland::*)(WXWaylandSurface * )>(_a, &WXWayland::surfaceAdded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WXWayland::*)(WXWaylandSurface * )>(_a, &WXWayland::surfaceRemoved, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WXWayland::*)(WXWaylandSurface * )>(_a, &WXWayland::toplevelAdded, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (WXWayland::*)(WXWaylandSurface * )>(_a, &WXWayland::toplevelRemoved, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (WXWayland::*)(WXWaylandSurface * , xcb_atom_t )>(_a, &WXWayland::windowPropertyChanged, 5))
            return;
    }
}

const QMetaObject *Waylib::Server::WXWayland::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WXWayland::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server9WXWaylandE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "WWaylandResource"))
        return static_cast< WWaylandResource*>(this);
    if (!strcmp(_clname, "WServerInterface"))
        return static_cast< WServerInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int Waylib::Server::WXWayland::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WXWayland::ready()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Waylib::Server::WXWayland::surfaceAdded(WXWaylandSurface * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Waylib::Server::WXWayland::surfaceRemoved(WXWaylandSurface * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Waylib::Server::WXWayland::toplevelAdded(WXWaylandSurface * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void Waylib::Server::WXWayland::toplevelRemoved(WXWaylandSurface * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void Waylib::Server::WXWayland::windowPropertyChanged(WXWaylandSurface * _t1, xcb_atom_t _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}
QT_WARNING_POP
