/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.freedesktop.DisplayManager.Seat.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef DISPLAYMANAGERSEAT_H
#define DISPLAYMANAGERSEAT_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>

/*
 * Proxy class for interface org.freedesktop.DisplayManager.Seat
 */
class DisplaySeat: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.freedesktop.DisplayManager.Seat"; }

public:
    DisplaySeat(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~DisplaySeat();

    Q_PROPERTY(bool CanSwitch READ canSwitch)
    inline bool canSwitch() const
    { return qvariant_cast< bool >(property("CanSwitch")); }

    Q_PROPERTY(bool HasGuestAccount READ hasGuestAccount)
    inline bool hasGuestAccount() const
    { return qvariant_cast< bool >(property("HasGuestAccount")); }

    Q_PROPERTY(QList<QDBusObjectPath> Sessions READ sessions)
    inline QList<QDBusObjectPath> sessions() const
    { return qvariant_cast< QList<QDBusObjectPath> >(property("Sessions")); }

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> Lock()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("Lock"), argumentList);
    }

    inline QDBusPendingReply<> SwitchToGreeter()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("SwitchToGreeter"), argumentList);
    }

    inline QDBusPendingReply<> SwitchToGuest(const QString &session_name)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(session_name);
        return asyncCallWithArgumentList(QStringLiteral("SwitchToGuest"), argumentList);
    }

    inline QDBusPendingReply<> SwitchToUser(const QString &username, const QString &session_name)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(username) << QVariant::fromValue(session_name);
        return asyncCallWithArgumentList(QStringLiteral("SwitchToUser"), argumentList);
    }

Q_SIGNALS: // SIGNALS
};

#endif
