/****************************************************************************
** Meta object code from reading C++ file 'rep_treelandwindowtree_source.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "rep_treelandwindowtree_source.h"
#include <QtRemoteObjects/qremoteobjectnode.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'rep_treelandwindowtree_source.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10WindowInfoE_t {};
} // unnamed namespace

template <> constexpr inline auto WindowInfo::qt_create_metaobjectdata<qt_meta_tag_ZN10WindowInfoE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "WindowInfo",
        "appId",
        "title",
        "output",
        "container",
        "workspace",
        "layer",
        "z",
        "type",
        "state",
        "visible",
        "active",
        "geometry",
        "QRectF",
        "titlebarGeometry",
        "boundingRect",
        "iconGeometry",
        "position",
        "QPointF"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'appId'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'title'
        QtMocHelpers::PropertyData<QString>(2, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'output'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'container'
        QtMocHelpers::PropertyData<QString>(4, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'workspace'
        QtMocHelpers::PropertyData<int>(5, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'layer'
        QtMocHelpers::PropertyData<int>(6, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'z'
        QtMocHelpers::PropertyData<int>(7, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'type'
        QtMocHelpers::PropertyData<int>(8, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'state'
        QtMocHelpers::PropertyData<int>(9, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'visible'
        QtMocHelpers::PropertyData<bool>(10, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'active'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'geometry'
        QtMocHelpers::PropertyData<QRectF>(12, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'titlebarGeometry'
        QtMocHelpers::PropertyData<QRectF>(14, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'boundingRect'
        QtMocHelpers::PropertyData<QRectF>(15, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'iconGeometry'
        QtMocHelpers::PropertyData<QRectF>(16, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'position'
        QtMocHelpers::PropertyData<QPointF>(17, 0x80000000 | 18, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WindowInfo, qt_meta_tag_ZN10WindowInfoE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject WindowInfo::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10WindowInfoE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10WindowInfoE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10WindowInfoE_t>.metaTypes,
    nullptr
} };

void WindowInfo::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<WindowInfo *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->appId(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->title(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->output(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->container(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->workspace(); break;
        case 5: *reinterpret_cast<int*>(_v) = _t->layer(); break;
        case 6: *reinterpret_cast<int*>(_v) = _t->z(); break;
        case 7: *reinterpret_cast<int*>(_v) = _t->type(); break;
        case 8: *reinterpret_cast<int*>(_v) = _t->state(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->visible(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->active(); break;
        case 11: *reinterpret_cast<QRectF*>(_v) = _t->geometry(); break;
        case 12: *reinterpret_cast<QRectF*>(_v) = _t->titlebarGeometry(); break;
        case 13: *reinterpret_cast<QRectF*>(_v) = _t->boundingRect(); break;
        case 14: *reinterpret_cast<QRectF*>(_v) = _t->iconGeometry(); break;
        case 15: *reinterpret_cast<QPointF*>(_v) = _t->position(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setAppId(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setTitle(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setOutput(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setContainer(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setWorkspace(*reinterpret_cast<int*>(_v)); break;
        case 5: _t->setLayer(*reinterpret_cast<int*>(_v)); break;
        case 6: _t->setZ(*reinterpret_cast<int*>(_v)); break;
        case 7: _t->setType(*reinterpret_cast<int*>(_v)); break;
        case 8: _t->setState(*reinterpret_cast<int*>(_v)); break;
        case 9: _t->setVisible(*reinterpret_cast<bool*>(_v)); break;
        case 10: _t->setActive(*reinterpret_cast<bool*>(_v)); break;
        case 11: _t->setGeometry(*reinterpret_cast<QRectF*>(_v)); break;
        case 12: _t->setTitlebarGeometry(*reinterpret_cast<QRectF*>(_v)); break;
        case 13: _t->setBoundingRect(*reinterpret_cast<QRectF*>(_v)); break;
        case 14: _t->setIconGeometry(*reinterpret_cast<QRectF*>(_v)); break;
        case 15: _t->setPosition(*reinterpret_cast<QPointF*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN13WorkspaceInfoE_t {};
} // unnamed namespace

template <> constexpr inline auto WorkspaceInfo::qt_create_metaobjectdata<qt_meta_tag_ZN13WorkspaceInfoE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "WorkspaceInfo",
        "id",
        "isActive",
        "windows",
        "QList<WindowInfo>"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'id'
        QtMocHelpers::PropertyData<int>(1, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'isActive'
        QtMocHelpers::PropertyData<bool>(2, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'windows'
        QtMocHelpers::PropertyData<QList<WindowInfo>>(3, 0x80000000 | 4, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WorkspaceInfo, qt_meta_tag_ZN13WorkspaceInfoE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject WorkspaceInfo::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13WorkspaceInfoE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13WorkspaceInfoE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13WorkspaceInfoE_t>.metaTypes,
    nullptr
} };

void WorkspaceInfo::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<WorkspaceInfo *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->id(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isActive(); break;
        case 2: *reinterpret_cast<QList<WindowInfo>*>(_v) = _t->windows(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setId(*reinterpret_cast<int*>(_v)); break;
        case 1: _t->setIsActive(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setWindows(*reinterpret_cast<QList<WindowInfo>*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN9LayerInfoE_t {};
} // unnamed namespace

template <> constexpr inline auto LayerInfo::qt_create_metaobjectdata<qt_meta_tag_ZN9LayerInfoE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LayerInfo",
        "name",
        "layer",
        "windows",
        "QList<WindowInfo>",
        "workspaces",
        "QList<WorkspaceInfo>"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'name'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'layer'
        QtMocHelpers::PropertyData<int>(2, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'windows'
        QtMocHelpers::PropertyData<QList<WindowInfo>>(3, 0x80000000 | 4, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'workspaces'
        QtMocHelpers::PropertyData<QList<WorkspaceInfo>>(5, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LayerInfo, qt_meta_tag_ZN9LayerInfoE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject LayerInfo::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9LayerInfoE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9LayerInfoE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9LayerInfoE_t>.metaTypes,
    nullptr
} };

void LayerInfo::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<LayerInfo *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->name(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->layer(); break;
        case 2: *reinterpret_cast<QList<WindowInfo>*>(_v) = _t->windows(); break;
        case 3: *reinterpret_cast<QList<WorkspaceInfo>*>(_v) = _t->workspaces(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setName(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setLayer(*reinterpret_cast<int*>(_v)); break;
        case 2: _t->setWindows(*reinterpret_cast<QList<WindowInfo>*>(_v)); break;
        case 3: _t->setWorkspaces(*reinterpret_cast<QList<WorkspaceInfo>*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN12TreelandInfoE_t {};
} // unnamed namespace

template <> constexpr inline auto TreelandInfo::qt_create_metaobjectdata<qt_meta_tag_ZN12TreelandInfoE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TreelandInfo",
        "currentMode",
        "layers",
        "QList<LayerInfo>"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'currentMode'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet),
        // property 'layers'
        QtMocHelpers::PropertyData<QList<LayerInfo>>(2, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TreelandInfo, qt_meta_tag_ZN12TreelandInfoE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TreelandInfo::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12TreelandInfoE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12TreelandInfoE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12TreelandInfoE_t>.metaTypes,
    nullptr
} };

void TreelandInfo::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<TreelandInfo *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->currentMode(); break;
        case 1: *reinterpret_cast<QList<LayerInfo>*>(_v) = _t->layers(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setCurrentMode(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setLayers(*reinterpret_cast<QList<LayerInfo>*>(_v)); break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN22WindowTreeRemoteSourceE_t {};
} // unnamed namespace

template <> constexpr inline auto WindowTreeRemoteSource::qt_create_metaobjectdata<qt_meta_tag_ZN22WindowTreeRemoteSourceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "WindowTreeRemoteSource",
        "RemoteObject Type",
        "WindowTreeRemote",
        "RemoteObject Signature",
        "779a43c3d9929da0bf2286419fc5cbfd5307e03d",
        "cursorPositionChanged",
        "",
        "QPointF",
        "cursorPosition",
        "getTreelandInfo",
        "TreelandInfo"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'cursorPositionChanged'
        QtMocHelpers::SignalData<void(QPointF)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'getTreelandInfo'
        QtMocHelpers::SlotData<TreelandInfo()>(9, 6, QMC::AccessPublic, 0x80000000 | 10),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'cursorPosition'
        QtMocHelpers::PropertyData<QPointF>(8, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<WindowTreeRemoteSource, qt_meta_tag_ZN22WindowTreeRemoteSourceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject WindowTreeRemoteSource::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22WindowTreeRemoteSourceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22WindowTreeRemoteSourceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22WindowTreeRemoteSourceE_t>.metaTypes,
    nullptr
} };

void WindowTreeRemoteSource::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WindowTreeRemoteSource *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->cursorPositionChanged((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1]))); break;
        case 1: { TreelandInfo _r = _t->getTreelandInfo();
            if (_a[0]) *reinterpret_cast<TreelandInfo*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WindowTreeRemoteSource::*)(QPointF )>(_a, &WindowTreeRemoteSource::cursorPositionChanged, 0))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QPointF*>(_v) = _t->cursorPosition(); break;
        default: break;
        }
    }
}

const QMetaObject *WindowTreeRemoteSource::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *WindowTreeRemoteSource::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22WindowTreeRemoteSourceE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int WindowTreeRemoteSource::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void WindowTreeRemoteSource::cursorPositionChanged(QPointF _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN28WindowTreeRemoteSimpleSourceE_t {};
} // unnamed namespace

template <> constexpr inline auto WindowTreeRemoteSimpleSource::qt_create_metaobjectdata<qt_meta_tag_ZN28WindowTreeRemoteSimpleSourceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "WindowTreeRemoteSimpleSource"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WindowTreeRemoteSimpleSource, qt_meta_tag_ZN28WindowTreeRemoteSimpleSourceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject WindowTreeRemoteSimpleSource::staticMetaObject = { {
    QMetaObject::SuperData::link<WindowTreeRemoteSource::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28WindowTreeRemoteSimpleSourceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28WindowTreeRemoteSimpleSourceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN28WindowTreeRemoteSimpleSourceE_t>.metaTypes,
    nullptr
} };

void WindowTreeRemoteSimpleSource::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WindowTreeRemoteSimpleSource *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *WindowTreeRemoteSimpleSource::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *WindowTreeRemoteSimpleSource::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28WindowTreeRemoteSimpleSourceE_t>.strings))
        return static_cast<void*>(this);
    return WindowTreeRemoteSource::qt_metacast(_clname);
}

int WindowTreeRemoteSimpleSource::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = WindowTreeRemoteSource::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
