/* Generated by wayland-scanner 1.26.0 */

/*
 * SPDX-FileCopyrightText: 2026 UnionTech Software Technology Co., Ltd.
 * SPDX-License-Identifier: MIT
 */

#include <stdbool.h>
#include <stdlib.h>
#include <stdint.h>
#include "wayland-util.h"

#ifndef __has_attribute
# define __has_attribute(x) 0  /* Compatibility with non-clang compilers. */
#endif

#if (__has_attribute(visibility) || defined(__GNUC__) && __GNUC__ >= 4)
#define WL_PRIVATE __attribute__ ((visibility("hidden")))
#else
#define WL_PRIVATE
#endif

extern const struct wl_interface treeland_prelaunch_splash_v2_interface;
extern const struct wl_interface wl_buffer_interface;

static const struct wl_interface *treeland_prelaunch_splash_v2_types[] = {
	&treeland_prelaunch_splash_v2_interface,
	NULL,
	NULL,
	NULL,
	&wl_buffer_interface,
};

static const struct wl_message treeland_prelaunch_splash_manager_v2_requests[] = {
	{ "destroy", "", treeland_prelaunch_splash_v2_types + 0 },
	{ "create_splash", "nsss?o", treeland_prelaunch_splash_v2_types + 0 },
};

WL_PRIVATE const struct wl_interface treeland_prelaunch_splash_manager_v2_interface = {
	"treeland_prelaunch_splash_manager_v2", 1,
	2, treeland_prelaunch_splash_manager_v2_requests,
	0, NULL,
};

static const struct wl_message treeland_prelaunch_splash_v2_requests[] = {
	{ "destroy", "", treeland_prelaunch_splash_v2_types + 0 },
};

WL_PRIVATE const struct wl_interface treeland_prelaunch_splash_v2_interface = {
	"treeland_prelaunch_splash_v2", 1,
	1, treeland_prelaunch_splash_v2_requests,
	0, NULL,
};

