/****************************************************************************
** Meta object code from reading C++ file 'wallpaperswitcheritem.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/wallpaper/wallpaperswitcheritem.h"
#include "workspace/workspace.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wallpaperswitcheritem.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN21WallpaperSwitcherItemE_t {};
} // unnamed namespace

template <> constexpr inline auto WallpaperSwitcherItem::qt_create_metaobjectdata<qt_meta_tag_ZN21WallpaperSwitcherItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "WallpaperSwitcherItem",
        "QML.Element",
        "WallpaperSwitcher",
        "QML.AddedInVersion",
        "256",
        "outputChanged",
        "",
        "workspaceChanged",
        "playChanged",
        "sourceChanged",
        "opacityDurationChanged",
        "slowDown",
        "output",
        "Waylib::Server::WOutput*",
        "workspace",
        "WorkspaceModel*",
        "play",
        "source",
        "opacityDuration"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'outputChanged'
        QtMocHelpers::SignalData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'workspaceChanged'
        QtMocHelpers::SignalData<void()>(7, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'playChanged'
        QtMocHelpers::SignalData<void()>(8, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sourceChanged'
        QtMocHelpers::SignalData<void()>(9, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'opacityDurationChanged'
        QtMocHelpers::SignalData<void()>(10, 6, QMC::AccessPublic, QMetaType::Void),
        // Method 'slowDown'
        QtMocHelpers::MethodData<void()>(11, 6, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'output'
        QtMocHelpers::PropertyData<Waylib::Server::WOutput*>(12, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 0),
        // property 'workspace'
        QtMocHelpers::PropertyData<WorkspaceModel*>(14, 0x80000000 | 15, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 1),
        // property 'play'
        QtMocHelpers::PropertyData<bool>(16, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 2),
        // property 'source'
        QtMocHelpers::PropertyData<QString>(17, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Final, 3),
        // property 'opacityDuration'
        QtMocHelpers::PropertyData<int>(18, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 4),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<WallpaperSwitcherItem, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject WallpaperSwitcherItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21WallpaperSwitcherItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21WallpaperSwitcherItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21WallpaperSwitcherItemE_t>.metaTypes,
    nullptr
} };

void WallpaperSwitcherItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WallpaperSwitcherItem *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->outputChanged(); break;
        case 1: _t->workspaceChanged(); break;
        case 2: _t->playChanged(); break;
        case 3: _t->sourceChanged(); break;
        case 4: _t->opacityDurationChanged(); break;
        case 5: _t->slowDown(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WallpaperSwitcherItem::*)()>(_a, &WallpaperSwitcherItem::outputChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WallpaperSwitcherItem::*)()>(_a, &WallpaperSwitcherItem::workspaceChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WallpaperSwitcherItem::*)()>(_a, &WallpaperSwitcherItem::playChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WallpaperSwitcherItem::*)()>(_a, &WallpaperSwitcherItem::sourceChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (WallpaperSwitcherItem::*)()>(_a, &WallpaperSwitcherItem::opacityDurationChanged, 4))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<Waylib::Server::WOutput**>(_v) = _t->output(); break;
        case 1: *reinterpret_cast<WorkspaceModel**>(_v) = _t->workspace(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->play(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->source(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->opacityDuration(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setOutput(*reinterpret_cast<Waylib::Server::WOutput**>(_v)); break;
        case 1: _t->setWorkspace(*reinterpret_cast<WorkspaceModel**>(_v)); break;
        case 2: _t->setPlay(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setOpacityDuration(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *WallpaperSwitcherItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *WallpaperSwitcherItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21WallpaperSwitcherItemE_t>.strings))
        return static_cast<void*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int WallpaperSwitcherItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void WallpaperSwitcherItem::outputChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void WallpaperSwitcherItem::workspaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void WallpaperSwitcherItem::playChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void WallpaperSwitcherItem::sourceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void WallpaperSwitcherItem::opacityDurationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
