/****************************************************************************
** Meta object code from reading C++ file 'woutputrenderwindow.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../waylib/src/server/qtquick/woutputrenderwindow.h"
#include "wcursor.h"
#include <wquickoutputlayout.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QPointer>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'woutputrenderwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Waylib6Server19WOutputRenderWindowE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WOutputRenderWindow::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server19WOutputRenderWindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WOutputRenderWindow",
        "QML.Element",
        "OutputRenderWindow",
        "widthChanged",
        "",
        "heightChanged",
        "outputViewportInitialized",
        "Waylib::Server::WOutputViewport*",
        "output",
        "initialized",
        "disableLayersChanged",
        "renderEnd",
        "QList<QPointer<WOutput>>",
        "committedOutputs",
        "effectiveDevicePixelRatioChanged",
        "scale",
        "render",
        "WOutputViewport*",
        "doCommit",
        "update",
        "setWidth",
        "arg",
        "setHeight",
        "markItemClipRectDirty",
        "QQuickItem*",
        "item",
        "width",
        "height",
        "disableLayers"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'widthChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'heightChanged'
        QtMocHelpers::SignalData<void()>(5, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'outputViewportInitialized'
        QtMocHelpers::SignalData<void(Waylib::Server::WOutputViewport *)>(6, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'initialized'
        QtMocHelpers::SignalData<void()>(9, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'disableLayersChanged'
        QtMocHelpers::SignalData<void()>(10, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'renderEnd'
        QtMocHelpers::SignalData<void(QList<QPointer<WOutput>>)>(11, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Signal 'effectiveDevicePixelRatioChanged'
        QtMocHelpers::SignalData<void(qreal)>(14, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 15 },
        }}),
        // Slot 'render'
        QtMocHelpers::SlotData<void()>(16, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'render'
        QtMocHelpers::SlotData<void(WOutputViewport *, bool)>(16, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 8 }, { QMetaType::Bool, 18 },
        }}),
        // Slot 'update'
        QtMocHelpers::SlotData<void()>(19, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'update'
        QtMocHelpers::SlotData<void(WOutputViewport *)>(19, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 17, 8 },
        }}),
        // Slot 'setWidth'
        QtMocHelpers::SlotData<void(qreal)>(20, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 21 },
        }}),
        // Slot 'setHeight'
        QtMocHelpers::SlotData<void(qreal)>(22, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 21 },
        }}),
        // Slot 'markItemClipRectDirty'
        QtMocHelpers::SlotData<void(QQuickItem *)>(23, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'width'
        QtMocHelpers::PropertyData<qreal>(26, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'height'
        QtMocHelpers::PropertyData<qreal>(27, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'disableLayers'
        QtMocHelpers::PropertyData<bool>(28, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 4),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<WOutputRenderWindow, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WOutputRenderWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server19WOutputRenderWindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server19WOutputRenderWindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server19WOutputRenderWindowE_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WOutputRenderWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WOutputRenderWindow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->widthChanged(); break;
        case 1: _t->heightChanged(); break;
        case 2: _t->outputViewportInitialized((*reinterpret_cast<std::add_pointer_t<Waylib::Server::WOutputViewport*>>(_a[1]))); break;
        case 3: _t->initialized(); break;
        case 4: _t->disableLayersChanged(); break;
        case 5: _t->renderEnd((*reinterpret_cast<std::add_pointer_t<QList<QPointer<WOutput>>>>(_a[1]))); break;
        case 6: _t->effectiveDevicePixelRatioChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 7: _t->render(); break;
        case 8: _t->render((*reinterpret_cast<std::add_pointer_t<WOutputViewport*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 9: _t->update(); break;
        case 10: _t->update((*reinterpret_cast<std::add_pointer_t<WOutputViewport*>>(_a[1]))); break;
        case 11: _t->setWidth((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 12: _t->setHeight((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 13: _t->markItemClipRectDirty((*reinterpret_cast<std::add_pointer_t<QQuickItem*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QPointer<WOutput>> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WOutputRenderWindow::*)()>(_a, &WOutputRenderWindow::widthChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputRenderWindow::*)()>(_a, &WOutputRenderWindow::heightChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputRenderWindow::*)(Waylib::Server::WOutputViewport * )>(_a, &WOutputRenderWindow::outputViewportInitialized, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputRenderWindow::*)()>(_a, &WOutputRenderWindow::initialized, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputRenderWindow::*)()>(_a, &WOutputRenderWindow::disableLayersChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputRenderWindow::*)(QList<QPointer<WOutput>> )>(_a, &WOutputRenderWindow::renderEnd, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputRenderWindow::*)(qreal )>(_a, &WOutputRenderWindow::effectiveDevicePixelRatioChanged, 6))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<qreal*>(_v) = _t->width(); break;
        case 1: *reinterpret_cast<qreal*>(_v) = _t->height(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->disableLayers(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setWidth(*reinterpret_cast<qreal*>(_v)); break;
        case 1: _t->setHeight(*reinterpret_cast<qreal*>(_v)); break;
        case 2: _t->setDisableLayers(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Waylib::Server::WOutputRenderWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WOutputRenderWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server19WOutputRenderWindowE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QQuickWindow::qt_metacast(_clname);
}

int Waylib::Server::WOutputRenderWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WOutputRenderWindow::widthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Waylib::Server::WOutputRenderWindow::heightChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Waylib::Server::WOutputRenderWindow::outputViewportInitialized(Waylib::Server::WOutputViewport * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Waylib::Server::WOutputRenderWindow::initialized()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Waylib::Server::WOutputRenderWindow::disableLayersChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Waylib::Server::WOutputRenderWindow::renderEnd(QList<QPointer<WOutput>> _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void Waylib::Server::WOutputRenderWindow::effectiveDevicePixelRatioChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}
QT_WARNING_POP
