/****************************************************************************
** Meta object code from reading C++ file 'wlayersurface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../waylib/src/server/protocols/wlayersurface.h"
#include "woutput.h"
#include "wcursor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wlayersurface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Waylib6Server13WLayerSurfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WLayerSurface::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server13WLayerSurfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WLayerSurface",
        "QML.Element",
        "WaylandLayerSurface",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Only create in C++",
        "RegisterEnumClassesUnscoped",
        "desiredSizeChanged",
        "",
        "surfaceChanged",
        "layerChanged",
        "ancherChanged",
        "leftMarginChanged",
        "rightMarginChanged",
        "topMarginChanged",
        "bottomMarginChanged",
        "exclusiveZoneChanged",
        "keyboardInteractivityChanged",
        "layerPropertiesChanged",
        "checkNewSize",
        "QSize",
        "size",
        "QSize*",
        "clipedSize",
        "getExclusiveZoneEdge",
        "AnchorType",
        "configureSize",
        "uint32_t",
        "newSize",
        "closed",
        "surface",
        "WSurface*",
        "desiredSize",
        "layer",
        "LayerType",
        "ancher",
        "AnchorTypes",
        "exclusiveZone",
        "int32_t",
        "leftMargin",
        "rightMargin",
        "topMargin",
        "bottomMargin",
        "keyboardInteractivity",
        "KeyboardInteractivity",
        "output",
        "WOutput*",
        "Background",
        "Bottom",
        "Top",
        "Overlay",
        "None",
        "Left",
        "Right",
        "Exclusive",
        "OnDemand"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'desiredSizeChanged'
        QtMocHelpers::SignalData<void()>(8, 9, QMC::AccessPublic, QMetaType::Void),
        // Signal 'surfaceChanged'
        QtMocHelpers::SignalData<void()>(10, 9, QMC::AccessPublic, QMetaType::Void),
        // Signal 'layerChanged'
        QtMocHelpers::SignalData<void()>(11, 9, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ancherChanged'
        QtMocHelpers::SignalData<void()>(12, 9, QMC::AccessPublic, QMetaType::Void),
        // Signal 'leftMarginChanged'
        QtMocHelpers::SignalData<void()>(13, 9, QMC::AccessPublic, QMetaType::Void),
        // Signal 'rightMarginChanged'
        QtMocHelpers::SignalData<void()>(14, 9, QMC::AccessPublic, QMetaType::Void),
        // Signal 'topMarginChanged'
        QtMocHelpers::SignalData<void()>(15, 9, QMC::AccessPublic, QMetaType::Void),
        // Signal 'bottomMarginChanged'
        QtMocHelpers::SignalData<void()>(16, 9, QMC::AccessPublic, QMetaType::Void),
        // Signal 'exclusiveZoneChanged'
        QtMocHelpers::SignalData<void()>(17, 9, QMC::AccessPublic, QMetaType::Void),
        // Signal 'keyboardInteractivityChanged'
        QtMocHelpers::SignalData<void()>(18, 9, QMC::AccessPublic, QMetaType::Void),
        // Signal 'layerPropertiesChanged'
        QtMocHelpers::SignalData<void()>(19, 9, QMC::AccessPublic, QMetaType::Void),
        // Slot 'checkNewSize'
        QtMocHelpers::SlotData<bool(const QSize &, QSize *)>(20, 9, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 21, 22 }, { 0x80000000 | 23, 24 },
        }}),
        // Slot 'checkNewSize'
        QtMocHelpers::SlotData<bool(const QSize &)>(20, 9, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Method 'getExclusiveZoneEdge'
        QtMocHelpers::MethodData<enum AnchorType() const>(25, 9, QMC::AccessPublic, 0x80000000 | 26),
        // Method 'configureSize'
        QtMocHelpers::MethodData<uint32_t(const QSize &)>(27, 9, QMC::AccessPublic, 0x80000000 | 28, {{
            { 0x80000000 | 21, 29 },
        }}),
        // Method 'closed'
        QtMocHelpers::MethodData<void()>(30, 9, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'surface'
        QtMocHelpers::PropertyData<WSurface*>(31, 0x80000000 | 32, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 1),
        // property 'desiredSize'
        QtMocHelpers::PropertyData<QSize>(33, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 'layer'
        QtMocHelpers::PropertyData<enum LayerType>(34, 0x80000000 | 35, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 2),
        // property 'ancher'
        QtMocHelpers::PropertyData<AnchorTypes>(36, 0x80000000 | 37, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 3),
        // property 'exclusiveZone'
        QtMocHelpers::PropertyData<int32_t>(38, 0x80000000 | 39, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 8),
        // property 'leftMargin'
        QtMocHelpers::PropertyData<int32_t>(40, 0x80000000 | 39, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 4),
        // property 'rightMargin'
        QtMocHelpers::PropertyData<int32_t>(41, 0x80000000 | 39, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 5),
        // property 'topMargin'
        QtMocHelpers::PropertyData<int32_t>(42, 0x80000000 | 39, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 6),
        // property 'bottomMargin'
        QtMocHelpers::PropertyData<int32_t>(43, 0x80000000 | 39, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 7),
        // property 'keyboardInteractivity'
        QtMocHelpers::PropertyData<enum KeyboardInteractivity>(44, 0x80000000 | 45, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 9),
        // property 'output'
        QtMocHelpers::PropertyData<WOutput*>(46, 0x80000000 | 47, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'LayerType'
        QtMocHelpers::EnumData<enum LayerType>(35, 35, QMC::EnumIsScoped).add({
            {   48, LayerType::Background },
            {   49, LayerType::Bottom },
            {   50, LayerType::Top },
            {   51, LayerType::Overlay },
        }),
        // enum 'AnchorType'
        QtMocHelpers::EnumData<enum AnchorType>(26, 26, QMC::EnumIsScoped).add({
            {   52, AnchorType::None },
            {   50, AnchorType::Top },
            {   49, AnchorType::Bottom },
            {   53, AnchorType::Left },
            {   54, AnchorType::Right },
        }),
        // enum 'KeyboardInteractivity'
        QtMocHelpers::EnumData<enum KeyboardInteractivity>(45, 45, QMC::EnumIsScoped).add({
            {   52, KeyboardInteractivity::None },
            {   55, KeyboardInteractivity::Exclusive },
            {   56, KeyboardInteractivity::OnDemand },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    4 },
    });
    return QtMocHelpers::metaObjectData<WLayerSurface, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WLayerSurface::staticMetaObject = { {
    QMetaObject::SuperData::link<WToplevelSurface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server13WLayerSurfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server13WLayerSurfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server13WLayerSurfaceE_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WLayerSurface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WLayerSurface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->desiredSizeChanged(); break;
        case 1: _t->surfaceChanged(); break;
        case 2: _t->layerChanged(); break;
        case 3: _t->ancherChanged(); break;
        case 4: _t->leftMarginChanged(); break;
        case 5: _t->rightMarginChanged(); break;
        case 6: _t->topMarginChanged(); break;
        case 7: _t->bottomMarginChanged(); break;
        case 8: _t->exclusiveZoneChanged(); break;
        case 9: _t->keyboardInteractivityChanged(); break;
        case 10: _t->layerPropertiesChanged(); break;
        case 11: { bool _r = _t->checkNewSize((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSize*>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 12: { bool _r = _t->checkNewSize((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 13: { enum AnchorType _r = _t->getExclusiveZoneEdge();
            if (_a[0]) *reinterpret_cast<enum AnchorType*>(_a[0]) = std::move(_r); }  break;
        case 14: { uint32_t _r = _t->configureSize((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1])));
            if (_a[0]) *reinterpret_cast<uint32_t*>(_a[0]) = std::move(_r); }  break;
        case 15: _t->closed(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WLayerSurface::*)()>(_a, &WLayerSurface::desiredSizeChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WLayerSurface::*)()>(_a, &WLayerSurface::surfaceChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WLayerSurface::*)()>(_a, &WLayerSurface::layerChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WLayerSurface::*)()>(_a, &WLayerSurface::ancherChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (WLayerSurface::*)()>(_a, &WLayerSurface::leftMarginChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (WLayerSurface::*)()>(_a, &WLayerSurface::rightMarginChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (WLayerSurface::*)()>(_a, &WLayerSurface::topMarginChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (WLayerSurface::*)()>(_a, &WLayerSurface::bottomMarginChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (WLayerSurface::*)()>(_a, &WLayerSurface::exclusiveZoneChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (WLayerSurface::*)()>(_a, &WLayerSurface::keyboardInteractivityChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (WLayerSurface::*)()>(_a, &WLayerSurface::layerPropertiesChanged, 10))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 10:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< WOutput* >(); break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< WSurface* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<WSurface**>(_v) = _t->surface(); break;
        case 1: *reinterpret_cast<QSize*>(_v) = _t->desiredSize(); break;
        case 2: *reinterpret_cast<enum LayerType*>(_v) = _t->layer(); break;
        case 3: *reinterpret_cast<AnchorTypes*>(_v) = _t->ancher(); break;
        case 4: *reinterpret_cast<int32_t*>(_v) = _t->exclusiveZone(); break;
        case 5: *reinterpret_cast<int32_t*>(_v) = _t->leftMargin(); break;
        case 6: *reinterpret_cast<int32_t*>(_v) = _t->rightMargin(); break;
        case 7: *reinterpret_cast<int32_t*>(_v) = _t->topMargin(); break;
        case 8: *reinterpret_cast<int32_t*>(_v) = _t->bottomMargin(); break;
        case 9: *reinterpret_cast<enum KeyboardInteractivity*>(_v) = _t->keyboardInteractivity(); break;
        case 10: *reinterpret_cast<WOutput**>(_v) = _t->output(); break;
        default: break;
        }
    }
}

const QMetaObject *Waylib::Server::WLayerSurface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WLayerSurface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server13WLayerSurfaceE_t>.strings))
        return static_cast<void*>(this);
    return WToplevelSurface::qt_metacast(_clname);
}

int Waylib::Server::WLayerSurface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = WToplevelSurface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 16;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WLayerSurface::desiredSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Waylib::Server::WLayerSurface::surfaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Waylib::Server::WLayerSurface::layerChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Waylib::Server::WLayerSurface::ancherChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Waylib::Server::WLayerSurface::leftMarginChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Waylib::Server::WLayerSurface::rightMarginChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Waylib::Server::WLayerSurface::topMarginChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void Waylib::Server::WLayerSurface::bottomMarginChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void Waylib::Server::WLayerSurface::exclusiveZoneChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void Waylib::Server::WLayerSurface::keyboardInteractivityChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void Waylib::Server::WLayerSurface::layerPropertiesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}
QT_WARNING_POP
