/****************************************************************************
** Meta object code from reading C++ file 'outputconfig.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../outputconfig.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'outputconfig.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12OutputConfigE_t {};
} // unnamed namespace

template <> constexpr inline auto OutputConfig::qt_create_metaobjectdata<qt_meta_tag_ZN12OutputConfigE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OutputConfig",
        "DConfigKeyList",
        "adaptiveSyncEnabled;brightness;colorTemperature;height;refresh;sca"
        "le;transform;width;x;y",
        "DConfigFileName",
        "org.deepin.dde.treeland.output",
        "DConfigFileVersion",
        "1.0",
        "configInitializeFailed",
        "",
        "configInitializeSucceed",
        "Dtk::Core::DConfig*",
        "config",
        "valueChanged",
        "key",
        "QVariant",
        "value",
        "adaptiveSyncEnabledChanged",
        "brightnessChanged",
        "colorTemperatureChanged",
        "heightChanged",
        "refreshChanged",
        "scaleChanged",
        "transformChanged",
        "widthChanged",
        "xChanged",
        "yChanged",
        "isInitializeSucceed",
        "isInitializeSucceeded",
        "isInitializeFailed",
        "isInitializing",
        "keyList",
        "isDefaultValue",
        "adaptiveSyncEnabledIsDefaultValue",
        "brightnessIsDefaultValue",
        "colorTemperatureIsDefaultValue",
        "heightIsDefaultValue",
        "refreshIsDefaultValue",
        "scaleIsDefaultValue",
        "transformIsDefaultValue",
        "widthIsDefaultValue",
        "xIsDefaultValue",
        "yIsDefaultValue",
        "adaptiveSyncEnabled",
        "brightness",
        "colorTemperature",
        "height",
        "refresh",
        "scale",
        "transform",
        "width",
        "x",
        "y"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'configInitializeFailed'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'configInitializeSucceed'
        QtMocHelpers::SignalData<void(Dtk::Core::DConfig *)>(9, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'valueChanged'
        QtMocHelpers::SignalData<void(const QString &, const QVariant &)>(12, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 }, { 0x80000000 | 14, 15 },
        }}),
        // Signal 'adaptiveSyncEnabledChanged'
        QtMocHelpers::SignalData<void()>(16, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'brightnessChanged'
        QtMocHelpers::SignalData<void()>(17, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'colorTemperatureChanged'
        QtMocHelpers::SignalData<void()>(18, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'heightChanged'
        QtMocHelpers::SignalData<void()>(19, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'refreshChanged'
        QtMocHelpers::SignalData<void()>(20, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'scaleChanged'
        QtMocHelpers::SignalData<void()>(21, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'transformChanged'
        QtMocHelpers::SignalData<void()>(22, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'widthChanged'
        QtMocHelpers::SignalData<void()>(23, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'xChanged'
        QtMocHelpers::SignalData<void()>(24, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'yChanged'
        QtMocHelpers::SignalData<void()>(25, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'config'
        QtMocHelpers::MethodData<Dtk::Core::DConfig *() const>(11, 8, QMC::AccessPublic, 0x80000000 | 10),
        // Method 'isInitializeSucceed'
        QtMocHelpers::MethodData<bool() const>(26, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'isInitializeSucceeded'
        QtMocHelpers::MethodData<bool() const>(27, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'isInitializeFailed'
        QtMocHelpers::MethodData<bool() const>(28, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'isInitializing'
        QtMocHelpers::MethodData<bool() const>(29, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'keyList'
        QtMocHelpers::MethodData<QStringList() const>(30, 8, QMC::AccessPublic, QMetaType::QStringList),
        // Method 'isDefaultValue'
        QtMocHelpers::MethodData<bool(const QString &) const>(31, 8, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 13 },
        }}),
        // Method 'adaptiveSyncEnabledIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(32, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'brightnessIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(33, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'colorTemperatureIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(34, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'heightIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(35, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'refreshIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(36, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'scaleIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(37, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'transformIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(38, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'widthIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(39, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'xIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(40, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'yIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(41, 8, QMC::AccessPublic, QMetaType::Bool),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'adaptiveSyncEnabled'
        QtMocHelpers::PropertyData<bool>(42, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 3),
        // property 'brightness'
        QtMocHelpers::PropertyData<double>(43, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 4),
        // property 'colorTemperature'
        QtMocHelpers::PropertyData<qlonglong>(44, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 5),
        // property 'height'
        QtMocHelpers::PropertyData<qlonglong>(45, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 6),
        // property 'refresh'
        QtMocHelpers::PropertyData<qlonglong>(46, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 7),
        // property 'scale'
        QtMocHelpers::PropertyData<double>(47, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 8),
        // property 'transform'
        QtMocHelpers::PropertyData<qlonglong>(48, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 9),
        // property 'width'
        QtMocHelpers::PropertyData<qlonglong>(49, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 10),
        // property 'x'
        QtMocHelpers::PropertyData<qlonglong>(50, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 11),
        // property 'y'
        QtMocHelpers::PropertyData<qlonglong>(51, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 12),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<OutputConfig, qt_meta_tag_ZN12OutputConfigE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject OutputConfig::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12OutputConfigE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12OutputConfigE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12OutputConfigE_t>.metaTypes,
    nullptr
} };

void OutputConfig::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OutputConfig *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->configInitializeFailed(); break;
        case 1: _t->configInitializeSucceed((*reinterpret_cast<std::add_pointer_t<Dtk::Core::DConfig*>>(_a[1]))); break;
        case 2: _t->valueChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 3: _t->adaptiveSyncEnabledChanged(); break;
        case 4: _t->brightnessChanged(); break;
        case 5: _t->colorTemperatureChanged(); break;
        case 6: _t->heightChanged(); break;
        case 7: _t->refreshChanged(); break;
        case 8: _t->scaleChanged(); break;
        case 9: _t->transformChanged(); break;
        case 10: _t->widthChanged(); break;
        case 11: _t->xChanged(); break;
        case 12: _t->yChanged(); break;
        case 13: { Dtk::Core::DConfig* _r = _t->config();
            if (_a[0]) *reinterpret_cast<Dtk::Core::DConfig**>(_a[0]) = std::move(_r); }  break;
        case 14: { bool _r = _t->isInitializeSucceed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 15: { bool _r = _t->isInitializeSucceeded();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 16: { bool _r = _t->isInitializeFailed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 17: { bool _r = _t->isInitializing();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 18: { QStringList _r = _t->keyList();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 19: { bool _r = _t->isDefaultValue((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 20: { bool _r = _t->adaptiveSyncEnabledIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 21: { bool _r = _t->brightnessIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 22: { bool _r = _t->colorTemperatureIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 23: { bool _r = _t->heightIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 24: { bool _r = _t->refreshIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 25: { bool _r = _t->scaleIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 26: { bool _r = _t->transformIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 27: { bool _r = _t->widthIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 28: { bool _r = _t->xIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 29: { bool _r = _t->yIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Dtk::Core::DConfig* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OutputConfig::*)()>(_a, &OutputConfig::configInitializeFailed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (OutputConfig::*)(Dtk::Core::DConfig * )>(_a, &OutputConfig::configInitializeSucceed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (OutputConfig::*)(const QString & , const QVariant & )>(_a, &OutputConfig::valueChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (OutputConfig::*)()>(_a, &OutputConfig::adaptiveSyncEnabledChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (OutputConfig::*)()>(_a, &OutputConfig::brightnessChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (OutputConfig::*)()>(_a, &OutputConfig::colorTemperatureChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (OutputConfig::*)()>(_a, &OutputConfig::heightChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (OutputConfig::*)()>(_a, &OutputConfig::refreshChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (OutputConfig::*)()>(_a, &OutputConfig::scaleChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (OutputConfig::*)()>(_a, &OutputConfig::transformChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (OutputConfig::*)()>(_a, &OutputConfig::widthChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (OutputConfig::*)()>(_a, &OutputConfig::xChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (OutputConfig::*)()>(_a, &OutputConfig::yChanged, 12))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->adaptiveSyncEnabled(); break;
        case 1: *reinterpret_cast<double*>(_v) = _t->brightness(); break;
        case 2: *reinterpret_cast<qlonglong*>(_v) = _t->colorTemperature(); break;
        case 3: *reinterpret_cast<qlonglong*>(_v) = _t->height(); break;
        case 4: *reinterpret_cast<qlonglong*>(_v) = _t->refresh(); break;
        case 5: *reinterpret_cast<double*>(_v) = _t->scale(); break;
        case 6: *reinterpret_cast<qlonglong*>(_v) = _t->transform(); break;
        case 7: *reinterpret_cast<qlonglong*>(_v) = _t->width(); break;
        case 8: *reinterpret_cast<qlonglong*>(_v) = _t->x(); break;
        case 9: *reinterpret_cast<qlonglong*>(_v) = _t->y(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setAdaptiveSyncEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setBrightness(*reinterpret_cast<double*>(_v)); break;
        case 2: _t->setColorTemperature(*reinterpret_cast<qlonglong*>(_v)); break;
        case 3: _t->setHeight(*reinterpret_cast<qlonglong*>(_v)); break;
        case 4: _t->setRefresh(*reinterpret_cast<qlonglong*>(_v)); break;
        case 5: _t->setScale(*reinterpret_cast<double*>(_v)); break;
        case 6: _t->setTransform(*reinterpret_cast<qlonglong*>(_v)); break;
        case 7: _t->setWidth(*reinterpret_cast<qlonglong*>(_v)); break;
        case 8: _t->setX(*reinterpret_cast<qlonglong*>(_v)); break;
        case 9: _t->setY(*reinterpret_cast<qlonglong*>(_v)); break;
        default: break;
        }
    }
    if (_c == QMetaObject::ResetProperty) {
        switch (_id) {
        case 0: _t->resetAdaptiveSyncEnabled(); break;
        case 1: _t->resetBrightness(); break;
        case 2: _t->resetColorTemperature(); break;
        case 3: _t->resetHeight(); break;
        case 4: _t->resetRefresh(); break;
        case 5: _t->resetScale(); break;
        case 6: _t->resetTransform(); break;
        case 7: _t->resetWidth(); break;
        case 8: _t->resetX(); break;
        case 9: _t->resetY(); break;
        default: break;
        }
    }
}

const QMetaObject *OutputConfig::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OutputConfig::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12OutputConfigE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int OutputConfig::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 30)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 30;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 30)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 30;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void OutputConfig::configInitializeFailed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void OutputConfig::configInitializeSucceed(Dtk::Core::DConfig * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void OutputConfig::valueChanged(const QString & _t1, const QVariant & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void OutputConfig::adaptiveSyncEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void OutputConfig::brightnessChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void OutputConfig::colorTemperatureChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void OutputConfig::heightChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void OutputConfig::refreshChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void OutputConfig::scaleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void OutputConfig::transformChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void OutputConfig::widthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void OutputConfig::xChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void OutputConfig::yChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}
QT_WARNING_POP
