/****************************************************************************
** Meta object code from reading C++ file 'wquicktextureproxy.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../waylib/src/server/qtquick/wquicktextureproxy.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wquicktextureproxy.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Waylib6Server18WQuickTextureProxyE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WQuickTextureProxy::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server18WQuickTextureProxyE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WQuickTextureProxy",
        "QML.Element",
        "TextureProxy",
        "sourceItemChanged",
        "",
        "sourceRectChanged",
        "fixedChanged",
        "hideSourceChanged",
        "mipmapChanged",
        "implicitWidthChanged",
        "implicitHeightChanged",
        "sourceItem",
        "QQuickItem*",
        "sourceRect",
        "QRectF",
        "implicitWidth",
        "implicitHeight",
        "hideSource",
        "mipmap"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'sourceItemChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sourceRectChanged'
        QtMocHelpers::SignalData<void()>(5, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fixedChanged'
        QtMocHelpers::SignalData<void()>(6, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hideSourceChanged'
        QtMocHelpers::SignalData<void()>(7, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'mipmapChanged'
        QtMocHelpers::SignalData<void()>(8, 4, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'sourceItem'
        QtMocHelpers::PropertyData<QQuickItem*>(11, 0x80000000 | 12, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'sourceRect'
        QtMocHelpers::PropertyData<QRectF>(13, 0x80000000 | 14, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'implicitWidth'
        QtMocHelpers::PropertyData<qreal>(15, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Final, 0x70000000 | 9),
        // property 'implicitHeight'
        QtMocHelpers::PropertyData<qreal>(16, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Final, 0x70000000 | 10),
        // property 'hideSource'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'mipmap'
        QtMocHelpers::PropertyData<bool>(18, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<WQuickTextureProxy, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WQuickTextureProxy::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server18WQuickTextureProxyE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server18WQuickTextureProxyE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server18WQuickTextureProxyE_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WQuickTextureProxy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WQuickTextureProxy *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sourceItemChanged(); break;
        case 1: _t->sourceRectChanged(); break;
        case 2: _t->fixedChanged(); break;
        case 3: _t->hideSourceChanged(); break;
        case 4: _t->mipmapChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WQuickTextureProxy::*)()>(_a, &WQuickTextureProxy::sourceItemChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WQuickTextureProxy::*)()>(_a, &WQuickTextureProxy::sourceRectChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WQuickTextureProxy::*)()>(_a, &WQuickTextureProxy::fixedChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WQuickTextureProxy::*)()>(_a, &WQuickTextureProxy::hideSourceChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (WQuickTextureProxy::*)()>(_a, &WQuickTextureProxy::mipmapChanged, 4))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItem* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QQuickItem**>(_v) = _t->sourceItem(); break;
        case 1: *reinterpret_cast<QRectF*>(_v) = _t->sourceRect(); break;
        case 2: *reinterpret_cast<qreal*>(_v) = _t->implicitWidth(); break;
        case 3: *reinterpret_cast<qreal*>(_v) = _t->implicitHeight(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->hideSource(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->mipmap(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSourceItem(*reinterpret_cast<QQuickItem**>(_v)); break;
        case 1: _t->setSourceRect(*reinterpret_cast<QRectF*>(_v)); break;
        case 4: _t->setHideSource(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setMipmap(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Waylib::Server::WQuickTextureProxy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WQuickTextureProxy::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server18WQuickTextureProxyE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "WObject"))
        return static_cast< WObject*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int Waylib::Server::WQuickTextureProxy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WQuickTextureProxy::sourceItemChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Waylib::Server::WQuickTextureProxy::sourceRectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Waylib::Server::WQuickTextureProxy::fixedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Waylib::Server::WQuickTextureProxy::hideSourceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Waylib::Server::WQuickTextureProxy::mipmapChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
namespace CheckNotifySignalValidity_ZN6Waylib6Server18WQuickTextureProxyE {
template<typename T> using has_nullary_implicitWidthChanged = decltype(std::declval<T>().implicitWidthChanged());
template<typename T> using has_unary_implicitWidthChanged = decltype(std::declval<T>().implicitWidthChanged(std::declval<qreal>()));
static_assert(qxp::is_detected_v<has_nullary_implicitWidthChanged, Waylib::Server::WQuickTextureProxy> || qxp::is_detected_v<has_unary_implicitWidthChanged, Waylib::Server::WQuickTextureProxy>,
              "NOTIFY signal implicitWidthChanged does not exist in class (or is private in its parent)");
template<typename T> using has_nullary_implicitHeightChanged = decltype(std::declval<T>().implicitHeightChanged());
template<typename T> using has_unary_implicitHeightChanged = decltype(std::declval<T>().implicitHeightChanged(std::declval<qreal>()));
static_assert(qxp::is_detected_v<has_nullary_implicitHeightChanged, Waylib::Server::WQuickTextureProxy> || qxp::is_detected_v<has_unary_implicitHeightChanged, Waylib::Server::WQuickTextureProxy>,
              "NOTIFY signal implicitHeightChanged does not exist in class (or is private in its parent)");
}
QT_WARNING_POP
