/****************************************************************************
** Meta object code from reading C++ file 'wxdgtoplevelsurface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../waylib/src/server/protocols/wxdgtoplevelsurface.h"
#include "woutput.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wxdgtoplevelsurface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Waylib6Server19WXdgToplevelSurfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WXdgToplevelSurface::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server19WXdgToplevelSurfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WXdgToplevelSurface",
        "QML.Element",
        "WaylandXdgToplevelSurface",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Only create in C++",
        "parentXdgSurfaceChanged",
        "",
        "resizeingChanged",
        "tagChanged",
        "descriptionChanged",
        "setResizeing",
        "resizeing",
        "setMaximize",
        "on",
        "setMinimize",
        "setActivate",
        "setFullScreen",
        "checkNewSize",
        "QSize",
        "size",
        "QSize*",
        "clipedSize",
        "resize",
        "close",
        "isResizeing",
        "parentXdgSurface",
        "WXdgSurface*",
        "tag",
        "description"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'parentXdgSurfaceChanged'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'resizeingChanged'
        QtMocHelpers::SignalData<void()>(9, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'tagChanged'
        QtMocHelpers::SignalData<void()>(10, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'descriptionChanged'
        QtMocHelpers::SignalData<void()>(11, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setResizeing'
        QtMocHelpers::SlotData<void(bool)>(12, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
        // Slot 'setMaximize'
        QtMocHelpers::SlotData<void(bool)>(14, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
        // Slot 'setMinimize'
        QtMocHelpers::SlotData<void(bool)>(16, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
        // Slot 'setActivate'
        QtMocHelpers::SlotData<void(bool)>(17, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
        // Slot 'setFullScreen'
        QtMocHelpers::SlotData<void(bool)>(18, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
        // Slot 'checkNewSize'
        QtMocHelpers::SlotData<bool(const QSize &, QSize *)>(19, 8, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 20, 21 }, { 0x80000000 | 22, 23 },
        }}),
        // Slot 'checkNewSize'
        QtMocHelpers::SlotData<bool(const QSize &)>(19, 8, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'resize'
        QtMocHelpers::SlotData<void(const QSize &)>(24, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'close'
        QtMocHelpers::SlotData<void()>(25, 8, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'isResizeing'
        QtMocHelpers::PropertyData<bool>(26, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 1),
        // property 'parentXdgSurface'
        QtMocHelpers::PropertyData<WXdgSurface*>(27, 0x80000000 | 28, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 0),
        // property 'tag'
        QtMocHelpers::PropertyData<QString>(29, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Final, 2),
        // property 'description'
        QtMocHelpers::PropertyData<QString>(30, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Final, 3),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<WXdgToplevelSurface, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WXdgToplevelSurface::staticMetaObject = { {
    QMetaObject::SuperData::link<WXdgSurface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server19WXdgToplevelSurfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server19WXdgToplevelSurfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server19WXdgToplevelSurfaceE_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WXdgToplevelSurface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WXdgToplevelSurface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->parentXdgSurfaceChanged(); break;
        case 1: _t->resizeingChanged(); break;
        case 2: _t->tagChanged(); break;
        case 3: _t->descriptionChanged(); break;
        case 4: _t->setResizeing((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->setMaximize((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->setMinimize((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->setActivate((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->setFullScreen((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: { bool _r = _t->checkNewSize((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSize*>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 10: { bool _r = _t->checkNewSize((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 11: _t->resize((*reinterpret_cast<std::add_pointer_t<QSize>>(_a[1]))); break;
        case 12: _t->close(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WXdgToplevelSurface::*)()>(_a, &WXdgToplevelSurface::parentXdgSurfaceChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WXdgToplevelSurface::*)()>(_a, &WXdgToplevelSurface::resizeingChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WXdgToplevelSurface::*)()>(_a, &WXdgToplevelSurface::tagChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WXdgToplevelSurface::*)()>(_a, &WXdgToplevelSurface::descriptionChanged, 3))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< WXdgSurface* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isResizeing(); break;
        case 1: *reinterpret_cast<WXdgSurface**>(_v) = _t->parentXdgSurface(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->tag(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->description(); break;
        default: break;
        }
    }
}

const QMetaObject *Waylib::Server::WXdgToplevelSurface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WXdgToplevelSurface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server19WXdgToplevelSurfaceE_t>.strings))
        return static_cast<void*>(this);
    return WXdgSurface::qt_metacast(_clname);
}

int Waylib::Server::WXdgToplevelSurface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = WXdgSurface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WXdgToplevelSurface::parentXdgSurfaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Waylib::Server::WXdgToplevelSurface::resizeingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Waylib::Server::WXdgToplevelSurface::tagChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Waylib::Server::WXdgToplevelSurface::descriptionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
