/****************************************************************************
** Meta object code from reading C++ file 'inputmanagerinterfacev1.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/modules/input-manager/inputmanagerinterfacev1.h"
#include "private/wserver_p.h"
#include <wsurface.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'inputmanagerinterfacev1.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN31TreelandInputManagerInterfaceV1E_t {};
} // unnamed namespace

template <> constexpr inline auto TreelandInputManagerInterfaceV1::qt_create_metaobjectdata<qt_meta_tag_ZN31TreelandInputManagerInterfaceV1E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TreelandInputManagerInterfaceV1",
        "mouseSettingsCreated",
        "",
        "MouseSettingsInterfaceV1*",
        "interface",
        "touchpadSettingsCreated",
        "TouchpadSettingsInterfaceV1*",
        "keyboardSettingsCreated",
        "KeyboardSettingsInterfaceV1*",
        "onInputAdded",
        "WInputDevice*",
        "input",
        "onInputRemoved",
        "DeviceTypes",
        "DeviceType",
        "Unknown",
        "Mouse",
        "TouchPad",
        "Keyboard"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'mouseSettingsCreated'
        QtMocHelpers::SignalData<void(MouseSettingsInterfaceV1 *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'touchpadSettingsCreated'
        QtMocHelpers::SignalData<void(TouchpadSettingsInterfaceV1 *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 4 },
        }}),
        // Signal 'keyboardSettingsCreated'
        QtMocHelpers::SignalData<void(KeyboardSettingsInterfaceV1 *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 4 },
        }}),
        // Slot 'onInputAdded'
        QtMocHelpers::SlotData<void(WInputDevice *)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'onInputRemoved'
        QtMocHelpers::SlotData<void(WInputDevice *)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // flag 'DeviceTypes'
        QtMocHelpers::EnumData<DeviceTypes>(13, 14, QMC::EnumIsFlag | QMC::EnumIsScoped).add({
            {   15, DeviceType::Unknown },
            {   16, DeviceType::Mouse },
            {   17, DeviceType::TouchPad },
            {   18, DeviceType::Keyboard },
        }),
    };
    return QtMocHelpers::metaObjectData<TreelandInputManagerInterfaceV1, qt_meta_tag_ZN31TreelandInputManagerInterfaceV1E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TreelandInputManagerInterfaceV1::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31TreelandInputManagerInterfaceV1E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31TreelandInputManagerInterfaceV1E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN31TreelandInputManagerInterfaceV1E_t>.metaTypes,
    nullptr
} };

void TreelandInputManagerInterfaceV1::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TreelandInputManagerInterfaceV1 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->mouseSettingsCreated((*reinterpret_cast<std::add_pointer_t<MouseSettingsInterfaceV1*>>(_a[1]))); break;
        case 1: _t->touchpadSettingsCreated((*reinterpret_cast<std::add_pointer_t<TouchpadSettingsInterfaceV1*>>(_a[1]))); break;
        case 2: _t->keyboardSettingsCreated((*reinterpret_cast<std::add_pointer_t<KeyboardSettingsInterfaceV1*>>(_a[1]))); break;
        case 3: _t->onInputAdded((*reinterpret_cast<std::add_pointer_t<WInputDevice*>>(_a[1]))); break;
        case 4: _t->onInputRemoved((*reinterpret_cast<std::add_pointer_t<WInputDevice*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MouseSettingsInterfaceV1* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TouchpadSettingsInterfaceV1* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KeyboardSettingsInterfaceV1* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TreelandInputManagerInterfaceV1::*)(MouseSettingsInterfaceV1 * )>(_a, &TreelandInputManagerInterfaceV1::mouseSettingsCreated, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandInputManagerInterfaceV1::*)(TouchpadSettingsInterfaceV1 * )>(_a, &TreelandInputManagerInterfaceV1::touchpadSettingsCreated, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandInputManagerInterfaceV1::*)(KeyboardSettingsInterfaceV1 * )>(_a, &TreelandInputManagerInterfaceV1::keyboardSettingsCreated, 2))
            return;
    }
}

const QMetaObject *TreelandInputManagerInterfaceV1::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TreelandInputManagerInterfaceV1::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31TreelandInputManagerInterfaceV1E_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "WServerInterface"))
        return static_cast< WServerInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int TreelandInputManagerInterfaceV1::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void TreelandInputManagerInterfaceV1::mouseSettingsCreated(MouseSettingsInterfaceV1 * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void TreelandInputManagerInterfaceV1::touchpadSettingsCreated(TouchpadSettingsInterfaceV1 * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void TreelandInputManagerInterfaceV1::keyboardSettingsCreated(KeyboardSettingsInterfaceV1 * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN28PointerDeviceConfigurationV1E_t {};
} // unnamed namespace

template <> constexpr inline auto PointerDeviceConfigurationV1::qt_create_metaobjectdata<qt_meta_tag_ZN28PointerDeviceConfigurationV1E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PointerDeviceConfigurationV1",
        "applied",
        "",
        "ChangeFlags",
        "changes",
        "FeatureFlags",
        "FeatureFlag",
        "NoFeature",
        "ScrollFactor",
        "HandMode",
        "AccelSpeed",
        "AccelProfile",
        "EventsMode",
        "NaturalScroll",
        "DisableWhileTyping",
        "TapToClick",
        "ChangeFlag",
        "NoChange",
        "ScrollFactorChanged",
        "HandedModeChanged",
        "AccelSpeedChanged",
        "AccelerationProfileChanged",
        "SendEventsModeChanged",
        "NaturalScrollChanged",
        "DisableWhileTypingChanged",
        "TapToClickChanged",
        "SendEventsModes",
        "SendEventsMode",
        "Enabled",
        "Disabled",
        "DisabledOnExternalMouse"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'applied'
        QtMocHelpers::SignalData<void(ChangeFlags)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // flag 'FeatureFlags'
        QtMocHelpers::EnumData<FeatureFlags>(5, 6, QMC::EnumIsFlag).add({
            {    7, FeatureFlag::NoFeature },
            {    8, FeatureFlag::ScrollFactor },
            {    9, FeatureFlag::HandMode },
            {   10, FeatureFlag::AccelSpeed },
            {   11, FeatureFlag::AccelProfile },
            {   12, FeatureFlag::EventsMode },
            {   13, FeatureFlag::NaturalScroll },
            {   14, FeatureFlag::DisableWhileTyping },
            {   15, FeatureFlag::TapToClick },
        }),
        // flag 'ChangeFlags'
        QtMocHelpers::EnumData<ChangeFlags>(3, 16, QMC::EnumIsFlag).add({
            {   17, ChangeFlag::NoChange },
            {   18, ChangeFlag::ScrollFactorChanged },
            {   19, ChangeFlag::HandedModeChanged },
            {   20, ChangeFlag::AccelSpeedChanged },
            {   21, ChangeFlag::AccelerationProfileChanged },
            {   22, ChangeFlag::SendEventsModeChanged },
            {   23, ChangeFlag::NaturalScrollChanged },
            {   24, ChangeFlag::DisableWhileTypingChanged },
            {   25, ChangeFlag::TapToClickChanged },
        }),
        // flag 'SendEventsModes'
        QtMocHelpers::EnumData<SendEventsModes>(26, 27, QMC::EnumIsFlag).add({
            {   28, SendEventsMode::Enabled },
            {   29, SendEventsMode::Disabled },
            {   30, SendEventsMode::DisabledOnExternalMouse },
        }),
    };
    return QtMocHelpers::metaObjectData<PointerDeviceConfigurationV1, qt_meta_tag_ZN28PointerDeviceConfigurationV1E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PointerDeviceConfigurationV1::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28PointerDeviceConfigurationV1E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28PointerDeviceConfigurationV1E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN28PointerDeviceConfigurationV1E_t>.metaTypes,
    nullptr
} };

void PointerDeviceConfigurationV1::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PointerDeviceConfigurationV1 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->applied((*reinterpret_cast<std::add_pointer_t<ChangeFlags>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PointerDeviceConfigurationV1::*)(ChangeFlags )>(_a, &PointerDeviceConfigurationV1::applied, 0))
            return;
    }
}

const QMetaObject *PointerDeviceConfigurationV1::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PointerDeviceConfigurationV1::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28PointerDeviceConfigurationV1E_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int PointerDeviceConfigurationV1::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void PointerDeviceConfigurationV1::applied(ChangeFlags _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN24MouseSettingsInterfaceV1E_t {};
} // unnamed namespace

template <> constexpr inline auto MouseSettingsInterfaceV1::qt_create_metaobjectdata<qt_meta_tag_ZN24MouseSettingsInterfaceV1E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MouseSettingsInterfaceV1",
        "pointerDeviceConfigurationCreated",
        "",
        "PointerDeviceConfigurationV1*",
        "config"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'pointerDeviceConfigurationCreated'
        QtMocHelpers::SignalData<void(PointerDeviceConfigurationV1 *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MouseSettingsInterfaceV1, qt_meta_tag_ZN24MouseSettingsInterfaceV1E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MouseSettingsInterfaceV1::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24MouseSettingsInterfaceV1E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24MouseSettingsInterfaceV1E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN24MouseSettingsInterfaceV1E_t>.metaTypes,
    nullptr
} };

void MouseSettingsInterfaceV1::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MouseSettingsInterfaceV1 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->pointerDeviceConfigurationCreated((*reinterpret_cast<std::add_pointer_t<PointerDeviceConfigurationV1*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PointerDeviceConfigurationV1* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MouseSettingsInterfaceV1::*)(PointerDeviceConfigurationV1 * )>(_a, &MouseSettingsInterfaceV1::pointerDeviceConfigurationCreated, 0))
            return;
    }
}

const QMetaObject *MouseSettingsInterfaceV1::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MouseSettingsInterfaceV1::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24MouseSettingsInterfaceV1E_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int MouseSettingsInterfaceV1::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void MouseSettingsInterfaceV1::pointerDeviceConfigurationCreated(PointerDeviceConfigurationV1 * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN27TouchpadSettingsInterfaceV1E_t {};
} // unnamed namespace

template <> constexpr inline auto TouchpadSettingsInterfaceV1::qt_create_metaobjectdata<qt_meta_tag_ZN27TouchpadSettingsInterfaceV1E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TouchpadSettingsInterfaceV1",
        "pointerDeviceConfigurationCreated",
        "",
        "PointerDeviceConfigurationV1*",
        "config"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'pointerDeviceConfigurationCreated'
        QtMocHelpers::SignalData<void(PointerDeviceConfigurationV1 *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TouchpadSettingsInterfaceV1, qt_meta_tag_ZN27TouchpadSettingsInterfaceV1E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TouchpadSettingsInterfaceV1::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27TouchpadSettingsInterfaceV1E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27TouchpadSettingsInterfaceV1E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN27TouchpadSettingsInterfaceV1E_t>.metaTypes,
    nullptr
} };

void TouchpadSettingsInterfaceV1::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TouchpadSettingsInterfaceV1 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->pointerDeviceConfigurationCreated((*reinterpret_cast<std::add_pointer_t<PointerDeviceConfigurationV1*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PointerDeviceConfigurationV1* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TouchpadSettingsInterfaceV1::*)(PointerDeviceConfigurationV1 * )>(_a, &TouchpadSettingsInterfaceV1::pointerDeviceConfigurationCreated, 0))
            return;
    }
}

const QMetaObject *TouchpadSettingsInterfaceV1::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TouchpadSettingsInterfaceV1::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27TouchpadSettingsInterfaceV1E_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int TouchpadSettingsInterfaceV1::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void TouchpadSettingsInterfaceV1::pointerDeviceConfigurationCreated(PointerDeviceConfigurationV1 * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN27KeyboardSettingsInterfaceV1E_t {};
} // unnamed namespace

template <> constexpr inline auto KeyboardSettingsInterfaceV1::qt_create_metaobjectdata<qt_meta_tag_ZN27KeyboardSettingsInterfaceV1E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KeyboardSettingsInterfaceV1",
        "applied",
        "",
        "ChangeFlags",
        "changes",
        "FeatureFlags",
        "FeatureFlag",
        "NoFeature",
        "NumLock",
        "ChangeFlag",
        "NoChange",
        "RepeatChanged",
        "NumLockChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'applied'
        QtMocHelpers::SignalData<void(ChangeFlags)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // flag 'FeatureFlags'
        QtMocHelpers::EnumData<FeatureFlags>(5, 6, QMC::EnumIsFlag).add({
            {    7, FeatureFlag::NoFeature },
            {    8, FeatureFlag::NumLock },
        }),
        // flag 'ChangeFlags'
        QtMocHelpers::EnumData<ChangeFlags>(3, 9, QMC::EnumIsFlag).add({
            {   10, ChangeFlag::NoChange },
            {   11, ChangeFlag::RepeatChanged },
            {   12, ChangeFlag::NumLockChanged },
        }),
    };
    return QtMocHelpers::metaObjectData<KeyboardSettingsInterfaceV1, qt_meta_tag_ZN27KeyboardSettingsInterfaceV1E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KeyboardSettingsInterfaceV1::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27KeyboardSettingsInterfaceV1E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27KeyboardSettingsInterfaceV1E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN27KeyboardSettingsInterfaceV1E_t>.metaTypes,
    nullptr
} };

void KeyboardSettingsInterfaceV1::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KeyboardSettingsInterfaceV1 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->applied((*reinterpret_cast<std::add_pointer_t<ChangeFlags>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KeyboardSettingsInterfaceV1::*)(ChangeFlags )>(_a, &KeyboardSettingsInterfaceV1::applied, 0))
            return;
    }
}

const QMetaObject *KeyboardSettingsInterfaceV1::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KeyboardSettingsInterfaceV1::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27KeyboardSettingsInterfaceV1E_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KeyboardSettingsInterfaceV1::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void KeyboardSettingsInterfaceV1::applied(ChangeFlags _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
