/****************************************************************************
** Meta object code from reading C++ file 'wglobal.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../waylib/src/server/kernel/wglobal.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wglobal.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Waylib6Server7WGlobalE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WGlobal::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server7WGlobalE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WGlobal",
        "QML.Element",
        "wglobal",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Use for enums",
        "ColorContentsMode",
        "DontCare",
        "Clear",
        "Preserve",
        "CursorShape",
        "Default",
        "Invalid",
        "ClientResource",
        "BottomLeftCorner",
        "BottomRightCorner",
        "TopLeftCorner",
        "TopRightCorner",
        "BottomSide",
        "LeftSide",
        "RightSide",
        "TopSide",
        "Grabbing",
        "Xterm",
        "Hand1",
        "Watch",
        "SWResize",
        "SEResize",
        "SResize",
        "WResize",
        "EResize",
        "EWResize",
        "NWResize",
        "NWSEResize",
        "NEResize",
        "NESWResize",
        "NSResize",
        "NResize",
        "AllScroll",
        "Text",
        "Pointer",
        "Wait",
        "ContextMenu",
        "Help",
        "Progress",
        "Cell",
        "Crosshair",
        "VerticalText",
        "Alias",
        "Copy",
        "Move",
        "NoDrop",
        "NotAllowed",
        "Grab",
        "ColResize",
        "RowResize",
        "ZoomIn",
        "ZoomOut",
        "DndAsk",
        "AllResize"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ColorContentsMode'
        QtMocHelpers::EnumData<enum ColorContentsMode>(7, 7, QMC::EnumIsScoped).add({
            {    8, ColorContentsMode::DontCare },
            {    9, ColorContentsMode::Clear },
            {   10, ColorContentsMode::Preserve },
        }),
        // enum 'CursorShape'
        QtMocHelpers::EnumData<enum CursorShape>(11, 11, QMC::EnumIsScoped).add({
            {   12, CursorShape::Default },
            {   13, CursorShape::Invalid },
            {   14, CursorShape::ClientResource },
            {   15, CursorShape::BottomLeftCorner },
            {   16, CursorShape::BottomRightCorner },
            {   17, CursorShape::TopLeftCorner },
            {   18, CursorShape::TopRightCorner },
            {   19, CursorShape::BottomSide },
            {   20, CursorShape::LeftSide },
            {   21, CursorShape::RightSide },
            {   22, CursorShape::TopSide },
            {   23, CursorShape::Grabbing },
            {   24, CursorShape::Xterm },
            {   25, CursorShape::Hand1 },
            {   26, CursorShape::Watch },
            {   27, CursorShape::SWResize },
            {   28, CursorShape::SEResize },
            {   29, CursorShape::SResize },
            {   30, CursorShape::WResize },
            {   31, CursorShape::EResize },
            {   32, CursorShape::EWResize },
            {   33, CursorShape::NWResize },
            {   34, CursorShape::NWSEResize },
            {   35, CursorShape::NEResize },
            {   36, CursorShape::NESWResize },
            {   37, CursorShape::NSResize },
            {   38, CursorShape::NResize },
            {   39, CursorShape::AllScroll },
            {   40, CursorShape::Text },
            {   41, CursorShape::Pointer },
            {   42, CursorShape::Wait },
            {   43, CursorShape::ContextMenu },
            {   44, CursorShape::Help },
            {   45, CursorShape::Progress },
            {   46, CursorShape::Cell },
            {   47, CursorShape::Crosshair },
            {   48, CursorShape::VerticalText },
            {   49, CursorShape::Alias },
            {   50, CursorShape::Copy },
            {   51, CursorShape::Move },
            {   52, CursorShape::NoDrop },
            {   53, CursorShape::NotAllowed },
            {   54, CursorShape::Grab },
            {   55, CursorShape::ColResize },
            {   56, CursorShape::RowResize },
            {   57, CursorShape::ZoomIn },
            {   58, CursorShape::ZoomOut },
            {   59, CursorShape::DndAsk },
            {   60, CursorShape::AllResize },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<WGlobal, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WGlobal::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server7WGlobalE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server7WGlobalE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server7WGlobalE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN6Waylib6Server20WCursorShapeQMLProxyE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WCursorShapeQMLProxy::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server20WCursorShapeQMLProxyE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WCursorShapeQMLProxy",
        "QML.Element",
        "CursorShape",
        "QML.Foreign",
        "WAYLIB_SERVER_NAMESPACE::WGlobal",
        "QML.ForeignIsNamespace",
        "true",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Use for enums"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
            {    9,   10 },
    });
    return QtMocHelpers::metaObjectData<WCursorShapeQMLProxy, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WCursorShapeQMLProxy::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server20WCursorShapeQMLProxyE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server20WCursorShapeQMLProxyE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server20WCursorShapeQMLProxyE_t>.metaTypes,
    nullptr
} };

QT_WARNING_POP
