/****************************************************************************
** Meta object code from reading C++ file 'capturev1impl.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../src/modules/capture/impl/capturev1impl.h"
#include "woutput.h"
#include <wsurface.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'capturev1impl.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN27treeland_capture_context_v1E_t {};
} // unnamed namespace

template <> constexpr inline auto treeland_capture_context_v1::qt_create_metaobjectdata<qt_meta_tag_ZN27treeland_capture_context_v1E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "treeland_capture_context_v1",
        "beforeDestroy",
        "",
        "newSession",
        "treeland_capture_session_v1*",
        "session",
        "selectSource",
        "capture",
        "treeland_capture_frame_v1*",
        "frame"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'beforeDestroy'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'newSession'
        QtMocHelpers::SignalData<void(treeland_capture_session_v1 *)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Signal 'selectSource'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'capture'
        QtMocHelpers::SignalData<void(treeland_capture_frame_v1 *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<treeland_capture_context_v1, qt_meta_tag_ZN27treeland_capture_context_v1E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject treeland_capture_context_v1::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27treeland_capture_context_v1E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27treeland_capture_context_v1E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN27treeland_capture_context_v1E_t>.metaTypes,
    nullptr
} };

void treeland_capture_context_v1::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<treeland_capture_context_v1 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->beforeDestroy(); break;
        case 1: _t->newSession((*reinterpret_cast<std::add_pointer_t<treeland_capture_session_v1*>>(_a[1]))); break;
        case 2: _t->selectSource(); break;
        case 3: _t->capture((*reinterpret_cast<std::add_pointer_t<treeland_capture_frame_v1*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< treeland_capture_session_v1* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< treeland_capture_frame_v1* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (treeland_capture_context_v1::*)()>(_a, &treeland_capture_context_v1::beforeDestroy, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (treeland_capture_context_v1::*)(treeland_capture_session_v1 * )>(_a, &treeland_capture_context_v1::newSession, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (treeland_capture_context_v1::*)()>(_a, &treeland_capture_context_v1::selectSource, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (treeland_capture_context_v1::*)(treeland_capture_frame_v1 * )>(_a, &treeland_capture_context_v1::capture, 3))
            return;
    }
}

const QMetaObject *treeland_capture_context_v1::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *treeland_capture_context_v1::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27treeland_capture_context_v1E_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int treeland_capture_context_v1::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void treeland_capture_context_v1::beforeDestroy()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void treeland_capture_context_v1::newSession(treeland_capture_session_v1 * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void treeland_capture_context_v1::selectSource()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void treeland_capture_context_v1::capture(treeland_capture_frame_v1 * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN25treeland_capture_frame_v1E_t {};
} // unnamed namespace

template <> constexpr inline auto treeland_capture_frame_v1::qt_create_metaobjectdata<qt_meta_tag_ZN25treeland_capture_frame_v1E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "treeland_capture_frame_v1",
        "copy",
        "",
        "wlr_buffer*",
        "buffer",
        "beforeDestroy"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'copy'
        QtMocHelpers::SignalData<void(wlr_buffer *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'beforeDestroy'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<treeland_capture_frame_v1, qt_meta_tag_ZN25treeland_capture_frame_v1E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject treeland_capture_frame_v1::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25treeland_capture_frame_v1E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25treeland_capture_frame_v1E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN25treeland_capture_frame_v1E_t>.metaTypes,
    nullptr
} };

void treeland_capture_frame_v1::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<treeland_capture_frame_v1 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->copy((*reinterpret_cast<std::add_pointer_t<wlr_buffer*>>(_a[1]))); break;
        case 1: _t->beforeDestroy(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (treeland_capture_frame_v1::*)(wlr_buffer * )>(_a, &treeland_capture_frame_v1::copy, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (treeland_capture_frame_v1::*)()>(_a, &treeland_capture_frame_v1::beforeDestroy, 1))
            return;
    }
}

const QMetaObject *treeland_capture_frame_v1::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *treeland_capture_frame_v1::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25treeland_capture_frame_v1E_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int treeland_capture_frame_v1::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void treeland_capture_frame_v1::copy(wlr_buffer * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void treeland_capture_frame_v1::beforeDestroy()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
namespace {
struct qt_meta_tag_ZN27treeland_capture_session_v1E_t {};
} // unnamed namespace

template <> constexpr inline auto treeland_capture_session_v1::qt_create_metaobjectdata<qt_meta_tag_ZN27treeland_capture_session_v1E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "treeland_capture_session_v1",
        "beforeDestroy",
        "",
        "start",
        "frameDone",
        "uint32_t",
        "tvSecHi",
        "tvSecLo",
        "tvUsec"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'beforeDestroy'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'start'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'frameDone'
        QtMocHelpers::SignalData<void(uint32_t, uint32_t, uint32_t)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 5, 7 }, { 0x80000000 | 5, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<treeland_capture_session_v1, qt_meta_tag_ZN27treeland_capture_session_v1E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject treeland_capture_session_v1::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27treeland_capture_session_v1E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27treeland_capture_session_v1E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN27treeland_capture_session_v1E_t>.metaTypes,
    nullptr
} };

void treeland_capture_session_v1::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<treeland_capture_session_v1 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->beforeDestroy(); break;
        case 1: _t->start(); break;
        case 2: _t->frameDone((*reinterpret_cast<std::add_pointer_t<uint32_t>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint32_t>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<uint32_t>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (treeland_capture_session_v1::*)()>(_a, &treeland_capture_session_v1::beforeDestroy, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (treeland_capture_session_v1::*)()>(_a, &treeland_capture_session_v1::start, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (treeland_capture_session_v1::*)(uint32_t , uint32_t , uint32_t )>(_a, &treeland_capture_session_v1::frameDone, 2))
            return;
    }
}

const QMetaObject *treeland_capture_session_v1::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *treeland_capture_session_v1::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27treeland_capture_session_v1E_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int treeland_capture_session_v1::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void treeland_capture_session_v1::beforeDestroy()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void treeland_capture_session_v1::start()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void treeland_capture_session_v1::frameDone(uint32_t _t1, uint32_t _t2, uint32_t _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3);
}
namespace {
struct qt_meta_tag_ZN27treeland_capture_manager_v1E_t {};
} // unnamed namespace

template <> constexpr inline auto treeland_capture_manager_v1::qt_create_metaobjectdata<qt_meta_tag_ZN27treeland_capture_manager_v1E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "treeland_capture_manager_v1",
        "newCaptureContext",
        "",
        "treeland_capture_context_v1*",
        "context"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'newCaptureContext'
        QtMocHelpers::SignalData<void(treeland_capture_context_v1 *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<treeland_capture_manager_v1, qt_meta_tag_ZN27treeland_capture_manager_v1E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject treeland_capture_manager_v1::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27treeland_capture_manager_v1E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27treeland_capture_manager_v1E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN27treeland_capture_manager_v1E_t>.metaTypes,
    nullptr
} };

void treeland_capture_manager_v1::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<treeland_capture_manager_v1 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->newCaptureContext((*reinterpret_cast<std::add_pointer_t<treeland_capture_context_v1*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< treeland_capture_context_v1* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (treeland_capture_manager_v1::*)(treeland_capture_context_v1 * )>(_a, &treeland_capture_manager_v1::newCaptureContext, 0))
            return;
    }
}

const QMetaObject *treeland_capture_manager_v1::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *treeland_capture_manager_v1::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN27treeland_capture_manager_v1E_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int treeland_capture_manager_v1::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void treeland_capture_manager_v1::newCaptureContext(treeland_capture_context_v1 * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
