/****************************************************************************
** Meta object code from reading C++ file 'workspace.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/workspace/workspace.h"
#include "woutput.h"
#include <woutput.h>
#include <output/output.h>
#include "core/rootsurfacecontainer.h"
#include "workspace/workspaceanimationcontroller.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'workspace.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18WorkspaceListModelE_t {};
} // unnamed namespace

template <> constexpr inline auto WorkspaceListModel::qt_create_metaobjectdata<qt_meta_tag_ZN18WorkspaceListModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "WorkspaceListModel",
        "QML.Element",
        "anonymous"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<WorkspaceListModel, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject WorkspaceListModel::staticMetaObject = { {
    QMetaObject::SuperData::link<ObjectListModel<WorkspaceModel>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18WorkspaceListModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18WorkspaceListModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18WorkspaceListModelE_t>.metaTypes,
    nullptr
} };

void WorkspaceListModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WorkspaceListModel *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *WorkspaceListModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *WorkspaceListModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18WorkspaceListModelE_t>.strings))
        return static_cast<void*>(this);
    return ObjectListModel<WorkspaceModel>::qt_metacast(_clname);
}

int WorkspaceListModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ObjectListModel<WorkspaceModel>::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN9WorkspaceE_t {};
} // unnamed namespace

template <> constexpr inline auto Workspace::qt_create_metaobjectdata<qt_meta_tag_ZN9WorkspaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Workspace",
        "QML.Element",
        "auto",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Created only in C++",
        "currentChanged",
        "",
        "currentFilterChanged",
        "currentIndexChanged",
        "countChanged",
        "workspaceAdded",
        "moveSurfaceTo",
        "SurfaceWrapper*",
        "surface",
        "workspaceId",
        "getLeftWorkspaceId",
        "getRightWorkspaceId",
        "moveModelTo",
        "destinationIndex",
        "createModel",
        "name",
        "visible",
        "removeModel",
        "index",
        "modelAt",
        "WorkspaceModel*",
        "modelFromId",
        "id",
        "switchToNext",
        "switchToPrev",
        "switchTo",
        "animated",
        "startPreviewing",
        "previewingItem",
        "stopPreviewing",
        "currentIndex",
        "current",
        "currentFilter",
        "SurfaceFilterProxyModel*",
        "showOnAllWorkspaceModel",
        "models",
        "WorkspaceListModel*",
        "animationController",
        "WorkspaceAnimationController*",
        "count"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'currentChanged'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'currentFilterChanged'
        QtMocHelpers::SignalData<void()>(9, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'currentIndexChanged'
        QtMocHelpers::SignalData<void()>(10, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'countChanged'
        QtMocHelpers::SignalData<void()>(11, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'workspaceAdded'
        QtMocHelpers::SignalData<void()>(12, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'moveSurfaceTo'
        QtMocHelpers::MethodData<void(SurfaceWrapper *, int)>(13, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 }, { QMetaType::Int, 16 },
        }}),
        // Method 'moveSurfaceTo'
        QtMocHelpers::MethodData<void(SurfaceWrapper *)>(13, 8, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Method 'getLeftWorkspaceId'
        QtMocHelpers::MethodData<int(int)>(17, 8, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::Int, 16 },
        }}),
        // Method 'getRightWorkspaceId'
        QtMocHelpers::MethodData<int(int)>(18, 8, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::Int, 16 },
        }}),
        // Method 'moveModelTo'
        QtMocHelpers::MethodData<void(int, int)>(19, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 16 }, { QMetaType::Int, 20 },
        }}),
        // Method 'createModel'
        QtMocHelpers::MethodData<int(const QString &, bool)>(21, 8, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 22 }, { QMetaType::Bool, 23 },
        }}),
        // Method 'createModel'
        QtMocHelpers::MethodData<int(const QString &)>(21, 8, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int, {{
            { QMetaType::QString, 22 },
        }}),
        // Method 'createModel'
        QtMocHelpers::MethodData<int()>(21, 8, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Int),
        // Method 'removeModel'
        QtMocHelpers::MethodData<void(int)>(24, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 25 },
        }}),
        // Method 'modelAt'
        QtMocHelpers::MethodData<WorkspaceModel *(int) const>(26, 8, QMC::AccessPublic, 0x80000000 | 27, {{
            { QMetaType::Int, 25 },
        }}),
        // Method 'modelFromId'
        QtMocHelpers::MethodData<WorkspaceModel *(int) const>(28, 8, QMC::AccessPublic, 0x80000000 | 27, {{
            { QMetaType::Int, 29 },
        }}),
        // Method 'switchToNext'
        QtMocHelpers::MethodData<void()>(30, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'switchToPrev'
        QtMocHelpers::MethodData<void()>(31, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'switchTo'
        QtMocHelpers::MethodData<void(int, bool)>(32, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 25 }, { QMetaType::Bool, 33 },
        }}),
        // Method 'switchTo'
        QtMocHelpers::MethodData<void(int)>(32, 8, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 25 },
        }}),
        // Method 'startPreviewing'
        QtMocHelpers::MethodData<void(SurfaceWrapper *)>(34, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 35 },
        }}),
        // Method 'stopPreviewing'
        QtMocHelpers::MethodData<void()>(36, 8, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'currentIndex'
        QtMocHelpers::PropertyData<int>(37, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 2),
        // property 'current'
        QtMocHelpers::PropertyData<WorkspaceModel*>(38, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 0),
        // property 'currentFilter'
        QtMocHelpers::PropertyData<SurfaceFilterProxyModel*>(39, 0x80000000 | 40, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 1),
        // property 'showOnAllWorkspaceModel'
        QtMocHelpers::PropertyData<WorkspaceModel*>(41, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant | QMC::Final),
        // property 'models'
        QtMocHelpers::PropertyData<WorkspaceListModel*>(42, 0x80000000 | 43, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant | QMC::Final),
        // property 'animationController'
        QtMocHelpers::PropertyData<WorkspaceAnimationController*>(44, 0x80000000 | 45, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant | QMC::Final),
        // property 'count'
        QtMocHelpers::PropertyData<int>(46, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Final, 3),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<Workspace, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Workspace::staticMetaObject = { {
    QMetaObject::SuperData::link<SurfaceContainer::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9WorkspaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9WorkspaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9WorkspaceE_t>.metaTypes,
    nullptr
} };

void Workspace::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Workspace *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->currentChanged(); break;
        case 1: _t->currentFilterChanged(); break;
        case 2: _t->currentIndexChanged(); break;
        case 3: _t->countChanged(); break;
        case 4: _t->workspaceAdded(); break;
        case 5: _t->moveSurfaceTo((*reinterpret_cast<std::add_pointer_t<SurfaceWrapper*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 6: _t->moveSurfaceTo((*reinterpret_cast<std::add_pointer_t<SurfaceWrapper*>>(_a[1]))); break;
        case 7: { int _r = _t->getLeftWorkspaceId((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 8: { int _r = _t->getRightWorkspaceId((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 9: _t->moveModelTo((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 10: { int _r = _t->createModel((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 11: { int _r = _t->createModel((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 12: { int _r = _t->createModel();
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 13: _t->removeModel((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 14: { WorkspaceModel* _r = _t->modelAt((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<WorkspaceModel**>(_a[0]) = std::move(_r); }  break;
        case 15: { WorkspaceModel* _r = _t->modelFromId((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<WorkspaceModel**>(_a[0]) = std::move(_r); }  break;
        case 16: _t->switchToNext(); break;
        case 17: _t->switchToPrev(); break;
        case 18: _t->switchTo((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 19: _t->switchTo((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 20: _t->startPreviewing((*reinterpret_cast<std::add_pointer_t<SurfaceWrapper*>>(_a[1]))); break;
        case 21: _t->stopPreviewing(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< SurfaceWrapper* >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< SurfaceWrapper* >(); break;
            }
            break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< SurfaceWrapper* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Workspace::*)()>(_a, &Workspace::currentChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Workspace::*)()>(_a, &Workspace::currentFilterChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Workspace::*)()>(_a, &Workspace::currentIndexChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Workspace::*)()>(_a, &Workspace::countChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (Workspace::*)()>(_a, &Workspace::workspaceAdded, 4))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< SurfaceFilterProxyModel* >(); break;
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< WorkspaceListModel* >(); break;
        case 3:
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< WorkspaceModel* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->currentIndex(); break;
        case 1: *reinterpret_cast<WorkspaceModel**>(_v) = _t->current(); break;
        case 2: *reinterpret_cast<SurfaceFilterProxyModel**>(_v) = _t->currentFilter(); break;
        case 3: *reinterpret_cast<WorkspaceModel**>(_v) = _t->showOnAllWorkspaceModel(); break;
        case 4: *reinterpret_cast<WorkspaceListModel**>(_v) = _t->models(); break;
        case 5: *reinterpret_cast<WorkspaceAnimationController**>(_v) = _t->animationController(); break;
        case 6: *reinterpret_cast<int*>(_v) = _t->count(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setCurrentIndex(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Workspace::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Workspace::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9WorkspaceE_t>.strings))
        return static_cast<void*>(this);
    return SurfaceContainer::qt_metacast(_clname);
}

int Workspace::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SurfaceContainer::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void Workspace::currentChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Workspace::currentFilterChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Workspace::currentIndexChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Workspace::countChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Workspace::workspaceAdded()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
