/* Generated by wayland-scanner 1.26.0 */

/*
 * SPDX-FileCopyrightText: 2024 UnionTech Software Technology Co., Ltd.
 * SPDX-License-Identifier: MIT
 */

#include <stdbool.h>
#include <stdlib.h>
#include <stdint.h>
#include "wayland-util.h"

#ifndef __has_attribute
# define __has_attribute(x) 0  /* Compatibility with non-clang compilers. */
#endif

#if (__has_attribute(visibility) || defined(__GNUC__) && __GNUC__ >= 4)
#define WL_PRIVATE __attribute__ ((visibility("hidden")))
#else
#define WL_PRIVATE
#endif

extern const struct wl_interface treeland_dock_preview_context_v1_interface;
extern const struct wl_interface treeland_foreign_toplevel_handle_v1_interface;
extern const struct wl_interface wl_output_interface;
extern const struct wl_interface wl_seat_interface;
extern const struct wl_interface wl_surface_interface;

static const struct wl_interface *treeland_foreign_toplevel_manager_v1_types[] = {
	NULL,
	NULL,
	NULL,
	NULL,
	&wl_surface_interface,
	&treeland_dock_preview_context_v1_interface,
	&treeland_foreign_toplevel_handle_v1_interface,
	&wl_seat_interface,
	&wl_surface_interface,
	NULL,
	NULL,
	NULL,
	NULL,
	&wl_output_interface,
	&wl_output_interface,
	&wl_output_interface,
	&treeland_foreign_toplevel_handle_v1_interface,
};

static const struct wl_message treeland_foreign_toplevel_manager_v1_requests[] = {
	{ "stop", "", treeland_foreign_toplevel_manager_v1_types + 0 },
	{ "get_dock_preview_context", "on", treeland_foreign_toplevel_manager_v1_types + 4 },
};

static const struct wl_message treeland_foreign_toplevel_manager_v1_events[] = {
	{ "toplevel", "n", treeland_foreign_toplevel_manager_v1_types + 6 },
	{ "finished", "", treeland_foreign_toplevel_manager_v1_types + 0 },
};

WL_PRIVATE const struct wl_interface treeland_foreign_toplevel_manager_v1_interface = {
	"treeland_foreign_toplevel_manager_v1", 2,
	2, treeland_foreign_toplevel_manager_v1_requests,
	2, treeland_foreign_toplevel_manager_v1_events,
};

static const struct wl_message treeland_foreign_toplevel_handle_v1_requests[] = {
	{ "set_maximized", "", treeland_foreign_toplevel_manager_v1_types + 0 },
	{ "unset_maximized", "", treeland_foreign_toplevel_manager_v1_types + 0 },
	{ "set_minimized", "", treeland_foreign_toplevel_manager_v1_types + 0 },
	{ "unset_minimized", "", treeland_foreign_toplevel_manager_v1_types + 0 },
	{ "activate", "o", treeland_foreign_toplevel_manager_v1_types + 7 },
	{ "close", "", treeland_foreign_toplevel_manager_v1_types + 0 },
	{ "set_rectangle", "oiiii", treeland_foreign_toplevel_manager_v1_types + 8 },
	{ "destroy", "", treeland_foreign_toplevel_manager_v1_types + 0 },
	{ "set_fullscreen", "?o", treeland_foreign_toplevel_manager_v1_types + 13 },
	{ "unset_fullscreen", "", treeland_foreign_toplevel_manager_v1_types + 0 },
};

static const struct wl_message treeland_foreign_toplevel_handle_v1_events[] = {
	{ "pid", "u", treeland_foreign_toplevel_manager_v1_types + 0 },
	{ "title", "s", treeland_foreign_toplevel_manager_v1_types + 0 },
	{ "app_id", "s", treeland_foreign_toplevel_manager_v1_types + 0 },
	{ "identifier", "u", treeland_foreign_toplevel_manager_v1_types + 0 },
	{ "output_enter", "o", treeland_foreign_toplevel_manager_v1_types + 14 },
	{ "output_leave", "o", treeland_foreign_toplevel_manager_v1_types + 15 },
	{ "state", "a", treeland_foreign_toplevel_manager_v1_types + 0 },
	{ "done", "", treeland_foreign_toplevel_manager_v1_types + 0 },
	{ "closed", "", treeland_foreign_toplevel_manager_v1_types + 0 },
	{ "parent", "?o", treeland_foreign_toplevel_manager_v1_types + 16 },
};

WL_PRIVATE const struct wl_interface treeland_foreign_toplevel_handle_v1_interface = {
	"treeland_foreign_toplevel_handle_v1", 2,
	10, treeland_foreign_toplevel_handle_v1_requests,
	10, treeland_foreign_toplevel_handle_v1_events,
};

static const struct wl_message treeland_dock_preview_context_v1_requests[] = {
	{ "show", "aiiu", treeland_foreign_toplevel_manager_v1_types + 0 },
	{ "show_tooltip", "siiu", treeland_foreign_toplevel_manager_v1_types + 0 },
	{ "close", "", treeland_foreign_toplevel_manager_v1_types + 0 },
	{ "destroy", "", treeland_foreign_toplevel_manager_v1_types + 0 },
};

static const struct wl_message treeland_dock_preview_context_v1_events[] = {
	{ "enter", "", treeland_foreign_toplevel_manager_v1_types + 0 },
	{ "leave", "", treeland_foreign_toplevel_manager_v1_types + 0 },
};

WL_PRIVATE const struct wl_interface treeland_dock_preview_context_v1_interface = {
	"treeland_dock_preview_context_v1", 1,
	4, treeland_dock_preview_context_v1_requests,
	2, treeland_dock_preview_context_v1_events,
};

