/****************************************************************************
** Meta object code from reading C++ file 'keyboardstatenotifymanagerinterfacev1.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/modules/keyboard-state-notify/keyboardstatenotifymanagerinterfacev1.h"
#include "private/wserver_p.h"
#include <wsurface.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'keyboardstatenotifymanagerinterfacev1.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN45TreelandKeyboardStateNotifyManagerInterfaceV1E_t {};
} // unnamed namespace

template <> constexpr inline auto TreelandKeyboardStateNotifyManagerInterfaceV1::qt_create_metaobjectdata<qt_meta_tag_ZN45TreelandKeyboardStateNotifyManagerInterfaceV1E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TreelandKeyboardStateNotifyManagerInterfaceV1",
        "keyboardStateWatcherCreated",
        "",
        "KeyboardStateWatcherV1*",
        "watcher"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'keyboardStateWatcherCreated'
        QtMocHelpers::SignalData<void(KeyboardStateWatcherV1 *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TreelandKeyboardStateNotifyManagerInterfaceV1, qt_meta_tag_ZN45TreelandKeyboardStateNotifyManagerInterfaceV1E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TreelandKeyboardStateNotifyManagerInterfaceV1::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN45TreelandKeyboardStateNotifyManagerInterfaceV1E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN45TreelandKeyboardStateNotifyManagerInterfaceV1E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN45TreelandKeyboardStateNotifyManagerInterfaceV1E_t>.metaTypes,
    nullptr
} };

void TreelandKeyboardStateNotifyManagerInterfaceV1::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TreelandKeyboardStateNotifyManagerInterfaceV1 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->keyboardStateWatcherCreated((*reinterpret_cast<std::add_pointer_t<KeyboardStateWatcherV1*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KeyboardStateWatcherV1* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TreelandKeyboardStateNotifyManagerInterfaceV1::*)(KeyboardStateWatcherV1 * )>(_a, &TreelandKeyboardStateNotifyManagerInterfaceV1::keyboardStateWatcherCreated, 0))
            return;
    }
}

const QMetaObject *TreelandKeyboardStateNotifyManagerInterfaceV1::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TreelandKeyboardStateNotifyManagerInterfaceV1::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN45TreelandKeyboardStateNotifyManagerInterfaceV1E_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "WServerInterface"))
        return static_cast< WServerInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int TreelandKeyboardStateNotifyManagerInterfaceV1::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void TreelandKeyboardStateNotifyManagerInterfaceV1::keyboardStateWatcherCreated(KeyboardStateWatcherV1 * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN22KeyboardStateWatcherV1E_t {};
} // unnamed namespace

template <> constexpr inline auto KeyboardStateWatcherV1::qt_create_metaobjectdata<qt_meta_tag_ZN22KeyboardStateWatcherV1E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KeyboardStateWatcherV1",
        "ModifierTypes",
        "ModifierType",
        "CapsLock",
        "NumLock",
        "WatchFlags",
        "WatchFlag",
        "WatchLocked",
        "WatchUnlocked",
        "ModifierState",
        "Unlocked",
        "Locked"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // flag 'ModifierTypes'
        QtMocHelpers::EnumData<ModifierTypes>(1, 2, QMC::EnumIsFlag).add({
            {    3, ModifierType::CapsLock },
            {    4, ModifierType::NumLock },
        }),
        // flag 'WatchFlags'
        QtMocHelpers::EnumData<WatchFlags>(5, 6, QMC::EnumIsFlag).add({
            {    7, WatchFlag::WatchLocked },
            {    8, WatchFlag::WatchUnlocked },
        }),
        // enum 'ModifierState'
        QtMocHelpers::EnumData<enum ModifierState>(9, 9, QMC::EnumFlags{}).add({
            {   10, ModifierState::Unlocked },
            {   11, ModifierState::Locked },
        }),
    };
    return QtMocHelpers::metaObjectData<KeyboardStateWatcherV1, qt_meta_tag_ZN22KeyboardStateWatcherV1E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KeyboardStateWatcherV1::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22KeyboardStateWatcherV1E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22KeyboardStateWatcherV1E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22KeyboardStateWatcherV1E_t>.metaTypes,
    nullptr
} };

void KeyboardStateWatcherV1::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KeyboardStateWatcherV1 *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KeyboardStateWatcherV1::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KeyboardStateWatcherV1::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22KeyboardStateWatcherV1E_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KeyboardStateWatcherV1::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
