/****************************************************************************
** Meta object code from reading C++ file 'surfacewrapper.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/surface/surfacewrapper.h"
#include "woutput.h"
#include <woutput.h>
#include <output/output.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QProperty>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'surfacewrapper.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14SurfaceWrapperE_t {};
} // unnamed namespace

template <> constexpr inline auto SurfaceWrapper::qt_create_metaobjectdata<qt_meta_tag_ZN14SurfaceWrapperE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SurfaceWrapper",
        "QML.Element",
        "auto",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "SurfaceWrapper objects are created by c++",
        "hasInitializeContainerChanged",
        "",
        "boundingRectChanged",
        "ownsOutputChanged",
        "appIdChanged",
        "normalGeometryChanged",
        "maximizedGeometryChanged",
        "fullscreenGeometryChanged",
        "tilingGeometryChanged",
        "positionAutomaticChanged",
        "previousSurfaceStateChanged",
        "surfaceStateChanged",
        "radiusChanged",
        "moveRequested",
        "resizeRequested",
        "Qt::Edges",
        "edges",
        "windowMenuRequested",
        "QPointF",
        "pos",
        "geometryChanged",
        "containerChanged",
        "visibleDecorationChanged",
        "clipInOutputChanged",
        "noDecorationChanged",
        "noTitleBarChanged",
        "noCornerRadiusChanged",
        "iconGeometryChanged",
        "workspaceIdChanged",
        "alwaysOnTopChanged",
        "showOnAllWorkspaceChanged",
        "activationRequested",
        "inactivationRequested",
        "splashCloseRequested",
        "skipSwitcherChanged",
        "skipDockPreViewChanged",
        "skipMutiTaskViewChanged",
        "isDDEShellSurfaceChanged",
        "surfaceRoleChanged",
        "autoPlaceYOffsetChanged",
        "clientRequstPosChanged",
        "blurChanged",
        "windowAnimationRunningChanged",
        "coverEnabledChanged",
        "aboutToBeInvalidated",
        "isActivatedChanged",
        "isIMCandidatePanelChanged",
        "resizableChanged",
        "maximizableChanged",
        "modalChanged",
        "attentionChanged",
        "surfaceItemCreated",
        "hasFocusCapabilityChanged",
        "prelaunchSplashChanged",
        "typeChanged",
        "minimize",
        "onAnimation",
        "restoreFromMinimized",
        "maximize",
        "unmaximize",
        "toggleMaximized",
        "enterFullscreen",
        "leaveFullscreen",
        "closeSurface",
        "onMappedChanged",
        "onSocketEnabledChanged",
        "stackBefore",
        "QQuickItem*",
        "item",
        "stackAfter",
        "stackToLast",
        "stackToFirst",
        "updateSurfaceSizeRatio",
        "onAnimationReady",
        "onAnimationFinished",
        "onPrelaunchGeometryAnimationReady",
        "onPrelaunchGeometryAnimationFinished",
        "onShowAnimationFinished",
        "onHideAnimationFinished",
        "onMinimizeAnimationFinished",
        "onShowDesktopAnimationFinished",
        "implicitWidthChanged",
        "implicitHeightChanged",
        "type",
        "Type",
        "appId",
        "implicitWidth",
        "implicitHeight",
        "surface",
        "Waylib::Server::WSurface*",
        "shellSurface",
        "Waylib::Server::WToplevelSurface*",
        "surfaceItem",
        "Waylib::Server::WSurfaceItem*",
        "prelaunchSplash",
        "boundingRect",
        "QRectF",
        "geometry",
        "normalGeometry",
        "maximizedGeometry",
        "fullscreenGeometry",
        "tilingGeometry",
        "ownsOutput",
        "Output*",
        "positionAutomatic",
        "previousSurfaceState",
        "State",
        "surfaceState",
        "radius",
        "container",
        "SurfaceContainer*",
        "titleBar",
        "decoration",
        "visibleDecoration",
        "clipInOutput",
        "noTitleBar",
        "noCornerRadius",
        "workspaceId",
        "alwaysOnTop",
        "showOnAllWorkspace",
        "skipSwitcher",
        "skipDockPreView",
        "skipMutiTaskView",
        "isDDEShellSurface",
        "surfaceRole",
        "SurfaceWrapper::SurfaceRole",
        "autoPlaceYOffset",
        "clientRequstPos",
        "QPoint",
        "blur",
        "isWindowAnimationRunning",
        "coverEnabled",
        "acceptKeyboardFocus",
        "isActivated",
        "isIMCandidatePanel",
        "isResizable",
        "isMaximizable",
        "modal",
        "XdgToplevel",
        "XdgPopup",
        "XWayland",
        "Layer",
        "InputPopup",
        "LockScreen",
        "SplashScreen",
        "Normal",
        "Maximized",
        "Minimized",
        "Fullscreen",
        "Tiling",
        "ActiveControlState",
        "MappedOrSplash",
        "UnMinimized",
        "HasInitializeContainer",
        "HasActivateCapability",
        "Full",
        "FocusControlState",
        "Mapped",
        "AcceptKeyboardFocus",
        "HasFocusCapability",
        "SurfaceRole",
        "Overlay"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'hasInitializeContainerChanged'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'boundingRectChanged'
        QtMocHelpers::SignalData<void()>(9, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ownsOutputChanged'
        QtMocHelpers::SignalData<void()>(10, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'appIdChanged'
        QtMocHelpers::SignalData<void()>(11, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'normalGeometryChanged'
        QtMocHelpers::SignalData<void()>(12, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'maximizedGeometryChanged'
        QtMocHelpers::SignalData<void()>(13, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fullscreenGeometryChanged'
        QtMocHelpers::SignalData<void()>(14, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'tilingGeometryChanged'
        QtMocHelpers::SignalData<void()>(15, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'positionAutomaticChanged'
        QtMocHelpers::SignalData<void()>(16, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'previousSurfaceStateChanged'
        QtMocHelpers::SignalData<void()>(17, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'surfaceStateChanged'
        QtMocHelpers::SignalData<void()>(18, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'radiusChanged'
        QtMocHelpers::SignalData<void()>(19, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'moveRequested'
        QtMocHelpers::SignalData<void()>(20, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'resizeRequested'
        QtMocHelpers::SignalData<void(Qt::Edges)>(21, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Signal 'windowMenuRequested'
        QtMocHelpers::SignalData<void(QPointF)>(24, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Signal 'geometryChanged'
        QtMocHelpers::SignalData<void()>(27, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'containerChanged'
        QtMocHelpers::SignalData<void()>(28, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'visibleDecorationChanged'
        QtMocHelpers::SignalData<void()>(29, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'clipInOutputChanged'
        QtMocHelpers::SignalData<void()>(30, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'noDecorationChanged'
        QtMocHelpers::SignalData<void()>(31, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'noTitleBarChanged'
        QtMocHelpers::SignalData<void()>(32, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'noCornerRadiusChanged'
        QtMocHelpers::SignalData<void()>(33, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'iconGeometryChanged'
        QtMocHelpers::SignalData<void()>(34, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'workspaceIdChanged'
        QtMocHelpers::SignalData<void()>(35, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'alwaysOnTopChanged'
        QtMocHelpers::SignalData<void()>(36, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'showOnAllWorkspaceChanged'
        QtMocHelpers::SignalData<void()>(37, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'activationRequested'
        QtMocHelpers::SignalData<void()>(38, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'inactivationRequested'
        QtMocHelpers::SignalData<void()>(39, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'splashCloseRequested'
        QtMocHelpers::SignalData<void()>(40, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'skipSwitcherChanged'
        QtMocHelpers::SignalData<void()>(41, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'skipDockPreViewChanged'
        QtMocHelpers::SignalData<void()>(42, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'skipMutiTaskViewChanged'
        QtMocHelpers::SignalData<void()>(43, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'isDDEShellSurfaceChanged'
        QtMocHelpers::SignalData<void()>(44, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'surfaceRoleChanged'
        QtMocHelpers::SignalData<void()>(45, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'autoPlaceYOffsetChanged'
        QtMocHelpers::SignalData<void()>(46, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'clientRequstPosChanged'
        QtMocHelpers::SignalData<void()>(47, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'blurChanged'
        QtMocHelpers::SignalData<void()>(48, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'windowAnimationRunningChanged'
        QtMocHelpers::SignalData<void()>(49, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'coverEnabledChanged'
        QtMocHelpers::SignalData<void()>(50, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'aboutToBeInvalidated'
        QtMocHelpers::SignalData<void()>(51, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'isActivatedChanged'
        QtMocHelpers::SignalData<void()>(52, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'isIMCandidatePanelChanged'
        QtMocHelpers::SignalData<void()>(53, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'resizableChanged'
        QtMocHelpers::SignalData<void()>(54, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'maximizableChanged'
        QtMocHelpers::SignalData<void()>(55, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'modalChanged'
        QtMocHelpers::SignalData<void()>(56, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'attentionChanged'
        QtMocHelpers::SignalData<void()>(57, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'surfaceItemCreated'
        QtMocHelpers::SignalData<void()>(58, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hasFocusCapabilityChanged'
        QtMocHelpers::SignalData<void()>(59, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'prelaunchSplashChanged'
        QtMocHelpers::SignalData<void()>(60, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'typeChanged'
        QtMocHelpers::SignalData<void()>(61, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'minimize'
        QtMocHelpers::SlotData<void(bool)>(62, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 63 },
        }}),
        // Slot 'minimize'
        QtMocHelpers::SlotData<void()>(62, 8, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'restoreFromMinimized'
        QtMocHelpers::SlotData<void(bool)>(64, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 63 },
        }}),
        // Slot 'restoreFromMinimized'
        QtMocHelpers::SlotData<void()>(64, 8, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'maximize'
        QtMocHelpers::SlotData<void()>(65, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'unmaximize'
        QtMocHelpers::SlotData<void()>(66, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toggleMaximized'
        QtMocHelpers::SlotData<void()>(67, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'enterFullscreen'
        QtMocHelpers::SlotData<void()>(68, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'leaveFullscreen'
        QtMocHelpers::SlotData<void()>(69, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'closeSurface'
        QtMocHelpers::SlotData<void()>(70, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onMappedChanged'
        QtMocHelpers::SlotData<void()>(71, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onSocketEnabledChanged'
        QtMocHelpers::SlotData<void()>(72, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'stackBefore'
        QtMocHelpers::SlotData<bool(QQuickItem *)>(73, 8, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 74, 75 },
        }}),
        // Slot 'stackAfter'
        QtMocHelpers::SlotData<bool(QQuickItem *)>(76, 8, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 74, 75 },
        }}),
        // Slot 'stackToLast'
        QtMocHelpers::SlotData<void()>(77, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'stackToFirst'
        QtMocHelpers::SlotData<void()>(78, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateSurfaceSizeRatio'
        QtMocHelpers::SlotData<void()>(79, 8, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onAnimationReady'
        QtMocHelpers::SlotData<void()>(80, 8, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onAnimationFinished'
        QtMocHelpers::SlotData<void()>(81, 8, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onPrelaunchGeometryAnimationReady'
        QtMocHelpers::SlotData<void()>(82, 8, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onPrelaunchGeometryAnimationFinished'
        QtMocHelpers::SlotData<void()>(83, 8, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onShowAnimationFinished'
        QtMocHelpers::SlotData<void()>(84, 8, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onHideAnimationFinished'
        QtMocHelpers::SlotData<void()>(85, 8, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onMinimizeAnimationFinished'
        QtMocHelpers::SlotData<void()>(86, 8, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onShowDesktopAnimationFinished'
        QtMocHelpers::SlotData<void()>(87, 8, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'type'
        QtMocHelpers::PropertyData<enum Type>(90, 0x80000000 | 91, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 49),
        // property 'appId'
        QtMocHelpers::PropertyData<QString>(92, QMetaType::QString, QMC::DefaultPropertyFlags, 3),
        // property 'implicitWidth'
        QtMocHelpers::PropertyData<qreal>(93, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Final, 0x70000000 | 88),
        // property 'implicitHeight'
        QtMocHelpers::PropertyData<qreal>(94, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Final, 0x70000000 | 89),
        // property 'surface'
        QtMocHelpers::PropertyData<Waylib::Server::WSurface*>(95, 0x80000000 | 96, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'shellSurface'
        QtMocHelpers::PropertyData<Waylib::Server::WToplevelSurface*>(97, 0x80000000 | 98, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'surfaceItem'
        QtMocHelpers::PropertyData<Waylib::Server::WSurfaceItem*>(99, 0x80000000 | 100, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 46),
        // property 'prelaunchSplash'
        QtMocHelpers::PropertyData<QQuickItem*>(101, 0x80000000 | 74, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 48),
        // property 'boundingRect'
        QtMocHelpers::PropertyData<QRectF>(102, 0x80000000 | 103, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 1),
        // property 'geometry'
        QtMocHelpers::PropertyData<QRectF>(104, 0x80000000 | 103, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 15),
        // property 'normalGeometry'
        QtMocHelpers::PropertyData<QRectF>(105, 0x80000000 | 103, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 4),
        // property 'maximizedGeometry'
        QtMocHelpers::PropertyData<QRectF>(106, 0x80000000 | 103, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 5),
        // property 'fullscreenGeometry'
        QtMocHelpers::PropertyData<QRectF>(107, 0x80000000 | 103, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 6),
        // property 'tilingGeometry'
        QtMocHelpers::PropertyData<QRectF>(108, 0x80000000 | 103, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 7),
        // property 'ownsOutput'
        QtMocHelpers::PropertyData<Output*>(109, 0x80000000 | 110, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 2),
        // property 'positionAutomatic'
        QtMocHelpers::PropertyData<bool>(111, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 8),
        // property 'previousSurfaceState'
        QtMocHelpers::PropertyData<enum State>(112, 0x80000000 | 113, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 9),
        // property 'surfaceState'
        QtMocHelpers::PropertyData<enum State>(114, 0x80000000 | 113, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final | QMC::Bindable, 10),
        // property 'radius'
        QtMocHelpers::PropertyData<qreal>(115, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Final, 11),
        // property 'container'
        QtMocHelpers::PropertyData<SurfaceContainer*>(116, 0x80000000 | 117, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 16),
        // property 'titleBar'
        QtMocHelpers::PropertyData<QQuickItem*>(118, 0x80000000 | 74, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 20),
        // property 'decoration'
        QtMocHelpers::PropertyData<QQuickItem*>(119, 0x80000000 | 74, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 19),
        // property 'visibleDecoration'
        QtMocHelpers::PropertyData<bool>(120, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 17),
        // property 'clipInOutput'
        QtMocHelpers::PropertyData<bool>(121, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 18),
        // property 'noTitleBar'
        QtMocHelpers::PropertyData<bool>(122, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 20),
        // property 'noCornerRadius'
        QtMocHelpers::PropertyData<bool>(123, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 21),
        // property 'workspaceId'
        QtMocHelpers::PropertyData<int>(124, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Final, 23),
        // property 'alwaysOnTop'
        QtMocHelpers::PropertyData<bool>(125, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 24),
        // property 'showOnAllWorkspace'
        QtMocHelpers::PropertyData<bool>(126, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 25),
        // property 'skipSwitcher'
        QtMocHelpers::PropertyData<bool>(127, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 29),
        // property 'skipDockPreView'
        QtMocHelpers::PropertyData<bool>(128, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 30),
        // property 'skipMutiTaskView'
        QtMocHelpers::PropertyData<bool>(129, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 31),
        // property 'isDDEShellSurface'
        QtMocHelpers::PropertyData<bool>(130, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 32),
        // property 'surfaceRole'
        QtMocHelpers::PropertyData<SurfaceWrapper::SurfaceRole>(131, 0x80000000 | 132, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 33),
        // property 'autoPlaceYOffset'
        QtMocHelpers::PropertyData<quint32>(133, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Final, 34),
        // property 'clientRequstPos'
        QtMocHelpers::PropertyData<QPoint>(134, 0x80000000 | 135, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 35),
        // property 'blur'
        QtMocHelpers::PropertyData<bool>(136, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 36),
        // property 'isWindowAnimationRunning'
        QtMocHelpers::PropertyData<bool>(137, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 37),
        // property 'coverEnabled'
        QtMocHelpers::PropertyData<bool>(138, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 38),
        // property 'acceptKeyboardFocus'
        QtMocHelpers::PropertyData<bool>(139, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final),
        // property 'isActivated'
        QtMocHelpers::PropertyData<bool>(140, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 40),
        // property 'isIMCandidatePanel'
        QtMocHelpers::PropertyData<bool>(141, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 41),
        // property 'isResizable'
        QtMocHelpers::PropertyData<bool>(142, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 42),
        // property 'isMaximizable'
        QtMocHelpers::PropertyData<bool>(143, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 43),
        // property 'modal'
        QtMocHelpers::PropertyData<bool>(144, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 44),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Type'
        QtMocHelpers::EnumData<enum Type>(91, 91, QMC::EnumIsScoped).add({
            {  145, Type::XdgToplevel },
            {  146, Type::XdgPopup },
            {  147, Type::XWayland },
            {  148, Type::Layer },
            {  149, Type::InputPopup },
            {  150, Type::LockScreen },
            {  151, Type::SplashScreen },
        }),
        // enum 'State'
        QtMocHelpers::EnumData<enum State>(113, 113, QMC::EnumIsScoped).add({
            {  152, State::Normal },
            {  153, State::Maximized },
            {  154, State::Minimized },
            {  155, State::Fullscreen },
            {  156, State::Tiling },
        }),
        // enum 'ActiveControlState'
        QtMocHelpers::EnumData<enum ActiveControlState>(157, 157, QMC::EnumIsScoped).add({
            {  158, ActiveControlState::MappedOrSplash },
            {  159, ActiveControlState::UnMinimized },
            {  160, ActiveControlState::HasInitializeContainer },
            {  161, ActiveControlState::HasActivateCapability },
            {  162, ActiveControlState::Full },
        }),
        // enum 'FocusControlState'
        QtMocHelpers::EnumData<enum FocusControlState>(163, 163, QMC::EnumIsScoped).add({
            {  164, FocusControlState::Mapped },
            {  159, FocusControlState::UnMinimized },
            {  165, FocusControlState::AcceptKeyboardFocus },
            {  166, FocusControlState::HasFocusCapability },
            {  162, FocusControlState::Full },
        }),
        // enum 'SurfaceRole'
        QtMocHelpers::EnumData<enum SurfaceRole>(167, 167, QMC::EnumIsScoped).add({
            {  152, SurfaceRole::Normal },
            {  168, SurfaceRole::Overlay },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<SurfaceWrapper, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject SurfaceWrapper::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14SurfaceWrapperE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14SurfaceWrapperE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14SurfaceWrapperE_t>.metaTypes,
    nullptr
} };

void SurfaceWrapper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SurfaceWrapper *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->hasInitializeContainerChanged(); break;
        case 1: _t->boundingRectChanged(); break;
        case 2: _t->ownsOutputChanged(); break;
        case 3: _t->appIdChanged(); break;
        case 4: _t->normalGeometryChanged(); break;
        case 5: _t->maximizedGeometryChanged(); break;
        case 6: _t->fullscreenGeometryChanged(); break;
        case 7: _t->tilingGeometryChanged(); break;
        case 8: _t->positionAutomaticChanged(); break;
        case 9: _t->previousSurfaceStateChanged(); break;
        case 10: _t->surfaceStateChanged(); break;
        case 11: _t->radiusChanged(); break;
        case 12: _t->moveRequested(); break;
        case 13: _t->resizeRequested((*reinterpret_cast<std::add_pointer_t<Qt::Edges>>(_a[1]))); break;
        case 14: _t->windowMenuRequested((*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[1]))); break;
        case 15: _t->geometryChanged(); break;
        case 16: _t->containerChanged(); break;
        case 17: _t->visibleDecorationChanged(); break;
        case 18: _t->clipInOutputChanged(); break;
        case 19: _t->noDecorationChanged(); break;
        case 20: _t->noTitleBarChanged(); break;
        case 21: _t->noCornerRadiusChanged(); break;
        case 22: _t->iconGeometryChanged(); break;
        case 23: _t->workspaceIdChanged(); break;
        case 24: _t->alwaysOnTopChanged(); break;
        case 25: _t->showOnAllWorkspaceChanged(); break;
        case 26: _t->activationRequested(); break;
        case 27: _t->inactivationRequested(); break;
        case 28: _t->splashCloseRequested(); break;
        case 29: _t->skipSwitcherChanged(); break;
        case 30: _t->skipDockPreViewChanged(); break;
        case 31: _t->skipMutiTaskViewChanged(); break;
        case 32: _t->isDDEShellSurfaceChanged(); break;
        case 33: _t->surfaceRoleChanged(); break;
        case 34: _t->autoPlaceYOffsetChanged(); break;
        case 35: _t->clientRequstPosChanged(); break;
        case 36: _t->blurChanged(); break;
        case 37: _t->windowAnimationRunningChanged(); break;
        case 38: _t->coverEnabledChanged(); break;
        case 39: _t->aboutToBeInvalidated(); break;
        case 40: _t->isActivatedChanged(); break;
        case 41: _t->isIMCandidatePanelChanged(); break;
        case 42: _t->resizableChanged(); break;
        case 43: _t->maximizableChanged(); break;
        case 44: _t->modalChanged(); break;
        case 45: _t->attentionChanged(); break;
        case 46: _t->surfaceItemCreated(); break;
        case 47: _t->hasFocusCapabilityChanged(); break;
        case 48: _t->prelaunchSplashChanged(); break;
        case 49: _t->typeChanged(); break;
        case 50: _t->minimize((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 51: _t->minimize(); break;
        case 52: _t->restoreFromMinimized((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 53: _t->restoreFromMinimized(); break;
        case 54: _t->maximize(); break;
        case 55: _t->unmaximize(); break;
        case 56: _t->toggleMaximized(); break;
        case 57: _t->enterFullscreen(); break;
        case 58: _t->leaveFullscreen(); break;
        case 59: _t->closeSurface(); break;
        case 60: _t->onMappedChanged(); break;
        case 61: _t->onSocketEnabledChanged(); break;
        case 62: { bool _r = _t->stackBefore((*reinterpret_cast<std::add_pointer_t<QQuickItem*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 63: { bool _r = _t->stackAfter((*reinterpret_cast<std::add_pointer_t<QQuickItem*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 64: _t->stackToLast(); break;
        case 65: _t->stackToFirst(); break;
        case 66: _t->updateSurfaceSizeRatio(); break;
        case 67: _t->onAnimationReady(); break;
        case 68: _t->onAnimationFinished(); break;
        case 69: _t->onPrelaunchGeometryAnimationReady(); break;
        case 70: _t->onPrelaunchGeometryAnimationFinished(); break;
        case 71: _t->onShowAnimationFinished(); break;
        case 72: _t->onHideAnimationFinished(); break;
        case 73: _t->onMinimizeAnimationFinished(); break;
        case 74: _t->onShowDesktopAnimationFinished(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 62:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QQuickItem* >(); break;
            }
            break;
        case 63:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QQuickItem* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::hasInitializeContainerChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::boundingRectChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::ownsOutputChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::appIdChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::normalGeometryChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::maximizedGeometryChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::fullscreenGeometryChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::tilingGeometryChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::positionAutomaticChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::previousSurfaceStateChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::surfaceStateChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::radiusChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::moveRequested, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)(Qt::Edges )>(_a, &SurfaceWrapper::resizeRequested, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)(QPointF )>(_a, &SurfaceWrapper::windowMenuRequested, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::geometryChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::containerChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::visibleDecorationChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::clipInOutputChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::noDecorationChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::noTitleBarChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::noCornerRadiusChanged, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::iconGeometryChanged, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::workspaceIdChanged, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::alwaysOnTopChanged, 24))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::showOnAllWorkspaceChanged, 25))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::activationRequested, 26))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::inactivationRequested, 27))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::splashCloseRequested, 28))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::skipSwitcherChanged, 29))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::skipDockPreViewChanged, 30))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::skipMutiTaskViewChanged, 31))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::isDDEShellSurfaceChanged, 32))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::surfaceRoleChanged, 33))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::autoPlaceYOffsetChanged, 34))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::clientRequstPosChanged, 35))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::blurChanged, 36))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::windowAnimationRunningChanged, 37))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::coverEnabledChanged, 38))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::aboutToBeInvalidated, 39))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::isActivatedChanged, 40))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::isIMCandidatePanelChanged, 41))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::resizableChanged, 42))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::maximizableChanged, 43))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::modalChanged, 44))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::attentionChanged, 45))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::surfaceItemCreated, 46))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::hasFocusCapabilityChanged, 47))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::prelaunchSplashChanged, 48))
            return;
        if (QtMocHelpers::indexOfMethod<void (SurfaceWrapper::*)()>(_a, &SurfaceWrapper::typeChanged, 49))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 21:
        case 20:
        case 7:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItem* >(); break;
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Waylib::Server::WSurface* >(); break;
        case 6:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Waylib::Server::WSurfaceItem* >(); break;
        case 5:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Waylib::Server::WToplevelSurface* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<enum Type*>(_v) = _t->type(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->appId(); break;
        case 2: *reinterpret_cast<qreal*>(_v) = _t->implicitWidth(); break;
        case 3: *reinterpret_cast<qreal*>(_v) = _t->implicitHeight(); break;
        case 4: *reinterpret_cast<Waylib::Server::WSurface**>(_v) = _t->surface(); break;
        case 5: *reinterpret_cast<Waylib::Server::WToplevelSurface**>(_v) = _t->shellSurface(); break;
        case 6: *reinterpret_cast<Waylib::Server::WSurfaceItem**>(_v) = _t->surfaceItem(); break;
        case 7: *reinterpret_cast<QQuickItem**>(_v) = _t->prelaunchSplash(); break;
        case 8: *reinterpret_cast<QRectF*>(_v) = _t->boundingRect(); break;
        case 9: *reinterpret_cast<QRectF*>(_v) = _t->geometry(); break;
        case 10: *reinterpret_cast<QRectF*>(_v) = _t->normalGeometry(); break;
        case 11: *reinterpret_cast<QRectF*>(_v) = _t->maximizedGeometry(); break;
        case 12: *reinterpret_cast<QRectF*>(_v) = _t->fullscreenGeometry(); break;
        case 13: *reinterpret_cast<QRectF*>(_v) = _t->tilingGeometry(); break;
        case 14: *reinterpret_cast<Output**>(_v) = _t->ownsOutput(); break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->positionAutomatic(); break;
        case 16: *reinterpret_cast<enum State*>(_v) = _t->previousSurfaceState(); break;
        case 17: *reinterpret_cast<enum State*>(_v) = _t->surfaceState(); break;
        case 18: *reinterpret_cast<qreal*>(_v) = _t->radius(); break;
        case 19: *reinterpret_cast<SurfaceContainer**>(_v) = _t->container(); break;
        case 20: *reinterpret_cast<QQuickItem**>(_v) = _t->titleBar(); break;
        case 21: *reinterpret_cast<QQuickItem**>(_v) = _t->decoration(); break;
        case 22: *reinterpret_cast<bool*>(_v) = _t->visibleDecoration(); break;
        case 23: *reinterpret_cast<bool*>(_v) = _t->clipInOutput(); break;
        case 24: *reinterpret_cast<bool*>(_v) = _t->noTitleBar(); break;
        case 25: *reinterpret_cast<bool*>(_v) = _t->noCornerRadius(); break;
        case 26: *reinterpret_cast<int*>(_v) = _t->workspaceId(); break;
        case 27: *reinterpret_cast<bool*>(_v) = _t->alwaysOnTop(); break;
        case 28: *reinterpret_cast<bool*>(_v) = _t->showOnAllWorkspace(); break;
        case 29: *reinterpret_cast<bool*>(_v) = _t->skipSwitcher(); break;
        case 30: *reinterpret_cast<bool*>(_v) = _t->skipDockPreView(); break;
        case 31: *reinterpret_cast<bool*>(_v) = _t->skipMutiTaskView(); break;
        case 32: *reinterpret_cast<bool*>(_v) = _t->isDDEShellSurface(); break;
        case 33: *reinterpret_cast<SurfaceWrapper::SurfaceRole*>(_v) = _t->surfaceRole(); break;
        case 34: *reinterpret_cast<quint32*>(_v) = _t->autoPlaceYOffset(); break;
        case 35: *reinterpret_cast<QPoint*>(_v) = _t->clientRequstPos(); break;
        case 36: *reinterpret_cast<bool*>(_v) = _t->blur(); break;
        case 37: *reinterpret_cast<bool*>(_v) = _t->isWindowAnimationRunning(); break;
        case 38: *reinterpret_cast<bool*>(_v) = _t->coverEnabled(); break;
        case 39: *reinterpret_cast<bool*>(_v) = _t->acceptKeyboardFocus(); break;
        case 40: *reinterpret_cast<bool*>(_v) = _t->isActivated(); break;
        case 41: *reinterpret_cast<bool*>(_v) = _t->isIMCandidatePanel(); break;
        case 42: *reinterpret_cast<bool*>(_v) = _t->isResizable(); break;
        case 43: *reinterpret_cast<bool*>(_v) = _t->isMaximizable(); break;
        case 44: *reinterpret_cast<bool*>(_v) = _t->modal(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 27: _t->setAlwaysOnTop(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
    if (_c == QMetaObject::BindableProperty) {
        switch (_id) {
        case 17: *static_cast<QUntypedBindable *>(_a[0]) = _t->bindableSurfaceState(); break;
        default: break;
        }
    }
}

const QMetaObject *SurfaceWrapper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SurfaceWrapper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14SurfaceWrapperE_t>.strings))
        return static_cast<void*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int SurfaceWrapper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 75)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 75;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 75)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 75;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 45;
    }
    return _id;
}

// SIGNAL 0
void SurfaceWrapper::hasInitializeContainerChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void SurfaceWrapper::boundingRectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void SurfaceWrapper::ownsOutputChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void SurfaceWrapper::appIdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void SurfaceWrapper::normalGeometryChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void SurfaceWrapper::maximizedGeometryChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void SurfaceWrapper::fullscreenGeometryChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void SurfaceWrapper::tilingGeometryChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void SurfaceWrapper::positionAutomaticChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void SurfaceWrapper::previousSurfaceStateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void SurfaceWrapper::surfaceStateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void SurfaceWrapper::radiusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void SurfaceWrapper::moveRequested()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void SurfaceWrapper::resizeRequested(Qt::Edges _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void SurfaceWrapper::windowMenuRequested(QPointF _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void SurfaceWrapper::geometryChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void SurfaceWrapper::containerChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void SurfaceWrapper::visibleDecorationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void SurfaceWrapper::clipInOutputChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void SurfaceWrapper::noDecorationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void SurfaceWrapper::noTitleBarChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 20, nullptr);
}

// SIGNAL 21
void SurfaceWrapper::noCornerRadiusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 21, nullptr);
}

// SIGNAL 22
void SurfaceWrapper::iconGeometryChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 22, nullptr);
}

// SIGNAL 23
void SurfaceWrapper::workspaceIdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 23, nullptr);
}

// SIGNAL 24
void SurfaceWrapper::alwaysOnTopChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 24, nullptr);
}

// SIGNAL 25
void SurfaceWrapper::showOnAllWorkspaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 25, nullptr);
}

// SIGNAL 26
void SurfaceWrapper::activationRequested()
{
    QMetaObject::activate(this, &staticMetaObject, 26, nullptr);
}

// SIGNAL 27
void SurfaceWrapper::inactivationRequested()
{
    QMetaObject::activate(this, &staticMetaObject, 27, nullptr);
}

// SIGNAL 28
void SurfaceWrapper::splashCloseRequested()
{
    QMetaObject::activate(this, &staticMetaObject, 28, nullptr);
}

// SIGNAL 29
void SurfaceWrapper::skipSwitcherChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 29, nullptr);
}

// SIGNAL 30
void SurfaceWrapper::skipDockPreViewChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 30, nullptr);
}

// SIGNAL 31
void SurfaceWrapper::skipMutiTaskViewChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 31, nullptr);
}

// SIGNAL 32
void SurfaceWrapper::isDDEShellSurfaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 32, nullptr);
}

// SIGNAL 33
void SurfaceWrapper::surfaceRoleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 33, nullptr);
}

// SIGNAL 34
void SurfaceWrapper::autoPlaceYOffsetChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 34, nullptr);
}

// SIGNAL 35
void SurfaceWrapper::clientRequstPosChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 35, nullptr);
}

// SIGNAL 36
void SurfaceWrapper::blurChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 36, nullptr);
}

// SIGNAL 37
void SurfaceWrapper::windowAnimationRunningChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 37, nullptr);
}

// SIGNAL 38
void SurfaceWrapper::coverEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 38, nullptr);
}

// SIGNAL 39
void SurfaceWrapper::aboutToBeInvalidated()
{
    QMetaObject::activate(this, &staticMetaObject, 39, nullptr);
}

// SIGNAL 40
void SurfaceWrapper::isActivatedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 40, nullptr);
}

// SIGNAL 41
void SurfaceWrapper::isIMCandidatePanelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 41, nullptr);
}

// SIGNAL 42
void SurfaceWrapper::resizableChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 42, nullptr);
}

// SIGNAL 43
void SurfaceWrapper::maximizableChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 43, nullptr);
}

// SIGNAL 44
void SurfaceWrapper::modalChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 44, nullptr);
}

// SIGNAL 45
void SurfaceWrapper::attentionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 45, nullptr);
}

// SIGNAL 46
void SurfaceWrapper::surfaceItemCreated()
{
    QMetaObject::activate(this, &staticMetaObject, 46, nullptr);
}

// SIGNAL 47
void SurfaceWrapper::hasFocusCapabilityChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 47, nullptr);
}

// SIGNAL 48
void SurfaceWrapper::prelaunchSplashChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 48, nullptr);
}

// SIGNAL 49
void SurfaceWrapper::typeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 49, nullptr);
}
namespace CheckNotifySignalValidity_ZN14SurfaceWrapperE {
template<typename T> using has_nullary_implicitWidthChanged = decltype(std::declval<T>().implicitWidthChanged());
template<typename T> using has_unary_implicitWidthChanged = decltype(std::declval<T>().implicitWidthChanged(std::declval<qreal>()));
static_assert(qxp::is_detected_v<has_nullary_implicitWidthChanged, SurfaceWrapper> || qxp::is_detected_v<has_unary_implicitWidthChanged, SurfaceWrapper>,
              "NOTIFY signal implicitWidthChanged does not exist in class (or is private in its parent)");
template<typename T> using has_nullary_implicitHeightChanged = decltype(std::declval<T>().implicitHeightChanged());
template<typename T> using has_unary_implicitHeightChanged = decltype(std::declval<T>().implicitHeightChanged(std::declval<qreal>()));
static_assert(qxp::is_detected_v<has_nullary_implicitHeightChanged, SurfaceWrapper> || qxp::is_detected_v<has_unary_implicitHeightChanged, SurfaceWrapper>,
              "NOTIFY signal implicitHeightChanged does not exist in class (or is private in its parent)");
}
QT_WARNING_POP
