/* Generated by wayland-scanner 1.26.0 */

/*
 * Copyright © 2012, 2013 Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include <stdbool.h>
#include <stdlib.h>
#include <stdint.h>
#include "wayland-util.h"

extern const struct wl_interface wl_seat_interface;
extern const struct wl_interface wl_surface_interface;
extern const struct wl_interface zwp_text_input_v1_interface;

static const struct wl_interface *text_input_unstable_v1_types[] = {
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	&wl_seat_interface,
	&wl_surface_interface,
	&wl_seat_interface,
	&wl_surface_interface,
	&zwp_text_input_v1_interface,
};

static const struct wl_message zwp_text_input_v1_requests[] = {
	{ "activate", "oo", text_input_unstable_v1_types + 5 },
	{ "deactivate", "o", text_input_unstable_v1_types + 7 },
	{ "show_input_panel", "", text_input_unstable_v1_types + 0 },
	{ "hide_input_panel", "", text_input_unstable_v1_types + 0 },
	{ "reset", "", text_input_unstable_v1_types + 0 },
	{ "set_surrounding_text", "suu", text_input_unstable_v1_types + 0 },
	{ "set_content_type", "uu", text_input_unstable_v1_types + 0 },
	{ "set_cursor_rectangle", "iiii", text_input_unstable_v1_types + 0 },
	{ "set_preferred_language", "s", text_input_unstable_v1_types + 0 },
	{ "commit_state", "u", text_input_unstable_v1_types + 0 },
	{ "invoke_action", "uu", text_input_unstable_v1_types + 0 },
};

static const struct wl_message zwp_text_input_v1_events[] = {
	{ "enter", "o", text_input_unstable_v1_types + 8 },
	{ "leave", "", text_input_unstable_v1_types + 0 },
	{ "modifiers_map", "a", text_input_unstable_v1_types + 0 },
	{ "input_panel_state", "u", text_input_unstable_v1_types + 0 },
	{ "preedit_string", "uss", text_input_unstable_v1_types + 0 },
	{ "preedit_styling", "uuu", text_input_unstable_v1_types + 0 },
	{ "preedit_cursor", "i", text_input_unstable_v1_types + 0 },
	{ "commit_string", "us", text_input_unstable_v1_types + 0 },
	{ "cursor_position", "ii", text_input_unstable_v1_types + 0 },
	{ "delete_surrounding_text", "iu", text_input_unstable_v1_types + 0 },
	{ "keysym", "uuuuu", text_input_unstable_v1_types + 0 },
	{ "language", "us", text_input_unstable_v1_types + 0 },
	{ "text_direction", "uu", text_input_unstable_v1_types + 0 },
};

WL_EXPORT const struct wl_interface zwp_text_input_v1_interface = {
	"zwp_text_input_v1", 1,
	11, zwp_text_input_v1_requests,
	13, zwp_text_input_v1_events,
};

static const struct wl_message zwp_text_input_manager_v1_requests[] = {
	{ "create_text_input", "n", text_input_unstable_v1_types + 9 },
};

WL_EXPORT const struct wl_interface zwp_text_input_manager_v1_interface = {
	"zwp_text_input_manager_v1", 1,
	1, zwp_text_input_manager_v1_requests,
	0, NULL,
};

