/****************************************************************************
** Meta object code from reading C++ file 'inputmanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/input/inputmanager.h"
#include "private/wserver_p.h"
#include <wsurface.h>
#include "seatuserconfig.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'inputmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12InputManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto InputManager::qt_create_metaobjectdata<qt_meta_tag_ZN12InputManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "InputManager",
        "onMouseSettingsCreated",
        "",
        "MouseSettingsInterfaceV1*",
        "interface",
        "onTouchpadSettingsCreated",
        "TouchpadSettingsInterfaceV1*",
        "onKeyboardSettingsCreated",
        "KeyboardSettingsInterfaceV1*",
        "onMousePointerConfigCreated",
        "PointerDeviceConfigurationV1*",
        "config",
        "onTouchpadPointerConfigCreated",
        "handleMousePointerConfigApplied",
        "PointerDeviceConfigurationV1::ChangeFlags",
        "changes",
        "handleTouchpadPointerConfigApplied",
        "handleKeyboardSettingsApplied",
        "KeyboardSettingsInterfaceV1::ChangeFlags",
        "onInputAdded",
        "WInputDevice*",
        "input",
        "onConfigInitializeSucceed"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'onMouseSettingsCreated'
        QtMocHelpers::SlotData<void(MouseSettingsInterfaceV1 *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'onTouchpadSettingsCreated'
        QtMocHelpers::SlotData<void(TouchpadSettingsInterfaceV1 *)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 4 },
        }}),
        // Slot 'onKeyboardSettingsCreated'
        QtMocHelpers::SlotData<void(KeyboardSettingsInterfaceV1 *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 4 },
        }}),
        // Slot 'onMousePointerConfigCreated'
        QtMocHelpers::SlotData<void(PointerDeviceConfigurationV1 *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'onTouchpadPointerConfigCreated'
        QtMocHelpers::SlotData<void(PointerDeviceConfigurationV1 *)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'handleMousePointerConfigApplied'
        QtMocHelpers::SlotData<void(PointerDeviceConfigurationV1::ChangeFlags)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'handleTouchpadPointerConfigApplied'
        QtMocHelpers::SlotData<void(PointerDeviceConfigurationV1::ChangeFlags)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'handleKeyboardSettingsApplied'
        QtMocHelpers::SlotData<void(KeyboardSettingsInterfaceV1::ChangeFlags)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 15 },
        }}),
        // Slot 'onInputAdded'
        QtMocHelpers::SlotData<void(WInputDevice *)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'onConfigInitializeSucceed'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<InputManager, qt_meta_tag_ZN12InputManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject InputManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12InputManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12InputManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12InputManagerE_t>.metaTypes,
    nullptr
} };

void InputManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<InputManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->onMouseSettingsCreated((*reinterpret_cast<std::add_pointer_t<MouseSettingsInterfaceV1*>>(_a[1]))); break;
        case 1: _t->onTouchpadSettingsCreated((*reinterpret_cast<std::add_pointer_t<TouchpadSettingsInterfaceV1*>>(_a[1]))); break;
        case 2: _t->onKeyboardSettingsCreated((*reinterpret_cast<std::add_pointer_t<KeyboardSettingsInterfaceV1*>>(_a[1]))); break;
        case 3: _t->onMousePointerConfigCreated((*reinterpret_cast<std::add_pointer_t<PointerDeviceConfigurationV1*>>(_a[1]))); break;
        case 4: _t->onTouchpadPointerConfigCreated((*reinterpret_cast<std::add_pointer_t<PointerDeviceConfigurationV1*>>(_a[1]))); break;
        case 5: _t->handleMousePointerConfigApplied((*reinterpret_cast<std::add_pointer_t<PointerDeviceConfigurationV1::ChangeFlags>>(_a[1]))); break;
        case 6: _t->handleTouchpadPointerConfigApplied((*reinterpret_cast<std::add_pointer_t<PointerDeviceConfigurationV1::ChangeFlags>>(_a[1]))); break;
        case 7: _t->handleKeyboardSettingsApplied((*reinterpret_cast<std::add_pointer_t<KeyboardSettingsInterfaceV1::ChangeFlags>>(_a[1]))); break;
        case 8: _t->onInputAdded((*reinterpret_cast<std::add_pointer_t<WInputDevice*>>(_a[1]))); break;
        case 9: _t->onConfigInitializeSucceed(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< MouseSettingsInterfaceV1* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TouchpadSettingsInterfaceV1* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< KeyboardSettingsInterfaceV1* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PointerDeviceConfigurationV1* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PointerDeviceConfigurationV1* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *InputManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *InputManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12InputManagerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int InputManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}
QT_WARNING_POP
