/****************************************************************************
** Meta object code from reading C++ file 'wallpaperitem.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/wallpaper/wallpaperitem.h"
#include "woutput.h"
#include "workspace/workspace.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wallpaperitem.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13WallpaperItemE_t {};
} // unnamed namespace

template <> constexpr inline auto WallpaperItem::qt_create_metaobjectdata<qt_meta_tag_ZN13WallpaperItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "WallpaperItem",
        "QML.Element",
        "Wallpaper",
        "QML.AddedInVersion",
        "256",
        "outputChanged",
        "",
        "workspaceChanged",
        "wallpaperRoleChanged",
        "wallpaperStateChanged",
        "playChanged",
        "handleCurrentuserChanged",
        "handleWorkspaceAdded",
        "handleWallpaperSurfaceAdded",
        "TreelandWallpaperSurfaceInterfaceV1*",
        "interface",
        "slowDown",
        "workspace",
        "WorkspaceModel*",
        "output",
        "Waylib::Server::WOutput*",
        "wallpaperRole",
        "WallpaperRole",
        "source",
        "wallpaperState",
        "WallpaperState",
        "play",
        "Desktop",
        "Lockscreen",
        "Normal",
        "Scale",
        "ScaleWithoutAnimation"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'outputChanged'
        QtMocHelpers::SignalData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'workspaceChanged'
        QtMocHelpers::SignalData<void()>(7, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'wallpaperRoleChanged'
        QtMocHelpers::SignalData<void()>(8, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'wallpaperStateChanged'
        QtMocHelpers::SignalData<void()>(9, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'playChanged'
        QtMocHelpers::SignalData<void()>(10, 6, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleCurrentuserChanged'
        QtMocHelpers::SlotData<void()>(11, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleWorkspaceAdded'
        QtMocHelpers::SlotData<void()>(12, 6, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleWallpaperSurfaceAdded'
        QtMocHelpers::SlotData<void(TreelandWallpaperSurfaceInterfaceV1 *)>(13, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Method 'slowDown'
        QtMocHelpers::MethodData<void()>(16, 6, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'workspace'
        QtMocHelpers::PropertyData<WorkspaceModel*>(17, 0x80000000 | 18, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 1),
        // property 'output'
        QtMocHelpers::PropertyData<Waylib::Server::WOutput*>(19, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 0),
        // property 'wallpaperRole'
        QtMocHelpers::PropertyData<enum WallpaperRole>(21, 0x80000000 | 22, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 2),
        // property 'source'
        QtMocHelpers::PropertyData<QString>(23, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Final),
        // property 'wallpaperState'
        QtMocHelpers::PropertyData<enum WallpaperState>(24, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 3),
        // property 'play'
        QtMocHelpers::PropertyData<bool>(26, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 4),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'WallpaperRole'
        QtMocHelpers::EnumData<enum WallpaperRole>(22, 22, QMC::EnumFlags{}).add({
            {   27, WallpaperRole::Desktop },
            {   28, WallpaperRole::Lockscreen },
        }),
        // enum 'WallpaperState'
        QtMocHelpers::EnumData<enum WallpaperState>(25, 25, QMC::EnumFlags{}).add({
            {   29, WallpaperState::Normal },
            {   30, WallpaperState::Scale },
            {   31, WallpaperState::ScaleWithoutAnimation },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<WallpaperItem, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject WallpaperItem::staticMetaObject = { {
    QMetaObject::SuperData::link<WSurfaceItemContent::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13WallpaperItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13WallpaperItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13WallpaperItemE_t>.metaTypes,
    nullptr
} };

void WallpaperItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WallpaperItem *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->outputChanged(); break;
        case 1: _t->workspaceChanged(); break;
        case 2: _t->wallpaperRoleChanged(); break;
        case 3: _t->wallpaperStateChanged(); break;
        case 4: _t->playChanged(); break;
        case 5: _t->handleCurrentuserChanged(); break;
        case 6: _t->handleWorkspaceAdded(); break;
        case 7: _t->handleWallpaperSurfaceAdded((*reinterpret_cast<std::add_pointer_t<TreelandWallpaperSurfaceInterfaceV1*>>(_a[1]))); break;
        case 8: _t->slowDown(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WallpaperItem::*)()>(_a, &WallpaperItem::outputChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WallpaperItem::*)()>(_a, &WallpaperItem::workspaceChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WallpaperItem::*)()>(_a, &WallpaperItem::wallpaperRoleChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WallpaperItem::*)()>(_a, &WallpaperItem::wallpaperStateChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (WallpaperItem::*)()>(_a, &WallpaperItem::playChanged, 4))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<WorkspaceModel**>(_v) = _t->workspace(); break;
        case 1: *reinterpret_cast<Waylib::Server::WOutput**>(_v) = _t->output(); break;
        case 2: *reinterpret_cast<enum WallpaperRole*>(_v) = _t->wallpaperRole(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->source(); break;
        case 4: *reinterpret_cast<enum WallpaperState*>(_v) = _t->wallpaperState(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->play(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setWorkspace(*reinterpret_cast<WorkspaceModel**>(_v)); break;
        case 1: _t->setOutput(*reinterpret_cast<Waylib::Server::WOutput**>(_v)); break;
        case 2: _t->setWallpaperRole(*reinterpret_cast<enum WallpaperRole*>(_v)); break;
        case 4: _t->setWallpaperState(*reinterpret_cast<enum WallpaperState*>(_v)); break;
        case 5: _t->setPlay(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *WallpaperItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *WallpaperItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13WallpaperItemE_t>.strings))
        return static_cast<void*>(this);
    return WSurfaceItemContent::qt_metacast(_clname);
}

int WallpaperItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = WSurfaceItemContent::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void WallpaperItem::outputChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void WallpaperItem::workspaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void WallpaperItem::wallpaperRoleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void WallpaperItem::wallpaperStateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void WallpaperItem::playChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
