/****************************************************************************
** Meta object code from reading C++ file 'woutputviewport.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../waylib/src/server/qtquick/woutputviewport.h"
#include "wcursor.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'woutputviewport.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Waylib6Server15WOutputViewportE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WOutputViewport::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server15WOutputViewportE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WOutputViewport",
        "QML.Element",
        "OutputViewport",
        "devicePixelRatioChanged",
        "",
        "offscreenChanged",
        "cacheBufferChanged",
        "colorContentsModeChanged",
        "outputRenderInitialized",
        "inputChanged",
        "liveChanged",
        "outputChanged",
        "viewportTransformChanged",
        "sourceRectChanged",
        "targetRectChanged",
        "ignoreViewportChanged",
        "disableHardwareLayersChanged",
        "ignoreSoftwareLayersChanged",
        "layersChanged",
        "hardwareLayersChanged",
        "dependsChanged",
        "setOutputScale",
        "scale",
        "rotateOutput",
        "WOutput::Transform",
        "t",
        "render",
        "doCommit",
        "invalidate",
        "mapToOutput",
        "QRectF",
        "QQuickItem*",
        "source",
        "geometry",
        "QPointF",
        "position",
        "input",
        "output",
        "WOutput*",
        "viewportTransform",
        "QQuickTransform*",
        "devicePixelRatio",
        "offscreen",
        "cacheBuffer",
        "colorContentsMode",
        "WGlobal::ColorContentsMode",
        "live",
        "sourceRect",
        "targetRect",
        "ignoreViewport",
        "disableHardwareLayers",
        "ignoreSoftwareLayers",
        "layers",
        "QList<Waylib::Server::WOutputLayer*>",
        "hardwareLayers",
        "depends",
        "QList<Waylib::Server::WOutputViewport*>"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'devicePixelRatioChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'offscreenChanged'
        QtMocHelpers::SignalData<void()>(5, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cacheBufferChanged'
        QtMocHelpers::SignalData<void()>(6, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'colorContentsModeChanged'
        QtMocHelpers::SignalData<void()>(7, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'outputRenderInitialized'
        QtMocHelpers::SignalData<void()>(8, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'inputChanged'
        QtMocHelpers::SignalData<void()>(9, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'liveChanged'
        QtMocHelpers::SignalData<void()>(10, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'outputChanged'
        QtMocHelpers::SignalData<void()>(11, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'viewportTransformChanged'
        QtMocHelpers::SignalData<void()>(12, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'sourceRectChanged'
        QtMocHelpers::SignalData<void()>(13, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'targetRectChanged'
        QtMocHelpers::SignalData<void()>(14, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ignoreViewportChanged'
        QtMocHelpers::SignalData<void()>(15, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'disableHardwareLayersChanged'
        QtMocHelpers::SignalData<void()>(16, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ignoreSoftwareLayersChanged'
        QtMocHelpers::SignalData<void()>(17, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'layersChanged'
        QtMocHelpers::SignalData<void()>(18, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hardwareLayersChanged'
        QtMocHelpers::SignalData<void()>(19, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'dependsChanged'
        QtMocHelpers::SignalData<void()>(20, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setOutputScale'
        QtMocHelpers::SlotData<void(float)>(21, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Float, 22 },
        }}),
        // Slot 'rotateOutput'
        QtMocHelpers::SlotData<void(WOutput::Transform)>(23, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Slot 'render'
        QtMocHelpers::SlotData<void(bool)>(26, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 27 },
        }}),
        // Method 'invalidate'
        QtMocHelpers::MethodData<void()>(28, 4, QMC::AccessPublic, QMetaType::Void),
        // Method 'mapToOutput'
        QtMocHelpers::MethodData<QRectF(QQuickItem *, const QRectF &) const>(29, 4, QMC::AccessPublic, 0x80000000 | 30, {{
            { 0x80000000 | 31, 32 }, { 0x80000000 | 30, 33 },
        }}),
        // Method 'mapToOutput'
        QtMocHelpers::MethodData<QPointF(QQuickItem *, const QPointF &) const>(29, 4, QMC::AccessPublic, 0x80000000 | 34, {{
            { 0x80000000 | 31, 32 }, { 0x80000000 | 34, 35 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'input'
        QtMocHelpers::PropertyData<QQuickItem*>(36, 0x80000000 | 31, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 5),
        // property 'output'
        QtMocHelpers::PropertyData<WOutput*>(37, 0x80000000 | 38, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Required, 7),
        // property 'viewportTransform'
        QtMocHelpers::PropertyData<QQuickTransform*>(39, 0x80000000 | 40, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 8),
        // property 'devicePixelRatio'
        QtMocHelpers::PropertyData<qreal>(41, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'offscreen'
        QtMocHelpers::PropertyData<bool>(42, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'cacheBuffer'
        QtMocHelpers::PropertyData<bool>(43, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 2),
        // property 'colorContentsMode'
        QtMocHelpers::PropertyData<WGlobal::ColorContentsMode>(44, 0x80000000 | 45, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 3),
        // property 'live'
        QtMocHelpers::PropertyData<bool>(46, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 6),
        // property 'sourceRect'
        QtMocHelpers::PropertyData<QRectF>(47, 0x80000000 | 30, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 9),
        // property 'targetRect'
        QtMocHelpers::PropertyData<QRectF>(48, 0x80000000 | 30, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 10),
        // property 'ignoreViewport'
        QtMocHelpers::PropertyData<bool>(49, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 11),
        // property 'disableHardwareLayers'
        QtMocHelpers::PropertyData<bool>(50, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 12),
        // property 'ignoreSoftwareLayers'
        QtMocHelpers::PropertyData<bool>(51, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 13),
        // property 'layers'
        QtMocHelpers::PropertyData<QList<Waylib::Server::WOutputLayer*>>(52, 0x80000000 | 53, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 14),
        // property 'hardwareLayers'
        QtMocHelpers::PropertyData<QList<Waylib::Server::WOutputLayer*>>(54, 0x80000000 | 53, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 15),
        // property 'depends'
        QtMocHelpers::PropertyData<QList<Waylib::Server::WOutputViewport*>>(55, 0x80000000 | 56, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 16),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<WOutputViewport, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN6Waylib6Server15WOutputViewportE[] = {
    QMetaObject::SuperData::link<Waylib::Server::WGlobal::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject Waylib::Server::WOutputViewport::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server15WOutputViewportE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server15WOutputViewportE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN6Waylib6Server15WOutputViewportE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server15WOutputViewportE_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WOutputViewport::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WOutputViewport *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->devicePixelRatioChanged(); break;
        case 1: _t->offscreenChanged(); break;
        case 2: _t->cacheBufferChanged(); break;
        case 3: _t->colorContentsModeChanged(); break;
        case 4: _t->outputRenderInitialized(); break;
        case 5: _t->inputChanged(); break;
        case 6: _t->liveChanged(); break;
        case 7: _t->outputChanged(); break;
        case 8: _t->viewportTransformChanged(); break;
        case 9: _t->sourceRectChanged(); break;
        case 10: _t->targetRectChanged(); break;
        case 11: _t->ignoreViewportChanged(); break;
        case 12: _t->disableHardwareLayersChanged(); break;
        case 13: _t->ignoreSoftwareLayersChanged(); break;
        case 14: _t->layersChanged(); break;
        case 15: _t->hardwareLayersChanged(); break;
        case 16: _t->dependsChanged(); break;
        case 17: _t->setOutputScale((*reinterpret_cast<std::add_pointer_t<float>>(_a[1]))); break;
        case 18: _t->rotateOutput((*reinterpret_cast<std::add_pointer_t<WOutput::Transform>>(_a[1]))); break;
        case 19: _t->render((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 20: _t->invalidate(); break;
        case 21: { QRectF _r = _t->mapToOutput((*reinterpret_cast<std::add_pointer_t<QQuickItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QRectF>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QRectF*>(_a[0]) = std::move(_r); }  break;
        case 22: { QPointF _r = _t->mapToOutput((*reinterpret_cast<std::add_pointer_t<QQuickItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QPointF*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QQuickItem* >(); break;
            }
            break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QQuickItem* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WOutputViewport::*)()>(_a, &WOutputViewport::devicePixelRatioChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputViewport::*)()>(_a, &WOutputViewport::offscreenChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputViewport::*)()>(_a, &WOutputViewport::cacheBufferChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputViewport::*)()>(_a, &WOutputViewport::colorContentsModeChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputViewport::*)()>(_a, &WOutputViewport::outputRenderInitialized, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputViewport::*)()>(_a, &WOutputViewport::inputChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputViewport::*)()>(_a, &WOutputViewport::liveChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputViewport::*)()>(_a, &WOutputViewport::outputChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputViewport::*)()>(_a, &WOutputViewport::viewportTransformChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputViewport::*)()>(_a, &WOutputViewport::sourceRectChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputViewport::*)()>(_a, &WOutputViewport::targetRectChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputViewport::*)()>(_a, &WOutputViewport::ignoreViewportChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputViewport::*)()>(_a, &WOutputViewport::disableHardwareLayersChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputViewport::*)()>(_a, &WOutputViewport::ignoreSoftwareLayersChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputViewport::*)()>(_a, &WOutputViewport::layersChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputViewport::*)()>(_a, &WOutputViewport::hardwareLayersChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputViewport::*)()>(_a, &WOutputViewport::dependsChanged, 16))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 15:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<Waylib::Server::WOutputViewport*> >(); break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItem* >(); break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickTransform* >(); break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< WOutput* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QQuickItem**>(_v) = _t->input(); break;
        case 1: *reinterpret_cast<WOutput**>(_v) = _t->output(); break;
        case 2: *reinterpret_cast<QQuickTransform**>(_v) = _t->viewportTransform(); break;
        case 3: *reinterpret_cast<qreal*>(_v) = _t->devicePixelRatio(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->offscreen(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->cacheBuffer(); break;
        case 6: *reinterpret_cast<WGlobal::ColorContentsMode*>(_v) = _t->colorContentsMode(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->live(); break;
        case 8: *reinterpret_cast<QRectF*>(_v) = _t->sourceRect(); break;
        case 9: *reinterpret_cast<QRectF*>(_v) = _t->targetRect(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->ignoreViewport(); break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->disableHardwareLayers(); break;
        case 12: *reinterpret_cast<bool*>(_v) = _t->ignoreSoftwareLayers(); break;
        case 13: *reinterpret_cast<QList<Waylib::Server::WOutputLayer*>*>(_v) = _t->layers(); break;
        case 14: *reinterpret_cast<QList<Waylib::Server::WOutputLayer*>*>(_v) = _t->hardwareLayers(); break;
        case 15: *reinterpret_cast<QList<Waylib::Server::WOutputViewport*>*>(_v) = _t->depends(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setInput(*reinterpret_cast<QQuickItem**>(_v)); break;
        case 1: _t->setOutput(*reinterpret_cast<WOutput**>(_v)); break;
        case 2: _t->setViewportTransform(*reinterpret_cast<QQuickTransform**>(_v)); break;
        case 3: _t->setDevicePixelRatio(*reinterpret_cast<qreal*>(_v)); break;
        case 4: _t->setOffscreen(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setCacheBuffer(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->setColorContentsMode(*reinterpret_cast<WGlobal::ColorContentsMode*>(_v)); break;
        case 7: _t->setLive(*reinterpret_cast<bool*>(_v)); break;
        case 8: _t->setSourceRect(*reinterpret_cast<QRectF*>(_v)); break;
        case 9: _t->setTargetRect(*reinterpret_cast<QRectF*>(_v)); break;
        case 10: _t->setIgnoreViewport(*reinterpret_cast<bool*>(_v)); break;
        case 11: _t->setDisableHardwareLayers(*reinterpret_cast<bool*>(_v)); break;
        case 12: _t->setIgnoreSoftwareLayers(*reinterpret_cast<bool*>(_v)); break;
        case 15: _t->setDepends(*reinterpret_cast<QList<Waylib::Server::WOutputViewport*>*>(_v)); break;
        default: break;
        }
    }
    if (_c == QMetaObject::ResetProperty) {
        switch (_id) {
        case 0: _t->resetInput(); break;
        case 8: _t->resetSourceRect(); break;
        case 9: _t->resetTargetRect(); break;
        default: break;
        }
    }
}

const QMetaObject *Waylib::Server::WOutputViewport::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WOutputViewport::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server15WOutputViewportE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "WTextureProviderProvider"))
        return static_cast< WTextureProviderProvider*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int Waylib::Server::WOutputViewport::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WOutputViewport::devicePixelRatioChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Waylib::Server::WOutputViewport::offscreenChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Waylib::Server::WOutputViewport::cacheBufferChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Waylib::Server::WOutputViewport::colorContentsModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Waylib::Server::WOutputViewport::outputRenderInitialized()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Waylib::Server::WOutputViewport::inputChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Waylib::Server::WOutputViewport::liveChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void Waylib::Server::WOutputViewport::outputChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void Waylib::Server::WOutputViewport::viewportTransformChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void Waylib::Server::WOutputViewport::sourceRectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void Waylib::Server::WOutputViewport::targetRectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void Waylib::Server::WOutputViewport::ignoreViewportChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void Waylib::Server::WOutputViewport::disableHardwareLayersChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void Waylib::Server::WOutputViewport::ignoreSoftwareLayersChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void Waylib::Server::WOutputViewport::layersChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void Waylib::Server::WOutputViewport::hardwareLayersChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void Waylib::Server::WOutputViewport::dependsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}
QT_WARNING_POP
