/****************************************************************************
** Meta object code from reading C++ file 'tquickradiuseffect.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/effects/tquickradiuseffect.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tquickradiuseffect.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18TQuickRadiusEffectE_t {};
} // unnamed namespace

template <> constexpr inline auto TQuickRadiusEffect::qt_create_metaobjectdata<qt_meta_tag_ZN18TQuickRadiusEffectE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TQuickRadiusEffect",
        "QML.Element",
        "TRadiusEffect",
        "sourceItemChanged",
        "",
        "hideSourceChanged",
        "radiusChanged",
        "topLeftRadiusChanged",
        "topRightRadiusChanged",
        "bottomLeftRadiusChanged",
        "bottomRightRadiusChanged",
        "sourceItemDestroyed",
        "item",
        "radius",
        "topLeftRadius",
        "topRightRadius",
        "bottomLeftRadius",
        "bottomRightRadius",
        "sourceItem",
        "QQuickItem*",
        "hideSource"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'sourceItemChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hideSourceChanged'
        QtMocHelpers::SignalData<void()>(5, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'radiusChanged'
        QtMocHelpers::SignalData<void()>(6, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'topLeftRadiusChanged'
        QtMocHelpers::SignalData<void()>(7, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'topRightRadiusChanged'
        QtMocHelpers::SignalData<void()>(8, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'bottomLeftRadiusChanged'
        QtMocHelpers::SignalData<void()>(9, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'bottomRightRadiusChanged'
        QtMocHelpers::SignalData<void()>(10, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'sourceItemDestroyed'
        QtMocHelpers::SlotData<void(QObject *)>(11, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QObjectStar, 12 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'radius'
        QtMocHelpers::PropertyData<qreal>(13, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'topLeftRadius'
        QtMocHelpers::PropertyData<qreal>(14, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet | QMC::Final, 3),
        // property 'topRightRadius'
        QtMocHelpers::PropertyData<qreal>(15, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet | QMC::Final, 4),
        // property 'bottomLeftRadius'
        QtMocHelpers::PropertyData<qreal>(16, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet | QMC::Final, 5),
        // property 'bottomRightRadius'
        QtMocHelpers::PropertyData<qreal>(17, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet | QMC::Final, 6),
        // property 'sourceItem'
        QtMocHelpers::PropertyData<QQuickItem*>(18, 0x80000000 | 19, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'hideSource'
        QtMocHelpers::PropertyData<bool>(20, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<TQuickRadiusEffect, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject TQuickRadiusEffect::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18TQuickRadiusEffectE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18TQuickRadiusEffectE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18TQuickRadiusEffectE_t>.metaTypes,
    nullptr
} };

void TQuickRadiusEffect::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TQuickRadiusEffect *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sourceItemChanged(); break;
        case 1: _t->hideSourceChanged(); break;
        case 2: _t->radiusChanged(); break;
        case 3: _t->topLeftRadiusChanged(); break;
        case 4: _t->topRightRadiusChanged(); break;
        case 5: _t->bottomLeftRadiusChanged(); break;
        case 6: _t->bottomRightRadiusChanged(); break;
        case 7: _t->sourceItemDestroyed((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TQuickRadiusEffect::*)()>(_a, &TQuickRadiusEffect::sourceItemChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TQuickRadiusEffect::*)()>(_a, &TQuickRadiusEffect::hideSourceChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TQuickRadiusEffect::*)()>(_a, &TQuickRadiusEffect::radiusChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TQuickRadiusEffect::*)()>(_a, &TQuickRadiusEffect::topLeftRadiusChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (TQuickRadiusEffect::*)()>(_a, &TQuickRadiusEffect::topRightRadiusChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (TQuickRadiusEffect::*)()>(_a, &TQuickRadiusEffect::bottomLeftRadiusChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (TQuickRadiusEffect::*)()>(_a, &TQuickRadiusEffect::bottomRightRadiusChanged, 6))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItem* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<qreal*>(_v) = _t->radius(); break;
        case 1: *reinterpret_cast<qreal*>(_v) = _t->topLeftRadius(); break;
        case 2: *reinterpret_cast<qreal*>(_v) = _t->topRightRadius(); break;
        case 3: *reinterpret_cast<qreal*>(_v) = _t->bottomLeftRadius(); break;
        case 4: *reinterpret_cast<qreal*>(_v) = _t->bottomRightRadius(); break;
        case 5: *reinterpret_cast<QQuickItem**>(_v) = _t->sourceItem(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->hideSource(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setRadius(*reinterpret_cast<qreal*>(_v)); break;
        case 1: _t->setTopLeftRadius(*reinterpret_cast<qreal*>(_v)); break;
        case 2: _t->setTopRightRadius(*reinterpret_cast<qreal*>(_v)); break;
        case 3: _t->setBottomLeftRadius(*reinterpret_cast<qreal*>(_v)); break;
        case 4: _t->setBottomRightRadius(*reinterpret_cast<qreal*>(_v)); break;
        case 5: _t->setSourceItem(*reinterpret_cast<QQuickItem**>(_v)); break;
        case 6: _t->setHideSource(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
    if (_c == QMetaObject::ResetProperty) {
        switch (_id) {
        case 1: _t->resetTopLeftRadius(); break;
        case 2: _t->resetTopRightRadius(); break;
        case 3: _t->resetBottomLeftRadius(); break;
        case 4: _t->resetBottomRightRadius(); break;
        default: break;
        }
    }
}

const QMetaObject *TQuickRadiusEffect::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TQuickRadiusEffect::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18TQuickRadiusEffectE_t>.strings))
        return static_cast<void*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int TQuickRadiusEffect::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void TQuickRadiusEffect::sourceItemChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void TQuickRadiusEffect::hideSourceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void TQuickRadiusEffect::radiusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void TQuickRadiusEffect::topLeftRadiusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void TQuickRadiusEffect::topRightRadiusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void TQuickRadiusEffect::bottomLeftRadiusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void TQuickRadiusEffect::bottomRightRadiusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
