/****************************************************************************
** Meta object code from reading C++ file 'wqmlhelper_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../waylib/src/server/qtquick/private/wqmlhelper_p.h"
#include "private/wserver_p.h"
#include "woutput.h"
#include "woutput.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wqmlhelper_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Waylib6Server10WQmlHelperE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WQmlHelper::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server10WQmlHelperE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WQmlHelper",
        "QML.Element",
        "WaylibHelper",
        "QML.Singleton",
        "true",
        "itemStackToTop",
        "",
        "QQuickItem*",
        "item",
        "setCursorShape",
        "WGlobal::CursorShape",
        "shape",
        "getEdges",
        "Qt::Edges",
        "QRectF",
        "rect",
        "QPointF",
        "pos",
        "edgeSize",
        "scaleSize",
        "QSizeF",
        "from",
        "to",
        "Qt::AspectRatioMode",
        "mode",
        "hasXWayland"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'itemStackToTop'
        QtMocHelpers::SlotData<void(QQuickItem *)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'setCursorShape'
        QtMocHelpers::SlotData<void(QQuickItem *, WGlobal::CursorShape)>(9, 6, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 }, { 0x80000000 | 10, 11 },
        }}),
        // Slot 'getEdges'
        QtMocHelpers::SlotData<Qt::Edges(const QRectF &, const QPointF &, qreal)>(12, 6, QMC::AccessPublic, 0x80000000 | 13, {{
            { 0x80000000 | 14, 15 }, { 0x80000000 | 16, 17 }, { QMetaType::QReal, 18 },
        }}),
        // Slot 'scaleSize'
        QtMocHelpers::SlotData<QSizeF(const QSizeF &, const QSizeF &, Qt::AspectRatioMode)>(19, 6, QMC::AccessPublic, 0x80000000 | 20, {{
            { 0x80000000 | 20, 21 }, { 0x80000000 | 20, 22 }, { 0x80000000 | 23, 24 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'hasXWayland'
        QtMocHelpers::PropertyData<bool>(25, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<WQmlHelper, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WQmlHelper::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server10WQmlHelperE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server10WQmlHelperE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server10WQmlHelperE_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WQmlHelper::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WQmlHelper *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->itemStackToTop((*reinterpret_cast<std::add_pointer_t<QQuickItem*>>(_a[1]))); break;
        case 1: _t->setCursorShape((*reinterpret_cast<std::add_pointer_t<QQuickItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<WGlobal::CursorShape>>(_a[2]))); break;
        case 2: { Qt::Edges _r = _t->getEdges((*reinterpret_cast<std::add_pointer_t<QRectF>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPointF>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[3])));
            if (_a[0]) *reinterpret_cast<Qt::Edges*>(_a[0]) = std::move(_r); }  break;
        case 3: { QSizeF _r = _t->scaleSize((*reinterpret_cast<std::add_pointer_t<QSizeF>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QSizeF>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<Qt::AspectRatioMode>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QSizeF*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->hasXWayland(); break;
        default: break;
        }
    }
}

const QMetaObject *Waylib::Server::WQmlHelper::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WQmlHelper::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server10WQmlHelperE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Waylib::Server::WQmlHelper::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}
QT_WARNING_POP
