/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was org.freedesktop.login1.Seat.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#ifndef LOGIN1SEAT_H
#define LOGIN1SEAT_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>
#include "loginddbustypes.h"

/*
 * Proxy class for interface org.freedesktop.login1.Seat
 */
class OrgFreedesktopLogin1SeatInterface: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    static inline const char *staticInterfaceName()
    { return "org.freedesktop.login1.Seat"; }

public:
    OrgFreedesktopLogin1SeatInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent = nullptr);

    ~OrgFreedesktopLogin1SeatInterface();

    Q_PROPERTY(NamedSessionPath ActiveSession READ activeSession)
    inline NamedSessionPath activeSession() const
    { return qvariant_cast< NamedSessionPath >(property("ActiveSession")); }

    Q_PROPERTY(bool CanGraphical READ canGraphical)
    inline bool canGraphical() const
    { return qvariant_cast< bool >(property("CanGraphical")); }

    Q_PROPERTY(bool CanMultiSession READ canMultiSession)
    inline bool canMultiSession() const
    { return qvariant_cast< bool >(property("CanMultiSession")); }

    Q_PROPERTY(bool CanTTY READ canTTY)
    inline bool canTTY() const
    { return qvariant_cast< bool >(property("CanTTY")); }

    Q_PROPERTY(QString Id READ id)
    inline QString id() const
    { return qvariant_cast< QString >(property("Id")); }

    Q_PROPERTY(bool IdleHint READ idleHint)
    inline bool idleHint() const
    { return qvariant_cast< bool >(property("IdleHint")); }

    Q_PROPERTY(qulonglong IdleSinceHint READ idleSinceHint)
    inline qulonglong idleSinceHint() const
    { return qvariant_cast< qulonglong >(property("IdleSinceHint")); }

    Q_PROPERTY(qulonglong IdleSinceHintMonotonic READ idleSinceHintMonotonic)
    inline qulonglong idleSinceHintMonotonic() const
    { return qvariant_cast< qulonglong >(property("IdleSinceHintMonotonic")); }

    Q_PROPERTY(QList<NamedSessionPath> Sessions READ sessions)
    inline QList<NamedSessionPath> sessions() const
    { return qvariant_cast< QList<NamedSessionPath> >(property("Sessions")); }

public Q_SLOTS: // METHODS
    inline QDBusPendingReply<> ActivateSession(const QString &in0)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0);
        return asyncCallWithArgumentList(QStringLiteral("ActivateSession"), argumentList);
    }

    inline QDBusPendingReply<> SwitchTo(uint in0)
    {
        QList<QVariant> argumentList;
        argumentList << QVariant::fromValue(in0);
        return asyncCallWithArgumentList(QStringLiteral("SwitchTo"), argumentList);
    }

    inline QDBusPendingReply<> SwitchToNext()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("SwitchToNext"), argumentList);
    }

    inline QDBusPendingReply<> SwitchToPrevious()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("SwitchToPrevious"), argumentList);
    }

    inline QDBusPendingReply<> Terminate()
    {
        QList<QVariant> argumentList;
        return asyncCallWithArgumentList(QStringLiteral("Terminate"), argumentList);
    }

Q_SIGNALS: // SIGNALS
};

namespace org {
  namespace freedesktop {
    namespace login1 {
      using Seat = ::OrgFreedesktopLogin1SeatInterface;
    }
  }
}

#endif
