/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<ddeshellattached.h>)
#  include <ddeshellattached.h>
#endif
#if __has_include(<fpsdisplaymanager.h>)
#  include <fpsdisplaymanager.h>
#endif
#if __has_include(<greeterproxy.h>)
#  include <greeterproxy.h>
#endif
#if __has_include(<helper.h>)
#  include <helper.h>
#endif
#if __has_include(<itemselector.h>)
#  include <itemselector.h>
#endif
#if __has_include(<lockscreen.h>)
#  include <lockscreen.h>
#endif
#if __has_include(<output.h>)
#  include <output.h>
#endif
#if __has_include(<personalizationmanagerinterfacev1.h>)
#  include <personalizationmanagerinterfacev1.h>
#endif
#if __has_include(<propertymonitor.h>)
#  include <propertymonitor.h>
#endif
#if __has_include(<rootsurfacecontainer.h>)
#  include <rootsurfacecontainer.h>
#endif
#if __has_include(<sessionmodel.h>)
#  include <sessionmodel.h>
#endif
#if __has_include(<surfacecontainer.h>)
#  include <surfacecontainer.h>
#endif
#if __has_include(<surfaceproxy.h>)
#  include <surfaceproxy.h>
#endif
#if __has_include(<surfacewrapper.h>)
#  include <surfacewrapper.h>
#endif
#if __has_include(<tquickradiuseffect.h>)
#  include <tquickradiuseffect.h>
#endif
#if __has_include(<usermodel.h>)
#  include <usermodel.h>
#endif
#if __has_include(<wallpaperitem.h>)
#  include <wallpaperitem.h>
#endif
#if __has_include(<wallpapersurface.h>)
#  include <wallpapersurface.h>
#endif
#if __has_include(<wallpaperswitcheritem.h>)
#  include <wallpaperswitcheritem.h>
#endif
#if __has_include(<windowpicker.h>)
#  include <windowpicker.h>
#endif
#if __has_include(<workspace.h>)
#  include <workspace.h>
#endif
#if __has_include(<workspaceanimationcontroller.h>)
#  include <workspaceanimationcontroller.h>
#endif
#if __has_include(<workspacemodel.h>)
#  include <workspacemodel.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
Q_QMLTYPE_EXPORT void qml_register_types_Treeland()
{
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    qmlRegisterTypesAndRevisions<DDEShellAttached>("Treeland", 2);
    qmlRegisterTypesAndRevisions<DDEShellHelper>("Treeland", 2);
    qmlRegisterTypesAndRevisions<FpsDisplayManager>("Treeland", 2);
    qmlRegisterTypesAndRevisions<GreeterProxy>("Treeland", 2);
    qmlRegisterTypesAndRevisions<Helper>("Treeland", 2);
    qmlRegisterEnum<Helper::OutputMode>("Helper::OutputMode");
    qmlRegisterEnum<Helper::CurrentMode>("Helper::CurrentMode");
    qmlRegisterTypesAndRevisions<ItemSelector>("Treeland", 2);
    qmlRegisterAnonymousType<QQuickItem, 254>("Treeland", 2);
    qmlRegisterEnum<ItemSelector::ItemType>("ItemSelector::ItemType");
    qmlRegisterTypesAndRevisions<LockScreen>("Treeland", 2);
    qmlRegisterEnum<LockScreen::CurrentMode>("LockScreen::CurrentMode");
    qmlRegisterTypesAndRevisions<Output>("Treeland", 2);
    qmlRegisterTypesAndRevisions<OutputListModel>("Treeland", 2);
    qmlRegisterTypesAndRevisions<Personalization>("Treeland", 2);
    qmlRegisterEnum<Personalization::BackgroundType>("Personalization::BackgroundType");
    qmlRegisterTypesAndRevisions<PropertyMonitor>("Treeland", 2);
    qmlRegisterTypesAndRevisions<SessionModel>("Treeland", 2);
    qmlRegisterAnonymousType<QAbstractItemModel, 254>("Treeland", 2);
    qmlRegisterEnum<SessionModel::SessionRole>("SessionModel::SessionRole");
    qmlRegisterTypesAndRevisions<SurfaceContainer>("Treeland", 2);
    qmlRegisterTypesAndRevisions<SurfaceFilterModel>("Treeland", 2);
    qmlRegisterTypesAndRevisions<SurfaceListModel>("Treeland", 2);
    qmlRegisterTypesAndRevisions<SurfaceProxy>("Treeland", 2);
    qmlRegisterTypesAndRevisions<SurfaceWrapper>("Treeland", 2);
    qmlRegisterEnum<SurfaceWrapper::Type>("SurfaceWrapper::Type");
    qmlRegisterEnum<SurfaceWrapper::State>("SurfaceWrapper::State");
    qmlRegisterEnum<SurfaceWrapper::ActiveControlState>("SurfaceWrapper::ActiveControlState");
    qmlRegisterEnum<SurfaceWrapper::FocusControlState>("SurfaceWrapper::FocusControlState");
    qmlRegisterEnum<SurfaceWrapper::SurfaceRole>("SurfaceWrapper::SurfaceRole");
    qmlRegisterTypesAndRevisions<TQuickRadiusEffect>("Treeland", 2);
    qmlRegisterTypesAndRevisions<UserModel>("Treeland", 2);
    qmlRegisterEnum<UserModel::UserRoles>("UserModel::UserRoles");
    qmlRegisterTypesAndRevisions<WallpaperItem>("Treeland", 2);
    qmlRegisterEnum<WallpaperItem::WallpaperRole>("WallpaperItem::WallpaperRole");
    qmlRegisterEnum<WallpaperItem::WallpaperState>("WallpaperItem::WallpaperState");
    qmlRegisterTypesAndRevisions<WallpaperSurface>("Treeland", 2);
    qmlRegisterTypesAndRevisions<WallpaperSwitcherItem>("Treeland", 2);
    qmlRegisterTypesAndRevisions<WindowOverlapChecker>("Treeland", 2);
    qmlRegisterTypesAndRevisions<WindowPicker>("Treeland", 2);
    qmlRegisterTypesAndRevisions<Workspace>("Treeland", 2);
    qmlRegisterTypesAndRevisions<WorkspaceAnimationController>("Treeland", 2);
    qmlRegisterEnum<WorkspaceAnimationController::Direction>("WorkspaceAnimationController::Direction");
    qmlRegisterTypesAndRevisions<WorkspaceListModel>("Treeland", 2);
    qmlRegisterTypesAndRevisions<WorkspaceModel>("Treeland", 2);
    QT_WARNING_POP
    qmlRegisterModule("Treeland", 2, 0);
}

static const QQmlModuleRegistration treelandRegistration("Treeland", qml_register_types_Treeland);
