/****************************************************************************
** Meta object code from reading C++ file 'wtextinputv1_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../waylib/src/server/protocols/private/wtextinputv1_p.h"
#include "private/wserver_p.h"
#include "wsurface.h"
#include "winputmethodv2_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wtextinputv1_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Waylib6Server12WTextInputV1E_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WTextInputV1::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server12WTextInputV1E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WTextInputV1",
        "activate",
        "",
        "deactivate",
        "sendEnter",
        "WSurface*",
        "surface",
        "sendLeave",
        "sendDone",
        "handleIMCommitted",
        "WInputMethodV2*",
        "im",
        "ContentHint",
        "CH_None",
        "CH_Default",
        "CH_Password",
        "CH_AutoCompletion",
        "CH_AutoCorrection",
        "CH_AutoCapitalization",
        "CH_Lowercase",
        "CH_Uppercase",
        "CH_Titlecase",
        "CH_HiddenText",
        "CH_SensitiveData",
        "CH_Latin",
        "CH_Multiline",
        "ContentPurpose",
        "CP_Normal",
        "CP_Alpha",
        "CP_Digits",
        "CP_Number",
        "CP_Phone",
        "CP_Url",
        "CP_Email",
        "CP_Name",
        "CP_Password",
        "CP_Date",
        "CP_Time",
        "CP_Datetime",
        "CP_Terminal",
        "PreeditStyle",
        "PS_Default",
        "PS_None",
        "PS_Active",
        "PS_Inactive",
        "PS_Highlight",
        "PS_Underline",
        "PS_Selection",
        "PS_Incorrect"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'activate'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'deactivate'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'sendEnter'
        QtMocHelpers::SlotData<void(WSurface *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'sendLeave'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'sendDone'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleIMCommitted'
        QtMocHelpers::SlotData<void(WInputMethodV2 *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ContentHint'
        QtMocHelpers::EnumData<enum ContentHint>(12, 12, QMC::EnumFlags{}).add({
            {   13, ContentHint::CH_None },
            {   14, ContentHint::CH_Default },
            {   15, ContentHint::CH_Password },
            {   16, ContentHint::CH_AutoCompletion },
            {   17, ContentHint::CH_AutoCorrection },
            {   18, ContentHint::CH_AutoCapitalization },
            {   19, ContentHint::CH_Lowercase },
            {   20, ContentHint::CH_Uppercase },
            {   21, ContentHint::CH_Titlecase },
            {   22, ContentHint::CH_HiddenText },
            {   23, ContentHint::CH_SensitiveData },
            {   24, ContentHint::CH_Latin },
            {   25, ContentHint::CH_Multiline },
        }),
        // enum 'ContentPurpose'
        QtMocHelpers::EnumData<enum ContentPurpose>(26, 26, QMC::EnumFlags{}).add({
            {   27, ContentPurpose::CP_Normal },
            {   28, ContentPurpose::CP_Alpha },
            {   29, ContentPurpose::CP_Digits },
            {   30, ContentPurpose::CP_Number },
            {   31, ContentPurpose::CP_Phone },
            {   32, ContentPurpose::CP_Url },
            {   33, ContentPurpose::CP_Email },
            {   34, ContentPurpose::CP_Name },
            {   35, ContentPurpose::CP_Password },
            {   36, ContentPurpose::CP_Date },
            {   37, ContentPurpose::CP_Time },
            {   38, ContentPurpose::CP_Datetime },
            {   39, ContentPurpose::CP_Terminal },
        }),
        // enum 'PreeditStyle'
        QtMocHelpers::EnumData<enum PreeditStyle>(40, 40, QMC::EnumFlags{}).add({
            {   41, PreeditStyle::PS_Default },
            {   42, PreeditStyle::PS_None },
            {   43, PreeditStyle::PS_Active },
            {   44, PreeditStyle::PS_Inactive },
            {   45, PreeditStyle::PS_Highlight },
            {   46, PreeditStyle::PS_Underline },
            {   47, PreeditStyle::PS_Selection },
            {   48, PreeditStyle::PS_Incorrect },
        }),
    };
    return QtMocHelpers::metaObjectData<WTextInputV1, qt_meta_tag_ZN6Waylib6Server12WTextInputV1E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WTextInputV1::staticMetaObject = { {
    QMetaObject::SuperData::link<WTextInput::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server12WTextInputV1E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server12WTextInputV1E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server12WTextInputV1E_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WTextInputV1::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WTextInputV1 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->activate(); break;
        case 1: _t->deactivate(); break;
        case 2: _t->sendEnter((*reinterpret_cast<std::add_pointer_t<WSurface*>>(_a[1]))); break;
        case 3: _t->sendLeave(); break;
        case 4: _t->sendDone(); break;
        case 5: _t->handleIMCommitted((*reinterpret_cast<std::add_pointer_t<WInputMethodV2*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WTextInputV1::*)()>(_a, &WTextInputV1::activate, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WTextInputV1::*)()>(_a, &WTextInputV1::deactivate, 1))
            return;
    }
}

const QMetaObject *Waylib::Server::WTextInputV1::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WTextInputV1::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server12WTextInputV1E_t>.strings))
        return static_cast<void*>(this);
    return WTextInput::qt_metacast(_clname);
}

int Waylib::Server::WTextInputV1::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = WTextInput::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WTextInputV1::activate()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Waylib::Server::WTextInputV1::deactivate()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
namespace {
struct qt_meta_tag_ZN6Waylib6Server19WTextInputManagerV1E_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WTextInputManagerV1::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server19WTextInputManagerV1E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WTextInputManagerV1",
        "newTextInput",
        "",
        "WTextInputV1*",
        "textInput"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'newTextInput'
        QtMocHelpers::SignalData<void(WTextInputV1 *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WTextInputManagerV1, qt_meta_tag_ZN6Waylib6Server19WTextInputManagerV1E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WTextInputManagerV1::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server19WTextInputManagerV1E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server19WTextInputManagerV1E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server19WTextInputManagerV1E_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WTextInputManagerV1::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WTextInputManagerV1 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->newTextInput((*reinterpret_cast<std::add_pointer_t<WTextInputV1*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< WTextInputV1* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WTextInputManagerV1::*)(WTextInputV1 * )>(_a, &WTextInputManagerV1::newTextInput, 0))
            return;
    }
}

const QMetaObject *Waylib::Server::WTextInputManagerV1::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WTextInputManagerV1::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server19WTextInputManagerV1E_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "WObject"))
        return static_cast< WObject*>(this);
    if (!strcmp(_clname, "WServerInterface"))
        return static_cast< WServerInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int Waylib::Server::WTextInputManagerV1::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WTextInputManagerV1::newTextInput(WTextInputV1 * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
