/****************************************************************************
** Meta object code from reading C++ file 'foreigntoplevelmanagerv1.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/modules/foreign-toplevel/foreigntoplevelmanagerv1.h"
#include "private/wserver_p.h"
#include "woutput.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'foreigntoplevelmanagerv1.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN33ForeignToplevelManagerInterfaceV1E_t {};
} // unnamed namespace

template <> constexpr inline auto ForeignToplevelManagerInterfaceV1::qt_create_metaobjectdata<qt_meta_tag_ZN33ForeignToplevelManagerInterfaceV1E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ForeignToplevelManagerInterfaceV1",
        "RegisterEnumClassesUnscoped",
        "false",
        "requestDockPreview",
        "",
        "std::vector<SurfaceWrapper*>",
        "surfaces",
        "WSurface*",
        "target",
        "QPoint",
        "abs",
        "PreviewDirection",
        "direction",
        "requestDockPreviewTooltip",
        "tooltip",
        "requestDockClose",
        "enterDockPreview",
        "relativeSurface",
        "leaveDockPreview",
        "top",
        "right",
        "bottom",
        "left"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'requestDockPreview'
        QtMocHelpers::SignalData<void(std::vector<SurfaceWrapper*>, WSurface *, QPoint, enum PreviewDirection)>(3, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { 0x80000000 | 7, 8 }, { 0x80000000 | 9, 10 }, { 0x80000000 | 11, 12 },
        }}),
        // Signal 'requestDockPreviewTooltip'
        QtMocHelpers::SignalData<void(QString, WSurface *, QPoint, enum PreviewDirection)>(13, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 }, { 0x80000000 | 7, 8 }, { 0x80000000 | 9, 10 }, { 0x80000000 | 11, 12 },
        }}),
        // Signal 'requestDockClose'
        QtMocHelpers::SignalData<void()>(15, 4, QMC::AccessPublic, QMetaType::Void),
        // Method 'enterDockPreview'
        QtMocHelpers::MethodData<void(WSurface *)>(16, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 17 },
        }}),
        // Method 'leaveDockPreview'
        QtMocHelpers::MethodData<void(WSurface *)>(18, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 17 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'PreviewDirection'
        QtMocHelpers::EnumData<enum PreviewDirection>(11, 11, QMC::EnumIsScoped).add({
            {   19, PreviewDirection::top },
            {   20, PreviewDirection::right },
            {   21, PreviewDirection::bottom },
            {   22, PreviewDirection::left },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<ForeignToplevelManagerInterfaceV1, qt_meta_tag_ZN33ForeignToplevelManagerInterfaceV1E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject ForeignToplevelManagerInterfaceV1::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN33ForeignToplevelManagerInterfaceV1E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN33ForeignToplevelManagerInterfaceV1E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN33ForeignToplevelManagerInterfaceV1E_t>.metaTypes,
    nullptr
} };

void ForeignToplevelManagerInterfaceV1::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ForeignToplevelManagerInterfaceV1 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->requestDockPreview((*reinterpret_cast<std::add_pointer_t<std::vector<SurfaceWrapper*>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<WSurface*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<enum PreviewDirection>>(_a[4]))); break;
        case 1: _t->requestDockPreviewTooltip((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<WSurface*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<enum PreviewDirection>>(_a[4]))); break;
        case 2: _t->requestDockClose(); break;
        case 3: _t->enterDockPreview((*reinterpret_cast<std::add_pointer_t<WSurface*>>(_a[1]))); break;
        case 4: _t->leaveDockPreview((*reinterpret_cast<std::add_pointer_t<WSurface*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< WSurface* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< WSurface* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< WSurface* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< WSurface* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ForeignToplevelManagerInterfaceV1::*)(std::vector<SurfaceWrapper*> , WSurface * , QPoint , PreviewDirection )>(_a, &ForeignToplevelManagerInterfaceV1::requestDockPreview, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ForeignToplevelManagerInterfaceV1::*)(QString , WSurface * , QPoint , PreviewDirection )>(_a, &ForeignToplevelManagerInterfaceV1::requestDockPreviewTooltip, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ForeignToplevelManagerInterfaceV1::*)()>(_a, &ForeignToplevelManagerInterfaceV1::requestDockClose, 2))
            return;
    }
}

const QMetaObject *ForeignToplevelManagerInterfaceV1::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ForeignToplevelManagerInterfaceV1::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN33ForeignToplevelManagerInterfaceV1E_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "WServerInterface"))
        return static_cast< WServerInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int ForeignToplevelManagerInterfaceV1::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void ForeignToplevelManagerInterfaceV1::requestDockPreview(std::vector<SurfaceWrapper*> _t1, WSurface * _t2, QPoint _t3, PreviewDirection _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 1
void ForeignToplevelManagerInterfaceV1::requestDockPreviewTooltip(QString _t1, WSurface * _t2, QPoint _t3, PreviewDirection _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 2
void ForeignToplevelManagerInterfaceV1::requestDockClose()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
