/****************************************************************************
** Meta object code from reading C++ file 'greeterproxy.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/greeter/greeterproxy.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'greeterproxy.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12GreeterProxyE_t {};
} // unnamed namespace

template <> constexpr inline auto GreeterProxy::qt_create_metaobjectdata<qt_meta_tag_ZN12GreeterProxyE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GreeterProxy",
        "QML.Element",
        "QML.Singleton",
        "true",
        "informationMessage",
        "",
        "message",
        "switchUser",
        "socketDisconnected",
        "hostNameChanged",
        "hostName",
        "canPowerOffChanged",
        "canPowerOff",
        "canRebootChanged",
        "canReboot",
        "canSuspendChanged",
        "canSuspend",
        "canHibernateChanged",
        "canHibernate",
        "canHybridSleepChanged",
        "canHybridSleep",
        "lockChanged",
        "isLocked",
        "failedAttemptsChanged",
        "failedAttempts",
        "showShutdownViewChanged",
        "showShutdownView",
        "showAnimationChanged",
        "showAnimation",
        "hasActiveSessionChanged",
        "hasActiveSession",
        "powerOff",
        "reboot",
        "suspend",
        "hibernate",
        "hybridSleep",
        "login",
        "user",
        "password",
        "sessionIndex",
        "lock",
        "unlock",
        "logout",
        "connected",
        "disconnected",
        "readyRead",
        "error",
        "onSessionNew",
        "id",
        "QDBusObjectPath",
        "session",
        "onSessionRemoved",
        "onSessionLock",
        "onSessionUnlock"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'informationMessage'
        QtMocHelpers::SignalData<void(const QString &)>(4, 5, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'switchUser'
        QtMocHelpers::SignalData<void()>(7, 5, QMC::AccessPublic, QMetaType::Void),
        // Signal 'socketDisconnected'
        QtMocHelpers::SignalData<void()>(8, 5, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hostNameChanged'
        QtMocHelpers::SignalData<void(const QString &)>(9, 5, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 },
        }}),
        // Signal 'canPowerOffChanged'
        QtMocHelpers::SignalData<void(bool)>(11, 5, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 12 },
        }}),
        // Signal 'canRebootChanged'
        QtMocHelpers::SignalData<void(bool)>(13, 5, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 14 },
        }}),
        // Signal 'canSuspendChanged'
        QtMocHelpers::SignalData<void(bool)>(15, 5, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Signal 'canHibernateChanged'
        QtMocHelpers::SignalData<void(bool)>(17, 5, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 18 },
        }}),
        // Signal 'canHybridSleepChanged'
        QtMocHelpers::SignalData<void(bool)>(19, 5, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 20 },
        }}),
        // Signal 'lockChanged'
        QtMocHelpers::SignalData<void(bool)>(21, 5, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 22 },
        }}),
        // Signal 'failedAttemptsChanged'
        QtMocHelpers::SignalData<void(int)>(23, 5, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 24 },
        }}),
        // Signal 'showShutdownViewChanged'
        QtMocHelpers::SignalData<void(bool)>(25, 5, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 26 },
        }}),
        // Signal 'showAnimationChanged'
        QtMocHelpers::SignalData<void(bool)>(27, 5, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 28 },
        }}),
        // Signal 'hasActiveSessionChanged'
        QtMocHelpers::SignalData<void(bool)>(29, 5, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 30 },
        }}),
        // Slot 'powerOff'
        QtMocHelpers::SlotData<void()>(31, 5, QMC::AccessPublic, QMetaType::Void),
        // Slot 'reboot'
        QtMocHelpers::SlotData<void()>(32, 5, QMC::AccessPublic, QMetaType::Void),
        // Slot 'suspend'
        QtMocHelpers::SlotData<void()>(33, 5, QMC::AccessPublic, QMetaType::Void),
        // Slot 'hibernate'
        QtMocHelpers::SlotData<void()>(34, 5, QMC::AccessPublic, QMetaType::Void),
        // Slot 'hybridSleep'
        QtMocHelpers::SlotData<void()>(35, 5, QMC::AccessPublic, QMetaType::Void),
        // Slot 'login'
        QtMocHelpers::SlotData<void(const QString &, const QString &, int)>(36, 5, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 37 }, { QMetaType::QString, 38 }, { QMetaType::Int, 39 },
        }}),
        // Slot 'lock'
        QtMocHelpers::SlotData<void()>(40, 5, QMC::AccessPublic, QMetaType::Void),
        // Slot 'unlock'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(41, 5, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 37 }, { QMetaType::QString, 38 },
        }}),
        // Slot 'logout'
        QtMocHelpers::SlotData<void()>(42, 5, QMC::AccessPublic, QMetaType::Void),
        // Slot 'connected'
        QtMocHelpers::SlotData<void()>(43, 5, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'disconnected'
        QtMocHelpers::SlotData<void()>(44, 5, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'readyRead'
        QtMocHelpers::SlotData<void()>(45, 5, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'error'
        QtMocHelpers::SlotData<void()>(46, 5, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onSessionNew'
        QtMocHelpers::SlotData<void(const QString &, const QDBusObjectPath &)>(47, 5, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 48 }, { 0x80000000 | 49, 50 },
        }}),
        // Slot 'onSessionRemoved'
        QtMocHelpers::SlotData<void(const QString &, const QDBusObjectPath &)>(51, 5, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 48 }, { 0x80000000 | 49, 50 },
        }}),
        // Slot 'onSessionLock'
        QtMocHelpers::SlotData<void()>(52, 5, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onSessionUnlock'
        QtMocHelpers::SlotData<void()>(53, 5, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'hostName'
        QtMocHelpers::PropertyData<QString>(10, QMetaType::QString, QMC::DefaultPropertyFlags, 3),
        // property 'canPowerOff'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags, 4),
        // property 'canReboot'
        QtMocHelpers::PropertyData<bool>(14, QMetaType::Bool, QMC::DefaultPropertyFlags, 5),
        // property 'canSuspend'
        QtMocHelpers::PropertyData<bool>(16, QMetaType::Bool, QMC::DefaultPropertyFlags, 6),
        // property 'canHibernate'
        QtMocHelpers::PropertyData<bool>(18, QMetaType::Bool, QMC::DefaultPropertyFlags, 7),
        // property 'canHybridSleep'
        QtMocHelpers::PropertyData<bool>(20, QMetaType::Bool, QMC::DefaultPropertyFlags, 8),
        // property 'isLocked'
        QtMocHelpers::PropertyData<bool>(22, QMetaType::Bool, QMC::DefaultPropertyFlags, 9),
        // property 'failedAttempts'
        QtMocHelpers::PropertyData<int>(24, QMetaType::Int, QMC::DefaultPropertyFlags, 10),
        // property 'showShutdownView'
        QtMocHelpers::PropertyData<bool>(26, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11),
        // property 'showAnimation'
        QtMocHelpers::PropertyData<bool>(28, QMetaType::Bool, QMC::DefaultPropertyFlags, 12),
        // property 'hasActiveSession'
        QtMocHelpers::PropertyData<bool>(30, QMetaType::Bool, QMC::DefaultPropertyFlags, 13),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    0 },
            {    2,    3 },
    });
    return QtMocHelpers::metaObjectData<GreeterProxy, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject GreeterProxy::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12GreeterProxyE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12GreeterProxyE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12GreeterProxyE_t>.metaTypes,
    nullptr
} };

void GreeterProxy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<GreeterProxy *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->informationMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->switchUser(); break;
        case 2: _t->socketDisconnected(); break;
        case 3: _t->hostNameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->canPowerOffChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->canRebootChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->canSuspendChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->canHibernateChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->canHybridSleepChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->lockChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->failedAttemptsChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->showShutdownViewChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->showAnimationChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->hasActiveSessionChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->powerOff(); break;
        case 15: _t->reboot(); break;
        case 16: _t->suspend(); break;
        case 17: _t->hibernate(); break;
        case 18: _t->hybridSleep(); break;
        case 19: _t->login((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 20: _t->lock(); break;
        case 21: _t->unlock((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 22: _t->logout(); break;
        case 23: _t->connected(); break;
        case 24: _t->disconnected(); break;
        case 25: _t->readyRead(); break;
        case 26: _t->error(); break;
        case 27: _t->onSessionNew((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[2]))); break;
        case 28: _t->onSessionRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusObjectPath>>(_a[2]))); break;
        case 29: _t->onSessionLock(); break;
        case 30: _t->onSessionUnlock(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 27:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            }
            break;
        case 28:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusObjectPath >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (GreeterProxy::*)(const QString & )>(_a, &GreeterProxy::informationMessage, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (GreeterProxy::*)()>(_a, &GreeterProxy::switchUser, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (GreeterProxy::*)()>(_a, &GreeterProxy::socketDisconnected, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (GreeterProxy::*)(const QString & )>(_a, &GreeterProxy::hostNameChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (GreeterProxy::*)(bool )>(_a, &GreeterProxy::canPowerOffChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (GreeterProxy::*)(bool )>(_a, &GreeterProxy::canRebootChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (GreeterProxy::*)(bool )>(_a, &GreeterProxy::canSuspendChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (GreeterProxy::*)(bool )>(_a, &GreeterProxy::canHibernateChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (GreeterProxy::*)(bool )>(_a, &GreeterProxy::canHybridSleepChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (GreeterProxy::*)(bool )>(_a, &GreeterProxy::lockChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (GreeterProxy::*)(int )>(_a, &GreeterProxy::failedAttemptsChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (GreeterProxy::*)(bool )>(_a, &GreeterProxy::showShutdownViewChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (GreeterProxy::*)(bool )>(_a, &GreeterProxy::showAnimationChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (GreeterProxy::*)(bool )>(_a, &GreeterProxy::hasActiveSessionChanged, 13))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->hostName(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->canPowerOff(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->canReboot(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->canSuspend(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->canHibernate(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->canHybridSleep(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->isLocked(); break;
        case 7: *reinterpret_cast<int*>(_v) = _t->failedAttempts(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->showShutdownView(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->showAnimation(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->hasActiveSession(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 8: _t->setShowShutdownView(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *GreeterProxy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GreeterProxy::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12GreeterProxyE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QDBusContext"))
        return static_cast< QDBusContext*>(this);
    return QObject::qt_metacast(_clname);
}

int GreeterProxy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 31)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 31;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 31)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 31;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void GreeterProxy::informationMessage(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void GreeterProxy::switchUser()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void GreeterProxy::socketDisconnected()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void GreeterProxy::hostNameChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void GreeterProxy::canPowerOffChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void GreeterProxy::canRebootChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void GreeterProxy::canSuspendChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void GreeterProxy::canHibernateChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void GreeterProxy::canHybridSleepChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void GreeterProxy::lockChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void GreeterProxy::failedAttemptsChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void GreeterProxy::showShutdownViewChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void GreeterProxy::showAnimationChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void GreeterProxy::hasActiveSessionChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}
QT_WARNING_POP
