/****************************************************************************
** Meta object code from reading C++ file 'winputmethodv2_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../waylib/src/server/protocols/private/winputmethodv2_p.h"
#include "private/wserver_p.h"
#include "woutput.h"
#include "wsurface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'winputmethodv2_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Waylib6Server14WInputMethodV2E_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WInputMethodV2::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server14WInputMethodV2E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WInputMethodV2",
        "committed",
        "",
        "newPopupSurface",
        "wlr_input_popup_surface_v2*",
        "surface",
        "newKeyboardGrab",
        "wlr_input_method_keyboard_grab_v2*",
        "keyboardGrab",
        "sendContentType",
        "hint",
        "purpose",
        "sendActivate",
        "sendDeactivate",
        "sendDone",
        "sendSurroundingText",
        "text",
        "cursor",
        "anchor",
        "sendTextChangeCause",
        "cause",
        "sendUnavailable"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'committed'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'newPopupSurface'
        QtMocHelpers::SignalData<void(wlr_input_popup_surface_v2 *)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 },
        }}),
        // Signal 'newKeyboardGrab'
        QtMocHelpers::SignalData<void(wlr_input_method_keyboard_grab_v2 *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'sendContentType'
        QtMocHelpers::SlotData<void(quint32, quint32)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 10 }, { QMetaType::UInt, 11 },
        }}),
        // Slot 'sendActivate'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'sendDeactivate'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'sendDone'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'sendSurroundingText'
        QtMocHelpers::SlotData<void(const QString &, quint32, quint32)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 }, { QMetaType::UInt, 17 }, { QMetaType::UInt, 18 },
        }}),
        // Slot 'sendTextChangeCause'
        QtMocHelpers::SlotData<void(quint32)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::UInt, 20 },
        }}),
        // Slot 'sendUnavailable'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WInputMethodV2, qt_meta_tag_ZN6Waylib6Server14WInputMethodV2E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WInputMethodV2::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server14WInputMethodV2E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server14WInputMethodV2E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server14WInputMethodV2E_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WInputMethodV2::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WInputMethodV2 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->committed(); break;
        case 1: _t->newPopupSurface((*reinterpret_cast<std::add_pointer_t<wlr_input_popup_surface_v2*>>(_a[1]))); break;
        case 2: _t->newKeyboardGrab((*reinterpret_cast<std::add_pointer_t<wlr_input_method_keyboard_grab_v2*>>(_a[1]))); break;
        case 3: _t->sendContentType((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<quint32>>(_a[2]))); break;
        case 4: _t->sendActivate(); break;
        case 5: _t->sendDeactivate(); break;
        case 6: _t->sendDone(); break;
        case 7: _t->sendSurroundingText((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<quint32>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<quint32>>(_a[3]))); break;
        case 8: _t->sendTextChangeCause((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1]))); break;
        case 9: _t->sendUnavailable(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WInputMethodV2::*)()>(_a, &WInputMethodV2::committed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WInputMethodV2::*)(wlr_input_popup_surface_v2 * )>(_a, &WInputMethodV2::newPopupSurface, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WInputMethodV2::*)(wlr_input_method_keyboard_grab_v2 * )>(_a, &WInputMethodV2::newKeyboardGrab, 2))
            return;
    }
}

const QMetaObject *Waylib::Server::WInputMethodV2::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WInputMethodV2::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server14WInputMethodV2E_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "WObject"))
        return static_cast< WObject*>(this);
    return QObject::qt_metacast(_clname);
}

int Waylib::Server::WInputMethodV2::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WInputMethodV2::committed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Waylib::Server::WInputMethodV2::newPopupSurface(wlr_input_popup_surface_v2 * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Waylib::Server::WInputMethodV2::newKeyboardGrab(wlr_input_method_keyboard_grab_v2 * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN6Waylib6Server21WInputMethodManagerV2E_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WInputMethodManagerV2::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server21WInputMethodManagerV2E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WInputMethodManagerV2",
        "newInputMethod",
        "",
        "wlr_input_method_v2*",
        "inputMethod"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'newInputMethod'
        QtMocHelpers::SignalData<void(wlr_input_method_v2 *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WInputMethodManagerV2, qt_meta_tag_ZN6Waylib6Server21WInputMethodManagerV2E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WInputMethodManagerV2::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server21WInputMethodManagerV2E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server21WInputMethodManagerV2E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server21WInputMethodManagerV2E_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WInputMethodManagerV2::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WInputMethodManagerV2 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->newInputMethod((*reinterpret_cast<std::add_pointer_t<wlr_input_method_v2*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WInputMethodManagerV2::*)(wlr_input_method_v2 * )>(_a, &WInputMethodManagerV2::newInputMethod, 0))
            return;
    }
}

const QMetaObject *Waylib::Server::WInputMethodManagerV2::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WInputMethodManagerV2::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server21WInputMethodManagerV2E_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "WObject"))
        return static_cast< WObject*>(this);
    if (!strcmp(_clname, "WServerInterface"))
        return static_cast< WServerInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int Waylib::Server::WInputMethodManagerV2::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WInputMethodManagerV2::newInputMethod(wlr_input_method_v2 * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
