/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<mpvvideoitem.h>)
#  include <mpvvideoitem.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
Q_QMLTYPE_EXPORT void qml_register_types_com_treeland_wallfactory()
{
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    qmlRegisterTypesAndRevisions<MpvVideoItem>("com.treeland.wallfactory", 1);
    qmlRegisterAnonymousType<QQuickItem, 254>("com.treeland.wallfactory", 1);
    qmlRegisterEnum<MpvVideoItem::AsyncIds>("MpvVideoItem::AsyncIds");
    qmlRegisterEnum<MpvVideoItem::Property>("MpvVideoItem::Property");
    qmlRegisterEnum<MpvVideoItem::VideoScaleMode>("MpvVideoItem::VideoScaleMode");
    QMetaType::fromType<QQuickFramebufferObject *>().id();
    QT_WARNING_POP
    qmlRegisterModule("com.treeland.wallfactory", 1, 0);
}

static const QQmlModuleRegistration comtreelandwallfactoryRegistration("com.treeland.wallfactory", qml_register_types_com_treeland_wallfactory);
