/* Generated by wayland-scanner 1.26.0 */

/*
 * Copyright © 2015, 2021 Red Hat Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this
 * software and its documentation for any purpose is hereby granted
 * without fee, provided that the above copyright notice appear in
 * all copies and that both that copyright notice and this permission
 * notice appear in supporting documentation, and that the name of
 * the copyright holders not be used in advertising or publicity
 * pertaining to distribution of the software without specific,
 * written prior permission.  The copyright holders make no
 * representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied
 * warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
 * THIS SOFTWARE.
 */

#include <stdbool.h>
#include <stdlib.h>
#include <stdint.h>
#include "wayland-util.h"

#ifndef __has_attribute
# define __has_attribute(x) 0  /* Compatibility with non-clang compilers. */
#endif

#if (__has_attribute(visibility) || defined(__GNUC__) && __GNUC__ >= 4)
#define WL_PRIVATE __attribute__ ((visibility("hidden")))
#else
#define WL_PRIVATE
#endif

extern const struct wl_interface wl_pointer_interface;
extern const struct wl_interface wl_surface_interface;
extern const struct wl_interface zwp_pointer_gesture_hold_v1_interface;
extern const struct wl_interface zwp_pointer_gesture_pinch_v1_interface;
extern const struct wl_interface zwp_pointer_gesture_swipe_v1_interface;

static const struct wl_interface *pointer_gestures_unstable_v1_types[] = {
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	&zwp_pointer_gesture_swipe_v1_interface,
	&wl_pointer_interface,
	&zwp_pointer_gesture_pinch_v1_interface,
	&wl_pointer_interface,
	&zwp_pointer_gesture_hold_v1_interface,
	&wl_pointer_interface,
	NULL,
	NULL,
	&wl_surface_interface,
	NULL,
	NULL,
	NULL,
	&wl_surface_interface,
	NULL,
	NULL,
	NULL,
	&wl_surface_interface,
	NULL,
};

static const struct wl_message zwp_pointer_gestures_v1_requests[] = {
	{ "get_swipe_gesture", "no", pointer_gestures_unstable_v1_types + 5 },
	{ "get_pinch_gesture", "no", pointer_gestures_unstable_v1_types + 7 },
	{ "release", "2", pointer_gestures_unstable_v1_types + 0 },
	{ "get_hold_gesture", "3no", pointer_gestures_unstable_v1_types + 9 },
};

WL_PRIVATE const struct wl_interface zwp_pointer_gestures_v1_interface = {
	"zwp_pointer_gestures_v1", 3,
	4, zwp_pointer_gestures_v1_requests,
	0, NULL,
};

static const struct wl_message zwp_pointer_gesture_swipe_v1_requests[] = {
	{ "destroy", "", pointer_gestures_unstable_v1_types + 0 },
};

static const struct wl_message zwp_pointer_gesture_swipe_v1_events[] = {
	{ "begin", "uuou", pointer_gestures_unstable_v1_types + 11 },
	{ "update", "uff", pointer_gestures_unstable_v1_types + 0 },
	{ "end", "uui", pointer_gestures_unstable_v1_types + 0 },
};

WL_PRIVATE const struct wl_interface zwp_pointer_gesture_swipe_v1_interface = {
	"zwp_pointer_gesture_swipe_v1", 3,
	1, zwp_pointer_gesture_swipe_v1_requests,
	3, zwp_pointer_gesture_swipe_v1_events,
};

static const struct wl_message zwp_pointer_gesture_pinch_v1_requests[] = {
	{ "destroy", "", pointer_gestures_unstable_v1_types + 0 },
};

static const struct wl_message zwp_pointer_gesture_pinch_v1_events[] = {
	{ "begin", "uuou", pointer_gestures_unstable_v1_types + 15 },
	{ "update", "uffff", pointer_gestures_unstable_v1_types + 0 },
	{ "end", "uui", pointer_gestures_unstable_v1_types + 0 },
};

WL_PRIVATE const struct wl_interface zwp_pointer_gesture_pinch_v1_interface = {
	"zwp_pointer_gesture_pinch_v1", 3,
	1, zwp_pointer_gesture_pinch_v1_requests,
	3, zwp_pointer_gesture_pinch_v1_events,
};

static const struct wl_message zwp_pointer_gesture_hold_v1_requests[] = {
	{ "destroy", "3", pointer_gestures_unstable_v1_types + 0 },
};

static const struct wl_message zwp_pointer_gesture_hold_v1_events[] = {
	{ "begin", "3uuou", pointer_gestures_unstable_v1_types + 19 },
	{ "end", "3uui", pointer_gestures_unstable_v1_types + 0 },
};

WL_PRIVATE const struct wl_interface zwp_pointer_gesture_hold_v1_interface = {
	"zwp_pointer_gesture_hold_v1", 3,
	1, zwp_pointer_gesture_hold_v1_requests,
	2, zwp_pointer_gesture_hold_v1_events,
};

