/****************************************************************************
** Meta object code from reading C++ file 'output.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/output/output.h"
#include "woutput.h"
#include <woutput.h>
#include <output/output.h>
#include "core/rootsurfacecontainer.h"
#include "wcursor.h"
#include <woutputitem.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'output.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6OutputE_t {};
} // unnamed namespace

template <> constexpr inline auto Output::qt_create_metaobjectdata<qt_meta_tag_ZN6OutputE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Output",
        "QML.Element",
        "anonymous",
        "exclusiveZoneChanged",
        "",
        "moveResizeFinised",
        "brightnessChanged",
        "colorTemperatureChanged",
        "enable",
        "updateOutputHardwareLayers",
        "setOutputColor",
        "brightness",
        "uint32_t",
        "colorTemperature",
        "std::function<void(bool)>",
        "resultCallback",
        "exclusiveZone",
        "QMargins",
        "validRect",
        "QRectF",
        "outputItem",
        "WOutputItem*",
        "minimizedSurfaces",
        "SurfaceListModel*",
        "screenViewport",
        "WOutputViewport*",
        "config",
        "OutputConfig*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'exclusiveZoneChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'moveResizeFinised'
        QtMocHelpers::SignalData<void()>(5, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'brightnessChanged'
        QtMocHelpers::SignalData<void()>(6, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'colorTemperatureChanged'
        QtMocHelpers::SignalData<void()>(7, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'enable'
        QtMocHelpers::SlotData<void()>(8, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateOutputHardwareLayers'
        QtMocHelpers::SlotData<void()>(9, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setOutputColor'
        QtMocHelpers::SlotData<void(qreal, uint32_t, std::function<void(bool)>)>(10, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 11 }, { 0x80000000 | 12, 13 }, { 0x80000000 | 14, 15 },
        }}),
        // Slot 'setOutputColor'
        QtMocHelpers::SlotData<void(qreal, uint32_t)>(10, 4, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QReal, 11 }, { 0x80000000 | 12, 13 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'exclusiveZone'
        QtMocHelpers::PropertyData<QMargins>(16, 0x80000000 | 17, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 0),
        // property 'validRect'
        QtMocHelpers::PropertyData<QRectF>(18, 0x80000000 | 19, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 0),
        // property 'outputItem'
        QtMocHelpers::PropertyData<WOutputItem*>(20, 0x80000000 | 21, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'minimizedSurfaces'
        QtMocHelpers::PropertyData<SurfaceListModel*>(22, 0x80000000 | 23, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'screenViewport'
        QtMocHelpers::PropertyData<WOutputViewport*>(24, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'config'
        QtMocHelpers::PropertyData<OutputConfig*>(26, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant | QMC::Final),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<Output, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Output::staticMetaObject = { {
    QMetaObject::SuperData::link<SurfaceListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6OutputE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6OutputE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6OutputE_t>.metaTypes,
    nullptr
} };

void Output::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Output *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->exclusiveZoneChanged(); break;
        case 1: _t->moveResizeFinised(); break;
        case 2: _t->brightnessChanged(); break;
        case 3: _t->colorTemperatureChanged(); break;
        case 4: _t->enable(); break;
        case 5: _t->updateOutputHardwareLayers(); break;
        case 6: _t->setOutputColor((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint32_t>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<std::function<void(bool)>>>(_a[3]))); break;
        case 7: _t->setOutputColor((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<uint32_t>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Output::*)()>(_a, &Output::exclusiveZoneChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Output::*)()>(_a, &Output::moveResizeFinised, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Output::*)()>(_a, &Output::brightnessChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Output::*)()>(_a, &Output::colorTemperatureChanged, 3))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< SurfaceListModel* >(); break;
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< WOutputViewport* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QMargins*>(_v) = _t->exclusiveZone(); break;
        case 1: *reinterpret_cast<QRectF*>(_v) = _t->validRect(); break;
        case 2: *reinterpret_cast<WOutputItem**>(_v) = _t->m_item; break;
        case 3: *reinterpret_cast<SurfaceListModel**>(_v) = _t->minimizedSurfaces; break;
        case 4: *reinterpret_cast<WOutputViewport**>(_v) = _t->m_outputViewport; break;
        case 5: *reinterpret_cast<OutputConfig**>(_v) = _t->config(); break;
        default: break;
        }
    }
}

const QMetaObject *Output::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Output::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6OutputE_t>.strings))
        return static_cast<void*>(this);
    return SurfaceListModel::qt_metacast(_clname);
}

int Output::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SurfaceListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void Output::exclusiveZoneChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Output::moveResizeFinised()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Output::brightnessChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Output::colorTemperatureChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
