/****************************************************************************
** Meta object code from reading C++ file 'treelanduserconfig.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../treelanduserconfig.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'treelanduserconfig.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18TreelandUserConfigE_t {};
} // unnamed namespace

template <> constexpr inline auto TreelandUserConfig::qt_create_metaobjectdata<qt_meta_tag_ZN18TreelandUserConfigE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TreelandUserConfig",
        "DConfigKeyList",
        "activeColor;blurAmount;blurMultiplier;blurStrength;currentWorkspac"
        "e;cursorBlink;cursorBlinkTime;cursorSize;cursorThemeName;defaultBa"
        "ckground;dndDragThreshold;doubleClickDistance;doubleClickTime;edge"
        "InnerDelayMs;edgeSideTriggerDistance;edgeTopTriggerDistance;font;f"
        "ontSize;glassBezel;glassEnabled;glassProfilePower;glassSaturation;"
        "glassSpecular;glassThickness;highlightBorderWidth;iconThemeName;lo"
        "stScreen;maxWorkspace;minMultitaskviewSurfaceHeight;monoFont;multi"
        "taskviewAnimationDuration;multitaskviewBottomContentMargin;multita"
        "skviewCellPadding;multitaskviewEasingCurveType;multitaskviewHorizo"
        "ntalMargin;multitaskviewLoadFactor;multitaskviewPaddingOpacity;mul"
        "titaskviewTopContentMargin;normalWindowHeight;numWorkspace;preferD"
        "ark;showOnLock;themeName;titleBoxCornerRadius;wallpaperConfig;wind"
        "owHeightStep;windowOpacity;windowRadius;windowThemeType;windowTitl"
        "ebarHeight;workspaceThumbCornerRadius;workspaceThumbHeight;workspa"
        "ceThumbMargin",
        "DConfigFileName",
        "org.deepin.dde.treeland.user",
        "DConfigFileVersion",
        "1.0",
        "configInitializeFailed",
        "",
        "configInitializeSucceed",
        "Dtk::Core::DConfig*",
        "config",
        "valueChanged",
        "key",
        "QVariant",
        "value",
        "activeColorChanged",
        "blurAmountChanged",
        "blurMultiplierChanged",
        "blurStrengthChanged",
        "currentWorkspaceChanged",
        "cursorBlinkChanged",
        "cursorBlinkTimeChanged",
        "cursorSizeChanged",
        "cursorThemeNameChanged",
        "defaultBackgroundChanged",
        "dndDragThresholdChanged",
        "doubleClickDistanceChanged",
        "doubleClickTimeChanged",
        "edgeInnerDelayMsChanged",
        "edgeSideTriggerDistanceChanged",
        "edgeTopTriggerDistanceChanged",
        "fontChanged",
        "fontSizeChanged",
        "glassBezelChanged",
        "glassEnabledChanged",
        "glassProfilePowerChanged",
        "glassSaturationChanged",
        "glassSpecularChanged",
        "glassThicknessChanged",
        "highlightBorderWidthChanged",
        "iconThemeNameChanged",
        "lostScreenChanged",
        "maxWorkspaceChanged",
        "minMultitaskviewSurfaceHeightChanged",
        "monoFontChanged",
        "multitaskviewAnimationDurationChanged",
        "multitaskviewBottomContentMarginChanged",
        "multitaskviewCellPaddingChanged",
        "multitaskviewEasingCurveTypeChanged",
        "multitaskviewHorizontalMarginChanged",
        "multitaskviewLoadFactorChanged",
        "multitaskviewPaddingOpacityChanged",
        "multitaskviewTopContentMarginChanged",
        "normalWindowHeightChanged",
        "numWorkspaceChanged",
        "preferDarkChanged",
        "showOnLockChanged",
        "themeNameChanged",
        "titleBoxCornerRadiusChanged",
        "wallpaperConfigChanged",
        "windowHeightStepChanged",
        "windowOpacityChanged",
        "windowRadiusChanged",
        "windowThemeTypeChanged",
        "windowTitlebarHeightChanged",
        "workspaceThumbCornerRadiusChanged",
        "workspaceThumbHeightChanged",
        "workspaceThumbMarginChanged",
        "isInitializeSucceed",
        "isInitializeSucceeded",
        "isInitializeFailed",
        "isInitializing",
        "keyList",
        "isDefaultValue",
        "activeColorIsDefaultValue",
        "blurAmountIsDefaultValue",
        "blurMultiplierIsDefaultValue",
        "blurStrengthIsDefaultValue",
        "currentWorkspaceIsDefaultValue",
        "cursorBlinkIsDefaultValue",
        "cursorBlinkTimeIsDefaultValue",
        "cursorSizeIsDefaultValue",
        "cursorThemeNameIsDefaultValue",
        "defaultBackgroundIsDefaultValue",
        "dndDragThresholdIsDefaultValue",
        "doubleClickDistanceIsDefaultValue",
        "doubleClickTimeIsDefaultValue",
        "edgeInnerDelayMsIsDefaultValue",
        "edgeSideTriggerDistanceIsDefaultValue",
        "edgeTopTriggerDistanceIsDefaultValue",
        "fontIsDefaultValue",
        "fontSizeIsDefaultValue",
        "glassBezelIsDefaultValue",
        "glassEnabledIsDefaultValue",
        "glassProfilePowerIsDefaultValue",
        "glassSaturationIsDefaultValue",
        "glassSpecularIsDefaultValue",
        "glassThicknessIsDefaultValue",
        "highlightBorderWidthIsDefaultValue",
        "iconThemeNameIsDefaultValue",
        "lostScreenIsDefaultValue",
        "maxWorkspaceIsDefaultValue",
        "minMultitaskviewSurfaceHeightIsDefaultValue",
        "monoFontIsDefaultValue",
        "multitaskviewAnimationDurationIsDefaultValue",
        "multitaskviewBottomContentMarginIsDefaultValue",
        "multitaskviewCellPaddingIsDefaultValue",
        "multitaskviewEasingCurveTypeIsDefaultValue",
        "multitaskviewHorizontalMarginIsDefaultValue",
        "multitaskviewLoadFactorIsDefaultValue",
        "multitaskviewPaddingOpacityIsDefaultValue",
        "multitaskviewTopContentMarginIsDefaultValue",
        "normalWindowHeightIsDefaultValue",
        "numWorkspaceIsDefaultValue",
        "preferDarkIsDefaultValue",
        "showOnLockIsDefaultValue",
        "themeNameIsDefaultValue",
        "titleBoxCornerRadiusIsDefaultValue",
        "wallpaperConfigIsDefaultValue",
        "windowHeightStepIsDefaultValue",
        "windowOpacityIsDefaultValue",
        "windowRadiusIsDefaultValue",
        "windowThemeTypeIsDefaultValue",
        "windowTitlebarHeightIsDefaultValue",
        "workspaceThumbCornerRadiusIsDefaultValue",
        "workspaceThumbHeightIsDefaultValue",
        "workspaceThumbMarginIsDefaultValue",
        "activeColor",
        "blurAmount",
        "blurMultiplier",
        "blurStrength",
        "currentWorkspace",
        "cursorBlink",
        "cursorBlinkTime",
        "cursorSize",
        "cursorThemeName",
        "defaultBackground",
        "dndDragThreshold",
        "doubleClickDistance",
        "doubleClickTime",
        "edgeInnerDelayMs",
        "edgeSideTriggerDistance",
        "edgeTopTriggerDistance",
        "font",
        "fontSize",
        "glassBezel",
        "glassEnabled",
        "glassProfilePower",
        "glassSaturation",
        "glassSpecular",
        "glassThickness",
        "highlightBorderWidth",
        "iconThemeName",
        "lostScreen",
        "maxWorkspace",
        "minMultitaskviewSurfaceHeight",
        "monoFont",
        "multitaskviewAnimationDuration",
        "multitaskviewBottomContentMargin",
        "multitaskviewCellPadding",
        "multitaskviewEasingCurveType",
        "multitaskviewHorizontalMargin",
        "multitaskviewLoadFactor",
        "multitaskviewPaddingOpacity",
        "multitaskviewTopContentMargin",
        "normalWindowHeight",
        "numWorkspace",
        "preferDark",
        "showOnLock",
        "themeName",
        "titleBoxCornerRadius",
        "wallpaperConfig",
        "windowHeightStep",
        "windowOpacity",
        "windowRadius",
        "windowThemeType",
        "windowTitlebarHeight",
        "workspaceThumbCornerRadius",
        "workspaceThumbHeight",
        "workspaceThumbMargin"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'configInitializeFailed'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'configInitializeSucceed'
        QtMocHelpers::SignalData<void(Dtk::Core::DConfig *)>(9, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'valueChanged'
        QtMocHelpers::SignalData<void(const QString &, const QVariant &)>(12, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 }, { 0x80000000 | 14, 15 },
        }}),
        // Signal 'activeColorChanged'
        QtMocHelpers::SignalData<void()>(16, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'blurAmountChanged'
        QtMocHelpers::SignalData<void()>(17, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'blurMultiplierChanged'
        QtMocHelpers::SignalData<void()>(18, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'blurStrengthChanged'
        QtMocHelpers::SignalData<void()>(19, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'currentWorkspaceChanged'
        QtMocHelpers::SignalData<void()>(20, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cursorBlinkChanged'
        QtMocHelpers::SignalData<void()>(21, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cursorBlinkTimeChanged'
        QtMocHelpers::SignalData<void()>(22, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cursorSizeChanged'
        QtMocHelpers::SignalData<void()>(23, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cursorThemeNameChanged'
        QtMocHelpers::SignalData<void()>(24, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'defaultBackgroundChanged'
        QtMocHelpers::SignalData<void()>(25, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'dndDragThresholdChanged'
        QtMocHelpers::SignalData<void()>(26, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'doubleClickDistanceChanged'
        QtMocHelpers::SignalData<void()>(27, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'doubleClickTimeChanged'
        QtMocHelpers::SignalData<void()>(28, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'edgeInnerDelayMsChanged'
        QtMocHelpers::SignalData<void()>(29, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'edgeSideTriggerDistanceChanged'
        QtMocHelpers::SignalData<void()>(30, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'edgeTopTriggerDistanceChanged'
        QtMocHelpers::SignalData<void()>(31, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fontChanged'
        QtMocHelpers::SignalData<void()>(32, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fontSizeChanged'
        QtMocHelpers::SignalData<void()>(33, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'glassBezelChanged'
        QtMocHelpers::SignalData<void()>(34, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'glassEnabledChanged'
        QtMocHelpers::SignalData<void()>(35, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'glassProfilePowerChanged'
        QtMocHelpers::SignalData<void()>(36, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'glassSaturationChanged'
        QtMocHelpers::SignalData<void()>(37, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'glassSpecularChanged'
        QtMocHelpers::SignalData<void()>(38, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'glassThicknessChanged'
        QtMocHelpers::SignalData<void()>(39, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'highlightBorderWidthChanged'
        QtMocHelpers::SignalData<void()>(40, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'iconThemeNameChanged'
        QtMocHelpers::SignalData<void()>(41, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'lostScreenChanged'
        QtMocHelpers::SignalData<void()>(42, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'maxWorkspaceChanged'
        QtMocHelpers::SignalData<void()>(43, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'minMultitaskviewSurfaceHeightChanged'
        QtMocHelpers::SignalData<void()>(44, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'monoFontChanged'
        QtMocHelpers::SignalData<void()>(45, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'multitaskviewAnimationDurationChanged'
        QtMocHelpers::SignalData<void()>(46, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'multitaskviewBottomContentMarginChanged'
        QtMocHelpers::SignalData<void()>(47, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'multitaskviewCellPaddingChanged'
        QtMocHelpers::SignalData<void()>(48, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'multitaskviewEasingCurveTypeChanged'
        QtMocHelpers::SignalData<void()>(49, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'multitaskviewHorizontalMarginChanged'
        QtMocHelpers::SignalData<void()>(50, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'multitaskviewLoadFactorChanged'
        QtMocHelpers::SignalData<void()>(51, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'multitaskviewPaddingOpacityChanged'
        QtMocHelpers::SignalData<void()>(52, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'multitaskviewTopContentMarginChanged'
        QtMocHelpers::SignalData<void()>(53, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'normalWindowHeightChanged'
        QtMocHelpers::SignalData<void()>(54, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'numWorkspaceChanged'
        QtMocHelpers::SignalData<void()>(55, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'preferDarkChanged'
        QtMocHelpers::SignalData<void()>(56, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'showOnLockChanged'
        QtMocHelpers::SignalData<void()>(57, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'themeNameChanged'
        QtMocHelpers::SignalData<void()>(58, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'titleBoxCornerRadiusChanged'
        QtMocHelpers::SignalData<void()>(59, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'wallpaperConfigChanged'
        QtMocHelpers::SignalData<void()>(60, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'windowHeightStepChanged'
        QtMocHelpers::SignalData<void()>(61, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'windowOpacityChanged'
        QtMocHelpers::SignalData<void()>(62, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'windowRadiusChanged'
        QtMocHelpers::SignalData<void()>(63, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'windowThemeTypeChanged'
        QtMocHelpers::SignalData<void()>(64, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'windowTitlebarHeightChanged'
        QtMocHelpers::SignalData<void()>(65, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'workspaceThumbCornerRadiusChanged'
        QtMocHelpers::SignalData<void()>(66, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'workspaceThumbHeightChanged'
        QtMocHelpers::SignalData<void()>(67, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'workspaceThumbMarginChanged'
        QtMocHelpers::SignalData<void()>(68, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'config'
        QtMocHelpers::MethodData<Dtk::Core::DConfig *() const>(11, 8, QMC::AccessPublic, 0x80000000 | 10),
        // Method 'isInitializeSucceed'
        QtMocHelpers::MethodData<bool() const>(69, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'isInitializeSucceeded'
        QtMocHelpers::MethodData<bool() const>(70, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'isInitializeFailed'
        QtMocHelpers::MethodData<bool() const>(71, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'isInitializing'
        QtMocHelpers::MethodData<bool() const>(72, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'keyList'
        QtMocHelpers::MethodData<QStringList() const>(73, 8, QMC::AccessPublic, QMetaType::QStringList),
        // Method 'isDefaultValue'
        QtMocHelpers::MethodData<bool(const QString &) const>(74, 8, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 13 },
        }}),
        // Method 'activeColorIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(75, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'blurAmountIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(76, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'blurMultiplierIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(77, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'blurStrengthIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(78, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'currentWorkspaceIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(79, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'cursorBlinkIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(80, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'cursorBlinkTimeIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(81, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'cursorSizeIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(82, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'cursorThemeNameIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(83, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'defaultBackgroundIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(84, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'dndDragThresholdIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(85, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'doubleClickDistanceIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(86, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'doubleClickTimeIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(87, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'edgeInnerDelayMsIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(88, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'edgeSideTriggerDistanceIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(89, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'edgeTopTriggerDistanceIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(90, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'fontIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(91, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'fontSizeIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(92, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'glassBezelIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(93, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'glassEnabledIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(94, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'glassProfilePowerIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(95, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'glassSaturationIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(96, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'glassSpecularIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(97, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'glassThicknessIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(98, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'highlightBorderWidthIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(99, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'iconThemeNameIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(100, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'lostScreenIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(101, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'maxWorkspaceIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(102, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'minMultitaskviewSurfaceHeightIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(103, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'monoFontIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(104, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'multitaskviewAnimationDurationIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(105, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'multitaskviewBottomContentMarginIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(106, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'multitaskviewCellPaddingIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(107, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'multitaskviewEasingCurveTypeIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(108, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'multitaskviewHorizontalMarginIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(109, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'multitaskviewLoadFactorIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(110, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'multitaskviewPaddingOpacityIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(111, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'multitaskviewTopContentMarginIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(112, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'normalWindowHeightIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(113, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'numWorkspaceIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(114, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'preferDarkIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(115, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'showOnLockIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(116, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'themeNameIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(117, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'titleBoxCornerRadiusIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(118, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'wallpaperConfigIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(119, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'windowHeightStepIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(120, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'windowOpacityIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(121, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'windowRadiusIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(122, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'windowThemeTypeIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(123, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'windowTitlebarHeightIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(124, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'workspaceThumbCornerRadiusIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(125, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'workspaceThumbHeightIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(126, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'workspaceThumbMarginIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(127, 8, QMC::AccessPublic, QMetaType::Bool),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'activeColor'
        QtMocHelpers::PropertyData<QString>(128, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 3),
        // property 'blurAmount'
        QtMocHelpers::PropertyData<double>(129, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 4),
        // property 'blurMultiplier'
        QtMocHelpers::PropertyData<double>(130, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 5),
        // property 'blurStrength'
        QtMocHelpers::PropertyData<qlonglong>(131, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 6),
        // property 'currentWorkspace'
        QtMocHelpers::PropertyData<qlonglong>(132, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 7),
        // property 'cursorBlink'
        QtMocHelpers::PropertyData<bool>(133, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 8),
        // property 'cursorBlinkTime'
        QtMocHelpers::PropertyData<qlonglong>(134, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 9),
        // property 'cursorSize'
        QtMocHelpers::PropertyData<qlonglong>(135, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 10),
        // property 'cursorThemeName'
        QtMocHelpers::PropertyData<QString>(136, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 11),
        // property 'defaultBackground'
        QtMocHelpers::PropertyData<QString>(137, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 12),
        // property 'dndDragThreshold'
        QtMocHelpers::PropertyData<qlonglong>(138, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 13),
        // property 'doubleClickDistance'
        QtMocHelpers::PropertyData<qlonglong>(139, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 14),
        // property 'doubleClickTime'
        QtMocHelpers::PropertyData<qlonglong>(140, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 15),
        // property 'edgeInnerDelayMs'
        QtMocHelpers::PropertyData<qlonglong>(141, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 16),
        // property 'edgeSideTriggerDistance'
        QtMocHelpers::PropertyData<qlonglong>(142, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 17),
        // property 'edgeTopTriggerDistance'
        QtMocHelpers::PropertyData<qlonglong>(143, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 18),
        // property 'font'
        QtMocHelpers::PropertyData<QString>(144, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 19),
        // property 'fontSize'
        QtMocHelpers::PropertyData<qlonglong>(145, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 20),
        // property 'glassBezel'
        QtMocHelpers::PropertyData<qlonglong>(146, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 21),
        // property 'glassEnabled'
        QtMocHelpers::PropertyData<bool>(147, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 22),
        // property 'glassProfilePower'
        QtMocHelpers::PropertyData<double>(148, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 23),
        // property 'glassSaturation'
        QtMocHelpers::PropertyData<double>(149, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 24),
        // property 'glassSpecular'
        QtMocHelpers::PropertyData<double>(150, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 25),
        // property 'glassThickness'
        QtMocHelpers::PropertyData<qlonglong>(151, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 26),
        // property 'highlightBorderWidth'
        QtMocHelpers::PropertyData<qlonglong>(152, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 27),
        // property 'iconThemeName'
        QtMocHelpers::PropertyData<QString>(153, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 28),
        // property 'lostScreen'
        QtMocHelpers::PropertyData<QString>(154, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 29),
        // property 'maxWorkspace'
        QtMocHelpers::PropertyData<qlonglong>(155, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 30),
        // property 'minMultitaskviewSurfaceHeight'
        QtMocHelpers::PropertyData<qlonglong>(156, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 31),
        // property 'monoFont'
        QtMocHelpers::PropertyData<QString>(157, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 32),
        // property 'multitaskviewAnimationDuration'
        QtMocHelpers::PropertyData<qlonglong>(158, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 33),
        // property 'multitaskviewBottomContentMargin'
        QtMocHelpers::PropertyData<qlonglong>(159, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 34),
        // property 'multitaskviewCellPadding'
        QtMocHelpers::PropertyData<qlonglong>(160, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 35),
        // property 'multitaskviewEasingCurveType'
        QtMocHelpers::PropertyData<qlonglong>(161, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 36),
        // property 'multitaskviewHorizontalMargin'
        QtMocHelpers::PropertyData<qlonglong>(162, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 37),
        // property 'multitaskviewLoadFactor'
        QtMocHelpers::PropertyData<double>(163, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 38),
        // property 'multitaskviewPaddingOpacity'
        QtMocHelpers::PropertyData<double>(164, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 39),
        // property 'multitaskviewTopContentMargin'
        QtMocHelpers::PropertyData<qlonglong>(165, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 40),
        // property 'normalWindowHeight'
        QtMocHelpers::PropertyData<qlonglong>(166, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 41),
        // property 'numWorkspace'
        QtMocHelpers::PropertyData<qlonglong>(167, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 42),
        // property 'preferDark'
        QtMocHelpers::PropertyData<bool>(168, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 43),
        // property 'showOnLock'
        QtMocHelpers::PropertyData<bool>(169, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 44),
        // property 'themeName'
        QtMocHelpers::PropertyData<QString>(170, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 45),
        // property 'titleBoxCornerRadius'
        QtMocHelpers::PropertyData<qlonglong>(171, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 46),
        // property 'wallpaperConfig'
        QtMocHelpers::PropertyData<QString>(172, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 47),
        // property 'windowHeightStep'
        QtMocHelpers::PropertyData<qlonglong>(173, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 48),
        // property 'windowOpacity'
        QtMocHelpers::PropertyData<qlonglong>(174, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 49),
        // property 'windowRadius'
        QtMocHelpers::PropertyData<qlonglong>(175, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 50),
        // property 'windowThemeType'
        QtMocHelpers::PropertyData<qlonglong>(176, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 51),
        // property 'windowTitlebarHeight'
        QtMocHelpers::PropertyData<qlonglong>(177, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 52),
        // property 'workspaceThumbCornerRadius'
        QtMocHelpers::PropertyData<qlonglong>(178, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 53),
        // property 'workspaceThumbHeight'
        QtMocHelpers::PropertyData<qlonglong>(179, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 54),
        // property 'workspaceThumbMargin'
        QtMocHelpers::PropertyData<qlonglong>(180, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 55),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<TreelandUserConfig, qt_meta_tag_ZN18TreelandUserConfigE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject TreelandUserConfig::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18TreelandUserConfigE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18TreelandUserConfigE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18TreelandUserConfigE_t>.metaTypes,
    nullptr
} };

void TreelandUserConfig::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TreelandUserConfig *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->configInitializeFailed(); break;
        case 1: _t->configInitializeSucceed((*reinterpret_cast<std::add_pointer_t<Dtk::Core::DConfig*>>(_a[1]))); break;
        case 2: _t->valueChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 3: _t->activeColorChanged(); break;
        case 4: _t->blurAmountChanged(); break;
        case 5: _t->blurMultiplierChanged(); break;
        case 6: _t->blurStrengthChanged(); break;
        case 7: _t->currentWorkspaceChanged(); break;
        case 8: _t->cursorBlinkChanged(); break;
        case 9: _t->cursorBlinkTimeChanged(); break;
        case 10: _t->cursorSizeChanged(); break;
        case 11: _t->cursorThemeNameChanged(); break;
        case 12: _t->defaultBackgroundChanged(); break;
        case 13: _t->dndDragThresholdChanged(); break;
        case 14: _t->doubleClickDistanceChanged(); break;
        case 15: _t->doubleClickTimeChanged(); break;
        case 16: _t->edgeInnerDelayMsChanged(); break;
        case 17: _t->edgeSideTriggerDistanceChanged(); break;
        case 18: _t->edgeTopTriggerDistanceChanged(); break;
        case 19: _t->fontChanged(); break;
        case 20: _t->fontSizeChanged(); break;
        case 21: _t->glassBezelChanged(); break;
        case 22: _t->glassEnabledChanged(); break;
        case 23: _t->glassProfilePowerChanged(); break;
        case 24: _t->glassSaturationChanged(); break;
        case 25: _t->glassSpecularChanged(); break;
        case 26: _t->glassThicknessChanged(); break;
        case 27: _t->highlightBorderWidthChanged(); break;
        case 28: _t->iconThemeNameChanged(); break;
        case 29: _t->lostScreenChanged(); break;
        case 30: _t->maxWorkspaceChanged(); break;
        case 31: _t->minMultitaskviewSurfaceHeightChanged(); break;
        case 32: _t->monoFontChanged(); break;
        case 33: _t->multitaskviewAnimationDurationChanged(); break;
        case 34: _t->multitaskviewBottomContentMarginChanged(); break;
        case 35: _t->multitaskviewCellPaddingChanged(); break;
        case 36: _t->multitaskviewEasingCurveTypeChanged(); break;
        case 37: _t->multitaskviewHorizontalMarginChanged(); break;
        case 38: _t->multitaskviewLoadFactorChanged(); break;
        case 39: _t->multitaskviewPaddingOpacityChanged(); break;
        case 40: _t->multitaskviewTopContentMarginChanged(); break;
        case 41: _t->normalWindowHeightChanged(); break;
        case 42: _t->numWorkspaceChanged(); break;
        case 43: _t->preferDarkChanged(); break;
        case 44: _t->showOnLockChanged(); break;
        case 45: _t->themeNameChanged(); break;
        case 46: _t->titleBoxCornerRadiusChanged(); break;
        case 47: _t->wallpaperConfigChanged(); break;
        case 48: _t->windowHeightStepChanged(); break;
        case 49: _t->windowOpacityChanged(); break;
        case 50: _t->windowRadiusChanged(); break;
        case 51: _t->windowThemeTypeChanged(); break;
        case 52: _t->windowTitlebarHeightChanged(); break;
        case 53: _t->workspaceThumbCornerRadiusChanged(); break;
        case 54: _t->workspaceThumbHeightChanged(); break;
        case 55: _t->workspaceThumbMarginChanged(); break;
        case 56: { Dtk::Core::DConfig* _r = _t->config();
            if (_a[0]) *reinterpret_cast<Dtk::Core::DConfig**>(_a[0]) = std::move(_r); }  break;
        case 57: { bool _r = _t->isInitializeSucceed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 58: { bool _r = _t->isInitializeSucceeded();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 59: { bool _r = _t->isInitializeFailed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 60: { bool _r = _t->isInitializing();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 61: { QStringList _r = _t->keyList();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 62: { bool _r = _t->isDefaultValue((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 63: { bool _r = _t->activeColorIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 64: { bool _r = _t->blurAmountIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 65: { bool _r = _t->blurMultiplierIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 66: { bool _r = _t->blurStrengthIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 67: { bool _r = _t->currentWorkspaceIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 68: { bool _r = _t->cursorBlinkIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 69: { bool _r = _t->cursorBlinkTimeIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 70: { bool _r = _t->cursorSizeIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 71: { bool _r = _t->cursorThemeNameIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 72: { bool _r = _t->defaultBackgroundIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 73: { bool _r = _t->dndDragThresholdIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 74: { bool _r = _t->doubleClickDistanceIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 75: { bool _r = _t->doubleClickTimeIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 76: { bool _r = _t->edgeInnerDelayMsIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 77: { bool _r = _t->edgeSideTriggerDistanceIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 78: { bool _r = _t->edgeTopTriggerDistanceIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 79: { bool _r = _t->fontIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 80: { bool _r = _t->fontSizeIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 81: { bool _r = _t->glassBezelIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 82: { bool _r = _t->glassEnabledIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 83: { bool _r = _t->glassProfilePowerIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 84: { bool _r = _t->glassSaturationIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 85: { bool _r = _t->glassSpecularIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 86: { bool _r = _t->glassThicknessIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 87: { bool _r = _t->highlightBorderWidthIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 88: { bool _r = _t->iconThemeNameIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 89: { bool _r = _t->lostScreenIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 90: { bool _r = _t->maxWorkspaceIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 91: { bool _r = _t->minMultitaskviewSurfaceHeightIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 92: { bool _r = _t->monoFontIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 93: { bool _r = _t->multitaskviewAnimationDurationIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 94: { bool _r = _t->multitaskviewBottomContentMarginIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 95: { bool _r = _t->multitaskviewCellPaddingIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 96: { bool _r = _t->multitaskviewEasingCurveTypeIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 97: { bool _r = _t->multitaskviewHorizontalMarginIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 98: { bool _r = _t->multitaskviewLoadFactorIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 99: { bool _r = _t->multitaskviewPaddingOpacityIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 100: { bool _r = _t->multitaskviewTopContentMarginIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 101: { bool _r = _t->normalWindowHeightIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 102: { bool _r = _t->numWorkspaceIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 103: { bool _r = _t->preferDarkIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 104: { bool _r = _t->showOnLockIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 105: { bool _r = _t->themeNameIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 106: { bool _r = _t->titleBoxCornerRadiusIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 107: { bool _r = _t->wallpaperConfigIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 108: { bool _r = _t->windowHeightStepIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 109: { bool _r = _t->windowOpacityIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 110: { bool _r = _t->windowRadiusIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 111: { bool _r = _t->windowThemeTypeIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 112: { bool _r = _t->windowTitlebarHeightIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 113: { bool _r = _t->workspaceThumbCornerRadiusIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 114: { bool _r = _t->workspaceThumbHeightIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 115: { bool _r = _t->workspaceThumbMarginIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Dtk::Core::DConfig* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::configInitializeFailed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)(Dtk::Core::DConfig * )>(_a, &TreelandUserConfig::configInitializeSucceed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)(const QString & , const QVariant & )>(_a, &TreelandUserConfig::valueChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::activeColorChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::blurAmountChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::blurMultiplierChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::blurStrengthChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::currentWorkspaceChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::cursorBlinkChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::cursorBlinkTimeChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::cursorSizeChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::cursorThemeNameChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::defaultBackgroundChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::dndDragThresholdChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::doubleClickDistanceChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::doubleClickTimeChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::edgeInnerDelayMsChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::edgeSideTriggerDistanceChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::edgeTopTriggerDistanceChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::fontChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::fontSizeChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::glassBezelChanged, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::glassEnabledChanged, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::glassProfilePowerChanged, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::glassSaturationChanged, 24))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::glassSpecularChanged, 25))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::glassThicknessChanged, 26))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::highlightBorderWidthChanged, 27))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::iconThemeNameChanged, 28))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::lostScreenChanged, 29))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::maxWorkspaceChanged, 30))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::minMultitaskviewSurfaceHeightChanged, 31))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::monoFontChanged, 32))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::multitaskviewAnimationDurationChanged, 33))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::multitaskviewBottomContentMarginChanged, 34))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::multitaskviewCellPaddingChanged, 35))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::multitaskviewEasingCurveTypeChanged, 36))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::multitaskviewHorizontalMarginChanged, 37))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::multitaskviewLoadFactorChanged, 38))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::multitaskviewPaddingOpacityChanged, 39))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::multitaskviewTopContentMarginChanged, 40))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::normalWindowHeightChanged, 41))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::numWorkspaceChanged, 42))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::preferDarkChanged, 43))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::showOnLockChanged, 44))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::themeNameChanged, 45))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::titleBoxCornerRadiusChanged, 46))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::wallpaperConfigChanged, 47))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::windowHeightStepChanged, 48))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::windowOpacityChanged, 49))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::windowRadiusChanged, 50))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::windowThemeTypeChanged, 51))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::windowTitlebarHeightChanged, 52))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::workspaceThumbCornerRadiusChanged, 53))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::workspaceThumbHeightChanged, 54))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandUserConfig::*)()>(_a, &TreelandUserConfig::workspaceThumbMarginChanged, 55))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->activeColor(); break;
        case 1: *reinterpret_cast<double*>(_v) = _t->blurAmount(); break;
        case 2: *reinterpret_cast<double*>(_v) = _t->blurMultiplier(); break;
        case 3: *reinterpret_cast<qlonglong*>(_v) = _t->blurStrength(); break;
        case 4: *reinterpret_cast<qlonglong*>(_v) = _t->currentWorkspace(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->cursorBlink(); break;
        case 6: *reinterpret_cast<qlonglong*>(_v) = _t->cursorBlinkTime(); break;
        case 7: *reinterpret_cast<qlonglong*>(_v) = _t->cursorSize(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->cursorThemeName(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->defaultBackground(); break;
        case 10: *reinterpret_cast<qlonglong*>(_v) = _t->dndDragThreshold(); break;
        case 11: *reinterpret_cast<qlonglong*>(_v) = _t->doubleClickDistance(); break;
        case 12: *reinterpret_cast<qlonglong*>(_v) = _t->doubleClickTime(); break;
        case 13: *reinterpret_cast<qlonglong*>(_v) = _t->edgeInnerDelayMs(); break;
        case 14: *reinterpret_cast<qlonglong*>(_v) = _t->edgeSideTriggerDistance(); break;
        case 15: *reinterpret_cast<qlonglong*>(_v) = _t->edgeTopTriggerDistance(); break;
        case 16: *reinterpret_cast<QString*>(_v) = _t->font(); break;
        case 17: *reinterpret_cast<qlonglong*>(_v) = _t->fontSize(); break;
        case 18: *reinterpret_cast<qlonglong*>(_v) = _t->glassBezel(); break;
        case 19: *reinterpret_cast<bool*>(_v) = _t->glassEnabled(); break;
        case 20: *reinterpret_cast<double*>(_v) = _t->glassProfilePower(); break;
        case 21: *reinterpret_cast<double*>(_v) = _t->glassSaturation(); break;
        case 22: *reinterpret_cast<double*>(_v) = _t->glassSpecular(); break;
        case 23: *reinterpret_cast<qlonglong*>(_v) = _t->glassThickness(); break;
        case 24: *reinterpret_cast<qlonglong*>(_v) = _t->highlightBorderWidth(); break;
        case 25: *reinterpret_cast<QString*>(_v) = _t->iconThemeName(); break;
        case 26: *reinterpret_cast<QString*>(_v) = _t->lostScreen(); break;
        case 27: *reinterpret_cast<qlonglong*>(_v) = _t->maxWorkspace(); break;
        case 28: *reinterpret_cast<qlonglong*>(_v) = _t->minMultitaskviewSurfaceHeight(); break;
        case 29: *reinterpret_cast<QString*>(_v) = _t->monoFont(); break;
        case 30: *reinterpret_cast<qlonglong*>(_v) = _t->multitaskviewAnimationDuration(); break;
        case 31: *reinterpret_cast<qlonglong*>(_v) = _t->multitaskviewBottomContentMargin(); break;
        case 32: *reinterpret_cast<qlonglong*>(_v) = _t->multitaskviewCellPadding(); break;
        case 33: *reinterpret_cast<qlonglong*>(_v) = _t->multitaskviewEasingCurveType(); break;
        case 34: *reinterpret_cast<qlonglong*>(_v) = _t->multitaskviewHorizontalMargin(); break;
        case 35: *reinterpret_cast<double*>(_v) = _t->multitaskviewLoadFactor(); break;
        case 36: *reinterpret_cast<double*>(_v) = _t->multitaskviewPaddingOpacity(); break;
        case 37: *reinterpret_cast<qlonglong*>(_v) = _t->multitaskviewTopContentMargin(); break;
        case 38: *reinterpret_cast<qlonglong*>(_v) = _t->normalWindowHeight(); break;
        case 39: *reinterpret_cast<qlonglong*>(_v) = _t->numWorkspace(); break;
        case 40: *reinterpret_cast<bool*>(_v) = _t->preferDark(); break;
        case 41: *reinterpret_cast<bool*>(_v) = _t->showOnLock(); break;
        case 42: *reinterpret_cast<QString*>(_v) = _t->themeName(); break;
        case 43: *reinterpret_cast<qlonglong*>(_v) = _t->titleBoxCornerRadius(); break;
        case 44: *reinterpret_cast<QString*>(_v) = _t->wallpaperConfig(); break;
        case 45: *reinterpret_cast<qlonglong*>(_v) = _t->windowHeightStep(); break;
        case 46: *reinterpret_cast<qlonglong*>(_v) = _t->windowOpacity(); break;
        case 47: *reinterpret_cast<qlonglong*>(_v) = _t->windowRadius(); break;
        case 48: *reinterpret_cast<qlonglong*>(_v) = _t->windowThemeType(); break;
        case 49: *reinterpret_cast<qlonglong*>(_v) = _t->windowTitlebarHeight(); break;
        case 50: *reinterpret_cast<qlonglong*>(_v) = _t->workspaceThumbCornerRadius(); break;
        case 51: *reinterpret_cast<qlonglong*>(_v) = _t->workspaceThumbHeight(); break;
        case 52: *reinterpret_cast<qlonglong*>(_v) = _t->workspaceThumbMargin(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setActiveColor(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setBlurAmount(*reinterpret_cast<double*>(_v)); break;
        case 2: _t->setBlurMultiplier(*reinterpret_cast<double*>(_v)); break;
        case 3: _t->setBlurStrength(*reinterpret_cast<qlonglong*>(_v)); break;
        case 4: _t->setCurrentWorkspace(*reinterpret_cast<qlonglong*>(_v)); break;
        case 5: _t->setCursorBlink(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->setCursorBlinkTime(*reinterpret_cast<qlonglong*>(_v)); break;
        case 7: _t->setCursorSize(*reinterpret_cast<qlonglong*>(_v)); break;
        case 8: _t->setCursorThemeName(*reinterpret_cast<QString*>(_v)); break;
        case 9: _t->setDefaultBackground(*reinterpret_cast<QString*>(_v)); break;
        case 10: _t->setDndDragThreshold(*reinterpret_cast<qlonglong*>(_v)); break;
        case 11: _t->setDoubleClickDistance(*reinterpret_cast<qlonglong*>(_v)); break;
        case 12: _t->setDoubleClickTime(*reinterpret_cast<qlonglong*>(_v)); break;
        case 13: _t->setEdgeInnerDelayMs(*reinterpret_cast<qlonglong*>(_v)); break;
        case 14: _t->setEdgeSideTriggerDistance(*reinterpret_cast<qlonglong*>(_v)); break;
        case 15: _t->setEdgeTopTriggerDistance(*reinterpret_cast<qlonglong*>(_v)); break;
        case 16: _t->setFont(*reinterpret_cast<QString*>(_v)); break;
        case 17: _t->setFontSize(*reinterpret_cast<qlonglong*>(_v)); break;
        case 18: _t->setGlassBezel(*reinterpret_cast<qlonglong*>(_v)); break;
        case 19: _t->setGlassEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 20: _t->setGlassProfilePower(*reinterpret_cast<double*>(_v)); break;
        case 21: _t->setGlassSaturation(*reinterpret_cast<double*>(_v)); break;
        case 22: _t->setGlassSpecular(*reinterpret_cast<double*>(_v)); break;
        case 23: _t->setGlassThickness(*reinterpret_cast<qlonglong*>(_v)); break;
        case 24: _t->setHighlightBorderWidth(*reinterpret_cast<qlonglong*>(_v)); break;
        case 25: _t->setIconThemeName(*reinterpret_cast<QString*>(_v)); break;
        case 26: _t->setLostScreen(*reinterpret_cast<QString*>(_v)); break;
        case 27: _t->setMaxWorkspace(*reinterpret_cast<qlonglong*>(_v)); break;
        case 28: _t->setMinMultitaskviewSurfaceHeight(*reinterpret_cast<qlonglong*>(_v)); break;
        case 29: _t->setMonoFont(*reinterpret_cast<QString*>(_v)); break;
        case 30: _t->setMultitaskviewAnimationDuration(*reinterpret_cast<qlonglong*>(_v)); break;
        case 31: _t->setMultitaskviewBottomContentMargin(*reinterpret_cast<qlonglong*>(_v)); break;
        case 32: _t->setMultitaskviewCellPadding(*reinterpret_cast<qlonglong*>(_v)); break;
        case 33: _t->setMultitaskviewEasingCurveType(*reinterpret_cast<qlonglong*>(_v)); break;
        case 34: _t->setMultitaskviewHorizontalMargin(*reinterpret_cast<qlonglong*>(_v)); break;
        case 35: _t->setMultitaskviewLoadFactor(*reinterpret_cast<double*>(_v)); break;
        case 36: _t->setMultitaskviewPaddingOpacity(*reinterpret_cast<double*>(_v)); break;
        case 37: _t->setMultitaskviewTopContentMargin(*reinterpret_cast<qlonglong*>(_v)); break;
        case 38: _t->setNormalWindowHeight(*reinterpret_cast<qlonglong*>(_v)); break;
        case 39: _t->setNumWorkspace(*reinterpret_cast<qlonglong*>(_v)); break;
        case 40: _t->setPreferDark(*reinterpret_cast<bool*>(_v)); break;
        case 41: _t->setShowOnLock(*reinterpret_cast<bool*>(_v)); break;
        case 42: _t->setThemeName(*reinterpret_cast<QString*>(_v)); break;
        case 43: _t->setTitleBoxCornerRadius(*reinterpret_cast<qlonglong*>(_v)); break;
        case 44: _t->setWallpaperConfig(*reinterpret_cast<QString*>(_v)); break;
        case 45: _t->setWindowHeightStep(*reinterpret_cast<qlonglong*>(_v)); break;
        case 46: _t->setWindowOpacity(*reinterpret_cast<qlonglong*>(_v)); break;
        case 47: _t->setWindowRadius(*reinterpret_cast<qlonglong*>(_v)); break;
        case 48: _t->setWindowThemeType(*reinterpret_cast<qlonglong*>(_v)); break;
        case 49: _t->setWindowTitlebarHeight(*reinterpret_cast<qlonglong*>(_v)); break;
        case 50: _t->setWorkspaceThumbCornerRadius(*reinterpret_cast<qlonglong*>(_v)); break;
        case 51: _t->setWorkspaceThumbHeight(*reinterpret_cast<qlonglong*>(_v)); break;
        case 52: _t->setWorkspaceThumbMargin(*reinterpret_cast<qlonglong*>(_v)); break;
        default: break;
        }
    }
    if (_c == QMetaObject::ResetProperty) {
        switch (_id) {
        case 0: _t->resetActiveColor(); break;
        case 1: _t->resetBlurAmount(); break;
        case 2: _t->resetBlurMultiplier(); break;
        case 3: _t->resetBlurStrength(); break;
        case 4: _t->resetCurrentWorkspace(); break;
        case 5: _t->resetCursorBlink(); break;
        case 6: _t->resetCursorBlinkTime(); break;
        case 7: _t->resetCursorSize(); break;
        case 8: _t->resetCursorThemeName(); break;
        case 9: _t->resetDefaultBackground(); break;
        case 10: _t->resetDndDragThreshold(); break;
        case 11: _t->resetDoubleClickDistance(); break;
        case 12: _t->resetDoubleClickTime(); break;
        case 13: _t->resetEdgeInnerDelayMs(); break;
        case 14: _t->resetEdgeSideTriggerDistance(); break;
        case 15: _t->resetEdgeTopTriggerDistance(); break;
        case 16: _t->resetFont(); break;
        case 17: _t->resetFontSize(); break;
        case 18: _t->resetGlassBezel(); break;
        case 19: _t->resetGlassEnabled(); break;
        case 20: _t->resetGlassProfilePower(); break;
        case 21: _t->resetGlassSaturation(); break;
        case 22: _t->resetGlassSpecular(); break;
        case 23: _t->resetGlassThickness(); break;
        case 24: _t->resetHighlightBorderWidth(); break;
        case 25: _t->resetIconThemeName(); break;
        case 26: _t->resetLostScreen(); break;
        case 27: _t->resetMaxWorkspace(); break;
        case 28: _t->resetMinMultitaskviewSurfaceHeight(); break;
        case 29: _t->resetMonoFont(); break;
        case 30: _t->resetMultitaskviewAnimationDuration(); break;
        case 31: _t->resetMultitaskviewBottomContentMargin(); break;
        case 32: _t->resetMultitaskviewCellPadding(); break;
        case 33: _t->resetMultitaskviewEasingCurveType(); break;
        case 34: _t->resetMultitaskviewHorizontalMargin(); break;
        case 35: _t->resetMultitaskviewLoadFactor(); break;
        case 36: _t->resetMultitaskviewPaddingOpacity(); break;
        case 37: _t->resetMultitaskviewTopContentMargin(); break;
        case 38: _t->resetNormalWindowHeight(); break;
        case 39: _t->resetNumWorkspace(); break;
        case 40: _t->resetPreferDark(); break;
        case 41: _t->resetShowOnLock(); break;
        case 42: _t->resetThemeName(); break;
        case 43: _t->resetTitleBoxCornerRadius(); break;
        case 44: _t->resetWallpaperConfig(); break;
        case 45: _t->resetWindowHeightStep(); break;
        case 46: _t->resetWindowOpacity(); break;
        case 47: _t->resetWindowRadius(); break;
        case 48: _t->resetWindowThemeType(); break;
        case 49: _t->resetWindowTitlebarHeight(); break;
        case 50: _t->resetWorkspaceThumbCornerRadius(); break;
        case 51: _t->resetWorkspaceThumbHeight(); break;
        case 52: _t->resetWorkspaceThumbMargin(); break;
        default: break;
        }
    }
}

const QMetaObject *TreelandUserConfig::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TreelandUserConfig::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18TreelandUserConfigE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int TreelandUserConfig::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 116)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 116;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 116)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 116;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 53;
    }
    return _id;
}

// SIGNAL 0
void TreelandUserConfig::configInitializeFailed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void TreelandUserConfig::configInitializeSucceed(Dtk::Core::DConfig * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void TreelandUserConfig::valueChanged(const QString & _t1, const QVariant & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void TreelandUserConfig::activeColorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void TreelandUserConfig::blurAmountChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void TreelandUserConfig::blurMultiplierChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void TreelandUserConfig::blurStrengthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void TreelandUserConfig::currentWorkspaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void TreelandUserConfig::cursorBlinkChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void TreelandUserConfig::cursorBlinkTimeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void TreelandUserConfig::cursorSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void TreelandUserConfig::cursorThemeNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void TreelandUserConfig::defaultBackgroundChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void TreelandUserConfig::dndDragThresholdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void TreelandUserConfig::doubleClickDistanceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void TreelandUserConfig::doubleClickTimeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void TreelandUserConfig::edgeInnerDelayMsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void TreelandUserConfig::edgeSideTriggerDistanceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void TreelandUserConfig::edgeTopTriggerDistanceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void TreelandUserConfig::fontChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void TreelandUserConfig::fontSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 20, nullptr);
}

// SIGNAL 21
void TreelandUserConfig::glassBezelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 21, nullptr);
}

// SIGNAL 22
void TreelandUserConfig::glassEnabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 22, nullptr);
}

// SIGNAL 23
void TreelandUserConfig::glassProfilePowerChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 23, nullptr);
}

// SIGNAL 24
void TreelandUserConfig::glassSaturationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 24, nullptr);
}

// SIGNAL 25
void TreelandUserConfig::glassSpecularChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 25, nullptr);
}

// SIGNAL 26
void TreelandUserConfig::glassThicknessChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 26, nullptr);
}

// SIGNAL 27
void TreelandUserConfig::highlightBorderWidthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 27, nullptr);
}

// SIGNAL 28
void TreelandUserConfig::iconThemeNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 28, nullptr);
}

// SIGNAL 29
void TreelandUserConfig::lostScreenChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 29, nullptr);
}

// SIGNAL 30
void TreelandUserConfig::maxWorkspaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 30, nullptr);
}

// SIGNAL 31
void TreelandUserConfig::minMultitaskviewSurfaceHeightChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 31, nullptr);
}

// SIGNAL 32
void TreelandUserConfig::monoFontChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 32, nullptr);
}

// SIGNAL 33
void TreelandUserConfig::multitaskviewAnimationDurationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 33, nullptr);
}

// SIGNAL 34
void TreelandUserConfig::multitaskviewBottomContentMarginChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 34, nullptr);
}

// SIGNAL 35
void TreelandUserConfig::multitaskviewCellPaddingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 35, nullptr);
}

// SIGNAL 36
void TreelandUserConfig::multitaskviewEasingCurveTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 36, nullptr);
}

// SIGNAL 37
void TreelandUserConfig::multitaskviewHorizontalMarginChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 37, nullptr);
}

// SIGNAL 38
void TreelandUserConfig::multitaskviewLoadFactorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 38, nullptr);
}

// SIGNAL 39
void TreelandUserConfig::multitaskviewPaddingOpacityChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 39, nullptr);
}

// SIGNAL 40
void TreelandUserConfig::multitaskviewTopContentMarginChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 40, nullptr);
}

// SIGNAL 41
void TreelandUserConfig::normalWindowHeightChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 41, nullptr);
}

// SIGNAL 42
void TreelandUserConfig::numWorkspaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 42, nullptr);
}

// SIGNAL 43
void TreelandUserConfig::preferDarkChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 43, nullptr);
}

// SIGNAL 44
void TreelandUserConfig::showOnLockChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 44, nullptr);
}

// SIGNAL 45
void TreelandUserConfig::themeNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 45, nullptr);
}

// SIGNAL 46
void TreelandUserConfig::titleBoxCornerRadiusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 46, nullptr);
}

// SIGNAL 47
void TreelandUserConfig::wallpaperConfigChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 47, nullptr);
}

// SIGNAL 48
void TreelandUserConfig::windowHeightStepChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 48, nullptr);
}

// SIGNAL 49
void TreelandUserConfig::windowOpacityChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 49, nullptr);
}

// SIGNAL 50
void TreelandUserConfig::windowRadiusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 50, nullptr);
}

// SIGNAL 51
void TreelandUserConfig::windowThemeTypeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 51, nullptr);
}

// SIGNAL 52
void TreelandUserConfig::windowTitlebarHeightChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 52, nullptr);
}

// SIGNAL 53
void TreelandUserConfig::workspaceThumbCornerRadiusChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 53, nullptr);
}

// SIGNAL 54
void TreelandUserConfig::workspaceThumbHeightChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 54, nullptr);
}

// SIGNAL 55
void TreelandUserConfig::workspaceThumbMarginChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 55, nullptr);
}
QT_WARNING_POP
