/****************************************************************************
** Meta object code from reading C++ file 'foreigntoplevelhandlev1.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/modules/foreign-toplevel/foreigntoplevelhandlev1.h"
#include <woutput.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'foreigntoplevelhandlev1.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN23ForeignToplevelHandleV1E_t {};
} // unnamed namespace

template <> constexpr inline auto ForeignToplevelHandleV1::qt_create_metaobjectdata<qt_meta_tag_ZN23ForeignToplevelHandleV1E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ForeignToplevelHandleV1",
        "requestMaximize",
        "",
        "maximized",
        "requestMinimize",
        "minimized",
        "requestActivate",
        "Waylib::Server::WSeat*",
        "seat",
        "requestFullscreen",
        "fullscreen",
        "Waylib::Server::WOutput*",
        "output",
        "requestClose",
        "rectangleChanged",
        "Waylib::Server::WSurface*",
        "surface",
        "QRect",
        "rect",
        "State",
        "Maximized",
        "Minimized",
        "Activated",
        "Fullscreen",
        "Attention"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'requestMaximize'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'requestMinimize'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Signal 'requestActivate'
        QtMocHelpers::SignalData<void(Waylib::Server::WSeat *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'requestFullscreen'
        QtMocHelpers::SignalData<void(bool, Waylib::Server::WOutput *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 10 }, { 0x80000000 | 11, 12 },
        }}),
        // Signal 'requestClose'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'rectangleChanged'
        QtMocHelpers::SignalData<void(Waylib::Server::WSurface *, const QRect &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 }, { 0x80000000 | 17, 18 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'State'
        QtMocHelpers::EnumData<enum State>(19, 19, QMC::EnumIsScoped).add({
            {   20, State::Maximized },
            {   21, State::Minimized },
            {   22, State::Activated },
            {   23, State::Fullscreen },
            {   24, State::Attention },
        }),
    };
    return QtMocHelpers::metaObjectData<ForeignToplevelHandleV1, qt_meta_tag_ZN23ForeignToplevelHandleV1E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ForeignToplevelHandleV1::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23ForeignToplevelHandleV1E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23ForeignToplevelHandleV1E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN23ForeignToplevelHandleV1E_t>.metaTypes,
    nullptr
} };

void ForeignToplevelHandleV1::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ForeignToplevelHandleV1 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->requestMaximize((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->requestMinimize((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->requestActivate((*reinterpret_cast<std::add_pointer_t<Waylib::Server::WSeat*>>(_a[1]))); break;
        case 3: _t->requestFullscreen((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Waylib::Server::WOutput*>>(_a[2]))); break;
        case 4: _t->requestClose(); break;
        case 5: _t->rectangleChanged((*reinterpret_cast<std::add_pointer_t<Waylib::Server::WSurface*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QRect>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ForeignToplevelHandleV1::*)(bool )>(_a, &ForeignToplevelHandleV1::requestMaximize, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ForeignToplevelHandleV1::*)(bool )>(_a, &ForeignToplevelHandleV1::requestMinimize, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ForeignToplevelHandleV1::*)(Waylib::Server::WSeat * )>(_a, &ForeignToplevelHandleV1::requestActivate, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ForeignToplevelHandleV1::*)(bool , Waylib::Server::WOutput * )>(_a, &ForeignToplevelHandleV1::requestFullscreen, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ForeignToplevelHandleV1::*)()>(_a, &ForeignToplevelHandleV1::requestClose, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ForeignToplevelHandleV1::*)(Waylib::Server::WSurface * , const QRect & )>(_a, &ForeignToplevelHandleV1::rectangleChanged, 5))
            return;
    }
}

const QMetaObject *ForeignToplevelHandleV1::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ForeignToplevelHandleV1::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23ForeignToplevelHandleV1E_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Waylib::Server::WObject"))
        return static_cast< Waylib::Server::WObject*>(this);
    return QObject::qt_metacast(_clname);
}

int ForeignToplevelHandleV1::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void ForeignToplevelHandleV1::requestMaximize(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void ForeignToplevelHandleV1::requestMinimize(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void ForeignToplevelHandleV1::requestActivate(Waylib::Server::WSeat * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void ForeignToplevelHandleV1::requestFullscreen(bool _t1, Waylib::Server::WOutput * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void ForeignToplevelHandleV1::requestClose()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void ForeignToplevelHandleV1::rectangleChanged(Waylib::Server::WSurface * _t1, const QRect & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}
QT_WARNING_POP
