/****************************************************************************
** Meta object code from reading C++ file 'treelandconfig.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../treelandconfig.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'treelandconfig.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14TreelandConfigE_t {};
} // unnamed namespace

template <> constexpr inline auto TreelandConfig::qt_create_metaobjectdata<qt_meta_tag_ZN14TreelandConfigE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TreelandConfig",
        "DConfigKeyList",
        "copyOutputName;copyOutputOutputs;createCopyOutput;debugSource;enab"
        "lePrelaunchSplash;forceSoftwareCursor;keyboardNumLock;prelaunchSpl"
        "ashTimeoutMs;primaryOutputId;quickSwitchTimeout;showOtherUserOptio"
        "n;singleOutputId",
        "DConfigFileName",
        "org.deepin.dde.treeland",
        "DConfigFileVersion",
        "1.0",
        "configInitializeFailed",
        "",
        "configInitializeSucceed",
        "Dtk::Core::DConfig*",
        "config",
        "valueChanged",
        "key",
        "QVariant",
        "value",
        "copyOutputNameChanged",
        "copyOutputOutputsChanged",
        "createCopyOutputChanged",
        "debugSourceChanged",
        "enablePrelaunchSplashChanged",
        "forceSoftwareCursorChanged",
        "keyboardNumLockChanged",
        "prelaunchSplashTimeoutMsChanged",
        "primaryOutputIdChanged",
        "quickSwitchTimeoutChanged",
        "showOtherUserOptionChanged",
        "singleOutputIdChanged",
        "isInitializeSucceed",
        "isInitializeSucceeded",
        "isInitializeFailed",
        "isInitializing",
        "keyList",
        "isDefaultValue",
        "copyOutputNameIsDefaultValue",
        "copyOutputOutputsIsDefaultValue",
        "createCopyOutputIsDefaultValue",
        "debugSourceIsDefaultValue",
        "enablePrelaunchSplashIsDefaultValue",
        "forceSoftwareCursorIsDefaultValue",
        "keyboardNumLockIsDefaultValue",
        "prelaunchSplashTimeoutMsIsDefaultValue",
        "primaryOutputIdIsDefaultValue",
        "quickSwitchTimeoutIsDefaultValue",
        "showOtherUserOptionIsDefaultValue",
        "singleOutputIdIsDefaultValue",
        "copyOutputName",
        "copyOutputOutputs",
        "createCopyOutput",
        "debugSource",
        "enablePrelaunchSplash",
        "forceSoftwareCursor",
        "keyboardNumLock",
        "prelaunchSplashTimeoutMs",
        "primaryOutputId",
        "quickSwitchTimeout",
        "showOtherUserOption",
        "singleOutputId"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'configInitializeFailed'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'configInitializeSucceed'
        QtMocHelpers::SignalData<void(Dtk::Core::DConfig *)>(9, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'valueChanged'
        QtMocHelpers::SignalData<void(const QString &, const QVariant &)>(12, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 }, { 0x80000000 | 14, 15 },
        }}),
        // Signal 'copyOutputNameChanged'
        QtMocHelpers::SignalData<void()>(16, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'copyOutputOutputsChanged'
        QtMocHelpers::SignalData<void()>(17, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'createCopyOutputChanged'
        QtMocHelpers::SignalData<void()>(18, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'debugSourceChanged'
        QtMocHelpers::SignalData<void()>(19, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'enablePrelaunchSplashChanged'
        QtMocHelpers::SignalData<void()>(20, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'forceSoftwareCursorChanged'
        QtMocHelpers::SignalData<void()>(21, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'keyboardNumLockChanged'
        QtMocHelpers::SignalData<void()>(22, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'prelaunchSplashTimeoutMsChanged'
        QtMocHelpers::SignalData<void()>(23, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'primaryOutputIdChanged'
        QtMocHelpers::SignalData<void()>(24, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'quickSwitchTimeoutChanged'
        QtMocHelpers::SignalData<void()>(25, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'showOtherUserOptionChanged'
        QtMocHelpers::SignalData<void()>(26, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'singleOutputIdChanged'
        QtMocHelpers::SignalData<void()>(27, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'config'
        QtMocHelpers::MethodData<Dtk::Core::DConfig *() const>(11, 8, QMC::AccessPublic, 0x80000000 | 10),
        // Method 'isInitializeSucceed'
        QtMocHelpers::MethodData<bool() const>(28, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'isInitializeSucceeded'
        QtMocHelpers::MethodData<bool() const>(29, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'isInitializeFailed'
        QtMocHelpers::MethodData<bool() const>(30, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'isInitializing'
        QtMocHelpers::MethodData<bool() const>(31, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'keyList'
        QtMocHelpers::MethodData<QStringList() const>(32, 8, QMC::AccessPublic, QMetaType::QStringList),
        // Method 'isDefaultValue'
        QtMocHelpers::MethodData<bool(const QString &) const>(33, 8, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 13 },
        }}),
        // Method 'copyOutputNameIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(34, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'copyOutputOutputsIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(35, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'createCopyOutputIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(36, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'debugSourceIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(37, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'enablePrelaunchSplashIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(38, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'forceSoftwareCursorIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(39, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'keyboardNumLockIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(40, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'prelaunchSplashTimeoutMsIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(41, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'primaryOutputIdIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(42, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'quickSwitchTimeoutIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(43, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'showOtherUserOptionIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(44, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'singleOutputIdIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(45, 8, QMC::AccessPublic, QMetaType::Bool),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'copyOutputName'
        QtMocHelpers::PropertyData<QString>(46, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 3),
        // property 'copyOutputOutputs'
        QtMocHelpers::PropertyData<QString>(47, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 4),
        // property 'createCopyOutput'
        QtMocHelpers::PropertyData<bool>(48, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 5),
        // property 'debugSource'
        QtMocHelpers::PropertyData<bool>(49, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 6),
        // property 'enablePrelaunchSplash'
        QtMocHelpers::PropertyData<bool>(50, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 7),
        // property 'forceSoftwareCursor'
        QtMocHelpers::PropertyData<bool>(51, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 8),
        // property 'keyboardNumLock'
        QtMocHelpers::PropertyData<bool>(52, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 9),
        // property 'prelaunchSplashTimeoutMs'
        QtMocHelpers::PropertyData<qlonglong>(53, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 10),
        // property 'primaryOutputId'
        QtMocHelpers::PropertyData<QString>(54, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 11),
        // property 'quickSwitchTimeout'
        QtMocHelpers::PropertyData<qlonglong>(55, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 12),
        // property 'showOtherUserOption'
        QtMocHelpers::PropertyData<bool>(56, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 13),
        // property 'singleOutputId'
        QtMocHelpers::PropertyData<QString>(57, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 14),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<TreelandConfig, qt_meta_tag_ZN14TreelandConfigE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject TreelandConfig::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14TreelandConfigE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14TreelandConfigE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14TreelandConfigE_t>.metaTypes,
    nullptr
} };

void TreelandConfig::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TreelandConfig *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->configInitializeFailed(); break;
        case 1: _t->configInitializeSucceed((*reinterpret_cast<std::add_pointer_t<Dtk::Core::DConfig*>>(_a[1]))); break;
        case 2: _t->valueChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 3: _t->copyOutputNameChanged(); break;
        case 4: _t->copyOutputOutputsChanged(); break;
        case 5: _t->createCopyOutputChanged(); break;
        case 6: _t->debugSourceChanged(); break;
        case 7: _t->enablePrelaunchSplashChanged(); break;
        case 8: _t->forceSoftwareCursorChanged(); break;
        case 9: _t->keyboardNumLockChanged(); break;
        case 10: _t->prelaunchSplashTimeoutMsChanged(); break;
        case 11: _t->primaryOutputIdChanged(); break;
        case 12: _t->quickSwitchTimeoutChanged(); break;
        case 13: _t->showOtherUserOptionChanged(); break;
        case 14: _t->singleOutputIdChanged(); break;
        case 15: { Dtk::Core::DConfig* _r = _t->config();
            if (_a[0]) *reinterpret_cast<Dtk::Core::DConfig**>(_a[0]) = std::move(_r); }  break;
        case 16: { bool _r = _t->isInitializeSucceed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 17: { bool _r = _t->isInitializeSucceeded();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 18: { bool _r = _t->isInitializeFailed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 19: { bool _r = _t->isInitializing();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 20: { QStringList _r = _t->keyList();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 21: { bool _r = _t->isDefaultValue((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 22: { bool _r = _t->copyOutputNameIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 23: { bool _r = _t->copyOutputOutputsIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 24: { bool _r = _t->createCopyOutputIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 25: { bool _r = _t->debugSourceIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 26: { bool _r = _t->enablePrelaunchSplashIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 27: { bool _r = _t->forceSoftwareCursorIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 28: { bool _r = _t->keyboardNumLockIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 29: { bool _r = _t->prelaunchSplashTimeoutMsIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 30: { bool _r = _t->primaryOutputIdIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 31: { bool _r = _t->quickSwitchTimeoutIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 32: { bool _r = _t->showOtherUserOptionIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 33: { bool _r = _t->singleOutputIdIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Dtk::Core::DConfig* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TreelandConfig::*)()>(_a, &TreelandConfig::configInitializeFailed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandConfig::*)(Dtk::Core::DConfig * )>(_a, &TreelandConfig::configInitializeSucceed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandConfig::*)(const QString & , const QVariant & )>(_a, &TreelandConfig::valueChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandConfig::*)()>(_a, &TreelandConfig::copyOutputNameChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandConfig::*)()>(_a, &TreelandConfig::copyOutputOutputsChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandConfig::*)()>(_a, &TreelandConfig::createCopyOutputChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandConfig::*)()>(_a, &TreelandConfig::debugSourceChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandConfig::*)()>(_a, &TreelandConfig::enablePrelaunchSplashChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandConfig::*)()>(_a, &TreelandConfig::forceSoftwareCursorChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandConfig::*)()>(_a, &TreelandConfig::keyboardNumLockChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandConfig::*)()>(_a, &TreelandConfig::prelaunchSplashTimeoutMsChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandConfig::*)()>(_a, &TreelandConfig::primaryOutputIdChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandConfig::*)()>(_a, &TreelandConfig::quickSwitchTimeoutChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandConfig::*)()>(_a, &TreelandConfig::showOtherUserOptionChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandConfig::*)()>(_a, &TreelandConfig::singleOutputIdChanged, 14))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->copyOutputName(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->copyOutputOutputs(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->createCopyOutput(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->debugSource(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->enablePrelaunchSplash(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->forceSoftwareCursor(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->keyboardNumLock(); break;
        case 7: *reinterpret_cast<qlonglong*>(_v) = _t->prelaunchSplashTimeoutMs(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->primaryOutputId(); break;
        case 9: *reinterpret_cast<qlonglong*>(_v) = _t->quickSwitchTimeout(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->showOtherUserOption(); break;
        case 11: *reinterpret_cast<QString*>(_v) = _t->singleOutputId(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setCopyOutputName(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setCopyOutputOutputs(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setCreateCopyOutput(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setDebugSource(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setEnablePrelaunchSplash(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setForceSoftwareCursor(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->setKeyboardNumLock(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setPrelaunchSplashTimeoutMs(*reinterpret_cast<qlonglong*>(_v)); break;
        case 8: _t->setPrimaryOutputId(*reinterpret_cast<QString*>(_v)); break;
        case 9: _t->setQuickSwitchTimeout(*reinterpret_cast<qlonglong*>(_v)); break;
        case 10: _t->setShowOtherUserOption(*reinterpret_cast<bool*>(_v)); break;
        case 11: _t->setSingleOutputId(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
    if (_c == QMetaObject::ResetProperty) {
        switch (_id) {
        case 0: _t->resetCopyOutputName(); break;
        case 1: _t->resetCopyOutputOutputs(); break;
        case 2: _t->resetCreateCopyOutput(); break;
        case 3: _t->resetDebugSource(); break;
        case 4: _t->resetEnablePrelaunchSplash(); break;
        case 5: _t->resetForceSoftwareCursor(); break;
        case 6: _t->resetKeyboardNumLock(); break;
        case 7: _t->resetPrelaunchSplashTimeoutMs(); break;
        case 8: _t->resetPrimaryOutputId(); break;
        case 9: _t->resetQuickSwitchTimeout(); break;
        case 10: _t->resetShowOtherUserOption(); break;
        case 11: _t->resetSingleOutputId(); break;
        default: break;
        }
    }
}

const QMetaObject *TreelandConfig::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TreelandConfig::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14TreelandConfigE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int TreelandConfig::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 34)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 34;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 34)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 34;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void TreelandConfig::configInitializeFailed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void TreelandConfig::configInitializeSucceed(Dtk::Core::DConfig * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void TreelandConfig::valueChanged(const QString & _t1, const QVariant & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void TreelandConfig::copyOutputNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void TreelandConfig::copyOutputOutputsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void TreelandConfig::createCopyOutputChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void TreelandConfig::debugSourceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void TreelandConfig::enablePrelaunchSplashChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void TreelandConfig::forceSoftwareCursorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void TreelandConfig::keyboardNumLockChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void TreelandConfig::prelaunchSplashTimeoutMsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void TreelandConfig::primaryOutputIdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void TreelandConfig::quickSwitchTimeoutChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void TreelandConfig::showOtherUserOptionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void TreelandConfig::singleOutputIdChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}
QT_WARNING_POP
