// Copyright (C) 2023 JiDe Zhang <zhangjide@deepin.org>.
// SPDX-License-Identifier: Apache-2.0 OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only

#include "wtypes.h"

WAYLIB_SERVER_BEGIN_NAMESPACE

WAYLIB_SERVER_END_NAMESPACE
