/****************************************************************************
** Meta object code from reading C++ file 'Login1Seat.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "Login1Seat.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'Login1Seat.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN33OrgFreedesktopLogin1SeatInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto OrgFreedesktopLogin1SeatInterface::qt_create_metaobjectdata<qt_meta_tag_ZN33OrgFreedesktopLogin1SeatInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "OrgFreedesktopLogin1SeatInterface",
        "ActivateSession",
        "QDBusPendingReply<>",
        "",
        "in0",
        "SwitchTo",
        "SwitchToNext",
        "SwitchToPrevious",
        "Terminate",
        "ActiveSession",
        "NamedSessionPath",
        "CanGraphical",
        "CanMultiSession",
        "CanTTY",
        "Id",
        "IdleHint",
        "IdleSinceHint",
        "IdleSinceHintMonotonic",
        "Sessions",
        "QList<NamedSessionPath>"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'ActivateSession'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &)>(1, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { QMetaType::QString, 4 },
        }}),
        // Slot 'SwitchTo'
        QtMocHelpers::SlotData<QDBusPendingReply<>(uint)>(5, 3, QMC::AccessPublic, 0x80000000 | 2, {{
            { QMetaType::UInt, 4 },
        }}),
        // Slot 'SwitchToNext'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(6, 3, QMC::AccessPublic, 0x80000000 | 2),
        // Slot 'SwitchToPrevious'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(7, 3, QMC::AccessPublic, 0x80000000 | 2),
        // Slot 'Terminate'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(8, 3, QMC::AccessPublic, 0x80000000 | 2),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'ActiveSession'
        QtMocHelpers::PropertyData<NamedSessionPath>(9, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
        // property 'CanGraphical'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'CanMultiSession'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'CanTTY'
        QtMocHelpers::PropertyData<bool>(13, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'Id'
        QtMocHelpers::PropertyData<QString>(14, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'IdleHint'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'IdleSinceHint'
        QtMocHelpers::PropertyData<qulonglong>(16, QMetaType::ULongLong, QMC::DefaultPropertyFlags),
        // property 'IdleSinceHintMonotonic'
        QtMocHelpers::PropertyData<qulonglong>(17, QMetaType::ULongLong, QMC::DefaultPropertyFlags),
        // property 'Sessions'
        QtMocHelpers::PropertyData<QList<NamedSessionPath>>(18, 0x80000000 | 19, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OrgFreedesktopLogin1SeatInterface, qt_meta_tag_ZN33OrgFreedesktopLogin1SeatInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject OrgFreedesktopLogin1SeatInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN33OrgFreedesktopLogin1SeatInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN33OrgFreedesktopLogin1SeatInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN33OrgFreedesktopLogin1SeatInterfaceE_t>.metaTypes,
    nullptr
} };

void OrgFreedesktopLogin1SeatInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OrgFreedesktopLogin1SeatInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QDBusPendingReply<> _r = _t->ActivateSession((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 1: { QDBusPendingReply<> _r = _t->SwitchTo((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 2: { QDBusPendingReply<> _r = _t->SwitchToNext();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 3: { QDBusPendingReply<> _r = _t->SwitchToPrevious();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 4: { QDBusPendingReply<> _r = _t->Terminate();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<NamedSessionPath*>(_v) = _t->activeSession(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->canGraphical(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->canMultiSession(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->canTTY(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->id(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->idleHint(); break;
        case 6: *reinterpret_cast<qulonglong*>(_v) = _t->idleSinceHint(); break;
        case 7: *reinterpret_cast<qulonglong*>(_v) = _t->idleSinceHintMonotonic(); break;
        case 8: *reinterpret_cast<QList<NamedSessionPath>*>(_v) = _t->sessions(); break;
        default: break;
        }
    }
}

const QMetaObject *OrgFreedesktopLogin1SeatInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OrgFreedesktopLogin1SeatInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN33OrgFreedesktopLogin1SeatInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int OrgFreedesktopLogin1SeatInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}
QT_WARNING_POP
