/****************************************************************************
** Meta object code from reading C++ file 'wqmlcreator_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../waylib/src/server/qtquick/private/wqmlcreator_p.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QSharedPointer>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wqmlcreator_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Waylib6Server22WQmlCreatorDataWatcherE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WQmlCreatorDataWatcher::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server22WQmlCreatorDataWatcherE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WQmlCreatorDataWatcher",
        "QML.Element",
        "DynamicCreatorDataWatcher",
        "added",
        "",
        "owner",
        "QJSValue",
        "initialProperties",
        "removed"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'added'
        QtMocHelpers::SignalData<void(QObject *, const QJSValue &)>(3, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QObjectStar, 5 }, { 0x80000000 | 6, 7 },
        }}),
        // Signal 'removed'
        QtMocHelpers::SignalData<void(QObject *, const QJSValue &)>(8, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QObjectStar, 5 }, { 0x80000000 | 6, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<WQmlCreatorDataWatcher, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WQmlCreatorDataWatcher::staticMetaObject = { {
    QMetaObject::SuperData::link<WAbstractCreatorComponent::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server22WQmlCreatorDataWatcherE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server22WQmlCreatorDataWatcherE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server22WQmlCreatorDataWatcherE_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WQmlCreatorDataWatcher::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WQmlCreatorDataWatcher *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->added((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[2]))); break;
        case 1: _t->removed((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QJSValue >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QJSValue >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WQmlCreatorDataWatcher::*)(QObject * , const QJSValue & )>(_a, &WQmlCreatorDataWatcher::added, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WQmlCreatorDataWatcher::*)(QObject * , const QJSValue & )>(_a, &WQmlCreatorDataWatcher::removed, 1))
            return;
    }
}

const QMetaObject *Waylib::Server::WQmlCreatorDataWatcher::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WQmlCreatorDataWatcher::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server22WQmlCreatorDataWatcherE_t>.strings))
        return static_cast<void*>(this);
    return WAbstractCreatorComponent::qt_metacast(_clname);
}

int Waylib::Server::WQmlCreatorDataWatcher::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = WAbstractCreatorComponent::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WQmlCreatorDataWatcher::added(QObject * _t1, const QJSValue & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void Waylib::Server::WQmlCreatorDataWatcher::removed(QObject * _t1, const QJSValue & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN6Waylib6Server20WQmlCreatorComponentE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WQmlCreatorComponent::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server20WQmlCreatorComponentE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WQmlCreatorComponent",
        "QML.Element",
        "DynamicCreatorComponent",
        "DefaultProperty",
        "delegate",
        "parentChanged",
        "",
        "chooserRoleChanged",
        "chooserRoleValueChanged",
        "autoDestroyChanged",
        "objectAdded",
        "object",
        "QJSValue",
        "initialProperties",
        "objectRemoved",
        "create",
        "QSharedPointer<WQmlCreatorDelegateData>",
        "data",
        "parent",
        "destroyObject",
        "QQmlComponent*",
        "chooserRole",
        "chooserRoleValue",
        "QVariant",
        "autoDestroy"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'parentChanged'
        QtMocHelpers::SignalData<void()>(5, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'chooserRoleChanged'
        QtMocHelpers::SignalData<void()>(7, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'chooserRoleValueChanged'
        QtMocHelpers::SignalData<void()>(8, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'autoDestroyChanged'
        QtMocHelpers::SignalData<void()>(9, 6, QMC::AccessPublic, QMetaType::Void),
        // Signal 'objectAdded'
        QtMocHelpers::SignalData<void(QObject *, const QJSValue &)>(10, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QObjectStar, 11 }, { 0x80000000 | 12, 13 },
        }}),
        // Signal 'objectRemoved'
        QtMocHelpers::SignalData<void(QObject *, const QJSValue &)>(14, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QObjectStar, 11 }, { 0x80000000 | 12, 13 },
        }}),
        // Slot 'create'
        QtMocHelpers::SlotData<void(QSharedPointer<WQmlCreatorDelegateData>, QObject *, const QJSValue &)>(15, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 16, 17 }, { QMetaType::QObjectStar, 18 }, { 0x80000000 | 12, 13 },
        }}),
        // Method 'destroyObject'
        QtMocHelpers::MethodData<void(QObject *)>(19, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QObjectStar, 11 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'delegate'
        QtMocHelpers::PropertyData<QQmlComponent*>(4, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final | QMC::Required),
        // property 'parent'
        QtMocHelpers::PropertyData<QObject*>(18, QMetaType::QObjectStar, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 0),
        // property 'chooserRole'
        QtMocHelpers::PropertyData<QString>(21, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 1),
        // property 'chooserRoleValue'
        QtMocHelpers::PropertyData<QVariant>(22, 0x80000000 | 23, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 2),
        // property 'autoDestroy'
        QtMocHelpers::PropertyData<bool>(24, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 3),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<WQmlCreatorComponent, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WQmlCreatorComponent::staticMetaObject = { {
    QMetaObject::SuperData::link<WAbstractCreatorComponent::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server20WQmlCreatorComponentE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server20WQmlCreatorComponentE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server20WQmlCreatorComponentE_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WQmlCreatorComponent::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WQmlCreatorComponent *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->parentChanged(); break;
        case 1: _t->chooserRoleChanged(); break;
        case 2: _t->chooserRoleValueChanged(); break;
        case 3: _t->autoDestroyChanged(); break;
        case 4: _t->objectAdded((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[2]))); break;
        case 5: _t->objectRemoved((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[2]))); break;
        case 6: _t->create((*reinterpret_cast<std::add_pointer_t<QSharedPointer<WQmlCreatorDelegateData>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[3]))); break;
        case 7: _t->destroyObject((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QJSValue >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QJSValue >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QJSValue >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WQmlCreatorComponent::*)()>(_a, &WQmlCreatorComponent::parentChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WQmlCreatorComponent::*)()>(_a, &WQmlCreatorComponent::chooserRoleChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WQmlCreatorComponent::*)()>(_a, &WQmlCreatorComponent::chooserRoleValueChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WQmlCreatorComponent::*)()>(_a, &WQmlCreatorComponent::autoDestroyChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (WQmlCreatorComponent::*)(QObject * , const QJSValue & )>(_a, &WQmlCreatorComponent::objectAdded, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (WQmlCreatorComponent::*)(QObject * , const QJSValue & )>(_a, &WQmlCreatorComponent::objectRemoved, 5))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlComponent* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QQmlComponent**>(_v) = _t->delegate(); break;
        case 1: *reinterpret_cast<QObject**>(_v) = _t->parent(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->chooserRole(); break;
        case 3: *reinterpret_cast<QVariant*>(_v) = _t->chooserRoleValue(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->autoDestroy(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setDelegate(*reinterpret_cast<QQmlComponent**>(_v)); break;
        case 1: _t->setParent(*reinterpret_cast<QObject**>(_v)); break;
        case 2: _t->setChooserRole(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setChooserRoleValue(*reinterpret_cast<QVariant*>(_v)); break;
        case 4: _t->setAutoDestroy(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Waylib::Server::WQmlCreatorComponent::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WQmlCreatorComponent::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server20WQmlCreatorComponentE_t>.strings))
        return static_cast<void*>(this);
    return WAbstractCreatorComponent::qt_metacast(_clname);
}

int Waylib::Server::WQmlCreatorComponent::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = WAbstractCreatorComponent::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WQmlCreatorComponent::parentChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Waylib::Server::WQmlCreatorComponent::chooserRoleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Waylib::Server::WQmlCreatorComponent::chooserRoleValueChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Waylib::Server::WQmlCreatorComponent::autoDestroyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Waylib::Server::WQmlCreatorComponent::objectAdded(QObject * _t1, const QJSValue & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2);
}

// SIGNAL 5
void Waylib::Server::WQmlCreatorComponent::objectRemoved(QObject * _t1, const QJSValue & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}
QT_WARNING_POP
