/****************************************************************************
** Meta object code from reading C++ file 'wtextinputv3_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../waylib/src/server/protocols/private/wtextinputv3_p.h"
#include "private/wserver_p.h"
#include "wsurface.h"
#include "winputmethodv2_p.h"
#include "wsurface.h"
#include "wseat.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wtextinputv3_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Waylib6Server12WTextInputV3E_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WTextInputV3::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server12WTextInputV3E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WTextInputV3",
        "sendEnter",
        "",
        "WSurface*",
        "surface",
        "sendLeave",
        "sendDone",
        "handleIMCommitted",
        "WInputMethodV2*",
        "im",
        "Feature",
        "SurroundingText",
        "ContentType",
        "CursorRect",
        "ChangeCause",
        "InputMethod",
        "Other",
        "ContentHint",
        "None",
        "Completion",
        "Spellcheck",
        "AutoCapitalization",
        "Lowercase",
        "Uppercase",
        "Titlecase",
        "HiddenText",
        "SensitiveData",
        "Latin",
        "Multiline",
        "ContentPurpose",
        "Normal",
        "Alpha",
        "Digits",
        "Number",
        "Phone",
        "Url",
        "Email",
        "Name",
        "Password",
        "Pin",
        "Date",
        "Time",
        "Datetime",
        "Terminal"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'sendEnter'
        QtMocHelpers::SlotData<void(WSurface *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'sendLeave'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'sendDone'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleIMCommitted'
        QtMocHelpers::SlotData<void(WInputMethodV2 *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Feature'
        QtMocHelpers::EnumData<enum Feature>(10, 10, QMC::EnumFlags{}).add({
            {   11, Feature::SurroundingText },
            {   12, Feature::ContentType },
            {   13, Feature::CursorRect },
        }),
        // enum 'ChangeCause'
        QtMocHelpers::EnumData<enum ChangeCause>(14, 14, QMC::EnumFlags{}).add({
            {   15, ChangeCause::InputMethod },
            {   16, ChangeCause::Other },
        }),
        // enum 'ContentHint'
        QtMocHelpers::EnumData<enum ContentHint>(17, 17, QMC::EnumFlags{}).add({
            {   18, ContentHint::None },
            {   19, ContentHint::Completion },
            {   20, ContentHint::Spellcheck },
            {   21, ContentHint::AutoCapitalization },
            {   22, ContentHint::Lowercase },
            {   23, ContentHint::Uppercase },
            {   24, ContentHint::Titlecase },
            {   25, ContentHint::HiddenText },
            {   26, ContentHint::SensitiveData },
            {   27, ContentHint::Latin },
            {   28, ContentHint::Multiline },
        }),
        // enum 'ContentPurpose'
        QtMocHelpers::EnumData<enum ContentPurpose>(29, 29, QMC::EnumFlags{}).add({
            {   30, ContentPurpose::Normal },
            {   31, ContentPurpose::Alpha },
            {   32, ContentPurpose::Digits },
            {   33, ContentPurpose::Number },
            {   34, ContentPurpose::Phone },
            {   35, ContentPurpose::Url },
            {   36, ContentPurpose::Email },
            {   37, ContentPurpose::Name },
            {   38, ContentPurpose::Password },
            {   39, ContentPurpose::Pin },
            {   40, ContentPurpose::Date },
            {   41, ContentPurpose::Time },
            {   42, ContentPurpose::Datetime },
            {   43, ContentPurpose::Terminal },
        }),
    };
    return QtMocHelpers::metaObjectData<WTextInputV3, qt_meta_tag_ZN6Waylib6Server12WTextInputV3E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WTextInputV3::staticMetaObject = { {
    QMetaObject::SuperData::link<WTextInput::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server12WTextInputV3E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server12WTextInputV3E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server12WTextInputV3E_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WTextInputV3::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WTextInputV3 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sendEnter((*reinterpret_cast<std::add_pointer_t<WSurface*>>(_a[1]))); break;
        case 1: _t->sendLeave(); break;
        case 2: _t->sendDone(); break;
        case 3: _t->handleIMCommitted((*reinterpret_cast<std::add_pointer_t<WInputMethodV2*>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *Waylib::Server::WTextInputV3::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WTextInputV3::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server12WTextInputV3E_t>.strings))
        return static_cast<void*>(this);
    return WTextInput::qt_metacast(_clname);
}

int Waylib::Server::WTextInputV3::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = WTextInput::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN6Waylib6Server19WTextInputManagerV3E_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WTextInputManagerV3::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server19WTextInputManagerV3E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WTextInputManagerV3",
        "newTextInput",
        "",
        "WTextInputV3*",
        "textInput"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'newTextInput'
        QtMocHelpers::SignalData<void(WTextInputV3 *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<WTextInputManagerV3, qt_meta_tag_ZN6Waylib6Server19WTextInputManagerV3E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WTextInputManagerV3::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server19WTextInputManagerV3E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server19WTextInputManagerV3E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server19WTextInputManagerV3E_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WTextInputManagerV3::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WTextInputManagerV3 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->newTextInput((*reinterpret_cast<std::add_pointer_t<WTextInputV3*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< WTextInputV3* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WTextInputManagerV3::*)(WTextInputV3 * )>(_a, &WTextInputManagerV3::newTextInput, 0))
            return;
    }
}

const QMetaObject *Waylib::Server::WTextInputManagerV3::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WTextInputManagerV3::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server19WTextInputManagerV3E_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "WObject"))
        return static_cast< WObject*>(this);
    if (!strcmp(_clname, "WServerInterface"))
        return static_cast< WServerInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int Waylib::Server::WTextInputManagerV3::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WTextInputManagerV3::newTextInput(WTextInputV3 * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
