/****************************************************************************
** Meta object code from reading C++ file 'woutputlayer.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../waylib/src/server/qtquick/woutputlayer.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'woutputlayer.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Waylib6Server12WOutputLayerE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WOutputLayer::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server12WOutputLayerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WOutputLayer",
        "QML.Element",
        "OutputLayer",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "OutputLayer is only available via attached properties",
        "QML.Attached",
        "WOutputLayer",
        "enabledChanged",
        "",
        "flagsChanged",
        "colorContentsModeChanged",
        "outputsChanged",
        "inOutputsByHardwareChanged",
        "zChanged",
        "keepLayerChanged",
        "forceChanged",
        "cursorHotSpotChanged",
        "enabled",
        "keepLayer",
        "force",
        "flags",
        "Flags",
        "colorContentsMode",
        "WGlobal::ColorContentsMode",
        "outputs",
        "QList<WOutputViewport*>",
        "inOutputsByHardware",
        "z",
        "cursorHotSpot",
        "QPointF",
        "Flag",
        "SizeSensitive",
        "DontClip",
        "NoAlpha",
        "Cursor"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'enabledChanged'
        QtMocHelpers::SignalData<void()>(9, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'flagsChanged'
        QtMocHelpers::SignalData<void()>(11, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'colorContentsModeChanged'
        QtMocHelpers::SignalData<void()>(12, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'outputsChanged'
        QtMocHelpers::SignalData<void()>(13, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'inOutputsByHardwareChanged'
        QtMocHelpers::SignalData<void()>(14, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'zChanged'
        QtMocHelpers::SignalData<void()>(15, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'keepLayerChanged'
        QtMocHelpers::SignalData<void()>(16, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'forceChanged'
        QtMocHelpers::SignalData<void()>(17, 10, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cursorHotSpotChanged'
        QtMocHelpers::SignalData<void()>(18, 10, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'enabled'
        QtMocHelpers::PropertyData<bool>(19, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 0),
        // property 'keepLayer'
        QtMocHelpers::PropertyData<bool>(20, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 6),
        // property 'force'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 7),
        // property 'flags'
        QtMocHelpers::PropertyData<Flags>(22, 0x80000000 | 23, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 1),
        // property 'colorContentsMode'
        QtMocHelpers::PropertyData<WGlobal::ColorContentsMode>(24, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 2),
        // property 'outputs'
        QtMocHelpers::PropertyData<QList<WOutputViewport*>>(26, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 3),
        // property 'inOutputsByHardware'
        QtMocHelpers::PropertyData<QList<WOutputViewport*>>(28, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 4),
        // property 'z'
        QtMocHelpers::PropertyData<int>(29, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 5),
        // property 'cursorHotSpot'
        QtMocHelpers::PropertyData<QPointF>(30, 0x80000000 | 31, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 8),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Flag'
        QtMocHelpers::EnumData<enum Flag>(32, 32, QMC::EnumFlags{}).add({
            {   33, Flag::SizeSensitive },
            {   34, Flag::DontClip },
            {   35, Flag::NoAlpha },
            {   36, Flag::Cursor },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
    });
    return QtMocHelpers::metaObjectData<WOutputLayer, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT static const QMetaObject::SuperData qt_meta_extradata_ZN6Waylib6Server12WOutputLayerE[] = {
    QMetaObject::SuperData::link<Waylib::Server::WGlobal::staticMetaObject>(),
    nullptr
};

Q_CONSTINIT const QMetaObject Waylib::Server::WOutputLayer::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server12WOutputLayerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server12WOutputLayerE_t>.data,
    qt_static_metacall,
    qt_meta_extradata_ZN6Waylib6Server12WOutputLayerE,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server12WOutputLayerE_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WOutputLayer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WOutputLayer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->enabledChanged(); break;
        case 1: _t->flagsChanged(); break;
        case 2: _t->colorContentsModeChanged(); break;
        case 3: _t->outputsChanged(); break;
        case 4: _t->inOutputsByHardwareChanged(); break;
        case 5: _t->zChanged(); break;
        case 6: _t->keepLayerChanged(); break;
        case 7: _t->forceChanged(); break;
        case 8: _t->cursorHotSpotChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WOutputLayer::*)()>(_a, &WOutputLayer::enabledChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputLayer::*)()>(_a, &WOutputLayer::flagsChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputLayer::*)()>(_a, &WOutputLayer::colorContentsModeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputLayer::*)()>(_a, &WOutputLayer::outputsChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputLayer::*)()>(_a, &WOutputLayer::inOutputsByHardwareChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputLayer::*)()>(_a, &WOutputLayer::zChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputLayer::*)()>(_a, &WOutputLayer::keepLayerChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputLayer::*)()>(_a, &WOutputLayer::forceChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputLayer::*)()>(_a, &WOutputLayer::cursorHotSpotChanged, 8))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->enabled(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->keepLayer(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->force(); break;
        case 3: *reinterpret_cast<Flags*>(_v) = _t->flags(); break;
        case 4: *reinterpret_cast<WGlobal::ColorContentsMode*>(_v) = _t->colorContentsMode(); break;
        case 5: *reinterpret_cast<QList<WOutputViewport*>*>(_v) = _t->outputs(); break;
        case 6: *reinterpret_cast<QList<WOutputViewport*>*>(_v) = _t->inOutputsByHardware(); break;
        case 7: *reinterpret_cast<int*>(_v) = _t->z(); break;
        case 8: *reinterpret_cast<QPointF*>(_v) = _t->cursorHotSpot(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setKeepLayer(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setForce(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setFlags(*reinterpret_cast<Flags*>(_v)); break;
        case 4: _t->setColorContentsMode(*reinterpret_cast<WGlobal::ColorContentsMode*>(_v)); break;
        case 5: _t->setOutputs(*reinterpret_cast<QList<WOutputViewport*>*>(_v)); break;
        case 7: _t->setZ(*reinterpret_cast<int*>(_v)); break;
        case 8: _t->setCursorHotSpot(*reinterpret_cast<QPointF*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Waylib::Server::WOutputLayer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WOutputLayer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server12WOutputLayerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Waylib::Server::WOutputLayer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WOutputLayer::enabledChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Waylib::Server::WOutputLayer::flagsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Waylib::Server::WOutputLayer::colorContentsModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Waylib::Server::WOutputLayer::outputsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Waylib::Server::WOutputLayer::inOutputsByHardwareChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Waylib::Server::WOutputLayer::zChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Waylib::Server::WOutputLayer::keepLayerChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void Waylib::Server::WOutputLayer::forceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void Waylib::Server::WOutputLayer::cursorHotSpotChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}
QT_WARNING_POP
