/****************************************************************************
** Meta object code from reading C++ file 'woutputitem.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../waylib/src/server/qtquick/woutputitem.h"
#include <wquickoutputlayout.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'woutputitem.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Waylib6Server11WOutputItemE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WOutputItem::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server11WOutputItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WOutputItem",
        "QML.Element",
        "OutputItem",
        "outputChanged",
        "",
        "layoutChanged",
        "devicePixelRatioChanged",
        "seatChanged",
        "cursorDelegateChanged",
        "cursorItemsChanged",
        "geometryChanged",
        "updateCursors",
        "output",
        "WOutput*",
        "layout",
        "WQuickOutputLayout*",
        "devicePixelRatio",
        "cursorDelegate",
        "QQmlComponent*",
        "cursorItems",
        "QList<QQuickItem*>"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'outputChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'layoutChanged'
        QtMocHelpers::SignalData<void()>(5, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'devicePixelRatioChanged'
        QtMocHelpers::SignalData<void()>(6, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'seatChanged'
        QtMocHelpers::SignalData<void()>(7, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cursorDelegateChanged'
        QtMocHelpers::SignalData<void()>(8, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cursorItemsChanged'
        QtMocHelpers::SignalData<void()>(9, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'geometryChanged'
        QtMocHelpers::SignalData<void()>(10, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateCursors'
        QtMocHelpers::SlotData<void()>(11, 4, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'output'
        QtMocHelpers::PropertyData<WOutput*>(12, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'layout'
        QtMocHelpers::PropertyData<WQuickOutputLayout*>(14, 0x80000000 | 15, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'devicePixelRatio'
        QtMocHelpers::PropertyData<qreal>(16, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'cursorDelegate'
        QtMocHelpers::PropertyData<QQmlComponent*>(17, 0x80000000 | 18, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4),
        // property 'cursorItems'
        QtMocHelpers::PropertyData<QList<QQuickItem*>>(19, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 5),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<WOutputItem, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WOutputItem::staticMetaObject = { {
    QMetaObject::SuperData::link<WQuickObserver::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server11WOutputItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server11WOutputItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server11WOutputItemE_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WOutputItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WOutputItem *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->outputChanged(); break;
        case 1: _t->layoutChanged(); break;
        case 2: _t->devicePixelRatioChanged(); break;
        case 3: _t->seatChanged(); break;
        case 4: _t->cursorDelegateChanged(); break;
        case 5: _t->cursorItemsChanged(); break;
        case 6: _t->geometryChanged(); break;
        case 7: _t->d_func()->updateCursors(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WOutputItem::*)()>(_a, &WOutputItem::outputChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputItem::*)()>(_a, &WOutputItem::layoutChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputItem::*)()>(_a, &WOutputItem::devicePixelRatioChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputItem::*)()>(_a, &WOutputItem::seatChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputItem::*)()>(_a, &WOutputItem::cursorDelegateChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputItem::*)()>(_a, &WOutputItem::cursorItemsChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (WOutputItem::*)()>(_a, &WOutputItem::geometryChanged, 6))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QQuickItem*> >(); break;
        case 3:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlComponent* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<WOutput**>(_v) = _t->output(); break;
        case 1: *reinterpret_cast<WQuickOutputLayout**>(_v) = _t->layout(); break;
        case 2: *reinterpret_cast<qreal*>(_v) = _t->devicePixelRatio(); break;
        case 3: *reinterpret_cast<QQmlComponent**>(_v) = _t->cursorDelegate(); break;
        case 4: *reinterpret_cast<QList<QQuickItem*>*>(_v) = _t->cursorItems(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setOutput(*reinterpret_cast<WOutput**>(_v)); break;
        case 1: _t->setLayout(*reinterpret_cast<WQuickOutputLayout**>(_v)); break;
        case 2: _t->setDevicePixelRatio(*reinterpret_cast<qreal*>(_v)); break;
        case 3: _t->setCursorDelegate(*reinterpret_cast<QQmlComponent**>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Waylib::Server::WOutputItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WOutputItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server11WOutputItemE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "WObject"))
        return static_cast< WObject*>(this);
    return WQuickObserver::qt_metacast(_clname);
}

int Waylib::Server::WOutputItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = WQuickObserver::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WOutputItem::outputChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Waylib::Server::WOutputItem::layoutChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Waylib::Server::WOutputItem::devicePixelRatioChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Waylib::Server::WOutputItem::seatChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Waylib::Server::WOutputItem::cursorDelegateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Waylib::Server::WOutputItem::cursorItemsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Waylib::Server::WOutputItem::geometryChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
