/****************************************************************************
** Meta object code from reading C++ file 'multitaskview.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../src/plugins/multitaskview/multitaskview.h"
#include "surface/surfacecontainer.h"
#include "workspace/workspacemodel.h"
#include "output/output.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'multitaskview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13MultitaskviewE_t {};
} // unnamed namespace

template <> constexpr inline auto Multitaskview::qt_create_metaobjectdata<qt_meta_tag_ZN13MultitaskviewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Multitaskview",
        "QML.Element",
        "auto",
        "actionFinished",
        "",
        "activeReasonChanged",
        "partialFactorChanged",
        "aboutToExit",
        "exit",
        "SurfaceWrapper*",
        "surface",
        "immediately",
        "enter",
        "ActiveReason",
        "reason",
        "status",
        "Status",
        "activeReason",
        "partialFactor",
        "Uninitialized",
        "Initialized",
        "Active",
        "Exited",
        "ShortcutKey",
        "Gesture",
        "ZOrder",
        "Background",
        "Overlay",
        "FloatingItem"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'actionFinished'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'activeReasonChanged'
        QtMocHelpers::SignalData<void()>(5, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'partialFactorChanged'
        QtMocHelpers::SignalData<void()>(6, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'aboutToExit'
        QtMocHelpers::SignalData<void()>(7, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'exit'
        QtMocHelpers::SlotData<void(SurfaceWrapper *, bool)>(8, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { QMetaType::Bool, 11 },
        }}),
        // Slot 'exit'
        QtMocHelpers::SlotData<void(SurfaceWrapper *)>(8, 4, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'exit'
        QtMocHelpers::SlotData<void()>(8, 4, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'enter'
        QtMocHelpers::SlotData<void(enum ActiveReason)>(12, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'status'
        QtMocHelpers::PropertyData<enum Status>(15, 0x80000000 | 16, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 0),
        // property 'activeReason'
        QtMocHelpers::PropertyData<enum ActiveReason>(17, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 1),
        // property 'partialFactor'
        QtMocHelpers::PropertyData<qreal>(18, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Final, 2),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Status'
        QtMocHelpers::EnumData<enum Status>(16, 16, QMC::EnumFlags{}).add({
            {   19, Status::Uninitialized },
            {   20, Status::Initialized },
            {   21, Status::Active },
            {   22, Status::Exited },
        }),
        // enum 'ActiveReason'
        QtMocHelpers::EnumData<enum ActiveReason>(13, 13, QMC::EnumFlags{}).add({
            {   23, ActiveReason::ShortcutKey },
            {   24, ActiveReason::Gesture },
        }),
        // enum 'ZOrder'
        QtMocHelpers::EnumData<enum ZOrder>(25, 25, QMC::EnumFlags{}).add({
            {   26, ZOrder::Background },
            {   27, ZOrder::Overlay },
            {   28, ZOrder::FloatingItem },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<Multitaskview, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Multitaskview::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13MultitaskviewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13MultitaskviewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13MultitaskviewE_t>.metaTypes,
    nullptr
} };

void Multitaskview::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Multitaskview *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->actionFinished(); break;
        case 1: _t->activeReasonChanged(); break;
        case 2: _t->partialFactorChanged(); break;
        case 3: _t->aboutToExit(); break;
        case 4: _t->exit((*reinterpret_cast<std::add_pointer_t<SurfaceWrapper*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 5: _t->exit((*reinterpret_cast<std::add_pointer_t<SurfaceWrapper*>>(_a[1]))); break;
        case 6: _t->exit(); break;
        case 7: _t->enter((*reinterpret_cast<std::add_pointer_t<enum ActiveReason>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (Multitaskview::*)()>(_a, &Multitaskview::actionFinished, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (Multitaskview::*)()>(_a, &Multitaskview::activeReasonChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (Multitaskview::*)()>(_a, &Multitaskview::partialFactorChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (Multitaskview::*)()>(_a, &Multitaskview::aboutToExit, 3))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<enum Status*>(_v) = _t->status(); break;
        case 1: *reinterpret_cast<enum ActiveReason*>(_v) = _t->activeReason(); break;
        case 2: *reinterpret_cast<qreal*>(_v) = _t->partialFactor(); break;
        default: break;
        }
    }
}

const QMetaObject *Multitaskview::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Multitaskview::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13MultitaskviewE_t>.strings))
        return static_cast<void*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int Multitaskview::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void Multitaskview::actionFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Multitaskview::activeReasonChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Multitaskview::partialFactorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Multitaskview::aboutToExit()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
namespace {
struct qt_meta_tag_ZN25MultitaskviewSurfaceModelE_t {};
} // unnamed namespace

template <> constexpr inline auto MultitaskviewSurfaceModel::qt_create_metaobjectdata<qt_meta_tag_ZN25MultitaskviewSurfaceModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MultitaskviewSurfaceModel",
        "QML.Element",
        "auto",
        "surfaceListModelChanged",
        "",
        "layoutAreaChanged",
        "modelReadyChanged",
        "rowsChanged",
        "workspaceChanged",
        "contentHeightChanged",
        "outputChanged",
        "countChanged",
        "calcLayout",
        "updateZOrder",
        "prevSameAppIndex",
        "index",
        "nextSameAppIndex",
        "workspace",
        "WorkspaceModel*",
        "layoutArea",
        "QRectF",
        "modelReady",
        "rows",
        "contentHeight",
        "output",
        "Output*",
        "count",
        "SurfaceModelRole",
        "SurfaceWrapperRole",
        "GeometryRole",
        "PaddingRole",
        "ZOrderRole",
        "MinimizedRole",
        "UpIndexRole",
        "DownIndexRole",
        "LeftIndexRole",
        "RightIndexRole"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'surfaceListModelChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'layoutAreaChanged'
        QtMocHelpers::SignalData<void()>(5, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'modelReadyChanged'
        QtMocHelpers::SignalData<void()>(6, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'rowsChanged'
        QtMocHelpers::SignalData<void()>(7, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'workspaceChanged'
        QtMocHelpers::SignalData<void()>(8, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'contentHeightChanged'
        QtMocHelpers::SignalData<void()>(9, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'outputChanged'
        QtMocHelpers::SignalData<void()>(10, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'countChanged'
        QtMocHelpers::SignalData<void()>(11, 4, QMC::AccessPublic, QMetaType::Void),
        // Method 'calcLayout'
        QtMocHelpers::MethodData<void()>(12, 4, QMC::AccessPublic, QMetaType::Void),
        // Method 'updateZOrder'
        QtMocHelpers::MethodData<void()>(13, 4, QMC::AccessPublic, QMetaType::Void),
        // Method 'prevSameAppIndex'
        QtMocHelpers::MethodData<uint(uint)>(14, 4, QMC::AccessPublic, QMetaType::UInt, {{
            { QMetaType::UInt, 15 },
        }}),
        // Method 'nextSameAppIndex'
        QtMocHelpers::MethodData<uint(uint)>(16, 4, QMC::AccessPublic, QMetaType::UInt, {{
            { QMetaType::UInt, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'workspace'
        QtMocHelpers::PropertyData<WorkspaceModel*>(17, 0x80000000 | 18, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final | QMC::Required, 4),
        // property 'layoutArea'
        QtMocHelpers::PropertyData<QRectF>(19, 0x80000000 | 20, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final | QMC::Required, 1),
        // property 'modelReady'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 2),
        // property 'rows'
        QtMocHelpers::PropertyData<uint>(22, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Final, 3),
        // property 'contentHeight'
        QtMocHelpers::PropertyData<qreal>(23, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Final, 5),
        // property 'output'
        QtMocHelpers::PropertyData<Output*>(24, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 6),
        // property 'count'
        QtMocHelpers::PropertyData<uint>(26, QMetaType::UInt, QMC::DefaultPropertyFlags | QMC::Final, 7),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'SurfaceModelRole'
        QtMocHelpers::EnumData<enum SurfaceModelRole>(27, 27, QMC::EnumFlags{}).add({
            {   28, SurfaceModelRole::SurfaceWrapperRole },
            {   29, SurfaceModelRole::GeometryRole },
            {   30, SurfaceModelRole::PaddingRole },
            {   31, SurfaceModelRole::ZOrderRole },
            {   32, SurfaceModelRole::MinimizedRole },
            {   33, SurfaceModelRole::UpIndexRole },
            {   34, SurfaceModelRole::DownIndexRole },
            {   35, SurfaceModelRole::LeftIndexRole },
            {   36, SurfaceModelRole::RightIndexRole },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<MultitaskviewSurfaceModel, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject MultitaskviewSurfaceModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25MultitaskviewSurfaceModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25MultitaskviewSurfaceModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN25MultitaskviewSurfaceModelE_t>.metaTypes,
    nullptr
} };

void MultitaskviewSurfaceModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MultitaskviewSurfaceModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->surfaceListModelChanged(); break;
        case 1: _t->layoutAreaChanged(); break;
        case 2: _t->modelReadyChanged(); break;
        case 3: _t->rowsChanged(); break;
        case 4: _t->workspaceChanged(); break;
        case 5: _t->contentHeightChanged(); break;
        case 6: _t->outputChanged(); break;
        case 7: _t->countChanged(); break;
        case 8: _t->calcLayout(); break;
        case 9: _t->updateZOrder(); break;
        case 10: { uint _r = _t->prevSameAppIndex((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])));
            if (_a[0]) *reinterpret_cast<uint*>(_a[0]) = std::move(_r); }  break;
        case 11: { uint _r = _t->nextSameAppIndex((*reinterpret_cast<std::add_pointer_t<uint>>(_a[1])));
            if (_a[0]) *reinterpret_cast<uint*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MultitaskviewSurfaceModel::*)()>(_a, &MultitaskviewSurfaceModel::surfaceListModelChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MultitaskviewSurfaceModel::*)()>(_a, &MultitaskviewSurfaceModel::layoutAreaChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MultitaskviewSurfaceModel::*)()>(_a, &MultitaskviewSurfaceModel::modelReadyChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (MultitaskviewSurfaceModel::*)()>(_a, &MultitaskviewSurfaceModel::rowsChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (MultitaskviewSurfaceModel::*)()>(_a, &MultitaskviewSurfaceModel::workspaceChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (MultitaskviewSurfaceModel::*)()>(_a, &MultitaskviewSurfaceModel::contentHeightChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (MultitaskviewSurfaceModel::*)()>(_a, &MultitaskviewSurfaceModel::outputChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (MultitaskviewSurfaceModel::*)()>(_a, &MultitaskviewSurfaceModel::countChanged, 7))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<WorkspaceModel**>(_v) = _t->workspace(); break;
        case 1: *reinterpret_cast<QRectF*>(_v) = _t->layoutArea(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->modelReady(); break;
        case 3: *reinterpret_cast<uint*>(_v) = _t->rows(); break;
        case 4: *reinterpret_cast<qreal*>(_v) = _t->contentHeight(); break;
        case 5: *reinterpret_cast<Output**>(_v) = _t->output(); break;
        case 6: *reinterpret_cast<uint*>(_v) = _t->count(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setWorkspace(*reinterpret_cast<WorkspaceModel**>(_v)); break;
        case 1: _t->setLayoutArea(*reinterpret_cast<QRectF*>(_v)); break;
        case 5: _t->setOutput(*reinterpret_cast<Output**>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *MultitaskviewSurfaceModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MultitaskviewSurfaceModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25MultitaskviewSurfaceModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int MultitaskviewSurfaceModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void MultitaskviewSurfaceModel::surfaceListModelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void MultitaskviewSurfaceModel::layoutAreaChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void MultitaskviewSurfaceModel::modelReadyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void MultitaskviewSurfaceModel::rowsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void MultitaskviewSurfaceModel::workspaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void MultitaskviewSurfaceModel::contentHeightChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void MultitaskviewSurfaceModel::outputChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void MultitaskviewSurfaceModel::countChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
