/****************************************************************************
** Meta object code from reading C++ file 'wqmlcreator.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../waylib/src/server/qtquick/wqmlcreator.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wqmlcreator.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Waylib6Server25WAbstractCreatorComponentE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WAbstractCreatorComponent::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server25WAbstractCreatorComponentE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WAbstractCreatorComponent",
        "QML.Element",
        "anonymous",
        "creatorChanged",
        "",
        "creator",
        "WQmlCreator*"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'creatorChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'creator'
        QtMocHelpers::PropertyData<WQmlCreator*>(5, 0x80000000 | 6, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 0),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<WAbstractCreatorComponent, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WAbstractCreatorComponent::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server25WAbstractCreatorComponentE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server25WAbstractCreatorComponentE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server25WAbstractCreatorComponentE_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WAbstractCreatorComponent::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WAbstractCreatorComponent *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->creatorChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WAbstractCreatorComponent::*)()>(_a, &WAbstractCreatorComponent::creatorChanged, 0))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< WQmlCreator* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<WQmlCreator**>(_v) = _t->creator(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setCreator(*reinterpret_cast<WQmlCreator**>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Waylib::Server::WAbstractCreatorComponent::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WAbstractCreatorComponent::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server25WAbstractCreatorComponentE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "WObject"))
        return static_cast< WObject*>(this);
    return QObject::qt_metacast(_clname);
}

int Waylib::Server::WAbstractCreatorComponent::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WAbstractCreatorComponent::creatorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
namespace {
struct qt_meta_tag_ZN6Waylib6Server11WQmlCreatorE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WQmlCreator::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server11WQmlCreatorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WQmlCreator",
        "QML.Element",
        "DynamicCreator",
        "objectAdded",
        "",
        "WAbstractCreatorComponent*",
        "component",
        "object",
        "QJSValue",
        "initialProperties",
        "objectRemoved",
        "countChanged",
        "add",
        "owner",
        "removeByOwner",
        "clear",
        "notify",
        "removeIf",
        "function",
        "getIf",
        "delegate",
        "get",
        "index",
        "getByOwner",
        "count"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'objectAdded'
        QtMocHelpers::SignalData<void(WAbstractCreatorComponent *, QObject *, const QJSValue &)>(3, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { QMetaType::QObjectStar, 7 }, { 0x80000000 | 8, 9 },
        }}),
        // Signal 'objectRemoved'
        QtMocHelpers::SignalData<void(WAbstractCreatorComponent *, QObject *, const QJSValue &)>(10, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 }, { QMetaType::QObjectStar, 7 }, { 0x80000000 | 8, 9 },
        }}),
        // Signal 'countChanged'
        QtMocHelpers::SignalData<void()>(11, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'add'
        QtMocHelpers::SlotData<void(const QJSValue &)>(12, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'add'
        QtMocHelpers::SlotData<void(QObject *, const QJSValue &)>(12, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QObjectStar, 13 }, { 0x80000000 | 8, 9 },
        }}),
        // Slot 'removeByOwner'
        QtMocHelpers::SlotData<bool(QObject *)>(14, 4, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QObjectStar, 13 },
        }}),
        // Slot 'clear'
        QtMocHelpers::SlotData<void(bool)>(15, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Slot 'removeIf'
        QtMocHelpers::SlotData<bool(QJSValue)>(17, 4, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 8, 18 },
        }}),
        // Slot 'getIf'
        QtMocHelpers::SlotData<QObject *(QJSValue) const>(19, 4, QMC::AccessPublic, QMetaType::QObjectStar, {{
            { 0x80000000 | 8, 18 },
        }}),
        // Slot 'getIf'
        QtMocHelpers::SlotData<QObject *(WAbstractCreatorComponent *, QJSValue) const>(19, 4, QMC::AccessPublic, QMetaType::QObjectStar, {{
            { 0x80000000 | 5, 20 }, { 0x80000000 | 8, 18 },
        }}),
        // Slot 'get'
        QtMocHelpers::SlotData<QObject *(int) const>(21, 4, QMC::AccessPublic, QMetaType::QObjectStar, {{
            { QMetaType::Int, 22 },
        }}),
        // Slot 'get'
        QtMocHelpers::SlotData<QObject *(WAbstractCreatorComponent *, int) const>(21, 4, QMC::AccessPublic, QMetaType::QObjectStar, {{
            { 0x80000000 | 5, 20 }, { QMetaType::Int, 22 },
        }}),
        // Slot 'getByOwner'
        QtMocHelpers::SlotData<QObject *(QObject *) const>(23, 4, QMC::AccessPublic, QMetaType::QObjectStar, {{
            { QMetaType::QObjectStar, 13 },
        }}),
        // Slot 'getByOwner'
        QtMocHelpers::SlotData<QObject *(WAbstractCreatorComponent *, QObject *) const>(23, 4, QMC::AccessPublic, QMetaType::QObjectStar, {{
            { 0x80000000 | 5, 20 }, { QMetaType::QObjectStar, 13 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'count'
        QtMocHelpers::PropertyData<int>(24, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Final, 2),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<WQmlCreator, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WQmlCreator::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server11WQmlCreatorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server11WQmlCreatorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server11WQmlCreatorE_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WQmlCreator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WQmlCreator *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->objectAdded((*reinterpret_cast<std::add_pointer_t<WAbstractCreatorComponent*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[3]))); break;
        case 1: _t->objectRemoved((*reinterpret_cast<std::add_pointer_t<WAbstractCreatorComponent*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[3]))); break;
        case 2: _t->countChanged(); break;
        case 3: _t->add((*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[1]))); break;
        case 4: _t->add((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[2]))); break;
        case 5: { bool _r = _t->removeByOwner((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 6: _t->clear((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: { bool _r = _t->removeIf((*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 8: { QObject* _r = _t->getIf((*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = std::move(_r); }  break;
        case 9: { QObject* _r = _t->getIf((*reinterpret_cast<std::add_pointer_t<WAbstractCreatorComponent*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = std::move(_r); }  break;
        case 10: { QObject* _r = _t->get((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = std::move(_r); }  break;
        case 11: { QObject* _r = _t->get((*reinterpret_cast<std::add_pointer_t<WAbstractCreatorComponent*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = std::move(_r); }  break;
        case 12: { QObject* _r = _t->getByOwner((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = std::move(_r); }  break;
        case 13: { QObject* _r = _t->getByOwner((*reinterpret_cast<std::add_pointer_t<WAbstractCreatorComponent*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QJSValue >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< WAbstractCreatorComponent* >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QJSValue >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< WAbstractCreatorComponent* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QJSValue >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QJSValue >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QJSValue >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QJSValue >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QJSValue >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< WAbstractCreatorComponent* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< WAbstractCreatorComponent* >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< WAbstractCreatorComponent* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WQmlCreator::*)(WAbstractCreatorComponent * , QObject * , const QJSValue & )>(_a, &WQmlCreator::objectAdded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WQmlCreator::*)(WAbstractCreatorComponent * , QObject * , const QJSValue & )>(_a, &WQmlCreator::objectRemoved, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WQmlCreator::*)()>(_a, &WQmlCreator::countChanged, 2))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->count(); break;
        default: break;
        }
    }
}

const QMetaObject *Waylib::Server::WQmlCreator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WQmlCreator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server11WQmlCreatorE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "WObject"))
        return static_cast< WObject*>(this);
    return QObject::qt_metacast(_clname);
}

int Waylib::Server::WQmlCreator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WQmlCreator::objectAdded(WAbstractCreatorComponent * _t1, QObject * _t2, const QJSValue & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void Waylib::Server::WQmlCreator::objectRemoved(WAbstractCreatorComponent * _t1, QObject * _t2, const QJSValue & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}

// SIGNAL 2
void Waylib::Server::WQmlCreator::countChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
