/****************************************************************************
** Meta object code from reading C++ file 'wxdgdecorationmanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../waylib/src/server/protocols/wxdgdecorationmanager.h"
#include "private/wserver_p.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wxdgdecorationmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Waylib6Server21WXdgDecorationManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WXdgDecorationManager::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server21WXdgDecorationManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WXdgDecorationManager",
        "QML.Element",
        "XdgDecorationManager",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Can't create in qml",
        "surfaceModeChanged",
        "",
        "Waylib::Server::WSurface*",
        "surface",
        "DecorationMode",
        "mode",
        "preferredModeChanged",
        "setModeBySurface",
        "modeBySurface",
        "preferredMode",
        "Undefined",
        "None",
        "Client",
        "Server"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'surfaceModeChanged'
        QtMocHelpers::SignalData<void(Waylib::Server::WSurface *, enum DecorationMode)>(7, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { 0x80000000 | 11, 12 },
        }}),
        // Signal 'preferredModeChanged'
        QtMocHelpers::SignalData<void(enum DecorationMode)>(13, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Method 'setModeBySurface'
        QtMocHelpers::MethodData<void(Waylib::Server::WSurface *, enum DecorationMode)>(14, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { 0x80000000 | 11, 12 },
        }}),
        // Method 'modeBySurface'
        QtMocHelpers::MethodData<enum DecorationMode(Waylib::Server::WSurface *) const>(15, 8, QMC::AccessPublic, 0x80000000 | 11, {{
            { 0x80000000 | 9, 10 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'preferredMode'
        QtMocHelpers::PropertyData<enum DecorationMode>(16, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'DecorationMode'
        QtMocHelpers::EnumData<enum DecorationMode>(11, 11, QMC::EnumFlags{}).add({
            {   17, DecorationMode::Undefined },
            {   18, DecorationMode::None },
            {   19, DecorationMode::Client },
            {   20, DecorationMode::Server },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<WXdgDecorationManager, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WXdgDecorationManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server21WXdgDecorationManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server21WXdgDecorationManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server21WXdgDecorationManagerE_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WXdgDecorationManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WXdgDecorationManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->surfaceModeChanged((*reinterpret_cast<std::add_pointer_t<Waylib::Server::WSurface*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<enum DecorationMode>>(_a[2]))); break;
        case 1: _t->preferredModeChanged((*reinterpret_cast<std::add_pointer_t<enum DecorationMode>>(_a[1]))); break;
        case 2: _t->setModeBySurface((*reinterpret_cast<std::add_pointer_t<Waylib::Server::WSurface*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<enum DecorationMode>>(_a[2]))); break;
        case 3: { enum DecorationMode _r = _t->modeBySurface((*reinterpret_cast<std::add_pointer_t<Waylib::Server::WSurface*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<enum DecorationMode*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WXdgDecorationManager::*)(Waylib::Server::WSurface * , DecorationMode )>(_a, &WXdgDecorationManager::surfaceModeChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WXdgDecorationManager::*)(DecorationMode )>(_a, &WXdgDecorationManager::preferredModeChanged, 1))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<enum DecorationMode*>(_v) = _t->preferredMode(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setPreferredMode(*reinterpret_cast<enum DecorationMode*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Waylib::Server::WXdgDecorationManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WXdgDecorationManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server21WXdgDecorationManagerE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "WObject"))
        return static_cast< WObject*>(this);
    if (!strcmp(_clname, "WServerInterface"))
        return static_cast< WServerInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int Waylib::Server::WXdgDecorationManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WXdgDecorationManager::surfaceModeChanged(Waylib::Server::WSurface * _t1, DecorationMode _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void Waylib::Server::WXdgDecorationManager::preferredModeChanged(DecorationMode _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
