/****************************************************************************
** Meta object code from reading C++ file 'wsurfaceitem.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../waylib/src/server/qtquick/wsurfaceitem.h"
#include "woutput.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wsurfaceitem.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Waylib6Server19WSurfaceItemContentE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WSurfaceItemContent::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server19WSurfaceItemContentE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WSurfaceItemContent",
        "QML.Element",
        "SurfaceItemContent",
        "surfaceChanged",
        "",
        "cacheLastBufferChanged",
        "liveChanged",
        "bufferOffsetChanged",
        "ignoreBufferOffsetChanged",
        "bufferSourceRectChanged",
        "devicePixelRatioChanged",
        "alphaModifierChanged",
        "invalidateSceneGraph",
        "implicitWidthChanged",
        "implicitHeightChanged",
        "surface",
        "WSurface*",
        "cacheLastBuffer",
        "live",
        "implicitWidth",
        "implicitHeight",
        "bufferOffset",
        "QPoint",
        "ignoreBufferOffset",
        "bufferSourceRect",
        "QRectF",
        "devicePixelRatio",
        "alphaModifier"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'surfaceChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'cacheLastBufferChanged'
        QtMocHelpers::SignalData<void()>(5, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'liveChanged'
        QtMocHelpers::SignalData<void()>(6, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'bufferOffsetChanged'
        QtMocHelpers::SignalData<void()>(7, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ignoreBufferOffsetChanged'
        QtMocHelpers::SignalData<void()>(8, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'bufferSourceRectChanged'
        QtMocHelpers::SignalData<void()>(9, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'devicePixelRatioChanged'
        QtMocHelpers::SignalData<void()>(10, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'alphaModifierChanged'
        QtMocHelpers::SignalData<void()>(11, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'invalidateSceneGraph'
        QtMocHelpers::SlotData<void()>(12, 4, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'surface'
        QtMocHelpers::PropertyData<WSurface*>(15, 0x80000000 | 16, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 0),
        // property 'cacheLastBuffer'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 1),
        // property 'live'
        QtMocHelpers::PropertyData<bool>(18, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 2),
        // property 'implicitWidth'
        QtMocHelpers::PropertyData<qreal>(19, QMetaType::QReal, QMC::DefaultPropertyFlags, 0x70000000 | 13),
        // property 'implicitHeight'
        QtMocHelpers::PropertyData<qreal>(20, QMetaType::QReal, QMC::DefaultPropertyFlags, 0x70000000 | 14),
        // property 'bufferOffset'
        QtMocHelpers::PropertyData<QPoint>(21, 0x80000000 | 22, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 3),
        // property 'ignoreBufferOffset'
        QtMocHelpers::PropertyData<bool>(23, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 4),
        // property 'bufferSourceRect'
        QtMocHelpers::PropertyData<QRectF>(24, 0x80000000 | 25, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 5),
        // property 'devicePixelRatio'
        QtMocHelpers::PropertyData<qreal>(26, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Final, 6),
        // property 'alphaModifier'
        QtMocHelpers::PropertyData<qreal>(27, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Final, 7),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<WSurfaceItemContent, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WSurfaceItemContent::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server19WSurfaceItemContentE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server19WSurfaceItemContentE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server19WSurfaceItemContentE_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WSurfaceItemContent::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WSurfaceItemContent *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->surfaceChanged(); break;
        case 1: _t->cacheLastBufferChanged(); break;
        case 2: _t->liveChanged(); break;
        case 3: _t->bufferOffsetChanged(); break;
        case 4: _t->ignoreBufferOffsetChanged(); break;
        case 5: _t->bufferSourceRectChanged(); break;
        case 6: _t->devicePixelRatioChanged(); break;
        case 7: _t->alphaModifierChanged(); break;
        case 8: _t->invalidateSceneGraph(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WSurfaceItemContent::*)()>(_a, &WSurfaceItemContent::surfaceChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSurfaceItemContent::*)()>(_a, &WSurfaceItemContent::cacheLastBufferChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSurfaceItemContent::*)()>(_a, &WSurfaceItemContent::liveChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSurfaceItemContent::*)()>(_a, &WSurfaceItemContent::bufferOffsetChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSurfaceItemContent::*)()>(_a, &WSurfaceItemContent::ignoreBufferOffsetChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSurfaceItemContent::*)()>(_a, &WSurfaceItemContent::bufferSourceRectChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSurfaceItemContent::*)()>(_a, &WSurfaceItemContent::devicePixelRatioChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSurfaceItemContent::*)()>(_a, &WSurfaceItemContent::alphaModifierChanged, 7))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< WSurface* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<WSurface**>(_v) = _t->surface(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->cacheLastBuffer(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->live(); break;
        case 3: *reinterpret_cast<qreal*>(_v) = _t->implicitWidth(); break;
        case 4: *reinterpret_cast<qreal*>(_v) = _t->implicitHeight(); break;
        case 5: *reinterpret_cast<QPoint*>(_v) = _t->bufferOffset(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->ignoreBufferOffset(); break;
        case 7: *reinterpret_cast<QRectF*>(_v) = _t->bufferSourceRect(); break;
        case 8: *reinterpret_cast<qreal*>(_v) = _t->devicePixelRatio(); break;
        case 9: *reinterpret_cast<qreal*>(_v) = _t->alphaModifier(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSurface(*reinterpret_cast<WSurface**>(_v)); break;
        case 1: _t->setCacheLastBuffer(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setLive(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->setIgnoreBufferOffset(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Waylib::Server::WSurfaceItemContent::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WSurfaceItemContent::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server19WSurfaceItemContentE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "WTextureProviderProvider"))
        return static_cast< WTextureProviderProvider*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int Waylib::Server::WSurfaceItemContent::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WSurfaceItemContent::surfaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Waylib::Server::WSurfaceItemContent::cacheLastBufferChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void Waylib::Server::WSurfaceItemContent::liveChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Waylib::Server::WSurfaceItemContent::bufferOffsetChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Waylib::Server::WSurfaceItemContent::ignoreBufferOffsetChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Waylib::Server::WSurfaceItemContent::bufferSourceRectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Waylib::Server::WSurfaceItemContent::devicePixelRatioChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void Waylib::Server::WSurfaceItemContent::alphaModifierChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
namespace CheckNotifySignalValidity_ZN6Waylib6Server19WSurfaceItemContentE {
template<typename T> using has_nullary_implicitWidthChanged = decltype(std::declval<T>().implicitWidthChanged());
template<typename T> using has_unary_implicitWidthChanged = decltype(std::declval<T>().implicitWidthChanged(std::declval<qreal>()));
static_assert(qxp::is_detected_v<has_nullary_implicitWidthChanged, Waylib::Server::WSurfaceItemContent> || qxp::is_detected_v<has_unary_implicitWidthChanged, Waylib::Server::WSurfaceItemContent>,
              "NOTIFY signal implicitWidthChanged does not exist in class (or is private in its parent)");
template<typename T> using has_nullary_implicitHeightChanged = decltype(std::declval<T>().implicitHeightChanged());
template<typename T> using has_unary_implicitHeightChanged = decltype(std::declval<T>().implicitHeightChanged(std::declval<qreal>()));
static_assert(qxp::is_detected_v<has_nullary_implicitHeightChanged, Waylib::Server::WSurfaceItemContent> || qxp::is_detected_v<has_unary_implicitHeightChanged, Waylib::Server::WSurfaceItemContent>,
              "NOTIFY signal implicitHeightChanged does not exist in class (or is private in its parent)");
}
namespace {
struct qt_meta_tag_ZN6Waylib6Server12WSurfaceItemE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WSurfaceItem::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server12WSurfaceItemE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WSurfaceItem",
        "QML.Element",
        "SurfaceItem",
        "surfaceChanged",
        "",
        "subsurfaceAdded",
        "Waylib::Server::WSurfaceItem*",
        "item",
        "subsurfaceRemoved",
        "resizeModeChanged",
        "effectiveVisibleChanged",
        "eventItemChanged",
        "flagsChanged",
        "topPaddingChanged",
        "bottomPaddingChanged",
        "leftPaddingChanged",
        "rightPaddingChanged",
        "surfaceSizeRatioChanged",
        "bufferScaleChanged",
        "contentItemChanged",
        "delegateChanged",
        "shellSurfaceChanged",
        "boundingRectChanged",
        "subsurfacesVisibleChanged",
        "readyChanged",
        "onSurfaceCommit",
        "resize",
        "ResizeMode",
        "mode",
        "implicitWidthChanged",
        "implicitHeightChanged",
        "surface",
        "WSurface*",
        "shellSurface",
        "WToplevelSurface*",
        "contentItem",
        "QQuickItem*",
        "eventItem",
        "resizeMode",
        "effectiveVisible",
        "flags",
        "Flags",
        "topPadding",
        "bottomPadding",
        "leftPadding",
        "rightPadding",
        "implicitWidth",
        "implicitHeight",
        "surfaceSizeRatio",
        "bufferScale",
        "delegate",
        "QQmlComponent*",
        "boundingRect",
        "QRectF",
        "subsurfacesVisible",
        "ready",
        "SizeFromSurface",
        "SizeToSurface",
        "ManualResize",
        "Flag",
        "DontCacheLastBuffer",
        "RejectEvent",
        "NonLive",
        "DelegateForSubsurface",
        "ZOrder",
        "BelowSubsurface",
        "ContentItem",
        "EventItem",
        "AboveSubsurface"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'surfaceChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'subsurfaceAdded'
        QtMocHelpers::SignalData<void(Waylib::Server::WSurfaceItem *)>(5, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'subsurfaceRemoved'
        QtMocHelpers::SignalData<void(Waylib::Server::WSurfaceItem *)>(8, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'resizeModeChanged'
        QtMocHelpers::SignalData<void()>(9, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'effectiveVisibleChanged'
        QtMocHelpers::SignalData<void()>(10, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'eventItemChanged'
        QtMocHelpers::SignalData<void()>(11, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'flagsChanged'
        QtMocHelpers::SignalData<void()>(12, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'topPaddingChanged'
        QtMocHelpers::SignalData<void()>(13, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'bottomPaddingChanged'
        QtMocHelpers::SignalData<void()>(14, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'leftPaddingChanged'
        QtMocHelpers::SignalData<void()>(15, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'rightPaddingChanged'
        QtMocHelpers::SignalData<void()>(16, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'surfaceSizeRatioChanged'
        QtMocHelpers::SignalData<void()>(17, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'bufferScaleChanged'
        QtMocHelpers::SignalData<void()>(18, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'contentItemChanged'
        QtMocHelpers::SignalData<void()>(19, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'delegateChanged'
        QtMocHelpers::SignalData<void()>(20, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'shellSurfaceChanged'
        QtMocHelpers::SignalData<void()>(21, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'boundingRectChanged'
        QtMocHelpers::SignalData<void()>(22, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'subsurfacesVisibleChanged'
        QtMocHelpers::SignalData<void()>(23, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'readyChanged'
        QtMocHelpers::SignalData<void()>(24, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onSurfaceCommit'
        QtMocHelpers::SlotData<void()>(25, 4, QMC::AccessProtected, QMetaType::Void),
        // Method 'resize'
        QtMocHelpers::MethodData<void(enum ResizeMode)>(26, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'surface'
        QtMocHelpers::PropertyData<WSurface*>(31, 0x80000000 | 32, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'shellSurface'
        QtMocHelpers::PropertyData<WToplevelSurface*>(33, 0x80000000 | 34, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 15),
        // property 'contentItem'
        QtMocHelpers::PropertyData<QQuickItem*>(35, 0x80000000 | 36, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 13),
        // property 'eventItem'
        QtMocHelpers::PropertyData<QQuickItem*>(37, 0x80000000 | 36, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 5),
        // property 'resizeMode'
        QtMocHelpers::PropertyData<enum ResizeMode>(38, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 3),
        // property 'effectiveVisible'
        QtMocHelpers::PropertyData<bool>(39, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 4),
        // property 'flags'
        QtMocHelpers::PropertyData<Flags>(40, 0x80000000 | 41, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 6),
        // property 'topPadding'
        QtMocHelpers::PropertyData<qreal>(42, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 7),
        // property 'bottomPadding'
        QtMocHelpers::PropertyData<qreal>(43, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 8),
        // property 'leftPadding'
        QtMocHelpers::PropertyData<qreal>(44, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 9),
        // property 'rightPadding'
        QtMocHelpers::PropertyData<qreal>(45, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 10),
        // property 'implicitWidth'
        QtMocHelpers::PropertyData<qreal>(46, QMetaType::QReal, QMC::DefaultPropertyFlags, 0x70000000 | 29),
        // property 'implicitHeight'
        QtMocHelpers::PropertyData<qreal>(47, QMetaType::QReal, QMC::DefaultPropertyFlags, 0x70000000 | 30),
        // property 'surfaceSizeRatio'
        QtMocHelpers::PropertyData<qreal>(48, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11),
        // property 'bufferScale'
        QtMocHelpers::PropertyData<qreal>(49, QMetaType::QReal, QMC::DefaultPropertyFlags, 12),
        // property 'delegate'
        QtMocHelpers::PropertyData<QQmlComponent*>(50, 0x80000000 | 51, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 14),
        // property 'boundingRect'
        QtMocHelpers::PropertyData<QRectF>(52, 0x80000000 | 53, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 16),
        // property 'subsurfacesVisible'
        QtMocHelpers::PropertyData<bool>(54, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 17),
        // property 'ready'
        QtMocHelpers::PropertyData<bool>(55, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 18),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ResizeMode'
        QtMocHelpers::EnumData<enum ResizeMode>(27, 27, QMC::EnumFlags{}).add({
            {   56, ResizeMode::SizeFromSurface },
            {   57, ResizeMode::SizeToSurface },
            {   58, ResizeMode::ManualResize },
        }),
        // enum 'Flag'
        QtMocHelpers::EnumData<enum Flag>(59, 59, QMC::EnumFlags{}).add({
            {   60, Flag::DontCacheLastBuffer },
            {   61, Flag::RejectEvent },
            {   62, Flag::NonLive },
            {   63, Flag::DelegateForSubsurface },
        }),
        // enum 'ZOrder'
        QtMocHelpers::EnumData<enum ZOrder>(64, 64, QMC::EnumIsScoped).add({
            {   65, ZOrder::BelowSubsurface },
            {   66, ZOrder::ContentItem },
            {   67, ZOrder::EventItem },
            {   68, ZOrder::AboveSubsurface },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<WSurfaceItem, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WSurfaceItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QQuickItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server12WSurfaceItemE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server12WSurfaceItemE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server12WSurfaceItemE_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WSurfaceItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WSurfaceItem *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->surfaceChanged(); break;
        case 1: _t->subsurfaceAdded((*reinterpret_cast<std::add_pointer_t<Waylib::Server::WSurfaceItem*>>(_a[1]))); break;
        case 2: _t->subsurfaceRemoved((*reinterpret_cast<std::add_pointer_t<Waylib::Server::WSurfaceItem*>>(_a[1]))); break;
        case 3: _t->resizeModeChanged(); break;
        case 4: _t->effectiveVisibleChanged(); break;
        case 5: _t->eventItemChanged(); break;
        case 6: _t->flagsChanged(); break;
        case 7: _t->topPaddingChanged(); break;
        case 8: _t->bottomPaddingChanged(); break;
        case 9: _t->leftPaddingChanged(); break;
        case 10: _t->rightPaddingChanged(); break;
        case 11: _t->surfaceSizeRatioChanged(); break;
        case 12: _t->bufferScaleChanged(); break;
        case 13: _t->contentItemChanged(); break;
        case 14: _t->delegateChanged(); break;
        case 15: _t->shellSurfaceChanged(); break;
        case 16: _t->boundingRectChanged(); break;
        case 17: _t->subsurfacesVisibleChanged(); break;
        case 18: _t->readyChanged(); break;
        case 19: _t->onSurfaceCommit(); break;
        case 20: _t->resize((*reinterpret_cast<std::add_pointer_t<enum ResizeMode>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Waylib::Server::WSurfaceItem* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Waylib::Server::WSurfaceItem* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WSurfaceItem::*)()>(_a, &WSurfaceItem::surfaceChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSurfaceItem::*)(Waylib::Server::WSurfaceItem * )>(_a, &WSurfaceItem::subsurfaceAdded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSurfaceItem::*)(Waylib::Server::WSurfaceItem * )>(_a, &WSurfaceItem::subsurfaceRemoved, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSurfaceItem::*)()>(_a, &WSurfaceItem::resizeModeChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSurfaceItem::*)()>(_a, &WSurfaceItem::effectiveVisibleChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSurfaceItem::*)()>(_a, &WSurfaceItem::eventItemChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSurfaceItem::*)()>(_a, &WSurfaceItem::flagsChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSurfaceItem::*)()>(_a, &WSurfaceItem::topPaddingChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSurfaceItem::*)()>(_a, &WSurfaceItem::bottomPaddingChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSurfaceItem::*)()>(_a, &WSurfaceItem::leftPaddingChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSurfaceItem::*)()>(_a, &WSurfaceItem::rightPaddingChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSurfaceItem::*)()>(_a, &WSurfaceItem::surfaceSizeRatioChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSurfaceItem::*)()>(_a, &WSurfaceItem::bufferScaleChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSurfaceItem::*)()>(_a, &WSurfaceItem::contentItemChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSurfaceItem::*)()>(_a, &WSurfaceItem::delegateChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSurfaceItem::*)()>(_a, &WSurfaceItem::shellSurfaceChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSurfaceItem::*)()>(_a, &WSurfaceItem::boundingRectChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSurfaceItem::*)()>(_a, &WSurfaceItem::subsurfacesVisibleChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSurfaceItem::*)()>(_a, &WSurfaceItem::readyChanged, 18))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 15:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlComponent* >(); break;
        case 3:
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQuickItem* >(); break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< WSurface* >(); break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< WToplevelSurface* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<WSurface**>(_v) = _t->surface(); break;
        case 1: *reinterpret_cast<WToplevelSurface**>(_v) = _t->shellSurface(); break;
        case 2: *reinterpret_cast<QQuickItem**>(_v) = _t->contentItem(); break;
        case 3: *reinterpret_cast<QQuickItem**>(_v) = _t->eventItem(); break;
        case 4: *reinterpret_cast<enum ResizeMode*>(_v) = _t->resizeMode(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->effectiveVisible(); break;
        case 6: *reinterpret_cast<Flags*>(_v) = _t->flags(); break;
        case 7: *reinterpret_cast<qreal*>(_v) = _t->topPadding(); break;
        case 8: *reinterpret_cast<qreal*>(_v) = _t->bottomPadding(); break;
        case 9: *reinterpret_cast<qreal*>(_v) = _t->leftPadding(); break;
        case 10: *reinterpret_cast<qreal*>(_v) = _t->rightPadding(); break;
        case 11: *reinterpret_cast<qreal*>(_v) = _t->implicitWidth(); break;
        case 12: *reinterpret_cast<qreal*>(_v) = _t->implicitHeight(); break;
        case 13: *reinterpret_cast<qreal*>(_v) = _t->surfaceSizeRatio(); break;
        case 14: *reinterpret_cast<qreal*>(_v) = _t->bufferScale(); break;
        case 15: *reinterpret_cast<QQmlComponent**>(_v) = _t->delegate(); break;
        case 16: *reinterpret_cast<QRectF*>(_v) = _t->boundingRect(); break;
        case 17: *reinterpret_cast<bool*>(_v) = _t->subsurfacesVisible(); break;
        case 18: *reinterpret_cast<bool*>(_v) = _t->isReady(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSurface(*reinterpret_cast<WSurface**>(_v)); break;
        case 1: _t->setShellSurface(*reinterpret_cast<WToplevelSurface**>(_v)); break;
        case 4: _t->setResizeMode(*reinterpret_cast<enum ResizeMode*>(_v)); break;
        case 6: _t->setFlags(*reinterpret_cast<Flags*>(_v)); break;
        case 7: _t->setTopPadding(*reinterpret_cast<qreal*>(_v)); break;
        case 8: _t->setBottomPadding(*reinterpret_cast<qreal*>(_v)); break;
        case 9: _t->setLeftPadding(*reinterpret_cast<qreal*>(_v)); break;
        case 10: _t->setRightPadding(*reinterpret_cast<qreal*>(_v)); break;
        case 13: _t->setSurfaceSizeRatio(*reinterpret_cast<qreal*>(_v)); break;
        case 15: _t->setDelegate(*reinterpret_cast<QQmlComponent**>(_v)); break;
        case 17: _t->setSubsurfacesVisible(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *Waylib::Server::WSurfaceItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WSurfaceItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server12WSurfaceItemE_t>.strings))
        return static_cast<void*>(this);
    return QQuickItem::qt_metacast(_clname);
}

int Waylib::Server::WSurfaceItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QQuickItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WSurfaceItem::surfaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Waylib::Server::WSurfaceItem::subsurfaceAdded(Waylib::Server::WSurfaceItem * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Waylib::Server::WSurfaceItem::subsurfaceRemoved(Waylib::Server::WSurfaceItem * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Waylib::Server::WSurfaceItem::resizeModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Waylib::Server::WSurfaceItem::effectiveVisibleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Waylib::Server::WSurfaceItem::eventItemChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Waylib::Server::WSurfaceItem::flagsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void Waylib::Server::WSurfaceItem::topPaddingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void Waylib::Server::WSurfaceItem::bottomPaddingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void Waylib::Server::WSurfaceItem::leftPaddingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void Waylib::Server::WSurfaceItem::rightPaddingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void Waylib::Server::WSurfaceItem::surfaceSizeRatioChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void Waylib::Server::WSurfaceItem::bufferScaleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void Waylib::Server::WSurfaceItem::contentItemChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void Waylib::Server::WSurfaceItem::delegateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void Waylib::Server::WSurfaceItem::shellSurfaceChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void Waylib::Server::WSurfaceItem::boundingRectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void Waylib::Server::WSurfaceItem::subsurfacesVisibleChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void Waylib::Server::WSurfaceItem::readyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}
namespace CheckNotifySignalValidity_ZN6Waylib6Server12WSurfaceItemE {
template<typename T> using has_nullary_implicitWidthChanged = decltype(std::declval<T>().implicitWidthChanged());
template<typename T> using has_unary_implicitWidthChanged = decltype(std::declval<T>().implicitWidthChanged(std::declval<qreal>()));
static_assert(qxp::is_detected_v<has_nullary_implicitWidthChanged, Waylib::Server::WSurfaceItem> || qxp::is_detected_v<has_unary_implicitWidthChanged, Waylib::Server::WSurfaceItem>,
              "NOTIFY signal implicitWidthChanged does not exist in class (or is private in its parent)");
template<typename T> using has_nullary_implicitHeightChanged = decltype(std::declval<T>().implicitHeightChanged());
template<typename T> using has_unary_implicitHeightChanged = decltype(std::declval<T>().implicitHeightChanged(std::declval<qreal>()));
static_assert(qxp::is_detected_v<has_nullary_implicitHeightChanged, Waylib::Server::WSurfaceItem> || qxp::is_detected_v<has_unary_implicitHeightChanged, Waylib::Server::WSurfaceItem>,
              "NOTIFY signal implicitHeightChanged does not exist in class (or is private in its parent)");
}
QT_WARNING_POP
