/* Generated by wayland-scanner 1.26.0 */

/*
 * SPDX-FileCopyrightText: 2024 UnionTech Software Technology Co., Ltd.
 * SPDX-License-Identifier: MIT
 */

#include <stdbool.h>
#include <stdlib.h>
#include <stdint.h>
#include "wayland-util.h"

#ifndef __has_attribute
# define __has_attribute(x) 0  /* Compatibility with non-clang compilers. */
#endif

#if (__has_attribute(visibility) || defined(__GNUC__) && __GNUC__ >= 4)
#define WL_PRIVATE __attribute__ ((visibility("hidden")))
#else
#define WL_PRIVATE
#endif

extern const struct wl_interface treeland_dde_active_v1_interface;
extern const struct wl_interface treeland_dde_shell_surface_v1_interface;
extern const struct wl_interface treeland_lockscreen_v1_interface;
extern const struct wl_interface treeland_multitaskview_v1_interface;
extern const struct wl_interface treeland_window_overlap_checker_interface;
extern const struct wl_interface treeland_window_picker_v1_interface;
extern const struct wl_interface wl_callback_interface;
extern const struct wl_interface wl_output_interface;
extern const struct wl_interface wl_seat_interface;
extern const struct wl_interface wl_surface_interface;

static const struct wl_interface *treeland_dde_shell_v1_types[] = {
	NULL,
	NULL,
	&treeland_window_overlap_checker_interface,
	&treeland_dde_shell_surface_v1_interface,
	&wl_surface_interface,
	&treeland_dde_active_v1_interface,
	&wl_seat_interface,
	&treeland_multitaskview_v1_interface,
	&treeland_window_picker_v1_interface,
	&treeland_lockscreen_v1_interface,
	&wl_callback_interface,
	NULL,
	&wl_surface_interface,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	&wl_output_interface,
};

static const struct wl_message treeland_dde_shell_manager_v1_requests[] = {
	{ "get_window_overlap_checker", "n", treeland_dde_shell_v1_types + 2 },
	{ "get_shell_surface", "no", treeland_dde_shell_v1_types + 3 },
	{ "get_treeland_dde_active", "no", treeland_dde_shell_v1_types + 5 },
	{ "get_treeland_multitaskview", "n", treeland_dde_shell_v1_types + 7 },
	{ "get_treeland_window_picker", "n", treeland_dde_shell_v1_types + 8 },
	{ "get_treeland_lockscreen", "n", treeland_dde_shell_v1_types + 9 },
	{ "set_xwindow_position_relative", "nuoff", treeland_dde_shell_v1_types + 10 },
	{ "destroy", "2", treeland_dde_shell_v1_types + 0 },
};

WL_PRIVATE const struct wl_interface treeland_dde_shell_manager_v1_interface = {
	"treeland_dde_shell_manager_v1", 2,
	8, treeland_dde_shell_manager_v1_requests,
	0, NULL,
};

static const struct wl_message treeland_window_overlap_checker_requests[] = {
	{ "update", "iiuo", treeland_dde_shell_v1_types + 15 },
	{ "destroy", "", treeland_dde_shell_v1_types + 0 },
};

static const struct wl_message treeland_window_overlap_checker_events[] = {
	{ "enter", "", treeland_dde_shell_v1_types + 0 },
	{ "leave", "", treeland_dde_shell_v1_types + 0 },
};

WL_PRIVATE const struct wl_interface treeland_window_overlap_checker_interface = {
	"treeland_window_overlap_checker", 1,
	2, treeland_window_overlap_checker_requests,
	2, treeland_window_overlap_checker_events,
};

static const struct wl_message treeland_dde_shell_surface_v1_requests[] = {
	{ "destroy", "", treeland_dde_shell_v1_types + 0 },
	{ "set_surface_position", "ii", treeland_dde_shell_v1_types + 0 },
	{ "set_role", "u", treeland_dde_shell_v1_types + 0 },
	{ "set_auto_placement", "u", treeland_dde_shell_v1_types + 0 },
	{ "set_skip_switcher", "u", treeland_dde_shell_v1_types + 0 },
	{ "set_skip_dock_preview", "u", treeland_dde_shell_v1_types + 0 },
	{ "set_skip_muti_task_view", "u", treeland_dde_shell_v1_types + 0 },
	{ "set_accept_keyboard_focus", "u", treeland_dde_shell_v1_types + 0 },
};

WL_PRIVATE const struct wl_interface treeland_dde_shell_surface_v1_interface = {
	"treeland_dde_shell_surface_v1", 1,
	8, treeland_dde_shell_surface_v1_requests,
	0, NULL,
};

static const struct wl_message treeland_dde_active_v1_requests[] = {
	{ "destroy", "", treeland_dde_shell_v1_types + 0 },
};

static const struct wl_message treeland_dde_active_v1_events[] = {
	{ "active_in", "u", treeland_dde_shell_v1_types + 0 },
	{ "active_out", "u", treeland_dde_shell_v1_types + 0 },
	{ "start_drag", "", treeland_dde_shell_v1_types + 0 },
	{ "drop", "", treeland_dde_shell_v1_types + 0 },
};

WL_PRIVATE const struct wl_interface treeland_dde_active_v1_interface = {
	"treeland_dde_active_v1", 1,
	1, treeland_dde_active_v1_requests,
	4, treeland_dde_active_v1_events,
};

static const struct wl_message treeland_multitaskview_v1_requests[] = {
	{ "destroy", "", treeland_dde_shell_v1_types + 0 },
	{ "toggle", "", treeland_dde_shell_v1_types + 0 },
};

WL_PRIVATE const struct wl_interface treeland_multitaskview_v1_interface = {
	"treeland_multitaskview_v1", 1,
	2, treeland_multitaskview_v1_requests,
	0, NULL,
};

static const struct wl_message treeland_window_picker_v1_requests[] = {
	{ "destroy", "", treeland_dde_shell_v1_types + 0 },
	{ "pick", "s", treeland_dde_shell_v1_types + 0 },
};

static const struct wl_message treeland_window_picker_v1_events[] = {
	{ "window", "i", treeland_dde_shell_v1_types + 0 },
};

WL_PRIVATE const struct wl_interface treeland_window_picker_v1_interface = {
	"treeland_window_picker_v1", 1,
	2, treeland_window_picker_v1_requests,
	1, treeland_window_picker_v1_events,
};

static const struct wl_message treeland_lockscreen_v1_requests[] = {
	{ "destroy", "", treeland_dde_shell_v1_types + 0 },
	{ "lock", "", treeland_dde_shell_v1_types + 0 },
	{ "shutdown", "", treeland_dde_shell_v1_types + 0 },
	{ "switch_user", "", treeland_dde_shell_v1_types + 0 },
};

WL_PRIVATE const struct wl_interface treeland_lockscreen_v1_interface = {
	"treeland_lockscreen_v1", 1,
	4, treeland_lockscreen_v1_requests,
	0, NULL,
};

