/****************************************************************************
** Meta object code from reading C++ file 'seatuserconfig.hpp'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../seatuserconfig.hpp"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'seatuserconfig.hpp' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15SeatUserDConfigE_t {};
} // unnamed namespace

template <> constexpr inline auto SeatUserDConfig::qt_create_metaobjectdata<qt_meta_tag_ZN15SeatUserDConfigE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SeatUserDConfig",
        "DConfigKeyList",
        "keyboardDelay;keyboardRate;mouseAccelSpeed;mouseAccelerationProfil"
        "e;mouseNaturalScroll;pointerHandMode;pointerScrollFactor;touchpadA"
        "ccelSpeed;touchpadAccelerationProfile;touchpadDisableWhileTyping;t"
        "ouchpadHoldTimeoutMs;touchpadNaturalScroll;touchpadSendEventsMode;"
        "touchpadTapToClick",
        "DConfigFileName",
        "org.deepin.dde.treeland.user.seat",
        "DConfigFileVersion",
        "1.0",
        "configInitializeFailed",
        "",
        "configInitializeSucceed",
        "Dtk::Core::DConfig*",
        "config",
        "valueChanged",
        "key",
        "QVariant",
        "value",
        "keyboardDelayChanged",
        "keyboardRateChanged",
        "mouseAccelSpeedChanged",
        "mouseAccelerationProfileChanged",
        "mouseNaturalScrollChanged",
        "pointerHandModeChanged",
        "pointerScrollFactorChanged",
        "touchpadAccelSpeedChanged",
        "touchpadAccelerationProfileChanged",
        "touchpadDisableWhileTypingChanged",
        "touchpadHoldTimeoutMsChanged",
        "touchpadNaturalScrollChanged",
        "touchpadSendEventsModeChanged",
        "touchpadTapToClickChanged",
        "isInitializeSucceed",
        "isInitializeSucceeded",
        "isInitializeFailed",
        "isInitializing",
        "keyList",
        "isDefaultValue",
        "keyboardDelayIsDefaultValue",
        "keyboardRateIsDefaultValue",
        "mouseAccelSpeedIsDefaultValue",
        "mouseAccelerationProfileIsDefaultValue",
        "mouseNaturalScrollIsDefaultValue",
        "pointerHandModeIsDefaultValue",
        "pointerScrollFactorIsDefaultValue",
        "touchpadAccelSpeedIsDefaultValue",
        "touchpadAccelerationProfileIsDefaultValue",
        "touchpadDisableWhileTypingIsDefaultValue",
        "touchpadHoldTimeoutMsIsDefaultValue",
        "touchpadNaturalScrollIsDefaultValue",
        "touchpadSendEventsModeIsDefaultValue",
        "touchpadTapToClickIsDefaultValue",
        "keyboardDelay",
        "keyboardRate",
        "mouseAccelSpeed",
        "mouseAccelerationProfile",
        "mouseNaturalScroll",
        "pointerHandMode",
        "pointerScrollFactor",
        "touchpadAccelSpeed",
        "touchpadAccelerationProfile",
        "touchpadDisableWhileTyping",
        "touchpadHoldTimeoutMs",
        "touchpadNaturalScroll",
        "touchpadSendEventsMode",
        "touchpadTapToClick"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'configInitializeFailed'
        QtMocHelpers::SignalData<void()>(7, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'configInitializeSucceed'
        QtMocHelpers::SignalData<void(Dtk::Core::DConfig *)>(9, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'valueChanged'
        QtMocHelpers::SignalData<void(const QString &, const QVariant &)>(12, 8, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 }, { 0x80000000 | 14, 15 },
        }}),
        // Signal 'keyboardDelayChanged'
        QtMocHelpers::SignalData<void()>(16, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'keyboardRateChanged'
        QtMocHelpers::SignalData<void()>(17, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'mouseAccelSpeedChanged'
        QtMocHelpers::SignalData<void()>(18, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'mouseAccelerationProfileChanged'
        QtMocHelpers::SignalData<void()>(19, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'mouseNaturalScrollChanged'
        QtMocHelpers::SignalData<void()>(20, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pointerHandModeChanged'
        QtMocHelpers::SignalData<void()>(21, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'pointerScrollFactorChanged'
        QtMocHelpers::SignalData<void()>(22, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'touchpadAccelSpeedChanged'
        QtMocHelpers::SignalData<void()>(23, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'touchpadAccelerationProfileChanged'
        QtMocHelpers::SignalData<void()>(24, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'touchpadDisableWhileTypingChanged'
        QtMocHelpers::SignalData<void()>(25, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'touchpadHoldTimeoutMsChanged'
        QtMocHelpers::SignalData<void()>(26, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'touchpadNaturalScrollChanged'
        QtMocHelpers::SignalData<void()>(27, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'touchpadSendEventsModeChanged'
        QtMocHelpers::SignalData<void()>(28, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'touchpadTapToClickChanged'
        QtMocHelpers::SignalData<void()>(29, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'config'
        QtMocHelpers::MethodData<Dtk::Core::DConfig *() const>(11, 8, QMC::AccessPublic, 0x80000000 | 10),
        // Method 'isInitializeSucceed'
        QtMocHelpers::MethodData<bool() const>(30, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'isInitializeSucceeded'
        QtMocHelpers::MethodData<bool() const>(31, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'isInitializeFailed'
        QtMocHelpers::MethodData<bool() const>(32, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'isInitializing'
        QtMocHelpers::MethodData<bool() const>(33, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'keyList'
        QtMocHelpers::MethodData<QStringList() const>(34, 8, QMC::AccessPublic, QMetaType::QStringList),
        // Method 'isDefaultValue'
        QtMocHelpers::MethodData<bool(const QString &) const>(35, 8, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 13 },
        }}),
        // Method 'keyboardDelayIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(36, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'keyboardRateIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(37, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'mouseAccelSpeedIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(38, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'mouseAccelerationProfileIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(39, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'mouseNaturalScrollIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(40, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'pointerHandModeIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(41, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'pointerScrollFactorIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(42, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'touchpadAccelSpeedIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(43, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'touchpadAccelerationProfileIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(44, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'touchpadDisableWhileTypingIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(45, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'touchpadHoldTimeoutMsIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(46, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'touchpadNaturalScrollIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(47, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'touchpadSendEventsModeIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(48, 8, QMC::AccessPublic, QMetaType::Bool),
        // Method 'touchpadTapToClickIsDefaultValue'
        QtMocHelpers::MethodData<bool() const>(49, 8, QMC::AccessPublic, QMetaType::Bool),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'keyboardDelay'
        QtMocHelpers::PropertyData<qlonglong>(50, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 3),
        // property 'keyboardRate'
        QtMocHelpers::PropertyData<qlonglong>(51, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 4),
        // property 'mouseAccelSpeed'
        QtMocHelpers::PropertyData<double>(52, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 5),
        // property 'mouseAccelerationProfile'
        QtMocHelpers::PropertyData<qlonglong>(53, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 6),
        // property 'mouseNaturalScroll'
        QtMocHelpers::PropertyData<bool>(54, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 7),
        // property 'pointerHandMode'
        QtMocHelpers::PropertyData<QString>(55, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 8),
        // property 'pointerScrollFactor'
        QtMocHelpers::PropertyData<double>(56, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 9),
        // property 'touchpadAccelSpeed'
        QtMocHelpers::PropertyData<double>(57, QMetaType::Double, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 10),
        // property 'touchpadAccelerationProfile'
        QtMocHelpers::PropertyData<qlonglong>(58, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 11),
        // property 'touchpadDisableWhileTyping'
        QtMocHelpers::PropertyData<bool>(59, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 12),
        // property 'touchpadHoldTimeoutMs'
        QtMocHelpers::PropertyData<qlonglong>(60, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 13),
        // property 'touchpadNaturalScroll'
        QtMocHelpers::PropertyData<bool>(61, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 14),
        // property 'touchpadSendEventsMode'
        QtMocHelpers::PropertyData<qlonglong>(62, QMetaType::LongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 15),
        // property 'touchpadTapToClick'
        QtMocHelpers::PropertyData<bool>(63, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::Resettable | QMC::StdCppSet, 16),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<SeatUserDConfig, qt_meta_tag_ZN15SeatUserDConfigE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject SeatUserDConfig::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15SeatUserDConfigE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15SeatUserDConfigE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15SeatUserDConfigE_t>.metaTypes,
    nullptr
} };

void SeatUserDConfig::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SeatUserDConfig *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->configInitializeFailed(); break;
        case 1: _t->configInitializeSucceed((*reinterpret_cast<std::add_pointer_t<Dtk::Core::DConfig*>>(_a[1]))); break;
        case 2: _t->valueChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2]))); break;
        case 3: _t->keyboardDelayChanged(); break;
        case 4: _t->keyboardRateChanged(); break;
        case 5: _t->mouseAccelSpeedChanged(); break;
        case 6: _t->mouseAccelerationProfileChanged(); break;
        case 7: _t->mouseNaturalScrollChanged(); break;
        case 8: _t->pointerHandModeChanged(); break;
        case 9: _t->pointerScrollFactorChanged(); break;
        case 10: _t->touchpadAccelSpeedChanged(); break;
        case 11: _t->touchpadAccelerationProfileChanged(); break;
        case 12: _t->touchpadDisableWhileTypingChanged(); break;
        case 13: _t->touchpadHoldTimeoutMsChanged(); break;
        case 14: _t->touchpadNaturalScrollChanged(); break;
        case 15: _t->touchpadSendEventsModeChanged(); break;
        case 16: _t->touchpadTapToClickChanged(); break;
        case 17: { Dtk::Core::DConfig* _r = _t->config();
            if (_a[0]) *reinterpret_cast<Dtk::Core::DConfig**>(_a[0]) = std::move(_r); }  break;
        case 18: { bool _r = _t->isInitializeSucceed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 19: { bool _r = _t->isInitializeSucceeded();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 20: { bool _r = _t->isInitializeFailed();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 21: { bool _r = _t->isInitializing();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 22: { QStringList _r = _t->keyList();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 23: { bool _r = _t->isDefaultValue((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 24: { bool _r = _t->keyboardDelayIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 25: { bool _r = _t->keyboardRateIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 26: { bool _r = _t->mouseAccelSpeedIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 27: { bool _r = _t->mouseAccelerationProfileIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 28: { bool _r = _t->mouseNaturalScrollIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 29: { bool _r = _t->pointerHandModeIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 30: { bool _r = _t->pointerScrollFactorIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 31: { bool _r = _t->touchpadAccelSpeedIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 32: { bool _r = _t->touchpadAccelerationProfileIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 33: { bool _r = _t->touchpadDisableWhileTypingIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 34: { bool _r = _t->touchpadHoldTimeoutMsIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 35: { bool _r = _t->touchpadNaturalScrollIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 36: { bool _r = _t->touchpadSendEventsModeIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 37: { bool _r = _t->touchpadTapToClickIsDefaultValue();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Dtk::Core::DConfig* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SeatUserDConfig::*)()>(_a, &SeatUserDConfig::configInitializeFailed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SeatUserDConfig::*)(Dtk::Core::DConfig * )>(_a, &SeatUserDConfig::configInitializeSucceed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (SeatUserDConfig::*)(const QString & , const QVariant & )>(_a, &SeatUserDConfig::valueChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (SeatUserDConfig::*)()>(_a, &SeatUserDConfig::keyboardDelayChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (SeatUserDConfig::*)()>(_a, &SeatUserDConfig::keyboardRateChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (SeatUserDConfig::*)()>(_a, &SeatUserDConfig::mouseAccelSpeedChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (SeatUserDConfig::*)()>(_a, &SeatUserDConfig::mouseAccelerationProfileChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (SeatUserDConfig::*)()>(_a, &SeatUserDConfig::mouseNaturalScrollChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (SeatUserDConfig::*)()>(_a, &SeatUserDConfig::pointerHandModeChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (SeatUserDConfig::*)()>(_a, &SeatUserDConfig::pointerScrollFactorChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (SeatUserDConfig::*)()>(_a, &SeatUserDConfig::touchpadAccelSpeedChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (SeatUserDConfig::*)()>(_a, &SeatUserDConfig::touchpadAccelerationProfileChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (SeatUserDConfig::*)()>(_a, &SeatUserDConfig::touchpadDisableWhileTypingChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (SeatUserDConfig::*)()>(_a, &SeatUserDConfig::touchpadHoldTimeoutMsChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (SeatUserDConfig::*)()>(_a, &SeatUserDConfig::touchpadNaturalScrollChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (SeatUserDConfig::*)()>(_a, &SeatUserDConfig::touchpadSendEventsModeChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (SeatUserDConfig::*)()>(_a, &SeatUserDConfig::touchpadTapToClickChanged, 16))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<qlonglong*>(_v) = _t->keyboardDelay(); break;
        case 1: *reinterpret_cast<qlonglong*>(_v) = _t->keyboardRate(); break;
        case 2: *reinterpret_cast<double*>(_v) = _t->mouseAccelSpeed(); break;
        case 3: *reinterpret_cast<qlonglong*>(_v) = _t->mouseAccelerationProfile(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->mouseNaturalScroll(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->pointerHandMode(); break;
        case 6: *reinterpret_cast<double*>(_v) = _t->pointerScrollFactor(); break;
        case 7: *reinterpret_cast<double*>(_v) = _t->touchpadAccelSpeed(); break;
        case 8: *reinterpret_cast<qlonglong*>(_v) = _t->touchpadAccelerationProfile(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->touchpadDisableWhileTyping(); break;
        case 10: *reinterpret_cast<qlonglong*>(_v) = _t->touchpadHoldTimeoutMs(); break;
        case 11: *reinterpret_cast<bool*>(_v) = _t->touchpadNaturalScroll(); break;
        case 12: *reinterpret_cast<qlonglong*>(_v) = _t->touchpadSendEventsMode(); break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->touchpadTapToClick(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setKeyboardDelay(*reinterpret_cast<qlonglong*>(_v)); break;
        case 1: _t->setKeyboardRate(*reinterpret_cast<qlonglong*>(_v)); break;
        case 2: _t->setMouseAccelSpeed(*reinterpret_cast<double*>(_v)); break;
        case 3: _t->setMouseAccelerationProfile(*reinterpret_cast<qlonglong*>(_v)); break;
        case 4: _t->setMouseNaturalScroll(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setPointerHandMode(*reinterpret_cast<QString*>(_v)); break;
        case 6: _t->setPointerScrollFactor(*reinterpret_cast<double*>(_v)); break;
        case 7: _t->setTouchpadAccelSpeed(*reinterpret_cast<double*>(_v)); break;
        case 8: _t->setTouchpadAccelerationProfile(*reinterpret_cast<qlonglong*>(_v)); break;
        case 9: _t->setTouchpadDisableWhileTyping(*reinterpret_cast<bool*>(_v)); break;
        case 10: _t->setTouchpadHoldTimeoutMs(*reinterpret_cast<qlonglong*>(_v)); break;
        case 11: _t->setTouchpadNaturalScroll(*reinterpret_cast<bool*>(_v)); break;
        case 12: _t->setTouchpadSendEventsMode(*reinterpret_cast<qlonglong*>(_v)); break;
        case 13: _t->setTouchpadTapToClick(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
    if (_c == QMetaObject::ResetProperty) {
        switch (_id) {
        case 0: _t->resetKeyboardDelay(); break;
        case 1: _t->resetKeyboardRate(); break;
        case 2: _t->resetMouseAccelSpeed(); break;
        case 3: _t->resetMouseAccelerationProfile(); break;
        case 4: _t->resetMouseNaturalScroll(); break;
        case 5: _t->resetPointerHandMode(); break;
        case 6: _t->resetPointerScrollFactor(); break;
        case 7: _t->resetTouchpadAccelSpeed(); break;
        case 8: _t->resetTouchpadAccelerationProfile(); break;
        case 9: _t->resetTouchpadDisableWhileTyping(); break;
        case 10: _t->resetTouchpadHoldTimeoutMs(); break;
        case 11: _t->resetTouchpadNaturalScroll(); break;
        case 12: _t->resetTouchpadSendEventsMode(); break;
        case 13: _t->resetTouchpadTapToClick(); break;
        default: break;
        }
    }
}

const QMetaObject *SeatUserDConfig::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SeatUserDConfig::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15SeatUserDConfigE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int SeatUserDConfig::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 38)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 38;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 38)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 38;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void SeatUserDConfig::configInitializeFailed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void SeatUserDConfig::configInitializeSucceed(Dtk::Core::DConfig * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void SeatUserDConfig::valueChanged(const QString & _t1, const QVariant & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void SeatUserDConfig::keyboardDelayChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void SeatUserDConfig::keyboardRateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void SeatUserDConfig::mouseAccelSpeedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void SeatUserDConfig::mouseAccelerationProfileChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void SeatUserDConfig::mouseNaturalScrollChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void SeatUserDConfig::pointerHandModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void SeatUserDConfig::pointerScrollFactorChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void SeatUserDConfig::touchpadAccelSpeedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void SeatUserDConfig::touchpadAccelerationProfileChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void SeatUserDConfig::touchpadDisableWhileTypingChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void SeatUserDConfig::touchpadHoldTimeoutMsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void SeatUserDConfig::touchpadNaturalScrollChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void SeatUserDConfig::touchpadSendEventsModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void SeatUserDConfig::touchpadTapToClickChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}
QT_WARNING_POP
