/****************************************************************************
** Meta object code from reading C++ file 'wallpapermanagerinterfacev1.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/modules/wallpaper/wallpapermanagerinterfacev1.h"
#include "private/wserver_p.h"
#include <wsurface.h>
#include "woutput.h"
#include "wcursor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wallpapermanagerinterfacev1.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN35TreelandWallpaperManagerInterfaceV1E_t {};
} // unnamed namespace

template <> constexpr inline auto TreelandWallpaperManagerInterfaceV1::qt_create_metaobjectdata<qt_meta_tag_ZN35TreelandWallpaperManagerInterfaceV1E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TreelandWallpaperManagerInterfaceV1",
        "wallpaperCreated",
        "",
        "TreelandWallpaperInterfaceV1*",
        "interface"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'wallpaperCreated'
        QtMocHelpers::SignalData<void(TreelandWallpaperInterfaceV1 *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TreelandWallpaperManagerInterfaceV1, qt_meta_tag_ZN35TreelandWallpaperManagerInterfaceV1E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TreelandWallpaperManagerInterfaceV1::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35TreelandWallpaperManagerInterfaceV1E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35TreelandWallpaperManagerInterfaceV1E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN35TreelandWallpaperManagerInterfaceV1E_t>.metaTypes,
    nullptr
} };

void TreelandWallpaperManagerInterfaceV1::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TreelandWallpaperManagerInterfaceV1 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->wallpaperCreated((*reinterpret_cast<std::add_pointer_t<TreelandWallpaperInterfaceV1*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TreelandWallpaperInterfaceV1* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TreelandWallpaperManagerInterfaceV1::*)(TreelandWallpaperInterfaceV1 * )>(_a, &TreelandWallpaperManagerInterfaceV1::wallpaperCreated, 0))
            return;
    }
}

const QMetaObject *TreelandWallpaperManagerInterfaceV1::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TreelandWallpaperManagerInterfaceV1::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35TreelandWallpaperManagerInterfaceV1E_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "WServerInterface"))
        return static_cast< WServerInterface*>(this);
    return QObject::qt_metacast(_clname);
}

int TreelandWallpaperManagerInterfaceV1::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void TreelandWallpaperManagerInterfaceV1::wallpaperCreated(TreelandWallpaperInterfaceV1 * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN28TreelandWallpaperInterfaceV1E_t {};
} // unnamed namespace

template <> constexpr inline auto TreelandWallpaperInterfaceV1::qt_create_metaobjectdata<qt_meta_tag_ZN28TreelandWallpaperInterfaceV1E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TreelandWallpaperInterfaceV1",
        "imageSourceChanged",
        "",
        "workspaceIndex",
        "fileSource",
        "WallpaperRoles",
        "roles",
        "videoSourceChanged",
        "WallpaperRole",
        "Desktop",
        "Lockscreen"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'imageSourceChanged'
        QtMocHelpers::SignalData<void(int, const QString &, WallpaperRoles)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::QString, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Signal 'videoSourceChanged'
        QtMocHelpers::SignalData<void(int, const QString &, WallpaperRoles)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::QString, 4 }, { 0x80000000 | 5, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // flag 'WallpaperRoles'
        QtMocHelpers::EnumData<WallpaperRoles>(5, 8, QMC::EnumIsFlag).add({
            {    9, WallpaperRole::Desktop },
            {   10, WallpaperRole::Lockscreen },
        }),
    };
    return QtMocHelpers::metaObjectData<TreelandWallpaperInterfaceV1, qt_meta_tag_ZN28TreelandWallpaperInterfaceV1E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TreelandWallpaperInterfaceV1::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28TreelandWallpaperInterfaceV1E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28TreelandWallpaperInterfaceV1E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN28TreelandWallpaperInterfaceV1E_t>.metaTypes,
    nullptr
} };

void TreelandWallpaperInterfaceV1::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TreelandWallpaperInterfaceV1 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->imageSourceChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<WallpaperRoles>>(_a[3]))); break;
        case 1: _t->videoSourceChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<WallpaperRoles>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TreelandWallpaperInterfaceV1::*)(int , const QString & , WallpaperRoles )>(_a, &TreelandWallpaperInterfaceV1::imageSourceChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TreelandWallpaperInterfaceV1::*)(int , const QString & , WallpaperRoles )>(_a, &TreelandWallpaperInterfaceV1::videoSourceChanged, 1))
            return;
    }
}

const QMetaObject *TreelandWallpaperInterfaceV1::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TreelandWallpaperInterfaceV1::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28TreelandWallpaperInterfaceV1E_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int TreelandWallpaperInterfaceV1::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void TreelandWallpaperInterfaceV1::imageSourceChanged(int _t1, const QString & _t2, WallpaperRoles _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void TreelandWallpaperInterfaceV1::videoSourceChanged(int _t1, const QString & _t2, WallpaperRoles _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
