/****************************************************************************
** Meta object code from reading C++ file 'wsessionlock.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../waylib/src/server/protocols/wsessionlock.h"
#include "private/wserver_p.h"
#include "wsessionlocksurface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'wsessionlock.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6Waylib6Server12WSessionLockE_t {};
} // unnamed namespace

template <> constexpr inline auto Waylib::Server::WSessionLock::qt_create_metaobjectdata<qt_meta_tag_ZN6Waylib6Server12WSessionLockE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Waylib::Server::WSessionLock",
        "QML.Element",
        "WSessionLock",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "Only create in C++",
        "surfaceAdded",
        "",
        "WSessionLockSurface*",
        "surface",
        "surfaceRemoved",
        "locked",
        "unlocked",
        "finished",
        "abandoned",
        "canceled",
        "lock",
        "finish",
        "LockState"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'surfaceAdded'
        QtMocHelpers::SignalData<void(WSessionLockSurface *)>(7, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'surfaceRemoved'
        QtMocHelpers::SignalData<void(WSessionLockSurface *)>(11, 8, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'locked'
        QtMocHelpers::SignalData<void()>(12, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'unlocked'
        QtMocHelpers::SignalData<void()>(13, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'finished'
        QtMocHelpers::SignalData<void()>(14, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'abandoned'
        QtMocHelpers::SignalData<void()>(15, 8, QMC::AccessPublic, QMetaType::Void),
        // Signal 'canceled'
        QtMocHelpers::SignalData<void()>(16, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'lock'
        QtMocHelpers::MethodData<void()>(17, 8, QMC::AccessPublic, QMetaType::Void),
        // Method 'finish'
        QtMocHelpers::MethodData<void()>(18, 8, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'locked'
        QtMocHelpers::PropertyData<enum LockState>(12, 0x80000000 | 19, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<WSessionLock, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject Waylib::Server::WSessionLock::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server12WSessionLockE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server12WSessionLockE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6Waylib6Server12WSessionLockE_t>.metaTypes,
    nullptr
} };

void Waylib::Server::WSessionLock::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<WSessionLock *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->surfaceAdded((*reinterpret_cast<std::add_pointer_t<WSessionLockSurface*>>(_a[1]))); break;
        case 1: _t->surfaceRemoved((*reinterpret_cast<std::add_pointer_t<WSessionLockSurface*>>(_a[1]))); break;
        case 2: _t->locked(); break;
        case 3: _t->unlocked(); break;
        case 4: _t->finished(); break;
        case 5: _t->abandoned(); break;
        case 6: _t->canceled(); break;
        case 7: _t->lock(); break;
        case 8: _t->finish(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (WSessionLock::*)(WSessionLockSurface * )>(_a, &WSessionLock::surfaceAdded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSessionLock::*)(WSessionLockSurface * )>(_a, &WSessionLock::surfaceRemoved, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSessionLock::*)()>(_a, &WSessionLock::locked, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSessionLock::*)()>(_a, &WSessionLock::unlocked, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSessionLock::*)()>(_a, &WSessionLock::finished, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSessionLock::*)()>(_a, &WSessionLock::abandoned, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (WSessionLock::*)()>(_a, &WSessionLock::canceled, 6))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<enum LockState*>(_v) = _t->lockState(); break;
        default: break;
        }
    }
}

const QMetaObject *Waylib::Server::WSessionLock::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Waylib::Server::WSessionLock::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6Waylib6Server12WSessionLockE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "WObject"))
        return static_cast< WObject*>(this);
    return QObject::qt_metacast(_clname);
}

int Waylib::Server::WSessionLock::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void Waylib::Server::WSessionLock::surfaceAdded(WSessionLockSurface * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Waylib::Server::WSessionLock::surfaceRemoved(WSessionLockSurface * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Waylib::Server::WSessionLock::locked()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void Waylib::Server::WSessionLock::unlocked()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void Waylib::Server::WSessionLock::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void Waylib::Server::WSessionLock::abandoned()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Waylib::Server::WSessionLock::canceled()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
