/****************************************************************************
** Meta object code from reading C++ file 'togglablegesture.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../src/input/togglablegesture.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'togglablegesture.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16TogglableGestureE_t {};
} // unnamed namespace

template <> constexpr inline auto TogglableGesture::qt_create_metaobjectdata<qt_meta_tag_ZN16TogglableGestureE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TogglableGesture",
        "inProgressChanged",
        "",
        "partialGestureFactorChanged",
        "factor",
        "activated",
        "deactivated",
        "hold",
        "longPressed",
        "statusChanged",
        "Status",
        "status",
        "toggle",
        "on",
        "stop",
        "inProgress",
        "partialGestureFactor",
        "Inactive",
        "Activating",
        "Deactivating",
        "Active",
        "Stopped"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'inProgressChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'partialGestureFactorChanged'
        QtMocHelpers::SignalData<void(qreal)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 4 },
        }}),
        // Signal 'activated'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'deactivated'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hold'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'longPressed'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'statusChanged'
        QtMocHelpers::SignalData<void(enum Status)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Method 'toggle'
        QtMocHelpers::MethodData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
        // Method 'toggle'
        QtMocHelpers::MethodData<void()>(12, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Method 'stop'
        QtMocHelpers::MethodData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'inProgress'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 0),
        // property 'partialGestureFactor'
        QtMocHelpers::PropertyData<qreal>(16, QMetaType::QReal, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet | QMC::Final, 1),
        // property 'status'
        QtMocHelpers::PropertyData<enum Status>(11, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet | QMC::Final, 6),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Status'
        QtMocHelpers::EnumData<enum Status>(10, 10, QMC::EnumFlags{}).add({
            {   17, Status::Inactive },
            {   18, Status::Activating },
            {   19, Status::Deactivating },
            {   20, Status::Active },
            {   21, Status::Stopped },
        }),
    };
    return QtMocHelpers::metaObjectData<TogglableGesture, qt_meta_tag_ZN16TogglableGestureE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TogglableGesture::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16TogglableGestureE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16TogglableGestureE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16TogglableGestureE_t>.metaTypes,
    nullptr
} };

void TogglableGesture::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TogglableGesture *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->inProgressChanged(); break;
        case 1: _t->partialGestureFactorChanged((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1]))); break;
        case 2: _t->activated(); break;
        case 3: _t->deactivated(); break;
        case 4: _t->hold(); break;
        case 5: _t->longPressed(); break;
        case 6: _t->statusChanged((*reinterpret_cast<std::add_pointer_t<enum Status>>(_a[1]))); break;
        case 7: _t->toggle((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->toggle(); break;
        case 9: _t->stop(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TogglableGesture::*)()>(_a, &TogglableGesture::inProgressChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TogglableGesture::*)(qreal )>(_a, &TogglableGesture::partialGestureFactorChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TogglableGesture::*)()>(_a, &TogglableGesture::activated, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TogglableGesture::*)()>(_a, &TogglableGesture::deactivated, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (TogglableGesture::*)()>(_a, &TogglableGesture::hold, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (TogglableGesture::*)()>(_a, &TogglableGesture::longPressed, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (TogglableGesture::*)(Status )>(_a, &TogglableGesture::statusChanged, 6))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->inProgress(); break;
        case 1: *reinterpret_cast<qreal*>(_v) = _t->partialGestureFactor(); break;
        case 2: *reinterpret_cast<enum Status*>(_v) = _t->status(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setInProgress(*reinterpret_cast<bool*>(_v)); break;
        case 1: _t->setPartialGestureFactor(*reinterpret_cast<qreal*>(_v)); break;
        case 2: _t->setStatus(*reinterpret_cast<enum Status*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *TogglableGesture::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TogglableGesture::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16TogglableGestureE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int TogglableGesture::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void TogglableGesture::inProgressChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void TogglableGesture::partialGestureFactorChanged(qreal _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void TogglableGesture::activated()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void TogglableGesture::deactivated()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void TogglableGesture::hold()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void TogglableGesture::longPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void TogglableGesture::statusChanged(Status _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}
QT_WARNING_POP
