/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<wbufferitem.h>)
#  include <wbufferitem.h>
#endif
#if __has_include(<wcursor.h>)
#  include <wcursor.h>
#endif
#if __has_include(<weventjunkman.h>)
#  include <weventjunkman.h>
#endif
#if __has_include(<wglobal.h>)
#  include <wglobal.h>
#endif
#if __has_include(<winputpopupsurface.h>)
#  include <winputpopupsurface.h>
#endif
#if __has_include(<winputpopupsurfaceitem.h>)
#  include <winputpopupsurfaceitem.h>
#endif
#if __has_include(<wlayersurface.h>)
#  include <wlayersurface.h>
#endif
#if __has_include(<wlayersurfaceitem.h>)
#  include <wlayersurfaceitem.h>
#endif
#if __has_include(<woutput.h>)
#  include <woutput.h>
#endif
#if __has_include(<woutputitem.h>)
#  include <woutputitem.h>
#endif
#if __has_include(<woutputitem_p.h>)
#  include <woutputitem_p.h>
#endif
#if __has_include(<woutputlayer.h>)
#  include <woutputlayer.h>
#endif
#if __has_include(<woutputlayoutitem.h>)
#  include <woutputlayoutitem.h>
#endif
#if __has_include(<woutputrenderwindow.h>)
#  include <woutputrenderwindow.h>
#endif
#if __has_include(<woutputviewport.h>)
#  include <woutputviewport.h>
#endif
#if __has_include(<wqmlcreator.h>)
#  include <wqmlcreator.h>
#endif
#if __has_include(<wqmlcreator_p.h>)
#  include <wqmlcreator_p.h>
#endif
#if __has_include(<wqmlhelper_p.h>)
#  include <wqmlhelper_p.h>
#endif
#if __has_include(<wquickcoordmapper_p.h>)
#  include <wquickcoordmapper_p.h>
#endif
#if __has_include(<wquickcursor.h>)
#  include <wquickcursor.h>
#endif
#if __has_include(<wquickobserver.h>)
#  include <wquickobserver.h>
#endif
#if __has_include(<wquickoutputlayout.h>)
#  include <wquickoutputlayout.h>
#endif
#if __has_include(<wquicksocketattached_p.h>)
#  include <wquicksocketattached_p.h>
#endif
#if __has_include(<wquicktextureproxy.h>)
#  include <wquicktextureproxy.h>
#endif
#if __has_include(<wrenderbufferblitter.h>)
#  include <wrenderbufferblitter.h>
#endif
#if __has_include(<wseat.h>)
#  include <wseat.h>
#endif
#if __has_include(<wsessionlock.h>)
#  include <wsessionlock.h>
#endif
#if __has_include(<wsessionlocksurface.h>)
#  include <wsessionlocksurface.h>
#endif
#if __has_include(<wsocket.h>)
#  include <wsocket.h>
#endif
#if __has_include(<wsurface.h>)
#  include <wsurface.h>
#endif
#if __has_include(<wsurfaceitem.h>)
#  include <wsurfaceitem.h>
#endif
#if __has_include(<wtoplevelsurface.h>)
#  include <wtoplevelsurface.h>
#endif
#if __has_include(<wxdgdecorationmanager.h>)
#  include <wxdgdecorationmanager.h>
#endif
#if __has_include(<wxdgpopupsurface.h>)
#  include <wxdgpopupsurface.h>
#endif
#if __has_include(<wxdgpopupsurfaceitem.h>)
#  include <wxdgpopupsurfaceitem.h>
#endif
#if __has_include(<wxdgsurface.h>)
#  include <wxdgsurface.h>
#endif
#if __has_include(<wxdgtoplevelsurface.h>)
#  include <wxdgtoplevelsurface.h>
#endif
#if __has_include(<wxdgtoplevelsurfaceitem.h>)
#  include <wxdgtoplevelsurfaceitem.h>
#endif
#if __has_include(<wxdgtopleveltagmanager.h>)
#  include <wxdgtopleveltagmanager.h>
#endif
#if __has_include(<wxwaylandsurface.h>)
#  include <wxwaylandsurface.h>
#endif
#if __has_include(<wxwaylandsurfaceitem.h>)
#  include <wxwaylandsurfaceitem.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
Q_QMLTYPE_EXPORT void qml_register_types_Waylib_Server()
{
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    qmlRegisterTypesAndRevisions<Waylib::Server::WAbstractCreatorComponent>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WBufferItem>("Waylib.Server", 1);
    qmlRegisterAnonymousType<QQuickItem, 254>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WClient>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WCursor>("Waylib.Server", 1);
    QMetaType::fromType<WAYLIB_SERVER_NAMESPACE::WGlobal>().id();
    qmlRegisterNamespaceAndRevisions(&WAYLIB_SERVER_NAMESPACE::WGlobal::staticMetaObject, "Waylib.Server", 1, nullptr, &Waylib::Server::WCursorShapeQMLProxy::staticMetaObject, nullptr);
    qmlRegisterTypesAndRevisions<Waylib::Server::WEventJunkman>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WGlobal>("Waylib.Server", 1);
    qmlRegisterEnum<Waylib::Server::WGlobal::ColorContentsMode>("Waylib::Server::WGlobal::ColorContentsMode");
    qmlRegisterEnum<Waylib::Server::WGlobal::CursorShape>("Waylib::Server::WGlobal::CursorShape");
    qmlRegisterTypesAndRevisions<Waylib::Server::WInputPopupSurface>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WInputPopupSurfaceItem>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WLayerSurface>("Waylib.Server", 1);
    qmlRegisterEnum<Waylib::Server::WLayerSurface::LayerType>("Waylib::Server::WLayerSurface::LayerType");
    qmlRegisterEnum<Waylib::Server::WLayerSurface::AnchorType>("Waylib::Server::WLayerSurface::AnchorType");
    qmlRegisterEnum<Waylib::Server::WLayerSurface::KeyboardInteractivity>("Waylib::Server::WLayerSurface::KeyboardInteractivity");
    qmlRegisterTypesAndRevisions<Waylib::Server::WLayerSurfaceItem>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WOutput>("Waylib.Server", 1);
    qmlRegisterEnum<Waylib::Server::WOutput::Transform>("Waylib::Server::WOutput::Transform");
    qmlRegisterTypesAndRevisions<Waylib::Server::WOutputCursor>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WOutputItem>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WOutputLayer>("Waylib.Server", 1);
    qmlRegisterEnum<Waylib::Server::WOutputLayer::Flag>("Waylib::Server::WOutputLayer::Flag");
    QMetaType::fromType<Waylib::Server::WOutputLayout *>().id();
    qmlRegisterEnum<Waylib::Server::WOutputLayout::Layer>("Waylib::Server::WOutputLayout::Layer");
    qmlRegisterTypesAndRevisions<Waylib::Server::WOutputLayoutItem>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WOutputRenderWindow>("Waylib.Server", 1);
    qmlRegisterAnonymousType<QQuickWindow, 254>("Waylib.Server", 1);
    qmlRegisterAnonymousType<QWindow, 254>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WOutputViewport>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WQmlCreator>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WQmlCreatorComponent>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WQmlCreatorDataWatcher>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WQmlHelper>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WQuickCoordMapper>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WQuickCoordMapperAttached>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WQuickCoordMapperHelper>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WQuickCursor>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WQuickCursorAttached>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WQuickObserver>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WQuickOutputLayout>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WQuickSocketAttached>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WQuickTextureProxy>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WRenderBufferBlitter>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WSeatEventFilter>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WSessionLock>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WSessionLockSurface>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WSurface>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WSurfaceItem>("Waylib.Server", 1);
    qmlRegisterEnum<Waylib::Server::WSurfaceItem::ResizeMode>("Waylib::Server::WSurfaceItem::ResizeMode");
    qmlRegisterEnum<Waylib::Server::WSurfaceItem::Flag>("Waylib::Server::WSurfaceItem::Flag");
    qmlRegisterEnum<Waylib::Server::WSurfaceItem::ZOrder>("Waylib::Server::WSurfaceItem::ZOrder");
    qmlRegisterTypesAndRevisions<Waylib::Server::WSurfaceItemContent>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WToplevelSurface>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WXWaylandSurface>("Waylib.Server", 1);
    qmlRegisterEnum<Waylib::Server::WXWaylandSurface::StackMode>("Waylib::Server::WXWaylandSurface::StackMode");
    qmlRegisterEnum<Waylib::Server::WXWaylandSurface::ConfigureFlag>("Waylib::Server::WXWaylandSurface::ConfigureFlag");
    qmlRegisterEnum<Waylib::Server::WXWaylandSurface::WindowType>("Waylib::Server::WXWaylandSurface::WindowType");
    qmlRegisterEnum<Waylib::Server::WXWaylandSurface::DecorationsFlag>("Waylib::Server::WXWaylandSurface::DecorationsFlag");
    qmlRegisterTypesAndRevisions<Waylib::Server::WXWaylandSurfaceItem>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WXdgDecorationManager>("Waylib.Server", 1);
    qmlRegisterEnum<Waylib::Server::WXdgDecorationManager::DecorationMode>("Waylib::Server::WXdgDecorationManager::DecorationMode");
    qmlRegisterTypesAndRevisions<Waylib::Server::WXdgPopupSurface>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WXdgPopupSurfaceItem>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WXdgSurface>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WXdgToplevelSurface>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WXdgToplevelSurfaceItem>("Waylib.Server", 1);
    qmlRegisterTypesAndRevisions<Waylib::Server::WXdgToplevelTagManagerV1>("Waylib.Server", 1);
    QT_WARNING_POP
    qmlRegisterModule("Waylib.Server", 1, 0);
}

static const QQmlModuleRegistration waylibServerRegistration("Waylib.Server", qml_register_types_Waylib_Server);
