/* Generated by wayland-scanner 1.26.0 */

/*
 * SPDX-FileCopyrightText: 2025 UnionTech Software Technology Co., Ltd.
 * SPDX-License-Identifier: MIT
 */

#include <stdbool.h>
#include <stdlib.h>
#include <stdint.h>
#include "wayland-util.h"

#ifndef __has_attribute
# define __has_attribute(x) 0  /* Compatibility with non-clang compilers. */
#endif

#if (__has_attribute(visibility) || defined(__GNUC__) && __GNUC__ >= 4)
#define WL_PRIVATE __attribute__ ((visibility("hidden")))
#else
#define WL_PRIVATE
#endif

extern const struct wl_interface treeland_wallpaper_v1_interface;
extern const struct wl_interface wl_output_interface;
extern const struct wl_interface wl_surface_interface;

static const struct wl_interface *treeland_wallpaper_manager_unstable_v1_types[] = {
	NULL,
	NULL,
	NULL,
	&treeland_wallpaper_v1_interface,
	&wl_output_interface,
	&wl_surface_interface,
};

static const struct wl_message treeland_wallpaper_manager_v1_requests[] = {
	{ "destroy", "", treeland_wallpaper_manager_unstable_v1_types + 0 },
	{ "get_treeland_wallpaper", "no?o", treeland_wallpaper_manager_unstable_v1_types + 3 },
};

WL_PRIVATE const struct wl_interface treeland_wallpaper_manager_v1_interface = {
	"treeland_wallpaper_manager_v1", 1,
	2, treeland_wallpaper_manager_v1_requests,
	0, NULL,
};

static const struct wl_message treeland_wallpaper_v1_requests[] = {
	{ "destroy", "", treeland_wallpaper_manager_unstable_v1_types + 0 },
	{ "set_image_source", "su", treeland_wallpaper_manager_unstable_v1_types + 0 },
	{ "set_video_source", "su", treeland_wallpaper_manager_unstable_v1_types + 0 },
};

static const struct wl_message treeland_wallpaper_v1_events[] = {
	{ "failed", "su", treeland_wallpaper_manager_unstable_v1_types + 0 },
	{ "changed", "uus", treeland_wallpaper_manager_unstable_v1_types + 0 },
};

WL_PRIVATE const struct wl_interface treeland_wallpaper_v1_interface = {
	"treeland_wallpaper_v1", 1,
	3, treeland_wallpaper_v1_requests,
	2, treeland_wallpaper_v1_events,
};

