// This file was generated by qtwaylandscanner
// source file is treeland-app-id-resolver-v1.xml

#ifndef QT_WAYLAND_TREELAND_APP_ID_RESOLVER_V1
#define QT_WAYLAND_TREELAND_APP_ID_RESOLVER_V1

#include "wayland-treeland-app-id-resolver-v1-client-protocol.h"
#include <QByteArray>
#include <QString>

struct wl_registry;

QT_BEGIN_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_GCC("-Wmissing-field-initializers")
QT_WARNING_DISABLE_CLANG("-Wmissing-field-initializers")

namespace QtWayland {
    class  treeland_app_id_resolver_manager_v1
    {
    public:
        treeland_app_id_resolver_manager_v1(struct ::wl_registry *registry, uint32_t id, int version);
        treeland_app_id_resolver_manager_v1(struct ::treeland_app_id_resolver_manager_v1 *object);
        treeland_app_id_resolver_manager_v1();

        virtual ~treeland_app_id_resolver_manager_v1();

        void init(struct ::wl_registry *registry, uint32_t id, int version);
        void init(struct ::treeland_app_id_resolver_manager_v1 *object);

        struct ::treeland_app_id_resolver_manager_v1 *object() { return m_treeland_app_id_resolver_manager_v1; }
        const struct ::treeland_app_id_resolver_manager_v1 *object() const { return m_treeland_app_id_resolver_manager_v1; }
        static treeland_app_id_resolver_manager_v1 *fromObject(struct ::treeland_app_id_resolver_manager_v1 *object);

        bool isInitialized() const;

        uint32_t version() const;
        static const struct ::wl_interface *interface();

        enum error {
            error_resolver_already_exists = 1, // A resolver is already registered in this session with the manager
        };

        void destroy();
        struct ::treeland_app_id_resolver_v1 *get_resolver();

    private:
        struct ::treeland_app_id_resolver_manager_v1 *m_treeland_app_id_resolver_manager_v1;
    };

    class  treeland_app_id_resolver_v1
    {
    public:
        treeland_app_id_resolver_v1(struct ::wl_registry *registry, uint32_t id, int version);
        treeland_app_id_resolver_v1(struct ::treeland_app_id_resolver_v1 *object);
        treeland_app_id_resolver_v1();

        virtual ~treeland_app_id_resolver_v1();

        void init(struct ::wl_registry *registry, uint32_t id, int version);
        void init(struct ::treeland_app_id_resolver_v1 *object);

        struct ::treeland_app_id_resolver_v1 *object() { return m_treeland_app_id_resolver_v1; }
        const struct ::treeland_app_id_resolver_v1 *object() const { return m_treeland_app_id_resolver_v1; }
        static treeland_app_id_resolver_v1 *fromObject(struct ::treeland_app_id_resolver_v1 *object);

        bool isInitialized() const;

        uint32_t version() const;
        static const struct ::wl_interface *interface();

        void respond(uint32_t request_id, const QString &app_id, const QString &sandbox_engine_name);
        void destroy();

    protected:
        virtual void treeland_app_id_resolver_v1_identify_request(uint32_t request_id, int32_t pidfd);

    private:
        void init_listener();
        static const struct treeland_app_id_resolver_v1_listener m_treeland_app_id_resolver_v1_listener;
        static void handle_identify_request(
            void *data,
            struct ::treeland_app_id_resolver_v1 *object,
            uint32_t request_id,
            int32_t pidfd);
        struct ::treeland_app_id_resolver_v1 *m_treeland_app_id_resolver_v1;
    };
}

QT_WARNING_POP
QT_END_NAMESPACE

#endif
